/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jan 5, 2008
 */

package eu.moonlight3d.ml3d.animation.basic;

import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;

/**
 * Create animation dialog. This is the dialog which used for 
 * creating new animations in a scene.
 *
 * @author gregor
 */
public class CreateAnimationDialog extends QDialog {
	/**
	 * Animation name input field
	 */
	private QLineEdit name;
	/**
	 * Animation start time input field
	 */
	private QLineEdit startTime;
	/**
	 * Animation end time input field
	 */
	private QLineEdit endTime;
	/**
	 * Animation duration input field
	 */
	private QLineEdit duration;
	/**
	 * Frames per second input field
	 */
	private QLineEdit framesPerSecond;
	/**
	 * Number of frames input field
	 */
	private QLineEdit numFrames;
	/**
	 * The dialog's OK button
	 */
	private QPushButton ok;
	/**
	 * The dialog's Cancel button
	 */
	private QPushButton cancel;

	/**
	 * Default constructor.
	 * 
	 * @param parent the dialog's parent window
	 */
	public CreateAnimationDialog(QWidget parent) {
		super(parent);
		
		setWindowTitle(L10N.translate("Create new animation"));
		
		QLabel nameLabel=new QLabel(L10N.translate("Animation name:"));
		name=new QLineEdit(this);
		name.setText("Animation");
		QHBoxLayout nameLayout=new QHBoxLayout();
		nameLayout.addWidget(nameLabel);
		nameLayout.addStretch(1);
		nameLayout.addWidget(name);
		
		QLabel startTimeLabel=new QLabel(L10N.translate("Start time:"));
		startTime=new QLineEdit(this);
		startTime.setValidator(new QDoubleValidator(startTime));
		startTime.setText("0.0");
		startTime.editingFinished.connect(this,"startTimeChanged()");
		QHBoxLayout startTimeLayout=new QHBoxLayout();
		startTimeLayout.addWidget(startTimeLabel);
		startTimeLayout.addStretch(1);
		startTimeLayout.addWidget(startTime);
		
		QLabel endTimeLabel=new QLabel(L10N.translate("End time:"));
		endTime=new QLineEdit(this);
		endTime.setValidator(new QDoubleValidator(endTime));
		endTime.setText("5.0");
		endTime.editingFinished.connect(this,"endTimeChanged()");
		QHBoxLayout endTimeLayout=new QHBoxLayout();
		endTimeLayout.addWidget(endTimeLabel);
		endTimeLayout.addStretch(1);
		endTimeLayout.addWidget(endTime);
		
		QLabel durationLabel=new QLabel(L10N.translate("Duration:"));
		duration=new QLineEdit(this);
		duration.setValidator(new QDoubleValidator(endTime));
		duration.setText("5.0");
		duration.editingFinished.connect(this,"durationChanged()");
		QHBoxLayout durationLayout=new QHBoxLayout();
		durationLayout.addWidget(durationLabel);
		durationLayout.addStretch(1);
		durationLayout.addWidget(duration);
		
		QLabel framesPerSecondLabel=new QLabel(L10N.translate("Frames per second:"));
		framesPerSecond=new QLineEdit(this);
		framesPerSecond.setValidator(new QDoubleValidator(framesPerSecond));
		framesPerSecond.setText("24.0");
		framesPerSecond.editingFinished.connect(this,"framesPerSecondChanged()");
		QHBoxLayout framesPerSecondLayout=new QHBoxLayout();
		framesPerSecondLayout.addWidget(framesPerSecondLabel);
		framesPerSecondLayout.addStretch(1);
		framesPerSecondLayout.addWidget(framesPerSecond);
		
		QLabel numFramesLabel=new QLabel(L10N.translate("Number of frames:"));
		numFrames=new QLineEdit();
		numFrames.setValidator(new QIntValidator(numFrames));
		numFrames.setText("120");
		numFrames.editingFinished.connect(this,"numFramesChanged()");
		QHBoxLayout numFramesLayout=new QHBoxLayout();
		numFramesLayout.addWidget(numFramesLabel);
		numFramesLayout.addStretch(1);
		numFramesLayout.addWidget(numFrames);
		
		ok=new QPushButton(L10N.translate("OK"));
		ok.clicked.connect(this,"okClicked()");
		cancel=new QPushButton(L10N.translate("Cancel"));
		cancel.clicked.connect(this,"cancelClicked()");
		QHBoxLayout bottomLayout=new QHBoxLayout();
		bottomLayout.addStretch(1);
		bottomLayout.addWidget(cancel);
		bottomLayout.addWidget(ok);
		
		QVBoxLayout layout=new QVBoxLayout(this);
		
		layout.addLayout(nameLayout);
		layout.addLayout(startTimeLayout);
		layout.addLayout(endTimeLayout);
		layout.addLayout(durationLayout);
		layout.addLayout(framesPerSecondLayout);
		layout.addLayout(numFramesLayout);
		layout.addStretch(1);
		layout.addLayout(bottomLayout);
		
		setLayout(layout);
	}
	
	/**
	 * Recalculate number of frames based on start time, end time
	 * and frames per second
	 */
	private void updateNumFrames() {
		double startTime=Double.parseDouble(this.startTime.text());
		double endTime=Double.parseDouble(this.endTime.text());
		double framesPerSecond=Double.parseDouble(this.framesPerSecond.text());
		double duration=endTime-startTime;
		
		int numFrames=(int)(duration*framesPerSecond);
		
		this.numFrames.setText(Integer.toString(numFrames));				
	}
	
	/**
	 * Recalculate duration based on start time and end time
	 */
	private void updateDuration() {
		double startTime=Double.parseDouble(this.startTime.text());
		double endTime=Double.parseDouble(this.endTime.text());
		double duration=endTime-startTime;		
		this.duration.setText(Double.toString(duration));
	}
	
	/**
	 * Slot reacting to changes in start time input field
	 */
	protected void startTimeChanged() {
		updateNumFrames();
		updateDuration();
	}
	
	/**
	 * Slot reacting to changes in end time input field
	 */
	protected void endTimeChanged() {
		updateNumFrames();
		updateDuration();
	}
	
	/**
	 * Recalculate end time based on start time and duration
	 */
	public void updateEndTime() {
		double startTime=Double.parseDouble(this.startTime.text());
		double duration=Double.parseDouble(this.duration.text());
		double endTime=startTime+duration;		
		this.endTime.setText(Double.toString(endTime));
	}
	
	/**
	 * Slot reacting to changes in duration input field
	 */
	protected void durationChanged() {
		updateNumFrames();
		updateEndTime();
	}
	
	/**
	 * Slot reacting to changes in frames per second input field
	 */
	protected void framesPerSecondChanged() {
		updateNumFrames();
	}
	
	/**
	 * Slot reacting to changes in number of frames input field
	 */
	protected void numFramesChanged() {
		// TODO update end time and duration
	}
	
	/**
	 * Slot reacting to user clicking OK by creating the animation and closing the dialog. 
	 */
	protected void okClicked() {
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Animation animation=document.getAnimationManager().createAnimation(name.text());
		animation.setStartTime(Double.parseDouble(startTime.text()));
		animation.setEndTime(Double.parseDouble(endTime.text()));
		animation.setFramesPerSecond(Double.parseDouble(framesPerSecond.text()));
		close();
	}
	
	/**
	 * Slot reacting to user clicking Cancel by closing the dialog.
	 */
	protected void cancelClicked() {
		close();
	}
}
