/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 17, 2006
 */
package eu.moonlight3d.ml3d.animation.basic;

import java.util.ArrayList;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.preferences.PreferencesGroup;
import eu.moonlight3d.framework.ui.DockWidgetFactory;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {

	public String getName() {
		return "eu.moonlight3d.ml3d.animation.basic.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencies=new ArrayList<String>();
		
		dependencies.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencies.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencies.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		dependencies.add("eu.moonlight3d.ml3d.animation.basicselectiontypes.Plugin");
		
		return dependencies;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
        PreferencesGroup preferences=new PreferencesGroup();
        preferences.setName("Timeline View");
        Property timelineCoarseBarColour=new Property(preferences);
        timelineCoarseBarColour.setName("timelineCoarseBarColour");
        timelineCoarseBarColour.setType(Value.Type.Colour);
        timelineCoarseBarColour.setValue(new Colour(0.156,0.305,0.576,1.0));
        Property timelineCoarseHandleColour=new Property(preferences);
        timelineCoarseHandleColour.setName("timelineCoarseHandleColour");
        timelineCoarseHandleColour.setType(Value.Type.Colour);
        timelineCoarseHandleColour.setValue(new Colour(0.575,0.701,0.854,1.0));
        
        state.getPreferencesManager().getCurrentPreferences().addPreferencesGroup(preferences);

        if(!state.getBatchMode()) {
			UIManager uiManager=state.getUIManager();
		
			uiManager.registerMainWidgetFactory(new TrackEditorFactory());
		
			uiManager.registerAction(new CreateAnimationAction(null));
			uiManager.registerAction(new DeleteAnimationAction(null));
			uiManager.registerAction(new AddAnimationTrackAction(null));
			uiManager.registerAction(new RemoveAnimationTrackAction(null));
			uiManager.registerAction(new OpenTrackEditorAction(null));

			uiManager.registerDockWidgetFactory(new DockWidgetFactory("TimelineView",L10N.translate("Timeline"),TimelineView.class));
		
			FrameLayout mainWindowLayout=uiManager.getLayout("mainwindow");
		
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"D_AnimationActions","menubar");
			MenuNode animationMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Animation"),"");
			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Animation..."),"CreateAnimationAction"));
			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Delete Animation"),"DeleteAnimationAction"));
//			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Open Track Editor..."),"OpenTrackEditorAction"));
			contribution.add(animationMenu);
		
			mainWindowLayout.addMenuContribution(contribution);
		
			mainWindowLayout.addDockWidget("TimelineView");
		
//			FrameLayout trackEditorLayout=new FrameLayout("trackeditor",L10N.translate("Track Editor"));
//			uiManager.addOrReplaceLayout(trackEditorLayout);
//		
//			contribution=new MenuNode(MenuNode.Type.Contribution,"TrackMenu","menubar");
//			animationMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Animation"),"");
//			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Animation..."),"CreateAnimationAction"));
//			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Delete Animation"),"DeleteAnimationAction"));
//			animationMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
//			animationMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Close"),"CloseCurrentFrameAction"));
//			contribution.add(animationMenu);
//			MenuNode trackMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Track"),"");
//			trackMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Add Track..."),"AddAnimationTrackAction"));
//			trackMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Remove Track"),"RemoveAnimationTrackAction"));
//			contribution.add(trackMenu);
//			trackEditorLayout.addMenuContribution(contribution);
		}
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager uiManager=State.getInstance().getUIManager();

			uiManager.unregisterAction("CreateAnimationAction");
			uiManager.unregisterAction("DeleteAnimationAction");
//			uiManager.unregisterAction("AddAnimationTrackAction");
//			uiManager.unregisterAction("RemoveAnimationTrackAction");
//			uiManager.unregisterAction("OpenTrackEditorAction");
		
			uiManager.unregisterMainWidgetFactory("trackeditor");
		
			uiManager.unregisterDockWidgetFactory("TimelineView");
		}
	}

}
