/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 17, 2006
 */
package eu.moonlight3d.ml3d.animation.basic;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QScrollArea;
import com.trolltech.qt.gui.QSplitter;
import com.trolltech.qt.gui.QTreeWidget;
import com.trolltech.qt.gui.QTreeWidgetItem;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.anim.AnimationTrack;
import eu.moonlight3d.ml3d.backend.threedim.anim.Manager;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;

public class TrackEditor extends QWidget {
	/**
	 * The drop down which lists all animations in the scene.
	 */
	private QComboBox animationList;
	/**
	 * Scroll area containing editors for each track
	 */
	private QScrollArea scrollArea;
	private QTreeWidget trackTree;
	private TrackView trackView;
	
	private Animation currentAnimation;

	/**
	 * Listener used to catch changes in the list of existing animations
	 * and update the animation list drop down accordingly.  
	 *
	 * @author gregor
	 */
	private class AnimationManagerListener implements eu.moonlight3d.ml3d.backend.threedim.anim.AnimationManagerListener {

		public void animationAdded(Animation animation) {
			rebuildAnimationList();
		}

		public void animationRemoved(Animation animation) {
			rebuildAnimationList();
		}
		
	}
	
	/**
	 * Listener used to catch changes in the currently selected animation
	 * to update the editor display accordingly. 
	 *
	 * @author gregor
	 */
	private class AnimationListener implements eu.moonlight3d.ml3d.backend.threedim.anim.AnimationListener {
		public void currentTimeUpdated(double oldTime, double newTime) {
			// TODO Auto-generated method stub
			
		}

		public void defaultsSet() {
			// TODO Auto-generated method stub
			
		}

		public void endTimeUpdated(double oldEndTime, double newEndTime) {
			// TODO Auto-generated method stub
			
		}

		public void startTimeUpdated(double oldStartTime, double newStartTime) {
			// TODO Auto-generated method stub
			
		}

		public void framesPerSecondUpdated(double oldFramesPerSecond, double newFramesPerSecond) {
			// TODO Auto-generated method stub
			
		}
		
		public void trackAdded(AnimationTrack track) {
			updateTrackTree();
		}

		public void trackRemoved(AnimationTrack track) {
			updateTrackTree();
		}

	}
	
	/**
	 * Instance of animation manager listener
	 */
	private AnimationManagerListener animationManagerListener;
	
	private AnimationListener animationListener;
	
	// TODO document lifecycle listener
	
	/**
	 * Default constructor for animation track editor widget
	 * 
	 * @param parent the parent for this widget
	 */
	public TrackEditor(QWidget parent) {
		super(parent);
		
		QLabel animationListLabel=new QLabel(L10N.translate("Animation: "));
		animationList=new QComboBox(this);
		animationList.setEditable(false);
		animationList.currentIndexChanged.connect(this,"animationChanged()");
		
		QHBoxLayout animationSelectionLayout=new QHBoxLayout();
		animationSelectionLayout.addWidget(animationListLabel);
		animationSelectionLayout.addWidget(animationList);
		animationSelectionLayout.addStretch(1);
		
		QSplitter splitter=new QSplitter(Qt.Orientation.Horizontal);
		
		trackTree=new QTreeWidget(splitter);
		trackTree.setColumnCount(1);
		trackTree.setHeaderLabel(L10N.translate("Tracks"));
		
		QTreeWidgetItem item=new QTreeWidgetItem();
		item.setFlags(Qt.ItemFlag.ItemIsUserCheckable,Qt.ItemFlag.ItemIsEnabled);
		item.setData(0,Qt.ItemDataRole.CheckStateRole,true);
		item.setText(0,"Test");
		
		trackTree.addTopLevelItem(item);
		
		scrollArea=new QScrollArea(splitter);
		trackView=new TrackView(scrollArea);
		scrollArea.setWidget(trackView);

		QVBoxLayout layout=new QVBoxLayout();
		layout.addLayout(animationSelectionLayout);		
		
		layout.addWidget(splitter);
		
		layout.setMargin(0);
		
		setLayout(layout);
		
		rebuildAnimationList();
		
		// register animation manager listener
		animationManagerListener=new AnimationManagerListener();
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Manager manager=document.getAnimationManager();
		manager.registerAnimationManagerListener(animationManagerListener);
		
		animationListener=new AnimationListener();
	}
	
	/**
	 * Rebuild the list of animations inside the animationList drop down
	 */
	private void rebuildAnimationList() {
		animationList.clear();
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Manager manager=document.getAnimationManager();
		for(Animation animation : manager.getAnimations()) {
			animationList.addItem(animation.getName());
		}
	}
	
	/**
	 * Handle the selection of a different animation in the editor
	 */
	protected void animationChanged() {
		// handle change of animation in track editor
		if(currentAnimation!=null) {
			currentAnimation.unregisterAnimationListener(animationListener);
		}

		String animationName=animationList.currentText();
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Manager manager=document.getAnimationManager();
		
		currentAnimation=manager.getAnimationByName(animationName);
		currentAnimation.registerAnimationListener(animationListener);
		trackView.setAnimation(currentAnimation);
		
		updateTrackTree();
	}
	
	/**
	 * Refresh the contents of the track tree
	 */
	protected void updateTrackTree() {
		trackTree.clear();
		
		for(AnimationTrack track : currentAnimation.getTracks()) {
			boolean inserted=false;
			for(int i=0;i<trackTree.topLevelItemCount();i++) {
				QTreeWidgetItem nodeItem=trackTree.topLevelItem(i);
				
				if(nodeItem.text(0).equals(track.getNodeName())) {
					QTreeWidgetItem propertyItem=new QTreeWidgetItem();
					propertyItem.setText(0, track.getPropertyName());
					propertyItem.setFlags(Qt.ItemFlag.ItemIsUserCheckable,Qt.ItemFlag.ItemIsEnabled);
					propertyItem.setData(0,Qt.ItemDataRole.CheckStateRole,true);
					nodeItem.addChild(propertyItem);
					
					inserted=true;
				}
			}
			if(!inserted) {
				QTreeWidgetItem nodeItem=new QTreeWidgetItem();
				nodeItem.setText(0, track.getNodeName());
				
				trackTree.addTopLevelItem(nodeItem);
				
				QTreeWidgetItem propertyItem=new QTreeWidgetItem();
				propertyItem.setText(0, track.getPropertyName());
				propertyItem.setFlags(Qt.ItemFlag.ItemIsUserCheckable,Qt.ItemFlag.ItemIsEnabled);
				propertyItem.setData(0,Qt.ItemDataRole.CheckStateRole,true);
				nodeItem.addChild(propertyItem);
			}
		}
	}

	@Override
	public void disposed() {
		// unregister animation manager listener
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Manager manager=document.getAnimationManager();
		manager.unregisterAnimationManagerListener(animationManagerListener);
		
		if(currentAnimation!=null) {
			currentAnimation.unregisterAnimationListener(animationListener);
		}
	}
}
