/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 6, 2006
 */
package eu.moonlight3d.ml3d.backend.threedim.anim;

public interface AnimationListener {
	/**
	 * This method is called whenever the start time for the animation is
	 * changed. The start time is the value at which the time value will start
	 * to be incremented when the animation is played
	 * 
	 * @param oldStartTime the old start time
	 * @param newStartTime the new start time
	 */
	public void startTimeUpdated(double oldStartTime, double newStartTime);

	/**
	 * This method is called whenever the end time for the animation is
	 * changed. The end time is the maximum value the time value will
	 * assume when it is incremented during animation playback.
	 *   
	 * @param oldEndTime the old end time
	 * @param newEndTime the new end time
	 */
	public void endTimeUpdated(double oldEndTime, double newEndTime);
	
	/**
	 * This method is called whenever the frames per second for the animation
	 * is changed.
	 * 
	 * @param oldFramesPerSecond the old frames per seconds value
	 * @param newFramesPerSecond the new frames per seconds value
	 */
	public void framesPerSecondUpdated(double oldFramesPerSecond, double newFramesPerSecond);
	
	/**
	 * This method is called whenever the current time for the
	 * animation is changed to another value.
	 *  
	 * @param oldTime the old time value
	 * @param newTime the new time value
	 */
	public void currentTimeUpdated(double oldTime, double newTime);
	
	/**
	 * This method is called when the animation is asked to set
	 * default values instead of the ones for the current time,
	 * effectively resetting the scene.
	 *
	 */
	public void defaultsSet();
	
	/**
	 * This method is called when a new animation track for a
	 * property was added to the animation.
	 * 
	 * @param track the newly added track
	 */
	public void trackAdded(AnimationTrack track);
	
	/**
	 * This method is called when an animation track for a
	 * property was removed from the animation
	 * 
	 * @param track the removed track
	 */
	public void trackRemoved(AnimationTrack track);
}
