/*
 * Moonlight|3D
 * Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Nov 16, 2005
 */
package eu.moonlight3d.ml3d.backend.threedim.anim;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.selection.PathElement;

/**
 * A generic animation track that controls the behaviour of an
 * operator graph node in time.
 *
 * @author gregor
 */
public interface AnimationTrack extends PathElement {
	/**
	 * Get the name of the operator graph node that is animated
	 * by this track.
	 * 
	 * @return the name of the operator graph node
	 */
	public String getNodeName();
	/**
	 * Get the name of the property of the operator graph node that
	 * is animated by this track.
	 * 
	 * @return the name of the property
	 */
	public String getPropertyName();
	/**
	 * Return the track's default value. The default value is the value
	 * that should be used when the animation that this track belongs
	 * to is not being played back.
	 * 
	 * @return the default value for this track
	 */
	public Value getDefaultValue();
	/**
	 * Return the track's value for a given time of the animation. The
	 * time value is given in seconds and may be outside of the time 
	 * range of the animation that this track belongs to. The track
	 * must always return a sensible interpolation or extrapolation to
	 * the given moment.
	 *  
	 * @param time the time for which to return a value
	 * @return the value for the given time
	 */
	public Value getValue(double time);
	/**
	 * Load the data for this track from an XML DOM tree with the
	 * given root node.
	 *  
	 * @param root the root node to load the data from
	 * @throws Exception if an error occurred during the loading
	 */
	public void loadXML(Element root) throws Exception;
	/**
	 * Save the data for this track to an XML DOM tree below the
	 * given root node.
	 *  
	 * @param document the node that the root node is attached to
	 * @param root the root node under which the data is added
	 * @throws DOMException
	 * @throws Exception
	 */
	public void saveXML(Document document, Element root) throws DOMException, Exception;
}
