/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Dec 5, 2008
 */

package eu.moonlight3d.ml3d.backend.threedim.document;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import eu.moonlight3d.common.document.extensible.DocumentContribution;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.og.Link;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.renderer.Manager;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngine;
import eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManager;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

public class BackendContribution implements DocumentContribution {
	/**
	 * The document's selection manager instance
	 */
	private eu.moonlight3d.ml3d.backend.selection.Manager selectionManager;
	/**
	 * The document's animation manager instance
	 */
	private eu.moonlight3d.ml3d.backend.threedim.anim.Manager animationManager;
	/**
	 * The document's operator graph manager instance
	 */
	private eu.moonlight3d.ml3d.backend.threedim.og.Manager ogManager;
	/**
	 * The document's scene graph manager instance
	 */
	private eu.moonlight3d.ml3d.backend.threedim.sg.Manager sgManager;
	/**
	 * The document's cache manager instance
	 */
	private eu.moonlight3d.ml3d.backend.threedim.cache.Manager cacheManager;
	/**
	 * The document's renderer manager instance
	 */
	private eu.moonlight3d.ml3d.backend.threedim.renderer.Manager rendererManager;
	/**
	 * The document's resource manager instance
	 */
	private ResourceManager resourceManager;
	
	/**
	 * Default constructor
	 * @param document document that this contribution belongs to
	 */
	BackendContribution(Document document) {
		document.setBackendContribution(this);
		selectionManager=new eu.moonlight3d.ml3d.backend.selection.Manager();
		animationManager=new eu.moonlight3d.ml3d.backend.threedim.anim.Manager(document);
		ogManager=new eu.moonlight3d.ml3d.backend.threedim.og.Manager(document);
		sgManager=new eu.moonlight3d.ml3d.backend.threedim.sg.Manager(document);
		cacheManager=new eu.moonlight3d.ml3d.backend.threedim.cache.Manager();
		rendererManager=new eu.moonlight3d.ml3d.backend.threedim.renderer.Manager();
		resourceManager=new eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManager(document);
        try {
			selectionManager.registerRootPathElement(animationManager);
	        selectionManager.registerRootPathElement(ogManager);
	        selectionManager.registerRootPathElement(resourceManager);
		} catch (Exception e) {
			// this exception cannot be thrown here; this block is actually unreachable
		}		
	}

	public void close() {
		selectionManager.clearSelections();
		ogManager.clear();
		cacheManager.clear();
		try {
			// update to force UI cleanup (FIXME there should be a cleaner way)
			ogManager.update();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String getName() {
		return "eu.moonlight3d.ml3d.backend.threedim.document.BackendContribution";
	}

	public boolean hasUnsavedChanges() {
		return true;
	}

	public boolean retainAlways() {
		return true;
	}

	public void update() {
		try {
			ogManager.update();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Get the document's selection manager
	 * 
	 * @return the document's selection manager
	 */
	eu.moonlight3d.ml3d.backend.selection.Manager getSelectionManager() {
		return selectionManager;
	}

	/**
	 * Get the document's animation manager
	 * 
	 * @return the document's animation manager
	 */
	eu.moonlight3d.ml3d.backend.threedim.anim.Manager getAnimationManager() {
		return animationManager;
	}

	/**
	 * Get the document's cache manager
	 * 
	 * @return the document's cache manager
	 */
	eu.moonlight3d.ml3d.backend.threedim.cache.Manager getCacheManager() {
		return cacheManager;
	}

	/**
	 * Get the document's operator graph manager
	 * 
	 * @return the document's operator graph manager
	 */
	eu.moonlight3d.ml3d.backend.threedim.og.Manager getOGManager() {
		return ogManager;
	}

	/**
	 * Get the document's scene graph manager
	 * 
	 * @return the document's scene graph manager
	 */
	eu.moonlight3d.ml3d.backend.threedim.sg.Manager getSGManager() {
		return sgManager;
	}

	/**
	 * Get the document's renderer manager
	 * 
	 * @return the document's renderer manager
	 */
	eu.moonlight3d.ml3d.backend.threedim.renderer.Manager getRendererManager() {
		return rendererManager;
	}

	/**
	 * Get the document's resource manager
	 * 
	 * @return the document's resource manager
	 */
	ResourceManager getResourceManager() {
		return resourceManager;
	}

	/**
     * Put the contents of the document's node cache into the XML
     * tree structure.
     * 
     * @param doc the DOM document in which to insert the cache contents
     * @param root the root node below which to insert the cache contents
     */
    private void exportCache(org.w3c.dom.Document doc, Element root) {
    	Element cacheRoot=doc.createElement("cache");
    	for(String nodeName : cacheManager.getSGs().keySet()) {
    		SceneGraph graph=cacheManager.getSG(nodeName);
    		Element nodeElement=doc.createElement("cachenode");
    		nodeElement.setAttribute("name",nodeName);
    		graph.saveToXML(doc,nodeElement);
    		cacheRoot.appendChild(nodeElement);
    	}
    	root.appendChild(cacheRoot);
    }
    
    /**
     * Put the contents of the document's operator graph into the XML
     * tree structure.
     * 
     * @param doc the DOM document in which to insert the OG contents
     * @param root the root node below which to insert the OG contents
     * @throws Exception 
     */
    private void exportOG(org.w3c.dom.Document doc, Element root) throws Exception {
        Element ogRoot=doc.createElement("og");
        root.appendChild(ogRoot);
        Element ogNodes=doc.createElement("nodes");
        ogRoot.appendChild(ogNodes);
        Element ogLinks=doc.createElement("links");
        ogRoot.appendChild(ogLinks);
        
        // iterate over OG nodes:
        List<eu.moonlight3d.ml3d.backend.threedim.og.Node> nodeList=ogManager.getNodeList();
        for(int i=0;i<nodeList.size();i++) {
            eu.moonlight3d.ml3d.backend.threedim.og.Node node=nodeList.get(i);
            
            Element docNode=doc.createElement("node");
            docNode.setAttribute("type",node.getType());
            docNode.setAttribute("name",node.getName());
            
            // iterate over properties and save them
            ArrayList<String> propertyList=node.getPropertyList();
            for(int j=0;j<propertyList.size();j++) {
                Property property=node.getProperty(propertyList.get(j));
                property.writeToFile(doc,docNode);
            }
            
            ogNodes.appendChild(docNode);
        }
        
        // iterate over OG links:
        ArrayList<Link> linkList=ogManager.getLinkList();
        for(int i=0;i<linkList.size();i++) {
            Link link=linkList.get(i);
            
            Element docLink=doc.createElement("link");
            Element input=doc.createElement("input");
            input.setAttribute("node",link.getInput().getNode().getName());
            input.setAttribute("slot",link.getInput().getName());
            docLink.appendChild(input);
            Element output=doc.createElement("output");
            output.setAttribute("node",link.getOutput().getNode().getName());
            output.setAttribute("slot",link.getOutput().getName());
            docLink.appendChild(output);
            
            ogLinks.appendChild(docLink);
        }
    }

    /**
     * Put the the document's render settings into the XML
     * tree structure.
     * 
     * @param doc the DOM document in which to insert the render settings
     * @param root the root node below which to insert the render settings
     * @throws DOMException 
     * @throws Exception 
     */
	private void exportRenderSettings(org.w3c.dom.Document doc, Element root) throws Exception {
		List<String> renderEngines=eu.moonlight3d.ml3d.backend.threedim.renderer.Manager.getRenderEngines();
		
		Element renderEnginesNode=doc.createElement("renderengines");
		
		for(int i=0;i<renderEngines.size();i++) {
			RenderEngine engine=Manager.getRenderEngine(renderEngines.get(i));
			PropertyContainer engineProperties=rendererManager.getProperties(engine.getName());
			
			Element engineNode=doc.createElement("engine");
			engineNode.setAttribute("name",engine.getName());
			
			ArrayList<String> propertyList=engineProperties.getPropertyList();
            for(int j=0;j<propertyList.size();j++) {
                Property property=engineProperties.getProperty(propertyList.get(j));
                property.writeToFile(doc,engineNode);
            }
            
            renderEnginesNode.appendChild(engineNode);
		}
		
		root.appendChild(renderEnginesNode);
	}
	
    /**
     * Put the contents of the document's animations into the XML
     * tree structure.
     * 
     * @param doc the DOM document in which to insert the animations
     * @param root the root node below which to insert the animations
     * @throws Exception 
     * @throws DOMException 
     */
	private void exportAnimations(org.w3c.dom.Document doc, Element root) throws Exception {
		List<Animation> animations=animationManager.getAnimations();
		
		Element animationsNode=doc.createElement("animations");
		
		for(Animation animation : animations) {
			animation.saveXML(doc, animationsNode);
		}
		
		root.appendChild(animationsNode);
	}
    
	public void write(org.w3c.dom.Document document, Element contributionRoot) throws Exception {
        // generate save file DOM tree here
        contributionRoot.setAttribute("fileformatversion","1");
        
        exportCache(document,contributionRoot);
        exportOG(document,contributionRoot);
        exportAnimations(document,contributionRoot);
        exportRenderSettings(document,contributionRoot);
        
        resourceManager.write(document, contributionRoot);
	}


    private void importCache(Element cacheRoot) throws Exception {
    	NodeList cacheList=cacheRoot.getElementsByTagName("cachenode");
    	for(int i=0;i<cacheList.getLength();i++) {
    		Element cacheNodeElement=(Element) cacheList.item(i);
    		try {
				SceneGraph sceneGraph=sgManager.loadFromXML(cacheNodeElement);
				cacheManager.addSG(cacheNodeElement.getAttribute("name"),sceneGraph);
			} catch (Exception e) {
                throw new Exception(L10N.translate("cannot load cached nodes from file"),e);
			}
    	}
    }
    
    private void readProperty(PropertyContainer propertyContainer, Node propertyNode) throws Exception {
        if(propertyNode.getNodeType()!=Node.ELEMENT_NODE) {
            return;
        }
        
        Element propertyElement=(Element)propertyNode;
        
        if(!propertyNode.getNodeName().equals("property")) {
            throw new Exception(L10N.translate("invalid file format: unknown tag found within <node> tag"));
        }
        String propertyName=propertyNode.getAttributes().getNamedItem("name").getNodeValue();
        eu.moonlight3d.framework.helper.Property property=propertyContainer.getProperty(propertyName);
        if(property==null) {
            throw new Exception(MessageFormat.format(L10N.translate("no such property in operator graph node: {0}"),propertyName));
        }
        property.readFromFile(propertyElement,true);
    }
    
    private void importOGNodes(Element ogRoot) throws Exception {
        // find node "nodes"
        NodeList nodesList=ogRoot.getElementsByTagName("nodes");
        if(nodesList.getLength()!=1) {
            throw new Exception(L10N.translate("invalid file format: more than one <nodes> tag within <og> tag"));
        }
        
        // iterate over children of nodes element:
        Element nodesRoot=(Element)nodesList.item(0);
        NodeList nodes=nodesRoot.getChildNodes();
        
        for(int i=0; i<nodes.getLength(); i++) {
            Node docOGNode=nodes.item(i);
            
            if(docOGNode.getNodeType()!=Node.ELEMENT_NODE) {
                continue;
            }
            
            if(!docOGNode.getNodeName().equals("node")) {
                throw new Exception(L10N.translate("invalid file format: unknown tag found within <nodes> tag"));
            }
            
            // create node and set name:
            Node attributeNode=docOGNode.getAttributes().getNamedItem("type");
            String typeName=attributeNode.getNodeValue();
            attributeNode=docOGNode.getAttributes().getNamedItem("name");
            String nodeName=attributeNode.getNodeValue();
            String loggerMessage;
            loggerMessage=MessageFormat.format(L10N.translate("creating OG node \"{0}\" of type \"{1}\""), nodeName, typeName);
            eu.moonlight3d.framework.State.getInstance().logger().info(loggerMessage);
            eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode=ogManager.createNodeByName(typeName, nodeName);
            
            // parse and set properties:
            NodeList propertyList=docOGNode.getChildNodes();
            for(int j=0;j<propertyList.getLength();j++) {
                Node propertyNode=propertyList.item(j);
                readProperty(ogNode,propertyNode);
            }
        }    	
    }
    
    private void importOGLinks(Element ogRoot) throws Exception {
        // find node "links"
        NodeList linksList=ogRoot.getElementsByTagName("links");
        if(linksList.getLength()!=1) {
            throw new Exception(L10N.translate("invalid file format: more than one <links> tag within <og> tag"));
        }
        Element linksRoot=(Element)linksList.item(0);
        NodeList links=linksRoot.getElementsByTagName("link");
        for(int i=0;i<links.getLength();i++) {
            Node linkNode=links.item(i);
            
            Element linkElement=(Element)linkNode;
            
            if(!linkNode.getNodeName().equals("link")) {
                throw new Exception(L10N.translate("invalid file format: unknown tag found within <links> tag"));
            }

            NodeList inputList=linkElement.getElementsByTagName("input");
            if(inputList.getLength()!=1) {
                throw new Exception(L10N.translate("invalid file format: expects exactly one <input> tag within <link> tag"));
            }
            Element inputElement=(Element)inputList.item(0);
            String inputNodeName=inputElement.getAttribute("node");
            String inputSlotName=inputElement.getAttribute("slot");
            Slot inputSlot=ogManager.getNodeByName(inputNodeName).getSlot(inputSlotName);

            NodeList outputList=linkElement.getElementsByTagName("output");
            if(outputList.getLength()!=1) {
                throw new Exception(L10N.translate("invalid file format: expects exactly one <output> tag within <link> tag"));
            }
            
            Element outputElement=(Element)outputList.item(0);
            String outputNodeName=outputElement.getAttribute("node");
            String outputSlotName=outputElement.getAttribute("slot");

            String loggerMessage;
            loggerMessage=MessageFormat.format(L10N.translate("linking {0}/{1} to {2}/{3}"),inputNodeName,inputSlotName,outputNodeName,outputSlotName);
            eu.moonlight3d.framework.State.getInstance().logger().info(loggerMessage);
            Slot outputSlot=ogManager.getNodeByName(outputNodeName).getSlot(outputSlotName);
            
            ogManager.addLink(inputSlot,outputSlot);
        }    	
    }
    
    private void importOG(Element ogRoot) throws Exception
    {
    	importOGNodes(ogRoot);
    	importOGLinks(ogRoot);
    }
    
    private void importAnimations(Element animationsRoot) throws Exception {
    	NodeList animationNodes=animationsRoot.getElementsByTagName("animation");
    	for(int i=0;i<animationNodes.getLength();i++) {
    		Element animationElement=(Element) animationNodes.item(i);
    		
    		String message=MessageFormat.format(L10N.translate("reading animation \"{0}\""),animationElement.getAttribute("name"));
    		State.getInstance().logger().info(message);
    		
    		Animation animation=animationManager.createAnimation(animationElement.getAttribute("name"));
    		animation.readXML(animationElement);
    	}
    }
    
	private void importRenderSettings(Element renderEngines) throws Exception {
        NodeList engineNodes=renderEngines.getElementsByTagName("engine");
        
        for(int i=0; i<engineNodes.getLength(); i++) {
            Element docEngineNode=(Element)engineNodes.item(i);
            
            // create node and set name:
            Node attributeNode=docEngineNode.getAttributes().getNamedItem("name");
            RenderEngine renderEngine=Manager.getRenderEngine(attributeNode.getNodeValue());
            PropertyContainer engineProperties=rendererManager.getProperties(attributeNode.getNodeValue());
            
            if(engineProperties==null) {
            	String warning=MessageFormat.format(L10N.translate("unknown rendering engine \"{0}\" encountered"), attributeNode.getNodeValue());
            	State.getInstance().logger().warning(warning);
            	continue;
            }
            
            String message=MessageFormat.format(L10N.translate("reading properties for engine \"{0}\""),renderEngine.getName());
            State.getInstance().logger().info(message);
            
            // parse and set properties:
            NodeList propertyList=docEngineNode.getChildNodes();
            for(int j=0;j<propertyList.getLength();j++) {
                Node propertyNode=propertyList.item(j);
				readProperty(engineProperties,propertyNode);
            }
        }
	}

	void read(org.w3c.dom.Document document, Element root) throws Exception
    {
		if (!root.getAttribute("fileformatversion").equals("1")) {
			throw new Exception(L10N.translate("file appears to be from an unsupported version of the program"));
		}

		// find cache element and read cache if present
		NodeList cacheList = root.getElementsByTagName("cache");
		if (cacheList.getLength() > 0) {
			Element cache = (Element) cacheList.item(0);
			importCache(cache);
		}

		// find og element
		NodeList ogList = root.getElementsByTagName("og");
		if (ogList.getLength() != 1) {
			throw new Exception(L10N.translate("invalid file format: more than one <og> tag in file"));
		}
		Element og = (Element) ogList.item(0);

		// read and convert DOM tree here
		importOG(og);

		// find animations element
		NodeList animationsList = root.getElementsByTagName("animations");
		if (animationsList.getLength() != 1) {
			throw new Exception(L10N.translate("invalid file format: more than one <animations> tag in file"));
		}
		Element animations = (Element) animationsList.item(0);
		importAnimations(animations);

		// load render engine settings
		NodeList renderSettingsList = root.getElementsByTagName("renderengines");
		if (renderSettingsList.getLength() != 1) {
			throw new Exception(L10N.translate("invalid file format: more than one <renderengines> tag in file"));
		}
		Element renderEngines = (Element) renderSettingsList.item(0);
		importRenderSettings(renderEngines);
		
		resourceManager.read(root);
    }
}
