/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 14, 2007
 */
package eu.moonlight3d.ml3d.backend.threedim.document;

import eu.moonlight3d.common.document.extensible.ExtensibleDocument;
import eu.moonlight3d.common.document.extensible.ExtensibleDocumentFactory;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManager;

/**
 * The document class for 3D scenes. It is a container class for all
 * data that belongs to a Moonlight|3D scene. 
 *
 * @author gregor
 */
public class Document extends ExtensibleDocument {
	private BackendContribution backendContribution;

	/**
	 * Wrapper around ExtensibleDocument constructor
	 * 
	 * @param factory factory that is creating this document
	 */
	Document(ExtensibleDocumentFactory factory) {
		super("MoonlightScene",factory);
	}
	
	/**
	 * Wrapper around ExtensibleDocument constructor
	 * 
	 * @param factory factory that is creating this document
	 * @param filename filename of the document being loaded
	 * 
	 * @throws Exception
	 */
	Document(ExtensibleDocumentFactory factory, String filename) throws Exception {
		super("MoonlightScene",factory, filename);
	}
	
	void setBackendContribution(BackendContribution backendContribution) {
		this.backendContribution=backendContribution;
	}
	
	/**
	 * Get the document's selection manager
	 * 
	 * @return the document's selection manager
	 */
	public eu.moonlight3d.ml3d.backend.selection.Manager getSelectionManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getSelectionManager();
	}

	/**
	 * Get the document's animation manager
	 * 
	 * @return the document's animation manager
	 */
	public eu.moonlight3d.ml3d.backend.threedim.anim.Manager getAnimationManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getAnimationManager();
	}

	/**
	 * Get the document's cache manager
	 * 
	 * @return the document's cache manager
	 */
	public eu.moonlight3d.ml3d.backend.threedim.cache.Manager getCacheManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getCacheManager();
	}

	/**
	 * Get the document's operator graph manager
	 * 
	 * @return the document's operator graph manager
	 */
	public eu.moonlight3d.ml3d.backend.threedim.og.Manager getOGManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getOGManager();
	}

	/**
	 * Get the document's scene graph manager
	 * 
	 * @return the document's scene graph manager
	 */
	public eu.moonlight3d.ml3d.backend.threedim.sg.Manager getSGManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getSGManager();
	}

	/**
	 * Get the document's renderer manager
	 * 
	 * @return the document's renderer manager
	 */
	public eu.moonlight3d.ml3d.backend.threedim.renderer.Manager getRendererManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getRendererManager();
	}

	/**
	 * Get the document's resource manager
	 * 
	 * @return the document's resource manager
	 */
	public ResourceManager getResourceManager() {
		if(backendContribution==null) {
			throw new IllegalStateException(L10N.translate("illegal internal state: no BackendContribution"));
		}
		return backendContribution.getResourceManager();
	}
}
