package eu.moonlight3d.ml3d.backend.threedim.helper;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.anim.AnimationTrack;
import eu.moonlight3d.ml3d.backend.threedim.anim.KeyframeAnimationTrack;
import eu.moonlight3d.ml3d.backend.threedim.anim.LinearValueInterpolator;
import eu.moonlight3d.ml3d.backend.threedim.anim.Manager;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;

/**
 * A collection of useful helper functions when
 * dealing with animation tracks.
 * 
 * @author gregor
 *
 */
public class AnimationHelper {
	public static void toggleKeyframe(Animation animation, Node node, Property property) throws Exception {
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		
		// find or create track
		AnimationTrack track=animation.getTrackByName(node.getName(), property.getName());
		
		if(track==null) {
			// add track
			KeyframeAnimationTrack keyframeTrack=new KeyframeAnimationTrack();
			keyframeTrack.setNodeName(node.getName());
			keyframeTrack.setPropertyName(property.getName());
			keyframeTrack.setStartExtrapolator(new LinearValueInterpolator());
			keyframeTrack.setEndExtrapolator(new LinearValueInterpolator());
			keyframeTrack.setDefaultValue(property);
			track=keyframeTrack;
			animation.addTrack(track);
		}
		
		// search for key frame at current time in this animation (if not set to default)
		
		if(animation.getCurrentTime()==null) {
			// TODO: change default value
		} else {
			// check existence of keyframe at current animation time
			if(track instanceof KeyframeAnimationTrack) {
				KeyframeAnimationTrack keyframeTrack=(KeyframeAnimationTrack) track;
				
				ArrayList<KeyframeAnimationTrack.KeyValuePair> keys=keyframeTrack.getKeyList();
				
				for(KeyframeAnimationTrack.KeyValuePair key : keys) {
					if(key.time==animation.getCurrentTime()) {
						// key exists, remove it
						keyframeTrack.removeKey(key);
						return;
					} else if(key.time>animation.getCurrentTime()) {
						/* already past keyframe position in list, so it is 
						 * not there and the search can be aborted
						 */
						break;
					}
				}
				
				// fallthrough, meaning that key does not exist
				// TODO create key (decide on default interpolator)
				keyframeTrack.insertKey(animation.getCurrentTime(), property, new LinearValueInterpolator());
			}
		}
	}

	/**
	 * Create or remove a keyframe on the given track on
	 * the current animation time.
	 * @throws Exception 
	 */
	public static void toggleKeyframe(String animationName, String nodeName, String propertyName) throws Exception {
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		
		Node node=document.getOGManager().getNodeByName(nodeName);
		
		Property property=node.getProperty(propertyName);
		if(property==null) {
			throw new IllegalArgumentException(L10N.translate("no property with given name exists"));
		}
		
		Manager animationManager=document.getAnimationManager();
		
		// find animation with given name
		Animation animation=animationManager.getAnimationByName(animationName);
		if(animation==null) {
			throw new IllegalArgumentException(L10N.translate("no animation with given name exists"));
		}
		
		toggleKeyframe(animation, node, property);
	}
}
