/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Dec 2, 2007
 */

package eu.moonlight3d.ml3d.backend.threedim.helper;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;

/**
 * A collection of helper functions that perform various common tasks 
 * on the operator graph.
 *
 * @author gregor
 */
public class OperatorGraphHelper {
	/**
	 * Insert a node into the operator graph after an existing node in
	 * the graph. Connect the input of the new node to the (only) output of the
	 * old node and connect the output of the new node to the input of the
	 * previously following node.
	 * 
	 * @param manager
	 * @param sourceNode
	 * @param newOperation
	 * @param inputSlotName
	 * @param outputSlotName
	 * @throws Exception
	 */
	public static void insert(eu.moonlight3d.ml3d.backend.threedim.og.Manager manager, String sourceNode, Node newOperation, String inputSlotName, String outputSlotName) throws Exception {
        String selectionName=sourceNode;
        String ownerOGNodeName=SelectionHelper.getOGNodeName(selectionName);
        String ownerOGSlotName=SelectionHelper.getOGSlotName(selectionName);
        String ownerSGNodeName=SelectionHelper.getSGNodeName(selectionName);
        eu.moonlight3d.framework.State.getInstance().logger().info("SG node name: " + ownerSGNodeName);

        // connect MoveObject OG node to parent OG node of currently active SG node
        eu.moonlight3d.ml3d.backend.threedim.og.Node ownerOGNode=manager.getNodeByName(ownerOGNodeName);
        eu.moonlight3d.ml3d.backend.threedim.og.Slot ownerOGSlot=ownerOGNode.getSlot(ownerOGSlotName);
        manager.insertNodeAfter(ownerOGSlot,newOperation.getSlot(inputSlotName),newOperation.getSlot(outputSlotName));		
	}
	
	public static void insert(String sourceNode, Node newOperation, String inputSlotName, String outputSlotName) throws Exception {
		Document document;
		document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		insert(document.getOGManager(),sourceNode,newOperation,inputSlotName,outputSlotName); 
	}

	public static Node insert(String sourceNode, String newOperationType, String newOperationName, String inputSlotName, String outputSlotName) throws Exception {
		Document document;
		document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Node node=document.getOGManager().createNodeByName(newOperationType, newOperationName);
		insert(sourceNode,node,inputSlotName,outputSlotName);
		return node;
	}

	public static Node insert(String sourceNode, String newOperationType, String inputSlotName, String outputSlotName) throws Exception {
		Document document;
		document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		Node node=document.getOGManager().createNodeByName(newOperationType);
		insert(sourceNode,node,inputSlotName,outputSlotName);
		return node;
	}
}
