/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 26, 2006
 */
package eu.moonlight3d.ml3d.backend.threedim.og;

/**
 * Listener interface that receives notifications from an operator
 * graph manager.
 *
 * @author gregor
 */
public interface OperatorGraphListener {
	/**
	 * A new node has been inserted into the operator graph.
	 * 
	 * @param node the node that has been inserted
	 */
	public void nodeInserted(Node node);
	/**
	 * A node has been removed from the operator graph.
	 * 
	 * @param node the node that has been removed
	 */
	public void nodeRemoved(Node node);
	/**
	 * A node has been updated.
	 * 
	 * @param node the node that was updated
	 */
	public void nodeUpdated(Node node);
	/**
	 * A node update has failed in a critical way. If an
	 * exception has been thrown, the exception is passed
	 * as well.
	 * 
	 * @param node the node that failed
	 * @param exception the exception that was thrown if available
	 */
	public void nodeUpdateFailed(Node node, Exception exception);
	/**
	 * The entire graph has been updated. All required node
	 * updates have been performed (with the exception of nodes
	 * where the update failed) and all notifications on successful
	 * and failed node updates for this update pass have been made
	 * before this method is called.
	 */
	public void graphUpdated();
	/**
	 * The operator graph has been cleared of all contents.
	 */
	public void graphCleared();
	/**
	 * A new link has been inserted into the operator graph.
	 * 
	 * @param link the link that has been inserted
	 */
	public void linkInserted(Link link);
	/**
	 * A link has been removed from the operator graph.
	 * 
	 * @param link the link that has been removed
	 */
	public void linkRemoved(Link link);
}
