/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 21, 2005
 */
package eu.moonlight3d.ml3d.backend.threedim.renderer;
/**
 * \package eu.moonlight3d.ml3d.backend.renderer
 * 
 * The management for the render engine plugisn resides here.
 * Access to render engines goes through here.
 * 
 * This package is part of the core program.
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import eu.moonlight3d.framework.helper.PropertyContainer;


/**
 * A manager for all supported render engines.
 *
 * @author gregor
 */
public class Manager {
	/**
	 * List of all registered render engines
	 */
    private static ArrayList<RenderEngine> renderEngines;

    static {
        renderEngines=new ArrayList<RenderEngine>();    	
    }
    
    /**
     * Register a new rendering engine.
     * 
     * @param engine the engine to register.
     */
    public static void addRenderEngine(RenderEngine engine)
    {
        renderEngines.add(engine);
    }
    
    /**
     * Find the render engine interface for the engine with the given name
     * and return it.
     * 
     * @param name the name of the engine to unregister
     * @return the RenderEngine instance that has been unregistered or null if it was not found
     */
    public static RenderEngine getRenderEngine(String name)
    {
        for(int i=0;i<renderEngines.size();i++) {
            if(renderEngines.get(i).getName().equals(name)) {
                return renderEngines.get(i);
            }
        }
        
        return null;
    }

    /**
     * @return a list of all registered rendering engines
     */
    public static List<String> getRenderEngines()
    {
    		ArrayList<String> engineList=new ArrayList<String>();
    		
    		for(int i=0;i<renderEngines.size();i++) {
    			engineList.add(renderEngines.get(i).getName());
    		}
    		
    		return engineList;
    }
    
    /**
     * Unregister the rendering engine with the given name.
     * 
     * @param name the name of the rendering engine to remove
     */
    public static void removeRenderEngine(String name)
    {
        for(int i=0;i<renderEngines.size();i++) {
            if(renderEngines.get(i).getName().equals(name)) {
                renderEngines.remove(i);
                return;
            }
        }        
    }

    /**
     * A map of render engine names to renderer properties.
     */
    private Map<String, PropertyContainer> rendererProperties;
    
    /**
     * Default constructor.
     */
    public Manager()
    {
    	rendererProperties=new HashMap<String, PropertyContainer>();
    	for(RenderEngine engine : renderEngines) {
    		rendererProperties.put(engine.getName(), engine.getProperties());
    	}
    }
    
    /**
     * Get the renderer properties for the render engine with the given
     * name.
     * 
     * @param engineName the name of the render engine
     * @return the properties for that specific engine
     */
    public PropertyContainer getProperties(String engineName) {
    	return rendererProperties.get(engineName);
    }
    
}
