/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Apr 13, 2006
 */
package eu.moonlight3d.ml3d.backend.threedim.sg;

import java.util.ArrayList;

import org.w3c.dom.Element;

import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.helper.Factory;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;

public class Manager {
	private static ArrayList<Factory> nodeFactories;
	
	static {
		nodeFactories=new ArrayList<Factory>();
		
		nodeFactories.add(new DefaultFactory("Instance",Instance.class));
		nodeFactories.add(new DefaultFactory("Group",Group.class));
	}
	
	public static void registerNodeFactory(Factory factory) {
		nodeFactories.add(factory);
	}
	
	public static void unregisterNodeFactory(String factoryName) {
		for(Factory factory : nodeFactories) {
			if(factory.getName().equals(factoryName)) {
				nodeFactories.remove(factory);
				return;
			}
		}
	}
	
	Document document;
	
	public Manager(Document document) {
		this.document=document;
	}
	
	Node createNode(String typeName) throws Exception {
		for(Factory factory : nodeFactories) {
			if(factory.getName().equals(typeName)) {
				Node node=(Node)factory.createNew();
				return node;
			}
		}
		
		throw new Exception("unable to find SG node factory for given type");
	}
	
	
	public SceneGraph loadFromXML(Element rootElement) throws Exception {
		SceneGraph sceneGraph=new SceneGraph(document);
		
		sceneGraph.loadFromXML(rootElement);
		
		return sceneGraph;
	}
}
