/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Dec 11, 2005
 */
package eu.moonlight3d.ml3d.images.png;

import java.util.ArrayList;

import libpng.PNG;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.framework.helper.FileType;

public class ImageFormat implements eu.moonlight3d.common.image.core.ImageFormat {

	public String getName() {
		return "PNG";
	}

	public FileType getFileType() {
		ArrayList<String> patterns=new ArrayList<String>();
		patterns.add("*.png");
		return new FileType("PNG",patterns);
	}

	public boolean isLoadable() {
		return true;
	}
	
	public boolean canLoadFile(String filename) {
		try {
			// TODO implement a proper check here that only tests the file header
			if(loadFile(filename)!=null) {
				return true;
			}
			return false;
		} catch(Exception e) {
			return false;
		}
	}
	
	public Framebuffer loadFile(String filename) throws Exception {
		Framebuffer buffer;
		int[] width=new int[1];
		int[] height=new int[1];
		
		if(!PNG.loadPNG(filename,width,height)) {
			throw new Exception("Error loading PNG file. Does the file exist?");
		}
		
		buffer=new Framebuffer(width[0],height[0]);
		buffer.addChannel("red");
		buffer.addChannel("green");
		buffer.addChannel("blue");
		buffer.addChannel("alpha");
		
		PNG.getLoadData(buffer.getChannel("red").getBuffer(),buffer.getChannel("green").getBuffer(),buffer.getChannel("blue").getBuffer(),buffer.getChannel("alpha").getBuffer());
		
		return buffer;
	}

	public boolean isSaveable() {
		return true;
	}
	
	public void saveFile(String filename, Framebuffer buffer) {
		float[] red, green, blue, alpha;
		
		red=buffer.getChannel("red").getBuffer();
		green=buffer.getChannel("green").getBuffer();
		blue=buffer.getChannel("blue").getBuffer();
		alpha=buffer.getChannel("alpha").getBuffer();
		
		PNG.savePNG(filename,buffer.getWidth(),buffer.getHeight(),red,green,blue,alpha);
	}

}
