/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2005
 */
package eu.moonlight3d.ml3d.images.viewer;

import java.util.ArrayList;
import java.util.List;


import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QImage;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QScrollArea;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.common.image.core.Channel;
import eu.moonlight3d.common.image.core.Framebuffer;

/**
 * This is the WindowSpace implementation of the UI test plugin
 * 
 * @author gregor
 */
public class ImageView extends QWidget {
	private class ImageCanvas extends QWidget {
		public ImageCanvas(QWidget parent) {
			super(parent);
		}
		
		@Override
		protected void paintEvent(QPaintEvent event) {
			if(image==null) {
				return;
			}
			QPainter painter=new QPainter();
			painter.begin(this);
			painter.drawImage(0,0,image);
			painter.end();
		}
	}
	
    QLabel label;
    QScrollArea scrollArea;
    QWidget canvas;
    Framebuffer framebuffer;
    QImage image;
    
    static ArrayList<ImageView> imageViews;
    
    static {
    		imageViews=new ArrayList<ImageView>();
    }
    
    public static List<ImageView> getImageViews() {
    		return (ArrayList<ImageView>) imageViews.clone();
    }
	
	public ImageView(QWidget parent) {
		super(parent);
        
		imageViews.add(this);
		
		scrollArea=new QScrollArea(this);
		
        canvas=new ImageCanvas(scrollArea);
        
        scrollArea.setWidget(canvas);
        
        QHBoxLayout layout=new QHBoxLayout();
        layout.addWidget(scrollArea);
        layout.setMargin(0);
        setLayout(layout);
    }

	public void setImage(Framebuffer framebuffer)
	{
		this.framebuffer=framebuffer;
		
		image=new QImage(framebuffer.getWidth(),framebuffer.getHeight(),QImage.Format.Format_RGB32);
		Channel red, green, blue;
		Colour colour=new Colour();
		
		red=framebuffer.getChannel("red");
		green=framebuffer.getChannel("green");
		blue=framebuffer.getChannel("blue");
		
		for(int j=0;j<framebuffer.getHeight();j++) {
			for(int i=0;i<framebuffer.getWidth();i++) {
				
				colour.red=red.getPixel(i,j);
				colour.green=green.getPixel(i,j);
				colour.blue=blue.getPixel(i,j);
				
				Colour.RGB rgb=colour.getsRGB8();

				image.setPixel(i, j, new QColor(rgb.red,rgb.green,rgb.blue).rgb());
			}
		}
		
		canvas.setFixedSize(framebuffer.getWidth(),framebuffer.getHeight());
	}
	
	public Framebuffer getImage()
	{
		return framebuffer;
	}
	
	@Override
	public void disposed() {
		for(int i=0;i<imageViews.size();i++) {
			if(imageViews.get(i)==this) {
				imageViews.remove(i);
				break;
			}
		}
	}
	
}
