/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 5, 2005
 */
package eu.moonlight3d.ml3d.images.viewer;

import java.util.HashMap;
import java.util.List;

import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;

public class SaveImageAction extends Action {
	public SaveImageAction(QWidget parent) {
		super("SaveImageAction",L10N.translate("Save image"),parent);
		triggered.connect(this,"run()");
	}

	@Override
	public void run() throws Exception {
		FormatManager formatManager=FormatManager.getInstance();
		List<ImageFormat> imageFormats=formatManager.getFormats();
		
		if(imageFormats.size()==0) {
			QMessageBox.warning(QApplication.activeWindow(),L10N.translate("No image format handlers present"),L10N.translate("There are no image format handlers loaded. The Image cannot be saved."));
			return;
		}
		
		HashMap<String, ImageFormat> imageFormatMap=new HashMap<String, ImageFormat>();
        String filterNames="";
        boolean usableFormatsPresent=false;
        for(ImageFormat imageFormat : imageFormats) {
        	if (imageFormat.isSaveable()) {
        		usableFormatsPresent=true;
				FileType fileType = imageFormat.getFileType();
				String filterName = fileType.getName() + " (";
				for (String pattern : fileType.getPatterns()) {
					filterName += pattern + " ";
				}
				filterName += ")";
				imageFormatMap.put(filterName, imageFormat);
				filterNames = filterNames + filterName + ";;";
			}
        }
        filterNames=filterNames+L10N.translate("By filename");
        
        if(!usableFormatsPresent) {
			QMessageBox.warning(QApplication.activeWindow(),L10N.translate("No image format handlers present"),L10N.translate("There are no suitable image format handlers loaded. The Image cannot be saved."));
			return;        	
        }

        QFileDialog.Filter filter=new QFileDialog.Filter(filterNames);
        
        String filename=QFileDialog.getSaveFileName(QApplication.activeWindow(),L10N.translate("Save rendered image"),"",filter);
        
        if(filename==null) {
        	return;
        }
        if(filename.equals("")) {
        	return;
        }

        List<ImageView> imageViews = ImageView.getImageViews();
        if(imageViews.size()==0) {
            return;
        }
        Framebuffer framebuffer = imageViews.get(0).getImage();
        
        String formatName=null;
        if(filter.selectedFilter.equals(L10N.translate("By filename"))) {
        	for(ImageFormat format : imageFormats) {
        		if(format.getFileType().matches(filename)) {
        			formatName=format.getName();
        		}
        	}
        } else {
        	formatName=imageFormatMap.get(filter.selectedFilter).getName();
        }
        if(formatName==null) {
        	QMessageBox.warning(QApplication.activeWindow(), L10N.translate("File format not recognised"), L10N.translate("The file extension you entered does not correspond to any known image format. The image cannot be saved."));
        	return;
        }
        
        formatManager.saveImage(framebuffer,filename,formatName);
	}

}
