/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jan 18, 2008
 */

package eu.moonlight3d.ml3d.modelling.basic;

import eu.moonlight3d.ml3d.backend.selection.SelectionType;

/**
 * This is a class for handlers for the Delete Selection Tool
 * which actually perform the deletion operation on the current
 * selection. There are different handlers for different types
 * of selection because the actual deletion operation needs to
 * be implemented differently in each case in general. 
 *
 * @author gregor
 */
public interface DeleteSelectionHandler {
	/**
	 * Check whether this handler can cope with this type of selection.
	 * 
	 * @param type the selection type to handle
	 * @return true if this handler can work on that particular type,
	 * false otherwise
	 * @throws Exception if anything goes wrong
	 */
	public boolean canHandleSelectionType(SelectionType type) throws Exception;
	
	/**
	 * Delete the current selection.
	 * 
	 * @throws Exception if anything goes wrong
	 */
	public void run() throws Exception;
}
