/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.basic;

import java.util.ArrayList;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.selection.SelectionType;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modeless tool that removes edges from a selected mesh.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class DeleteSelectionTool extends Tool {
	/**
	 * A map between selection type names and applicable handlers.
	 */
	private static ArrayList<DeleteSelectionHandler> handlers;
	
	static {
		handlers=new ArrayList<DeleteSelectionHandler>();
	}
	
	/**
	 * Register a {@link DeleteSelectionHandler}. This will be called to check
	 * if it can handle the current selection and if true, will be
	 * called to delete the selection.
	 * 
	 * @param handler the handler to register
	 */
	public static void registerHandler(DeleteSelectionHandler handler) {
		handlers.add(handler);
	}
	
	/**
	 * Unregister a {@link DeleteSelectionHandler}.
	 * 
	 * @param handler the handler to unregister.
	 */
	public static void unregisterHandler(DeleteSelectionHandler handler) {
		handlers.remove(handler);
	}
	
	/**
	 * Default constructor
	 */
	public DeleteSelectionTool() {
		super();
		setName("DeleteSelectionTool");
		setDisplayName(L10N.translate("Delete Selection"));
		setCategory("Edit");
	}

	@Override
	public void activated() {
		try {
	        State state=eu.moonlight3d.framework.State.getInstance();
	        // don't do anything if selected object is not a SG node
			Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			SelectionType type=selection.getType();
			
			for(DeleteSelectionHandler handler : handlers) {
				if(handler.canHandleSelectionType(selection.getType())) {
					handler.run();
					Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
			        ogManager.update();
			        selection.clearSelection(type);        
					return;
				}
			}
			
			state.logger().warning(L10N.translate("Cannot delete current selection"));
			
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		return false;
	}

}
