/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 1, 2009
 */

package eu.moonlight3d.ml3d.modelling.basic;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;

public class OGOutputMerger {
	private ArrayList<Merge> mergeNodes=new ArrayList<Merge>();

	public List<String> getOGNodeNames(List<String> selection) throws Exception {
		ArrayList<String> ogNodes;
		
		ogNodes=new ArrayList<String>();
		
        for(String name : selection) {
        	String ogNodeName=SelectionHelper.getOGNodeName(name);
        	if(!ogNodes.contains(ogNodeName)) {
        		ogNodes.add(ogNodeName);
        	}
        }
		return ogNodes;		
	}
	
	public List<String> getOGNodeNames(Selection selection) throws Exception {
		return getOGNodeNames(selection.getSelection());
	}
	
	public String mergeOGOutputSlots(List<String> ogNodes) throws Exception {
        State state=State.getInstance();
        Manager ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();

    	// insert required merge nodes
		Node firstNode=ogManager.getNodeByName(ogNodes.get(0));
    	for(int i=1;i<ogNodes.size();i++) {
    		Node secondNode=ogManager.getNodeByName(ogNodes.get(i));
    		
    		Node mergeNode=ogManager.createNodeByName("Merge");
    		Slot firstOutput=firstNode.getSlot("output");
    		Slot secondOutput=secondNode.getSlot("output");
    		
    		ogManager.addLink(firstOutput, mergeNode.getSlot("input1"));
    		ogManager.addLink(secondOutput, mergeNode.getSlot("input2"));
    		firstNode=mergeNode;
    		mergeNodes.add((Merge) mergeNode);
    	}
    	
    	return firstNode.getName();
	}
	
	public List<String> getMergedSGNodeNames(List<String> nodePaths) throws Exception {
    	// run OG update to get the name mappings
        State state=State.getInstance();
        Manager ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
    	ogManager.update();
    	
    	// update selection list with renamed objects from merge node output
    	for(Merge merge : mergeNodes) {
    		Map<String, String> nodeNameMap=merge.getNameMap();
    		
    		for(int i=0;i<nodePaths.size();i++) {
    			String name=nodePaths.get(i);
    			// apply name map if object comes from second (renamed) input of merge
    			if(SelectionHelper.getOGNodeName(name).equals(merge.getSlot("input2").getCurrentLink().getInput().getNode().getName())) {
    				String nodePath=SelectionHelper.getSGPath(name);
    				String[] nodePathElements = nodePath.split("/");
    				for(int j=0;j<nodePathElements.length;j++) {
    					nodePathElements[j]=nodeNameMap.get(nodePathElements[j]);
    				}
        			name="OperatorGraph/" + merge.getName() + "/output";
        			for(int j=0;j<nodePathElements.length;j++) {
        				name=name+"/"+nodePathElements[j];
        			}
        			nodePaths.set(i, name);
    			}
    		}
    	}
    	
    	return nodePaths;
	}
}
