/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package eu.moonlight3d.ml3d.modelling.basic;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.ModeManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	/**
	 * {@link TransformMode} instance
	 */
	private TransformMode transformMode;
	/**
	 * {@link TransformTool} instance
	 */
	private TransformTool transformTool;
	/**
	 * {@link DeleteSelectionTool} instance
	 */
	private DeleteSelectionTool deleteSelectionTool;
	/**
	 * {@link OGNodeElementCounter} instance
	 */
	private OGNodeElementCounter ogNodeElementCounter;
	/**
	 * {@link SGNodeElementCounter} instance
	 */
	private SGNodeElementCounter sgNodeElementCounter;

	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.basic.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencies=new ArrayList<String>();
		dependencies.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencies.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		return dependencies;
	}

	public void load() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CopyFromCache",CopyFromCache.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("Merge",Merge.class));

		State state=eu.moonlight3d.framework.State.getInstance();
		
		if(!state.getBatchMode()) {
	        UIManager manager=state.getUIManager();
	
			manager.registerAction(new SceneStatisticsAction(null));
			manager.registerAction(new TransformAction(null));
			manager.registerAction(new TranslateAction(null));
			manager.registerAction(new RotateAction(null));
			manager.registerAction(new ScaleAction(null));
			manager.registerAction(new ToolAction("DeleteSelectionAction",L10N.translate("Delete Selection"),null,"","","DeleteSelectionTool","Del"));
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"C_Modify","menubar");
			MenuNode objectsMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Modify"),"");
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Transform Selection"),"TransformAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Translate Selection"),"TranslateAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Rotate Selection"),"RotateAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Scale Selection"),"ScaleAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Delete Selection"),"DeleteSelectionAction"));
			contribution.add(objectsMenu);
			state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"CountElements","viewMenuExtensionPoint");
			contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Scene Statistics..."), "SceneStatisticsAction"));
			state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);			
	
			transformMode=new TransformMode();
			ModeManager.getInstance().addMode(transformMode);
			transformTool=new TransformTool(transformMode);
			ToolChestView.addTool(transformTool);
			deleteSelectionTool=new DeleteSelectionTool();
			ToolChestView.addTool(deleteSelectionTool);
			
			ogNodeElementCounter=new OGNodeElementCounter();
			SceneStatisticsDialog.addElementCounter(ogNodeElementCounter);
			sgNodeElementCounter=new SGNodeElementCounter();
			SceneStatisticsDialog.addElementCounter(sgNodeElementCounter);
		}
	}

	public void unload() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CopyFromCache");

		State state=eu.moonlight3d.framework.State.getInstance();
		if(!state.getBatchMode()) {
	        UIManager manager=state.getUIManager();
	
			manager.unregisterAction("TransformAction");
			manager.unregisterAction("TranslateAction");
			manager.unregisterAction("RotateAction");
			manager.unregisterAction("ScaleAction");
	
			ToolChestView.removeTool(transformTool);
			ToolChestView.removeTool(deleteSelectionTool);
			ModeManager.getInstance().removeMode(transformMode);

			SceneStatisticsDialog.removeElementCounter(ogNodeElementCounter);
			SceneStatisticsDialog.removeElementCounter(sgNodeElementCounter);
		}
	}
}
