/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Mar 23, 2009
 */

package eu.moonlight3d.ml3d.modelling.basic;

import java.text.MessageFormat;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.DrainSlotIterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.Instance;
import eu.moonlight3d.ml3d.backend.threedim.sg.MaterialNode;
import eu.moonlight3d.ml3d.backend.threedim.sg.Node;

public class SGNodeElementCounter implements ElementCounter {

	public String getCount() {
		State state=State.getInstance();
		Document document=(Document) state.getDocumentManager().getMainDocument();
		
		DrainSlotIterator slotIterator=new DrainSlotIterator(document.getOGManager());
		
		int totalCount=0;
		int instanceCount=0;
		int materialCount=0;
		while(slotIterator.hasNext()) {
			eu.moonlight3d.ml3d.backend.threedim.og.Slot slot=slotIterator.next();
			
			try {
				totalCount+= slot.getGraph().getNodes().size();
				
				for(Node node : slot.getGraph().getNodes()) {
					if(node instanceof Instance) {
						instanceCount++;
					} else if(node instanceof MaterialNode) {
						materialCount++;
					}
				}
			} catch (Exception e) {
				// should not happen
				ExceptionDialog.run(e);
			}
		}
		
		String result;
		
		result=MessageFormat.format(L10N.translate("{0} total ({1} instances and groups, {2} material nodes)"), totalCount,instanceCount,materialCount);
		
		return result;
	}

	public String getLabel() {
		return L10N.translate("Scene Graph Nodes: ");
	}

}
