/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Mar 23, 2009
 */

package eu.moonlight3d.ml3d.modelling.basic;

import java.util.ArrayList;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.localization.L10N;

public class SceneStatisticsDialog extends QDialog {
	private static ArrayList<ElementCounter> elementCounters;
	
	static {
		elementCounters=new ArrayList<ElementCounter>();
	}
	
	public static void addElementCounter(ElementCounter counter) {
		elementCounters.add(counter);
	}
	
	public static void removeElementCounter(ElementCounter counter) {
		elementCounters.remove(counter);
	}
	
	private QTextEdit countingResult;
	private QPushButton closeButton;	
	
	public SceneStatisticsDialog(QWidget parent) {
		super(parent);
		setWindowTitle(L10N.translate("Scene Statistics"));
		
		countingResult=new QTextEdit(this);
		countingResult.setReadOnly(true);
		
		closeButton=new QPushButton(this);
		closeButton.setText(L10N.translate("Close"));
		closeButton.clicked.connect(this, "close()");
		
		QVBoxLayout layout=new QVBoxLayout(this);
		layout.addWidget(countingResult);
		layout.addWidget(closeButton);
		layout.setAlignment(closeButton, Qt.AlignmentFlag.AlignCenter);
		setLayout(layout);
		
		countElements();
	}
	
	private void countElements() {
		String result="<html><h3>Scene Element Count:</h3><table>";
		
		for(ElementCounter counter : elementCounters) {
			String count=counter.getCount();
			if(count!=null) {
				result+="<tr><td>" + counter.getLabel() + "</td><td>" + count + "</td></tr>";
			}
		}
		
		result+="</table>";
		
		countingResult.setHtml(result);
	}
}
