/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.basic;


import com.trolltech.qt.gui.QCheckBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modal tool to transform the current selection if possible.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool eu.moonlight3d.ml3d.modelling.basic.TransformMode
 *
 * @author gregor
 */
public class TransformTool extends Tool {
	
	/**
	 * Default constructor taking the tool's mode instance as a parameter.
	 * @param mode the mode instance that this tool is assigned to.
	 */
	public TransformTool(TransformMode mode) {
		super(mode);
		setName("TransformTool");
		setDisplayName(L10N.translate("Transform Selection"));
		setCategory("Edit");
	}

	/**
	 * Translate checkbox in the option editior
	 */
	private QCheckBox translate;
	/**
	 * Rotate checkbox in the option editior
	 */
	private QCheckBox rotate;
	/**
	 * Scale checkbox in the option editior
	 */
	private QCheckBox scale;
	/**
	 * Snap to Grid checkbox in the option editior
	 */
	private QCheckBox snapToGrid;

	@Override
	public void activated() {
		// nothing to do here, mode takes care of rest
	}

	@Override
	public void deactivated() {
		// nothing to do here, mode takes care of rest
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);
		
		translate=new QCheckBox(optionEditor);
		translate.setText(L10N.translate("translate"));
		translate.setChecked(true);
		translate.toggled.connect(this,"translateChanged()");
		rotate=new QCheckBox(optionEditor);
		rotate.setText(L10N.translate("rotate"));
		rotate.setChecked(true);
		rotate.toggled.connect(this,"rotateChanged()");
		scale=new QCheckBox(optionEditor);
		scale.setText(L10N.translate("scale"));
		scale.setChecked(true);
		scale.toggled.connect(this,"scaleChanged()");
		snapToGrid=new QCheckBox(optionEditor);
		snapToGrid.setText(L10N.translate("snap to grid"));
		snapToGrid.setChecked(false);
		snapToGrid.toggled.connect(this,"snapToGridChanged()");
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(translate);
		layout.addWidget(rotate);
		layout.addWidget(scale);
		layout.addWidget(snapToGrid);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}
	
	/**
	 * Enable or disable translation
	 * 
	 * @param translateEnabled true if translation should be enabled
	 */
	public void setTranslateEnabled(boolean translateEnabled) {
		translate.setChecked(translateEnabled);
		((TransformMode)getMode()).setTranslateEnabled(translateEnabled);
	}

	/**
	 * Enable or disable rotation
	 * 
	 * @param rotateEnabled true if rotation should be enabled
	 */
	public void setRotateEnabled(boolean rotateEnabled) {
		rotate.setChecked(rotateEnabled);
		((TransformMode)getMode()).setRotateEnabled(rotateEnabled);
	}

	/**
	 * Enable or disable scale
	 * 
	 * @param scaleEnabled true if translation should be enabled
	 */
	public void setScaleEnabled(boolean scaleEnabled) {
		scale.setChecked(scaleEnabled);
		((TransformMode)getMode()).setScaleEnabled(scaleEnabled);
	}

	/**
	 * Handle toggling of the translate checkbox
	 */
	protected void translateChanged() {
		((TransformMode)getMode()).setTranslateEnabled(translate.isChecked());
	}
	
	/**
	 * Handle toggling of the rotate checkbox
	 */
	protected void rotateChanged() {
		((TransformMode)getMode()).setRotateEnabled(rotate.isChecked());
	}

	/**
	 * Handle toggling of the scale checkbox
	 */
	protected void scaleChanged() {
		((TransformMode)getMode()).setScaleEnabled(scale.isChecked());
	}
	
	/**
	 * Handle toggling of the snap to grid checkbox
	 */
	protected void snapToGridChanged() {
		((TransformMode)getMode()).setSnapToGridEnabled(snapToGrid.isChecked());		
	}
}
