/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 1, 2006
 */
package eu.moonlight3d.ml3d.modelling.camera;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.ui.mainwindow.view.RenderableNode;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Scene;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Viewport;


public class CameraRenderable extends RenderableNode {
    private final Vector3D position;
    private final Vector3D up;
    private final Vector3D right;
    private final Vector3D front;
    private Colour selectedColour;
    private Colour activeColour;
    private boolean active;
    private boolean selected;

    public CameraRenderable(Vector3D position, Vector3D front, Vector3D up, Vector3D right)
    {
    	this.position=position.clone();
    	this.front=front.clone();
    	this.up=up.clone();
    	this.right=right.clone();
        try {
			selectedColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("selectedColour").getColourValue();
	        activeColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("activeColour").getColourValue();
		} catch (Exception e) {
			e.printStackTrace();
			selectedColour=new Colour(1.0,0.0,0.0,1.0);
			activeColour=new Colour(1.0,0.5,0.5,1.0);
		}
    }

	@Override
	public void drawPreview(Viewport view, Scene scene) {
		// nothing to draw here
	}

	@Override
	public void drawVisibleStructure(Viewport view, Scene scene) {
        GL gl=view.getGL();
        
        if(active) {
            gl.glColor3d(activeColour.red,activeColour.green,activeColour.blue);
        } else if(selected) {
            gl.glColor3d(selectedColour.red,selectedColour.green,selectedColour.blue);
        } else {
            gl.glColor3d(1.0,1.0,1.0);
        }

        Vector3D pos2;
        
        gl.glBegin(GL.GL_LINES);

        gl.glVertex3d(position.X1,position.X2,position.X3);
        pos2=position.add(front).add(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        
        gl.glVertex3d(position.X1,position.X2,position.X3);
        pos2=position.add(front).add(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        
        gl.glVertex3d(position.X1,position.X2,position.X3);
        pos2=position.add(front).sub(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        gl.glVertex3d(position.X1,position.X2,position.X3);
        pos2=position.add(front).sub(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        pos2=position.add(front).add(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        pos2=position.add(front).add(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).sub(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        
        pos2=position.add(front).sub(up).sub(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).sub(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        
        pos2=position.add(front).sub(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up).add(right);
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        // up arrow
        pos2=position.add(front).add(up).add(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up.multiply(1.2)).add(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        pos2=position.add(front).add(up).sub(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up.multiply(1.2)).sub(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        pos2=position.add(front).add(up.multiply(1.3));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up.multiply(1.2)).add(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        pos2=position.add(front).add(up.multiply(1.3));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);
        pos2=position.add(front).add(up.multiply(1.2)).sub(right.multiply(0.1));
        gl.glVertex3d(pos2.X1,pos2.X2,pos2.X3);

        gl.glEnd();
	}

	@Override
	public void drawOverlay(Viewport view, Scene scene) {
		// nothing to draw here
	}

	@Override
	public void drawOverlay2D(Viewport view, Scene scene) {
		// nothing to draw here
	}

}
