/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 6, 2006
 */
package eu.moonlight3d.ml3d.modelling.camera;
/**
 * \package eu.moonlight3d.ml3d.modelling.camera
 *
 * This plugin contains the code for the camera object.
 * 
 * This package is a plugin depending on eu.moonlight3d.ml3d.plugins.ui.view and eu.moonlight3d.ml3d.plugins.ui.toolchest
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private CreateCameraTool createCameraTool;

	private CameraConverter cameraConverter;

	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.camera.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("Camera",Camera.class));
		
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateCamera",CreateCamera.class));
		
		if(!state.getBatchMode()) {
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        cameraConverter=new CameraConverter();
	        converter.registerSGNodeConverter(cameraConverter);
	        
			state.getUIManager().registerAction(new ToolAction("CreateCameraAction","Create Camera",null,"Camera","Cameras and Lights","CreateCameraTool"));
	
	        MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"F_Camera","menubar");
	        MenuNode lightMenu=new MenuNode(MenuNode.Type.Menu,"Camera","");
	        lightMenu.add(new MenuNode(MenuNode.Type.Entry,"Create Camera","CreateCameraAction"));
	        contribution.add(lightMenu);
	        state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);
	        
	        createCameraTool=new CreateCameraTool();
	        eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.addTool(createCameraTool);
		}
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("Camera");

		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateCamera");

		if(!state.getBatchMode()) {
			OpenGLConverter converter=OpenGLConverter.getInstance();
		    converter.unregisterSGNodeConverter(cameraConverter);
		    
		    state.getUIManager().unregisterAction("CreateCameraAction");
		
		    eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.removeTool(createCameraTool);
		}
	}

}
