/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 29, 2005
 */
package eu.moonlight3d.ml3d.modelling.lighting;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.Instance;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * An operator graph node that creates a light source.
 * 
 * @see eu.moonlight3d.ml3d.modelling.lighting.CreateLightTool
 *
 * @author gregor
 */
public class CreateLight extends Node {
    private Property name;
    private Property intensity;
    private Property colour;
    private Property type;
    private Property angle;
    private Property radius;
    private Property samples;

    public CreateLight()
    {
    	super();
        createSlot("output",Slot.Type.Output);
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Light");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        type=new Property(this);
        type.setName("type");
        type.setType(Property.Type.Integer);
        try {
        	type.setValue(0);
        } catch(Exception e) {
            // This exception shouldn't occur at all in this limited case
        	e.printStackTrace();
        }
        intensity=new Property(this);
        intensity.setName("intensity");
        intensity.setType(Property.Type.Float);
        try {
            intensity.setValue(100.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        colour=new Property(this);
        colour.setName("colour");
        colour.setType(Property.Type.Colour);
        try {
       		Colour c=new Colour();
       		c.red=1.0;
       		c.green=1.0;
       		c.blue=1.0;
       		c.alpha=1.0;
            colour.setValue(c);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        angle=new Property(this);
        angle.setName("angle");
        angle.setType(Property.Type.Float);
        try {
            angle.setValue(45.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        radius=new Property(this);
        radius.setName("radius");
        radius.setType(Property.Type.Float);
        try {
            radius.setValue(1.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        samples=new Property(this);
        samples.setName("samples");
        samples.setType(Property.Type.Integer);
        try {
        	samples.setValue(16);
        } catch(Exception e) {
            // This exception shouldn't occur at all in this limited case
        	e.printStackTrace();
        }
    }
    
    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.backend.og.Node#update()
     */
    @Override
    public void update() throws Exception
    {
    	SceneGraph sceneGraph=new SceneGraph(getDocument());
        Instance instance;
        
        instance=new eu.moonlight3d.ml3d.backend.threedim.sg.Instance();
        instance.setName(name.getStringValue() + "Instance");

        sceneGraph.addNode(instance);
        
        switch(type.getIntegerValue()) {
        case 0: // OmniLight
            OmniLight omniLight=new OmniLight();
            
            omniLight.setName(name.getStringValue());
            omniLight.setIntensity(intensity.getFloatValue());
            omniLight.setColour(colour.getColourValue());
            
            sceneGraph.addNode(omniLight);
            sceneGraph.addChild(instance,omniLight);
        	break;
        case 1: // SpotLight
            SpotLight spotLight=new SpotLight();
            
            spotLight.setName(name.getStringValue());
            spotLight.setIntensity(intensity.getFloatValue());
            spotLight.setAngle(angle.getFloatValue());
            spotLight.setColour(colour.getColourValue());
            
            sceneGraph.addNode(spotLight);
            sceneGraph.addChild(instance,spotLight);
        	break;
        case 2: // SphereLight
        	SphereLight sphereLight=new SphereLight();
        	
            sphereLight.setName(name.getStringValue());
            sphereLight.setIntensity(intensity.getFloatValue());
            sphereLight.setRadius(radius.getFloatValue());
            sphereLight.setColour(colour.getColourValue());
            sphereLight.setSamples(samples.getIntegerValue());
            
            sceneGraph.addNode(sphereLight);
            sceneGraph.addChild(instance,sphereLight);        	
        }
      
        getSlot("output").setGraph(sceneGraph);

    }

	@Override
	public void removeDependencies() {
	}
}
