/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 31, 2006
 */
package eu.moonlight3d.ml3d.modelling.lighting;

import eu.moonlight3d.ml3d.backend.threedim.sg.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.view.LightSource;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SGNodeConverter;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SceneNode;


/**
 * A scene graph converter that converts a light source to it's
 * representation in the 3D view.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.view.SGNodeConverter
 *
 * @author gregor
 */
public class LightConverter implements SGNodeConverter {

	public boolean canConvertNode(Node node) {
		if(node instanceof OmniLight || node instanceof SphereLight || node instanceof SpotLight) {
			return true;
		}
		return false;
	}

    public SceneNode[] convert(Node node) {
    	boolean selected=false;
    	boolean active=false;
    	
        LightRenderable lightRenderable=null;
        
        if(node instanceof OmniLight) {
        	lightRenderable=new LightRenderable(((OmniLight)node).getPosition(),LightRenderable.Type.Omni);
        } else if(node instanceof SphereLight) {
        	lightRenderable=new LightRenderable(((SphereLight)node).getPosition(),LightRenderable.Type.Sphere);
        	lightRenderable.setSphereRadius(((SphereLight)node).getRadius());
        } else if(node instanceof SpotLight) {
        	lightRenderable=new LightRenderable(((SpotLight)node).getPosition(),LightRenderable.Type.Directional);
        	lightRenderable.setDirection(((SpotLight)node).getDirection());
        }
        
        lightRenderable.setSelected(selected);
        lightRenderable.setActive(active);
		SceneNode[] results=new SceneNode[1];
		results[0]=lightRenderable;
		return results;
	}

	public LightSource[] convertLights(Node node) {
        LightSource lightSource=null;
        
        if(node instanceof OmniLight) {
            lightSource=new LightSource(LightSource.Type.Omni,((OmniLight)node).getPosition(),((OmniLight)node).getColour());
        } else if(node instanceof SphereLight) {
            lightSource=new LightSource(LightSource.Type.Omni,((SphereLight)node).getPosition(),((SphereLight)node).getColour());
        } else if(node instanceof SpotLight) {
            lightSource=new LightSource(LightSource.Type.Omni,((SpotLight)node).getPosition(),((SpotLight)node).getColour());
        }
        
        LightSource[] results=new LightSource[1];
        results[0]=lightSource;
        return results;
	}
}
