/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 31, 2006
 */
package eu.moonlight3d.ml3d.modelling.lighting;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.ui.mainwindow.view.RenderableNode;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Scene;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Viewport;


/**
 * A schematic representation of a light source in the 3D view scene
 * graph. This class draws a symbol representing a light source in
 * the 3D view.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.view.Renderable
 *
 * @author gregor
 */
public class LightRenderable extends RenderableNode {
    private Vector3D position;
    private boolean selected;
    private boolean active;

    private static final double radius=0.1;

    private Colour selectedColour;
    private Colour activeColour;
    
    enum Type {
    	Omni,
    	Directional,
    	Sphere
    }
    
    private Type type;
	private double sphereRadius;
	private Vector3D direction;

    public LightRenderable(Vector3D position, Type type)
    {
        this.position=position.clone();
        this.type=type;

        try {
			selectedColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("selectedColour").getColourValue();
	        activeColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("activeColour").getColourValue();
		} catch (Exception e) {
			e.printStackTrace();
			selectedColour=new Colour(1.0,0.0,0.0,1.0);
			activeColour=new Colour(1.0,0.5,0.5,1.0);
		}
    }
    
    void setSphereRadius(double sphereRadius) {
    	this.sphereRadius=sphereRadius;
    }
    
    void setDirection(Vector3D direction)  {
    	this.direction=direction;
    }

	@Override
	public void drawPreview(Viewport view, Scene scene) {
		// TODO Auto-generated method stub

	}
	
	private void drawLightSymbol(GL gl) {
		double radiusSqrt2 = radius / Math.sqrt(2);
		gl.glBegin(GL.GL_LINES);
		gl.glVertex3d(position.X1 - radius, position.X2, position.X3);
		gl.glVertex3d(position.X1 + radius, position.X2, position.X3);
		gl.glVertex3d(position.X1, position.X2 - radius, position.X3);
		gl.glVertex3d(position.X1, position.X2 + radius, position.X3);
		gl.glVertex3d(position.X1, position.X2, position.X3 - radius);
		gl.glVertex3d(position.X1, position.X2, position.X3 + radius);

		gl.glVertex3d(position.X1 - radiusSqrt2, position.X2 - radiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 + radiusSqrt2, position.X2 + radiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 + radiusSqrt2, position.X2 - radiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 - radiusSqrt2, position.X2 + radiusSqrt2, position.X3);

		gl.glVertex3d(position.X1 - radiusSqrt2, position.X2, position.X3 - radiusSqrt2);
		gl.glVertex3d(position.X1 + radiusSqrt2, position.X2, position.X3 + radiusSqrt2);
		gl.glVertex3d(position.X1 + radiusSqrt2, position.X2, position.X3 - radiusSqrt2);
		gl.glVertex3d(position.X1 - radiusSqrt2, position.X2, position.X3 + radiusSqrt2);

		gl.glVertex3d(position.X1, position.X2 - radiusSqrt2, position.X3 - radiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 + radiusSqrt2, position.X3 + radiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 + radiusSqrt2, position.X3 - radiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 - radiusSqrt2, position.X3 + radiusSqrt2);
		gl.glEnd();
	}
	
	private void drawDirectionalLightSymbol(GL gl) {
		drawLightSymbol(gl);
		
		Vector3D target=position.add(direction);
		
		gl.glBegin(GL.GL_LINES);
		gl.glVertex3d(position.X1,position.X2,position.X3);
		gl.glVertex3d(target.X1, target.X2, target.X3);
		gl.glEnd();
	}
	
	private void drawSphereLightSymbol(GL gl) {
		drawLightSymbol(gl);

		double sphereRadiusSqrt2 = sphereRadius / Math.sqrt(2);
		gl.glBegin(GL.GL_LINES);
		gl.glVertex3d(position.X1 - sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2 - sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 - sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2 + sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2 - sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2 + sphereRadiusSqrt2, position.X3);
		
		gl.glVertex3d(position.X1, position.X2 - sphereRadius, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2 - sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1, position.X2 - sphereRadius, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2 - sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1, position.X2 + sphereRadius, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2 + sphereRadiusSqrt2, position.X3);
		gl.glVertex3d(position.X1, position.X2 + sphereRadius, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2 + sphereRadiusSqrt2, position.X3);
		
		gl.glVertex3d(position.X1, position.X2, position.X3 - sphereRadius);
		gl.glVertex3d(position.X1, position.X2 - sphereRadiusSqrt2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 - sphereRadius);
		gl.glVertex3d(position.X1, position.X2 + sphereRadiusSqrt2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 + sphereRadius);
		gl.glVertex3d(position.X1, position.X2 - sphereRadiusSqrt2, position.X3 + sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 + sphereRadius);
		gl.glVertex3d(position.X1, position.X2 + sphereRadiusSqrt2, position.X3 + sphereRadiusSqrt2);
		
		gl.glVertex3d(position.X1, position.X2 - sphereRadius, position.X3);
		gl.glVertex3d(position.X1, position.X2 - sphereRadiusSqrt2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 - sphereRadius, position.X3);
		gl.glVertex3d(position.X1, position.X2 - sphereRadiusSqrt2, position.X3 + sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 + sphereRadius, position.X3);
		gl.glVertex3d(position.X1, position.X2 + sphereRadiusSqrt2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2 + sphereRadius, position.X3);
		gl.glVertex3d(position.X1, position.X2 + sphereRadiusSqrt2, position.X3 + sphereRadiusSqrt2);
		
		gl.glVertex3d(position.X1 - sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1 - sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2, position.X3 + sphereRadiusSqrt2);
		gl.glVertex3d(position.X1 + sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1 + sphereRadius, position.X2, position.X3);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2, position.X3 + sphereRadiusSqrt2);

		gl.glVertex3d(position.X1, position.X2, position.X3 - sphereRadius);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 - sphereRadius);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2, position.X3 - sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 + sphereRadius);
		gl.glVertex3d(position.X1 - sphereRadiusSqrt2, position.X2, position.X3 + sphereRadiusSqrt2);
		gl.glVertex3d(position.X1, position.X2, position.X3 + sphereRadius);
		gl.glVertex3d(position.X1 + sphereRadiusSqrt2, position.X2, position.X3 + sphereRadiusSqrt2);
		gl.glEnd();
	}

	@Override
	public void drawVisibleStructure(Viewport view, Scene scene) {
        GL gl=view.getGL();
        
        if(getActive()) {
            gl.glColor3d(activeColour.red,activeColour.green,activeColour.blue);
        } else if(getSelected()) {
            gl.glColor3d(selectedColour.red,selectedColour.green,selectedColour.blue);
        } else {
        	gl.glColor3d(1.0,1.0,0.0);
        }

        switch(type) {
        case Omni:
        	drawLightSymbol(gl);
        	break;
        case Directional:
        	drawDirectionalLightSymbol(gl);
        	break;
        case Sphere:
        	drawSphereLightSymbol(gl);
        	break;
        }
	}

	@Override
	public void drawOverlay(Viewport view, Scene scene) {
		// nothing to draw here
	}

	@Override
	public void drawOverlay2D(Viewport view, Scene scene) {
		// nothing to draw here
	}

    public void setSelected(boolean selected)
    {
      this.selected=selected;   
    }
   
    public boolean getSelected()
    {
        return selected;
    }
    
    public void setActive(boolean active)
    {
        this.active=active;
    }
    
    public boolean getActive()
    {
        return active;
    }
}
