/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 6, 2006
 */
package eu.moonlight3d.ml3d.modelling.lighting;
/**
 * \package eu.moonlight3d.ml3d.modelling.lighting
 *
 * This plugin contains the light source objects.
 * 
 * This package is a plugin depending on eu.moonlight3d.ml3d.plugins.ui.view and eu.moonlight3d.ml3d.plugins.ui.toolchest
 */


import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {

	private LightConverter lightConverter;
	private CreateLightTool createLightTool;

	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.lighting.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("OmniLight",OmniLight.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("SphereLight",OmniLight.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("SpotLight",OmniLight.class));

        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateLight",CreateLight.class));

        if(!state.getBatchMode()) {
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        lightConverter=new LightConverter();
	        converter.registerSGNodeConverter(lightConverter);
	
			UIManager manager=state.getUIManager();
	
			manager.registerAction(new ToolAction("CreateLightAction","Create Light",null,"Light","Cameras and Lights","CreateLightTool"));
	        
	        MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"E_Light","menubar");
	        MenuNode lightMenu=new MenuNode(MenuNode.Type.Menu,"Light","");
	        lightMenu.add(new MenuNode(MenuNode.Type.Entry,"Create Light Source","CreateLightAction"));
	        contribution.add(lightMenu);
	        manager.getLayout("mainwindow").addMenuContribution(contribution);
	        
	        createLightTool=new CreateLightTool();
	        eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.addTool(createLightTool);
        }
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("OmniLight");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("SphereLight");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("SpotLight");

        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateLight");

        if(!state.getBatchMode()) {
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        converter.unregisterSGNodeConverter(lightConverter);
	
			UIManager manager=state.getUIManager();
	
			manager.unregisterAction("CreateLightAction");
	
	        eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.removeTool(createLightTool);
        }
	}

}
