/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Aug 12, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * An operator graph node adding a single new vertex to a mesh.
 * 
 * @see eu.moonlight3d.ml3d.backend.threedim.og.Node
 *
 * @author gregor
 */
public class AddVertex extends Node {
    private Property name;
    private Property position;

    public AddVertex()
    {
        super();
        
        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);
        
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Nothing");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        position=new Property(this);
        position.setName("position");
        position.setType(Property.Type.Vector3D);
        try {
            position.setValue(new Vector3D());
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;
        Mesh mesh;
        
        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        if(!(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh)) {
            throw new Exception(L10N.translate("MoveVertices can only operate on mesh nodes"));
        }
        mesh=(Mesh)node;
        
        mesh.addVertex(position.getVectorValue());
        
        getSlot("output").setGraph(graph);
    }

	@Override
	public void removeDependencies() {
	}

}
