/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Apr 16, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * The Bevel operator bevels edges of mesh objects 
 *
 * @author gregor
 */
public class Bevel extends Node {
    private Property name;
    private Property amount;
    private Property recursionLevel;
    
    public Bevel()
    {
    	super();
        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);
        amount=new Property(this);
        amount.setName("amount");
        amount.setType(Property.Type.Float);
        try {
            amount.setValue(1.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        recursionLevel=new Property(this);
        recursionLevel.setName("recursionLevel");
        recursionLevel.setType(Property.Type.Integer);
        try {
            recursionLevel.setValue(1);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Box");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }
    
    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;
        
        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        if(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh) {
            bevel((Mesh)node);
        } else {
            throw new Exception(L10N.translate("bevel operator can only work on mesh objects"));
        }
        getSlot("output").setGraph(graph);        
    }

    private void bevel(Mesh mesh)
    {
        // TODO bevel given object
        /*
         * action plan for beveling:
         * 1. find all edges that have pairs of adjacent faces
         * 2. find all vertices that are shared between three faces
         * 3. for each found edge (and each pair of neighboring faces for each edge):
         *    3.1 build average face normal
         *    3.2 make a cut plane using average normal which is inset from edge by amount
         * 4. for each found vertex:
         *    4.1 build average face normal
         *    4.2 make a cut plane using average normal which is inset from vertex by amount
         * 5. for each cut plane:
         *    5.1 calculate intersections between edges of adjacent "source" elements and plane
         * 6. build faces from intersection points and old points
         * 7. delete obsolete faces
         * TODO fix problems in action plan (edge and vertex qualifying, etc...)
         */
    }

	@Override
	public void removeDependencies() {
	}

}
