/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 1, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * An operator graph node that generates a cone mesh.
 *
 * @author gregor
 */
public class CreateCone extends Node {
    protected Property name;
    protected Property radius;
    protected Property height;
    protected Property divisions;

    public CreateCone()
    {
    	super();
        createSlot("output", Slot.Type.Output);
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Cone");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        radius=new Property(this);
        radius.setName("radius");
        radius.setType(Property.Type.Float);
        try {
            radius.setValue(1.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        height=new Property(this);
        height.setName("height");
        height.setType(Property.Type.Float);
        try {
            height.setValue(1.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        divisions=new Property(this);
        divisions.setName("divisionsU");
        divisions.setType(Property.Type.Integer);
        try {
            divisions.setValue(32);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph=new SceneGraph(getDocument());
        eu.moonlight3d.ml3d.backend.threedim.sg.Instance instance;
        eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh;
        
        instance=new eu.moonlight3d.ml3d.backend.threedim.sg.Instance();
        instance.setName(name.getStringValue() + "Instance");
        
        mesh=new eu.moonlight3d.ml3d.modelling.mesh.Mesh();        
        mesh.setName(name.getStringValue());
        
        graph.addNode(mesh);
        graph.addNode(instance);
        graph.addChild(instance,mesh);
        
        int divisions = this.divisions.getIntegerValue();

		// generate vertices
		Vector3D vertex = new Vector3D();
		vertex.X1 = 0;
		vertex.X2 = 0;
		vertex.X3 = height.getFloatValue();
		mesh.addVertex(vertex);
		for (int i = 0; i < divisions; i++) {
			double phi = 2 * i * Math.PI / divisions;

			vertex.X1 = radius.getFloatValue() * Math.cos(phi);
			vertex.X2 = radius.getFloatValue() * Math.sin(phi);
			vertex.X3 = 0;
			int vertexID=mesh.addVertex(vertex);
			mesh.addEdge(vertexID,0); // add vertical edges
		}
		vertex.X1 = 0;
		vertex.X2 = 0;
		vertex.X3 = 0;
		mesh.addVertex(vertex);
		
		// add ring edges
		for(int i=0;i<divisions;i++) {
			mesh.addEdge(i+1,(i+1)%(divisions)+1);
		}
		// add radial edges
		for(int i=0;i<divisions;i++) {
			mesh.addEdge(i+1,divisions+1);
		}
		// add bottom faces
		// so far we've created (3*division) edges; we're interested in ring and radial edges 
		for(int i=0;i<divisions;i++) {
			ArrayList<Integer> edgeLoop=new ArrayList<Integer>();
			
			edgeLoop.clear();
			edgeLoop.add(divisions+i); // ring edge
			edgeLoop.add(2*divisions+i); // first radial edge
			edgeLoop.add(2*divisions+(i+1)%divisions); // second radial edge
			
			mesh.addFace(edgeLoop);
		}
		// add ring faces
		for(int i=0;i<divisions;i++) {
			ArrayList<Integer> edgeLoop=new ArrayList<Integer>();
			
			edgeLoop.add(i); // first vertical edge
			edgeLoop.add(divisions+(i)%divisions); // ring edge
			edgeLoop.add((i+1)%divisions); // second vertical edge
			
			mesh.addFace(edgeLoop);
		}
        
        getSlot("output").setGraph(graph);
    }

	@Override
	public void removeDependencies() {
	}
}
