/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A tool that adds a cone mesh to the scene.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class CreateConeTool extends Tool {
	/**
	 * The initial base radius of the cone created by this tool.
	 */
	private QLineEdit radius;
	/**
	 * The initial height of the cone created by this tool.
	 */
	private QLineEdit height;
	/**
	 * The initial count of subdivisions of the cone created by this tool.
	 */
	private QLineEdit divisions;
	
	/**
	 * Default constructor.
	 */
	public CreateConeTool() {
		super();
		setName("CreateConeTool");
		setDisplayName("Create Cone");
		setCategory("Create/Mesh");
	}

	@Override
	public void activated() {
        State state=eu.moonlight3d.framework.State.getInstance();

        if(Integer.parseInt(divisions.text())<3) {
			state.logger().warning("cannot create a cone with less than 3 divisions");
			return;
		}
		
        Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
        try {
        	Node createCone=ogManager.createNodeByName("CreateCone");
        	createCone.getProperty("radius").setValue(Double.parseDouble(radius.text()));
        	createCone.getProperty("height").setValue(Double.parseDouble(height.text()));
        	createCone.getProperty("divisionsU").setValue(Integer.parseInt(divisions.text()));
        	ogManager.update();
	        Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
	        selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode"));
	        selection.add("OperatorGraph/" + createCone.getName() + "/output/ConeInstance");
        } catch(Exception e) {
        	ExceptionDialog.run(e);
        }
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);

        QLabel radiusLabel=new QLabel(optionEditor);
        radiusLabel.setText("Cone radius:");
		radius=new QLineEdit(optionEditor);
		radius.setValidator(new QDoubleValidator(radius));
		radius.setText("1.0");
        
        QLabel heightLabel=new QLabel(optionEditor);
        heightLabel.setText("Cone height:");
		height=new QLineEdit(optionEditor);
		height.setValidator(new QDoubleValidator(height));
		height.setText("1.0");

        QLabel divisionsLabel=new QLabel(optionEditor);
        divisionsLabel.setText("Cone divisions:");
		divisions=new QLineEdit(optionEditor);
		divisions.setValidator(new QIntValidator(divisions));
		divisions.setText("32");
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(radiusLabel);
		layout.addWidget(radius);
		layout.addWidget(heightLabel);
		layout.addWidget(height);
		layout.addWidget(divisionsLabel);
		layout.addWidget(divisions);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
