/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modeless tool that adds a pipe mesh to the scene.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class CreatePipeTool extends Tool {
	/**
	 * The initial inner radius of the pipe created by this tool
	 */
	private QLineEdit radius;
	/**
	 * The initial height of the pipe created by this tool
	 */
	private QLineEdit height;
	/**
	 * The initial thickness of the pipe created by this tool
	 */
	private QLineEdit thickness;
	/**
	 * The initial number of subdivisions of the pipe created by this tool
	 */
	private QLineEdit divisions;
	
	/**
	 * Default constructor.
	 */
	public CreatePipeTool() {
		super();
		setName("CreatePipeTool");
		setDisplayName(L10N.translate("Create Pipe"));
		setCategory("Create/Mesh");
	}

	@Override
	public void activated() {
        State state=eu.moonlight3d.framework.State.getInstance();

        if(Integer.parseInt(divisions.text())<3) {
			state.logger().warning(L10N.translate("cannot create a pipe with less than 3 divisions"));
			return;
		}
		
        Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
        try {
			Node createPipe=ogManager.createNodeByName("CreatePipe");
        	createPipe.getProperty("radius").setValue(Double.parseDouble(radius.text()));
        	createPipe.getProperty("height").setValue(Double.parseDouble(height.text()));
        	createPipe.getProperty("thickness").setValue(Double.parseDouble(thickness.text()));
        	createPipe.getProperty("divisions").setValue(Integer.parseInt(divisions.text()));
	        ogManager.update();
	        Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
	        selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode"));
	        selection.add("OperatorGraph/" + createPipe.getName() + "/output/PipeInstance");
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);

        QLabel radiusLabel=new QLabel(optionEditor);
        radiusLabel.setText(L10N.translate("Pipe radius:"));
		radius=new QLineEdit(optionEditor);
		radius.setValidator(new QDoubleValidator(radius));
		radius.setText("1.0");
        
        QLabel heightLabel=new QLabel(optionEditor);
        heightLabel.setText(L10N.translate("Pipe height:"));
		height=new QLineEdit(optionEditor);
		height.setValidator(new QDoubleValidator(height));
		height.setText("1.0");

        QLabel thicknessLabel=new QLabel(optionEditor);
        thicknessLabel.setText(L10N.translate("Pipe thickness:"));
		thickness=new QLineEdit(optionEditor);
		thickness.setValidator(new QDoubleValidator(thickness));
		thickness.setText("1.0");

        QLabel divisionsLabel=new QLabel(optionEditor);
        divisionsLabel.setText(L10N.translate("Pipe divisions:"));
		divisions=new QLineEdit(optionEditor);
		divisions.setValidator(new QIntValidator(divisions));
		divisions.setText("32");
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(radiusLabel);
		layout.addWidget(radius);
		layout.addWidget(heightLabel);
		layout.addWidget(height);
		layout.addWidget(thicknessLabel);
		layout.addWidget(thickness);
		layout.addWidget(divisionsLabel);
		layout.addWidget(divisions);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
