/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Created on Jan 10, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * This is the implementation of the create plane operator graph node
 *
 * @author gregor
 */
public class CreatePlane extends eu.moonlight3d.ml3d.backend.threedim.og.Node {
    private Property size;
    private Property name;
    private Property divisionsX;
    private Property divisionsY;

    public CreatePlane() {
    	super();
        createSlot("output",Slot.Type.Output);
        size=new Property(this);
        size.setName("size");
        size.setType(Property.Type.Float);
        try {
            size.setValue(1.0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Plane");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }

        divisionsX=new Property(this);
        divisionsX.setName("divisionsX");
        divisionsX.setType(Property.Type.Integer);
        try {
        	divisionsX.setValue(0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }

        divisionsY=new Property(this);
        divisionsY.setName("divisionsY");
        divisionsY.setType(Property.Type.Integer);
        try {
        	divisionsY.setValue(0);
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.backend.og.Node#update()
     */
    @Override
    public void update() throws Exception
    {
    	SceneGraph graph=new SceneGraph(getDocument());
        eu.moonlight3d.ml3d.backend.threedim.sg.Instance instance;
        eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh;
        ArrayList<Integer> edges=new ArrayList<Integer>();

        instance=new eu.moonlight3d.ml3d.backend.threedim.sg.Instance();
        instance.setName(name.getStringValue() + "Instance");
        mesh=new eu.moonlight3d.ml3d.modelling.mesh.Mesh();
        mesh.setName(name.getStringValue());

        graph.addNode(instance);
        graph.addNode(mesh);
        graph.addChild(instance,mesh);

        // create vertices
        double size = this.size.getFloatValue();
        int vertX = divisionsX.getIntegerValue() + 2;
        int vertY = divisionsY.getIntegerValue() + 2;
        for (int y = 0; y < vertY; y++)
        	for (int x = 0; x < vertX; x++)
        	{
        		double vx = x * size / (vertX - 1) - size / 2;
        		double vy = y * size / (vertY - 1) - size / 2;
        		mesh.addVertex(new Vector3D(vx, vy, 0));
        	}

        // create the edges
    	for (int y = 0; y < vertY; y++)
    		for (int x = 0; x < vertX - 1; x++)
        	{
        		mesh.addEdge(vertX * y + x, vertX * y + x + 1);
        	}
        for (int x = 0; x < vertX; x++)
        	for (int y = 0; y < vertY - 1; y++)
        	{
        		mesh.addEdge(vertX * y + x, x + vertX * y + vertX);
        	}

        // create all faces
		int vertEdges = vertY * (vertX - 1);
		for (int x = 0; x < vertX - 1; x++)
			for (int y = 0; y < vertY - 1; y++)
			{
				edges.add((vertX - 1) * y + x);
				edges.add(vertEdges + y + (vertY - 1) * (x + 1));
				edges.add((vertX - 1) * (y + 1) + x);
				edges.add(vertEdges + y + (vertY - 1) * x);
				mesh.addFace(edges);
				edges.clear();
			}
		getSlot("output").setGraph(graph);

    }

	@Override
	public void removeDependencies() {
	}
}
