/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A tool that adds a simple planar mesh to the scene.
 *
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 * @author gregor
 */
public class CreatePlaneTool extends Tool
{
	/**
	 * The initial size of the plane created by this tool
	 */
	private QLineEdit size;

	/**
	 * The divisions of the plane algong the x-axis
	 */
	private QLineEdit divX;

	/**
	 * The divisions of the plane along the z-axis1
	 */
	private QLineEdit divY;

	/**
	 * Default constructor.
	 */
	public CreatePlaneTool()
	{
		super();
		setName("CreatePlaneTool");
		setDisplayName(L10N.translate("Create Plane"));
		setCategory("Create/Mesh");
	}

	@Override
	public void activated()
	{
		State state = eu.moonlight3d.framework.State.getInstance();
		Manager ogManager = ((Document) state.getDocumentManager().getMainDocument())
				.getOGManager();
		try
		{
			Node createPlane = ogManager.createNodeByName("CreatePlane");
			createPlane.getProperty("size").setValue(Double.parseDouble(size.text()));
			createPlane.getProperty("divisionsX").setValue(Integer.parseInt(divX.text()));
			createPlane.getProperty("divisionsY").setValue(Integer.parseInt(divY.text()));
			ogManager.update();
			Selection selection = ((Document) State.getInstance().getDocumentManager()
					.getMainDocument()).getSelectionManager().getSelection("viewSelection");
			selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager
					.getSelectionType("SGNode"));
			selection.add("OperatorGraph/" + createPlane.getName() + "/output/PlaneInstance");
		} catch (Exception e)
		{
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated()
	{
		// TODO Auto-generated method stub

	}

	@Override
	public QWidget getOptionEditor(QWidget parent)
	{
		QWidget optionEditor = new QWidget(parent);

		QLabel sizeLabel = new QLabel(optionEditor);
		sizeLabel.setText(L10N.translate("Plane size:"));
		size = new QLineEdit(optionEditor);
		size.setValidator(new QDoubleValidator(size));
		size.setText("1.0");

		QLabel divXLabel = new QLabel(optionEditor);
		divXLabel.setText(L10N.translate("Subdivisions x:"));
		divX = new QLineEdit(optionEditor);
		divX.setValidator(new QIntValidator(divX));
		divX.setText("0");

		QLabel divZLabel = new QLabel(optionEditor);
		divZLabel.setText(L10N.translate("Subdivisions y:"));
		divY = new QLineEdit(optionEditor);
		divY.setValidator(new QIntValidator(divY));
		divY.setText("0");

		QVBoxLayout layout = new QVBoxLayout();
		layout.addWidget(sizeLabel);
		layout.addWidget(size);
		layout.addWidget(divXLabel);
		layout.addWidget(divX);
		layout.addWidget(divZLabel);
		layout.addWidget(divY);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor()
	{
		return true;
	}

}
