/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 1, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * A operator graph node that generates a spehical mesh.
 *
 * @author gregor
 */
public class CreateSphere extends Node {
	protected Property name;

	protected Property radius;

	protected Property divisionsU;

	protected Property divisionsV;

	public CreateSphere() {
		super();
		createSlot("output", Slot.Type.Output);
		radius = new Property(this);
		radius.setName("radius");
		radius.setType(Property.Type.Float);
		try {
			radius.setValue(1.0);
		} catch (Exception e) {
			// This exception shouldn't occur at all in this limited case
			e.printStackTrace();
		}
		name = new Property(this);
		name.setName("name");
		name.setType(Property.Type.String);
		try {
			name.setValue("Sphere");
		} catch (Exception e) {
			// This exception shouldn't occur at all in this limited case
			e.printStackTrace();
		}
		divisionsU = new Property(this);
		divisionsU.setName("divisionsU");
		divisionsU.setType(Property.Type.Integer);
		try {
			divisionsU.setValue(16);
		} catch (Exception e) {
			// This exception shouldn't occur at all in this limited case
			e.printStackTrace();
		}
		divisionsV = new Property(this);
		divisionsV.setName("divisionsV");
		divisionsV.setType(Property.Type.Integer);
		try {
			divisionsV.setValue(16);
		} catch (Exception e) {
			// This exception shouldn't occur at all in this limited case
			e.printStackTrace();
		}
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=new SceneGraph(getDocument());
		eu.moonlight3d.ml3d.backend.threedim.sg.Instance instance;
		eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh;

		instance = new eu.moonlight3d.ml3d.backend.threedim.sg.Instance();
        instance.setName(name.getStringValue() + "Instance");
		mesh = new eu.moonlight3d.ml3d.modelling.mesh.Mesh();

		mesh.setName(name.getStringValue());
		graph.addNode(instance);
		graph.addNode(mesh);
		graph.addChild(instance,mesh);

		int divU = divisionsU.getIntegerValue();
		int divV = divisionsV.getIntegerValue();
		
		int vertexReserve=2+(divU-1)*(divV-1);
		int edgeReserve=2*divV+(divU-2)*divV+(divU-1)*(divV-1);
		int faceReserve=2*divV+(divU-2)*divV;
		int faceEdgeReserve=2*divV*3+(divU-2)*divV*4;
		mesh.setReserves(vertexReserve, edgeReserve, faceReserve, faceEdgeReserve);
		mesh.setAutoNormalsUpdate(false);
		
		long startTime, endTime;

		startTime=System.currentTimeMillis();
		// generate vertices (Note: poles are made up by single vertex)
		Vector3D vertex = new Vector3D();
		vertex.X1 = 0;
		vertex.X2 = 0;
		vertex.X3 = radius.getFloatValue();
		mesh.addVertex(vertex);
		for (int i = 0; i < divU - 1; i++) {
			double theta = (i + 1) * Math.PI / divU;
			for (int j = 0; j < divV; j++) {
				double phi = 2 * j * Math.PI / divV;

				vertex.X1 = radius.getFloatValue() * Math.sin(theta) * Math.cos(phi);
				vertex.X2 = radius.getFloatValue() * Math.sin(theta) * Math.sin(phi);
				vertex.X3 = radius.getFloatValue() * Math.cos(theta);
				mesh.addVertex(vertex);
			}
		}
		vertex.X1 = 0;
		vertex.X2 = 0;
		vertex.X3 = -radius.getFloatValue();
		int lastVertexID=mesh.addVertex(vertex);
		
		endTime=System.currentTimeMillis();
		System.out.println("vertex generation: " + Long.toString(endTime-startTime));

		startTime=System.currentTimeMillis();
//		ArrayList<Integer> edges = new ArrayList<Integer>();
		// add pole edges
		for(int i=0;i<divV;i++) {
//			edges.add(0);
//			edges.add(i+1);
//			edges.add(lastVertexID);
//			edges.add(lastVertexID-divV+i);
			mesh.addEdge(0,i+1);
			mesh.addEdge(lastVertexID,lastVertexID-divV+i);
		}
		// add longitudinal edges
		for(int i=0;i<divU-2;i++) {
			for(int j=0;j<divV;j++) {
				int vertexIndex=i*divV+j;
//				edges.add(1+vertexIndex);
//				edges.add(1+vertexIndex+divV);
				mesh.addEdge(1+vertexIndex,1+vertexIndex+divV);
			}
		}
		// add lateral edges
		for(int i=0;i<divU-1;i++) {
			// build edges at same latitude
			for(int j=0;j<divV-1;j++) {
				int vertexIndex=i*divV+j;
//				edges.add(1+vertexIndex);
//				edges.add(1+vertexIndex+1);
				mesh.addEdge(1+vertexIndex,1+vertexIndex+1);
			}
			// edge between first and last vertex (by index) of this latitude does not fit above scheme
//			edges.add(1+i*divV);
//			edges.add(1+(i+1)*divV-1);
			mesh.addEdge(1+i*divV,1+(i+1)*divV-1);
		}
		
		// calculate some important edge IDs
		int baseLongitudinalEdgeID=2*divV;
		int baseLateralEdgeID=2*divV+(divU-2)*divV;
		
//		mesh.addEdges(edges);

		endTime=System.currentTimeMillis();
		System.out.println("edge generation: " + Long.toString(endTime-startTime));
		ArrayList<Integer> edgeIDs=new ArrayList<Integer>();
//		ArrayList<ArrayList<Integer>> faces=new ArrayList<ArrayList<Integer>>();

		// add pole triangles
		for(int i=0;i<divV;i++) {
			// add top triangles
//			edgeIDs=new ArrayList<Integer>();
			edgeIDs.clear();
			
			edgeIDs.add(2*i);
			edgeIDs.add(baseLateralEdgeID+i);
			edgeIDs.add(2*((i+1)%divV)); // somewhat weird but working way to get the wraparound in the ID
			
//			faces.add(edgeIDs);
			mesh.addFace(edgeIDs,false);
			
			// add bottom triangles (same pattern as above)
//			edgeIDs=new ArrayList<Integer>();
			edgeIDs.clear();

			edgeIDs.add(2*i+1);
			edgeIDs.add(2*((i+1)%divV)+1);			
			edgeIDs.add(baseLateralEdgeID+(divU-2)*divV+i);
			
//			faces.add(edgeIDs);
			mesh.addFace(edgeIDs,false);
		}
		// add remaining quads
		for(int i=0;i<divU-2;i++) {
			for(int j=0;j<divV;j++) {
//				edgeIDs=new ArrayList<Integer>();
				edgeIDs.clear();
				
				// upper lateral edge
				edgeIDs.add(baseLateralEdgeID+i*divV+j);
				// first longitudinal edge
				edgeIDs.add(baseLongitudinalEdgeID+i*divV+j);
				// lower lateral edge
				edgeIDs.add(baseLateralEdgeID+(i+1)*divV+j);
				// second longitudinal edge
				edgeIDs.add(baseLongitudinalEdgeID+i*divV+(j+1)%divV);
				
//				faces.add(edgeIDs);
				mesh.addFace(edgeIDs,false);
			}
		}
//		mesh.addFaces(faces,false);
		endTime=System.currentTimeMillis();
		System.out.println("face generation: " + Long.toString(endTime-startTime));
		
		mesh.resetReserves();
		startTime=System.currentTimeMillis();
		mesh.setAutoNormalsUpdate(true);
		endTime=System.currentTimeMillis();
		System.out.println("normals update: " + Long.toString(endTime-startTime));		

		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}
}
