/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modeless tool that adds a spherical mesh to the scene.
 *
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class CreateSphereTool extends Tool {
	/**
	 * The initial radius of the sphere created by this tool
	 */
	private QLineEdit radius;
	/**
	 * The initial subdivions in U direction of the sphere created by this tool
	 */
	private QLineEdit divisionsU;
	/**
	 * The initial subdivions in V direction of the sphere created by this tool
	 */
	private QLineEdit divisionsV;
	
	/**
	 * Default constructor.
	 */
	public CreateSphereTool() {
		super();
		setName("CreateSphereTool");
		setDisplayName(L10N.translate("Create Sphere"));
		setCategory("Create/Mesh");
	}

	@Override
	public void activated() {
        State state=eu.moonlight3d.framework.State.getInstance();
        
        if(Integer.parseInt(divisionsU.text())<2) {
        	state.logger().warning(L10N.translate("cannot create a sphere with less then 2 divisions in U direction"));
        	return;
        }
        
        if(Integer.parseInt(divisionsV.text())<3) {
        	state.logger().warning(L10N.translate("cannot create a sphere with less then 3 divisions in V direction"));
        	return;
        }
        
        Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
        try {
			Node createSphere=ogManager.createNodeByName("CreateSphere");
        	createSphere.getProperty("radius").setValue(Double.parseDouble(radius.text()));
        	createSphere.getProperty("divisionsU").setValue(Integer.parseInt(divisionsU.text()));
        	createSphere.getProperty("divisionsV").setValue(Integer.parseInt(divisionsV.text()));
	        ogManager.update();
	        Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
	        selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode"));
	        selection.add("OperatorGraph/" + createSphere.getName() + "/output/SphereInstance");
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
		// nothing to do here
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);

        QLabel radiusLabel=new QLabel(optionEditor);
        radiusLabel.setText(L10N.translate("Sphere radius:"));
		radius=new QLineEdit(optionEditor);
		radius.setValidator(new QDoubleValidator(radius));
		radius.setText("1.0");
        
        QLabel divisionsULabel=new QLabel(optionEditor);
        divisionsULabel.setText(L10N.translate("Sphere U divisions:"));
		divisionsU=new QLineEdit(optionEditor);
		divisionsU.setValidator(new QIntValidator(divisionsU));
		divisionsU.setText("16");

        QLabel divisionsVLabel=new QLabel(optionEditor);
        divisionsVLabel.setText(L10N.translate("Sphere V divisions:"));
		divisionsV=new QLineEdit(optionEditor);
		divisionsV.setValidator(new QIntValidator(divisionsV));
		divisionsV.setText("16");
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(radiusLabel);
		layout.addWidget(radius);
		layout.addWidget(divisionsULabel);
		layout.addWidget(divisionsU);
		layout.addWidget(divisionsVLabel);
		layout.addWidget(divisionsV);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
