/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 23, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.Arrays;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * An operator graph node that removes a list of faces from a mesh.
 *
 * @author gregor
 */
public class DeleteFaces extends Node {
	/**
	 * Name of the mesh node to delete faces from
	 */
    private Property name;
    /**
     * The list of faces to delete
     */
    private Property faces;

    /**
     * Default constructor
     */
    public DeleteFaces()
    {
        super();

        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);

        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Nothing");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        faces=new Property(this);
        faces.setName("faces");
        faces.setType(Property.Type.IntegerArray);
        try {
            faces.setMaxSize(0);
        } catch (Exception e1) {
            // This exception shouldn't occur at all in this limited case
            e1.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;
        
        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        
        if(!(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh)) {
            throw new Exception(L10N.translate("DeleteFaces can only operate on mesh nodes"));
        }
        
        eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh=(eu.moonlight3d.ml3d.modelling.mesh.Mesh)node;
        
        int faceIDs[]=new int[faces.getSize()];
        
        for(int i=0;i<faces.getSize();i++) {
        	faceIDs[i]=faces.getArrayIntegerValue(i);
        }
        
        Arrays.sort(faceIDs);
        
        for(int i=faceIDs.length-1;i>=0;i--) {
          mesh.removeFace(faceIDs[i]);
        }
        
        getSlot("output").setGraph(graph);
    }

	@Override
	public void removeDependencies() {
	}
}
