/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 23, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.Arrays;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * An operator graph node that removes a list of vertices from a mesh.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class DeleteVertices extends Node {
    private Property name;
    private Property vertices;

    public DeleteVertices()
    {
        super();

        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);

        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Nothing");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        vertices=new Property(this);
        vertices.setName("vertices");
        vertices.setType(Property.Type.IntegerArray);
        try {
            vertices.setMaxSize(0);
        } catch (Exception e1) {
            // This exception shouldn't occur at all in this limited case
            e1.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;
        
        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        
        if(!(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh)) {
            throw new Exception(L10N.translate("DeleteVertices can only operate on mesh nodes"));
        }
        
        eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh=(eu.moonlight3d.ml3d.modelling.mesh.Mesh)node;
        
        int vertexIDs[]=new int[vertices.getSize()];
        
        for(int i=0;i<vertices.getSize();i++) {
        	vertexIDs[i]=vertices.getArrayIntegerValue(i);
        }
        
        Arrays.sort(vertexIDs);
        
        for(int i=vertexIDs.length-1;i>=0;i--) {
          mesh.removeVertex(vertexIDs[i]);
        }
        
        getSlot("output").setGraph(graph);
    }

	@Override
	public void removeDependencies() {
	}
}
