/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 26, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;
import java.util.HashMap;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * An operator graph node that extrudes a set of edges.
 *
 * @author gregor
 */
public class ExtrudeEdges extends Node {
	private Property name;
	private Property edges;
	private Property displacement;
	private HashMap<Integer, Integer> edgesMap;
	
	public ExtrudeEdges()
	{
		super();
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		name=new Property(this);
		name.setName("name");
		name.setType(Property.Type.String);
		try {
			name.setValue("none");
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
		edges=new Property(this);
		edges.setName("edges");
		edges.setType(Property.Type.IntegerArray);
		try {
			edges.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
		displacement=new Property(this);
		displacement.setName("displacement");
		displacement.setType(Property.Type.Vector3D);
		try {
			displacement.setValue(new Vector3D());
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();
		eu.moonlight3d.ml3d.backend.threedim.sg.Node node=graph.getNodeByName(name.getStringValue());
		
		if(node==null) {
			throw new Exception(L10N.translate("given node does not exist"));
		}
		
		if(!(node instanceof Mesh)) {
			throw new Exception(L10N.translate("given object is not a mesh"));
		}
		
		eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh=(Mesh)node;
		
		// first build a list of vertices to displace to avoid duplication of vertices
		ArrayList<Integer> vertices=new ArrayList<Integer>();
		for(int i=0;i<edges.getSize();i++) {
			Mesh.Edge edge=mesh.getEdge(edges.getArrayIntegerValue(i));
			for(int j=0;j<2;j++) {
				boolean present = false;
				for (int k = 0; k < vertices.size(); k++) {
					if (vertices.get(k) == edge.vertexID[j]) {
						present = true;
						break;
					}
				}
				if (!present) {
					vertices.add(edge.vertexID[j]);
				}
			}
		}
		
		// extrude vertices, build new edges between old and new vertices and build maps
		HashMap<Integer,Integer> forwardMap=new HashMap<Integer,Integer>();
		HashMap<Integer,Integer> backwardMap=new HashMap<Integer,Integer>();
		HashMap<Integer,Integer> vertexEdgeMap=new HashMap<Integer,Integer>();
		Vector3D displacement=this.displacement.getVectorValue();
		for(int i=0;i<vertices.size();i++) {
			int newID=mesh.addVertex(mesh.getVertexPosition(vertices.get(i)).add(displacement));
			int edgeID=mesh.addEdge(vertices.get(i),newID);
			forwardMap.put(vertices.get(i),newID);
			backwardMap.put(newID,vertices.get(i));
			vertexEdgeMap.put(vertices.get(i),edgeID);
		}
		
		// build new edges between new vertices and build faces
		edgesMap=new HashMap<Integer, Integer>();
		for(int i=0;i<edges.getSize();i++) {
			int newID[]=new int[2];
			Mesh.Edge edge=mesh.getEdge(edges.getArrayIntegerValue(i));
			newID[0]=forwardMap.get(edge.vertexID[0]);
			newID[1]=forwardMap.get(edge.vertexID[1]);
			
			int newEdgeID=mesh.addEdge(newID[0],newID[1]);
			edgesMap.put(edge.edgeID,newEdgeID);
			
			ArrayList<Integer> edgeList=new ArrayList<Integer>();
			
			edgeList.add(edge.edgeID);
			edgeList.add(vertexEdgeMap.get(edge.vertexID[0]));
			edgeList.add(newEdgeID);
			edgeList.add(vertexEdgeMap.get(edge.vertexID[1]));
			
			mesh.addFace(edgeList);
		}
		
		getSlot("output").setGraph(graph);
	}

	public HashMap<Integer, Integer> getEdgesMap() {
		return (HashMap<Integer, Integer>) edgesMap.clone();
	}

	@Override
	public void removeDependencies() {
	}
}
