/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QCheckBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modal tool to extrude the selected parts (edges, faces, vertices) of a mesh.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool eu.moonlight3d.ml3d.modelling.mesh.ExtrudeMode
 *
 * @author gregor
 */
public class ExtrudeTool extends Tool {
	/**
	 * Snap to grid checkbox in the option editor
	 */
	private QCheckBox snapToGrid;
	
	/**
	 * Default constructor taking the tool's mode instance as a parameter.
	 * @param mode the mode instance that this tool is assigned to.
	 */
	public ExtrudeTool(ExtrudeMode mode) {
		super(mode);
		setName("ExtrudeTool");
		setDisplayName(L10N.translate("Extrude Selection"));
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
		// activation is handled by the attached mode
	}

	@Override
	public void deactivated() {
		// deactivation is handled by the attached mode
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);
		
		snapToGrid=new QCheckBox(optionEditor);
		snapToGrid.setText(L10N.translate("snap to grid"));
		snapToGrid.setChecked(false);
		snapToGrid.toggled.connect(this,"snapToGridChanged()");
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(snapToGrid);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

	/**
	 * Handle toggling of the snap to grid checkbox
	 */
	protected void snapToGridChanged() {
		((ExtrudeMode)getMode()).setSnapToGridEnabled(snapToGrid.isChecked());		
	}
}
