/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 26, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.HashMap;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * An operator graph node that extrudes vertices of a mesh.
 *
 * @author gregor
 */
public class ExtrudeVertices extends Node {
	private Property name;
	private Property vertices;
	private Property displacement;
	private HashMap<Integer, Integer> verticesMap;
	
	public ExtrudeVertices()
	{
		super();
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		name=new Property(this);
		name.setName("name");
		name.setType(Property.Type.String);
		try {
			name.setValue("none");
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
		vertices=new Property(this);
		vertices.setName("vertices");
		vertices.setType(Property.Type.IntegerArray);
		try {
			vertices.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
		displacement=new Property(this);
		displacement.setName("displacement");
		displacement.setType(Property.Type.Vector3D);
		try {
			displacement.setValue(new Vector3D());
		} catch (Exception e) {
			// cannot happen
			e.printStackTrace();
		}
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();
		eu.moonlight3d.ml3d.backend.threedim.sg.Node node=graph.getNodeByName(name.getStringValue());
		
		if(node==null) {
			throw new Exception(L10N.translate("given node does not exist"));
		}
		
		if(!(node instanceof Mesh)) {
			throw new Exception(L10N.translate("given object is not a mesh"));
		}
		
		eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh=(Mesh)node;
		
		// extrude vertices, build new edges between old and new vertices
		verticesMap=new HashMap<Integer,Integer>();
		Vector3D displacement=this.displacement.getVectorValue();
		for(int i=0;i<vertices.getSize();i++) {
			int newID=mesh.addVertex(mesh.getVertexPosition(vertices.getArrayIntegerValue(i)).add(displacement));
			verticesMap.put(vertices.getArrayIntegerValue(i),newID);
			mesh.addEdge(vertices.getArrayIntegerValue(i),newID);
		}
		
		getSlot("output").setGraph(graph);
	}

	public HashMap<Integer, Integer> getVerticesMap() {
		return (HashMap<Integer, Integer>) verticesMap.clone();
	}

	@Override
	public void removeDependencies() {
	}
}
