/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on May 22, 2007
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.OperatorGraphHelper;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.Gizmo;

class ExtrudeVerticesGizmoListener extends ExtrudeGizmoListener {
	private eu.moonlight3d.ml3d.backend.threedim.og.Node currentOperation;
	private boolean didChange=false;
	private Manager ogManager;
	
	private Vector3D oldDisplacement;
	
	@Override
	public Vector3D getInitialPosition() throws Exception {
		State state=State.getInstance();
		Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");

		Mesh.Vertex vertex=(Mesh.Vertex)selection.getObjectFromSelectionString(selection.getActiveObject());
		
		return vertex.position;
	}
	
	private void createNewOperation(String name) throws Exception {
		State state=State.getInstance();
		
        ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
        
        // get name of currently active object, og node and slot
        Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
        String selectionName=selection.getActiveObject();
        String ownerOGNodeName=SelectionHelper.getOGNodeName(selectionName);
        String ownerSGNodeName=SelectionHelper.getSGNodeName(selectionName);
        
        currentOperation=OperatorGraphHelper.insert(selectionName, name, "input", "output");
        
        // pass name of SG node to OG node
        currentOperation.getProperty("name").setValue(ownerSGNodeName);
        // build list of vertices
        int index=0;
        for(Object i : selection.getSelectionObjects()) {
            // TODO: do sanity check on selection (should be one object only for now)
            currentOperation.getProperty("vertices").setArrayValue(index,((Mesh.Vertex)i).vertexID);
            index=index+1;
        }
        state.logger().info(currentOperation.getProperty("name").getStringValue());
        
        // update selection to point to output of new node
        ogManager.update();
        selection.updateBase("OperatorGraph/" + ownerOGNodeName,"OperatorGraph/" + currentOperation.getName());

        didChange=false;			
	}
	
	public void begin(Gizmo.DragMode mode) {
		if(mode==Gizmo.DragMode.TranslateX || mode==Gizmo.DragMode.TranslateY || mode==Gizmo.DragMode.TranslateZ ||
		   mode==Gizmo.DragMode.TranslateXY || mode==Gizmo.DragMode.TranslateXZ || mode==Gizmo.DragMode.TranslateYZ ||
		   mode==Gizmo.DragMode.TranslateXYZ) {
			if(currentOperation instanceof ExtrudeVertices) {
				try {
					oldDisplacement=currentOperation.getProperty("displacement").getVectorValue();
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("ExtrudeVertices");
					oldDisplacement = new Vector3D();
					oldDisplacement.clear();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		}
	}
	
	public void translate(Vector3D displacment) {
		try {
			// update displacement
			currentOperation.getProperty("displacement").setValue(oldDisplacement.add(displacment));
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
	}

	public void rotate(Vector3D center, Quaternion rotation) {
	}

	public void scale(Vector3D center, Vector3D amount) {
	}

	public void apply() {
		if(!didChange) {
            try {
            	// remove OG node because it is a NOP and restore any OG links
            	String previousOGNodeName=currentOperation.getSlot("input").getCurrentLink().getInput().getNode().getName();
                Selection selection = ((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
                selection.updateBase("OperatorGraph/" + currentOperation.getName(), "OperatorGraph/" + previousOGNodeName);
            	
            	if(currentOperation.getSlot("output").getCurrentLink()!=null) {
            		eu.moonlight3d.ml3d.backend.threedim.og.Slot previous, next;
            		previous=currentOperation.getSlot("input").getCurrentLink().getInput();
            		next=currentOperation.getSlot("output").getCurrentLink().getOutput();
            		ogManager.removeLink(currentOperation.getSlot("output").getCurrentLink());
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            		ogManager.addLink(previous, next);
            	} else {
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            	}
				ogManager.removeNode(currentOperation);
				currentOperation=null;
			} catch (Exception e) {
				// this should not happen
				e.printStackTrace();
			}
		}
	}

	public void end() {
		apply();
		currentOperation=null;
	}
}