/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jul 19, 2008
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;
import java.util.Arrays;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * Operator graph node that reorients faces and face
 * normals by flipping the order of edges in the face.
 *
 * @author gregor
 */
public class FlipFace extends Node {
	/**
	 * IDs of the faces for which to set the normals mode
	 */
	private Property faces;
	/**
	 * The name of the mesh to operate on
	 */
	private Property name;

	/**
	 * Default constructor
	 */
	public FlipFace() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		faces=new Property(this);
		faces.setName("faces");
		faces.setType(Value.Type.IntegerArray);
		faces.setMaxSize(0);
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		name.setValue("None");
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();

		Mesh mesh=(Mesh) graph.getNodeByName(name.getStringValue());
		
		int[] sortedFaces=new int[faces.getSize()];
		for(int i=0;i<faces.getSize();i++) {
			sortedFaces[i]=faces.getArrayIntegerValue(i);
		}
		Arrays.sort(sortedFaces);
		
		for(int i=sortedFaces.length-1;i>=0;i--) {
			int faceID=sortedFaces[i];
			int[] edges=mesh.getFaceEdges(faceID);
			mesh.removeFace(faceID);
			
			// reorder list of edges
			ArrayList<Integer> newEdges=new ArrayList<Integer>();
			for(int j=0;j<edges.length;j++) {
				newEdges.add(edges[edges.length-j-1]);
			}
			mesh.addFace(newEdges);
		}
		
		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}

}
