/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Face;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A tool that changes orientation of mesh faces
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class FlipFaceTool extends Tool {
	/**
	 * Default constructor.
	 */
	public FlipFaceTool() {
		super();
		setName("FlipFaceTool");
		setDisplayName(L10N.translate("Flip Face"));
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshFace")) {
                state.logger().warning(L10N.translate("current selection must be mesh faces"));
                return;            	
            }
            
            String selectionString = selection.getActiveObject();
            String ogNodeName = SelectionHelper.getOGNodeName(selectionString);
            String slotName = SelectionHelper.getOGSlotName(selectionString);
            String sgNodeName = SelectionHelper.getSGNodeName(selectionString);
            
            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            Node ogNode = ogManager.getNodeByName(ogNodeName);
            Slot slot = ogNode.getSlot(slotName);
            Node flipFace = ogManager.createNodeByName("FlipFace");
            flipFace.getProperty("name").setValue(sgNodeName);
            
           	int index=0;
           	for(Object object : selection.getSelectionObjects()) {
           		Mesh.Face face=(Face) object;
           		flipFace.getProperty("faces").setArrayValue(index, face.faceID);
           		index++;
           	}

            // set links
            ogManager.insertNodeAfter(slot,flipFace.getSlot("input"),flipFace.getSlot("output"));
			ogManager.update();
	        selection.updateBase(ogNode.getName(),flipFace.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		return false;
	}

}
