/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 28, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;
import java.util.TreeSet;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.StringTools;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Edge;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Face;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Vertex;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * This tool extends  the active mesh selection, it works for edge, vertex and face mode.  Depending
 * on the current mode, all adjacent vertices, edges or faces are added to the selection.
 * @author Stefan
 *
 */
public class GrowSelection extends Tool {

	/**
	 * Default constructor, initialises the object with all important data.
	 */
	public GrowSelection() {
		setName("GrowSelectionTool");
		setDisplayName(L10N.translate("Grow Selection"));
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            
            // don't do anything if selected object is not a SG node
    		Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getActiveObject()==null) {
            	State.getInstance().logger().info(L10N.translate("no active object to work on"));
            	return;
            }

      	if (selection.getType() != eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshVertex")
      			&& selection.getType() != eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshFace")
      			&& selection.getType() != eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshEdge")) {
      		state.logger().warning(L10N.translate("Current selection must be MeshVertex, MeshEdge or MeshFace!"));
      		return;
      	}
      	TreeSet< Integer> newSelection = new TreeSet<Integer>();
      	String appendix;
      	Mesh mesh = (Mesh) selection
					.getObjectFromSelectionString(SelectionHelper
							.stripAppendix(selection.getActiveObject()));
      	if (selection.getType() == eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshVertex"))
      	{
      		// Vertex mode
      		appendix = ":vertices[";
	      	
	      	for (Object o: selection.getSelectionObjects())
	      	{
	      		int[] edges = ((Vertex)o).edgeIDs;
	      		for (int e: edges)
	      		{
	      			int[] vertices2 = mesh.getEdgeVertices(e);
	      			for (int v: vertices2)
		      			// add vertices to the selection
		      			newSelection.add(v);
	      		}
	      	}
      	}
      	else if (selection.getType() == eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshEdge"))
      	{
      		// Edge mode
      		appendix = ":edges[";
      		for (Object o: selection.getSelectionObjects())
      		{
      			Edge edge = (Edge) o;
      			for (int i: edge.vertexID)
      			{
      				int[] edges = mesh.getVertexEdges(i);
      				for (int j: edges)
      					newSelection.add(j);
      			}
      		}
      	}     
      	else if (selection.getType() == eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshFace"))
      	{
      		// Face mode
      		appendix = ":faces[";
      		for (Object o: selection.getSelectionObjects())
      		{
      			Face face= (Face) o;
      			for (int i: face.edgeIDs)
      			{
      				int[] faces = mesh.getEdgeFaces(i);
      				for (int j: faces)
      					newSelection.add(j);
      			}
      		}
      	}	 else
      		// Cannot happen
      		throw new InternalError();
      	// set the new selection
      	String[] parts = StringTools.split(selection.getActiveObject(), ':');
      	ArrayList<String> strings = new ArrayList<String>(newSelection.size());
      	String path = parts[0]  + appendix;
      	for (int i: newSelection)
      		strings.add(path + i + "]");
      	selection.add(strings);
      	} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
