/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Mar 23, 2009
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import java.text.MessageFormat;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.DrainSlotIterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.Instance;
import eu.moonlight3d.ml3d.backend.threedim.sg.MaterialNode;
import eu.moonlight3d.ml3d.backend.threedim.sg.Node;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TreeIterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TypeFilter;
import eu.moonlight3d.ml3d.modelling.basic.ElementCounter;

public class InstancedMeshElementCounter implements ElementCounter {

	public String getCount() {
		State state=State.getInstance();
		Document document=(Document) state.getDocumentManager().getMainDocument();
		
		DrainSlotIterator slotIterator=new DrainSlotIterator(document.getOGManager());

		int vertexCount=0;
		int edgeCount=0;
		int faceCount=0;
		int triangulatedFaceCount=0;
		while(slotIterator.hasNext()) {
			eu.moonlight3d.ml3d.backend.threedim.og.Slot slot=slotIterator.next();
			
			try {
				TreeIterator treeIterator=new TreeIterator(slot.getGraph());
				
				treeIterator.addFilter(new TypeFilter("eu.moonlight3d.ml3d.modelling.mesh.Mesh",true));
				
				while(treeIterator.hasNext()) {
					Node[] nodes=treeIterator.next();
					Node node=nodes[nodes.length-1];
					if(node instanceof Mesh) {
						Mesh mesh=(Mesh) node;
						vertexCount+=mesh.getNumVertices();
						edgeCount+=mesh.getNumEdges();
						faceCount+=mesh.getNumFaces();
						for(int i=0;i<mesh.getNumFaces();i++) {
							triangulatedFaceCount+=mesh.getFaceEdges(i).length-2;
						}
					}
				}
			} catch (Exception e) {
				// should not happen
				ExceptionDialog.run(e);
			}
		}
		
		String result;
		
		result=MessageFormat.format(L10N.translate("{0} vertices, {1} edges, {2} faces ({3} triangles)"), vertexCount, edgeCount, faceCount, triangulatedFaceCount);
		
		return result;
	}

	public String getLabel() {
		return L10N.translate("Mesh complexity (instanced): ");
	}

}
