/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.OperatorGraphHelper;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modeless tool that inserts a new edge between two selected mesh vertices.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class MakeEdgeTool extends Tool {
	/**
	 * Default constructor
	 */
	public MakeEdgeTool() {
		setName("MakeEdgeTool");
		setDisplayName("Make Edge");
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
        try {
            State state = eu.moonlight3d.framework.State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshVertex")) {
                state.logger().warning(L10N.translate("current selection must be two mesh vertices"));
                return;
            }
            if(selection.getActiveObject()==null) {
            	State.getInstance().logger().info(L10N.translate("no active object to work on"));
            	return;
            }
            
            String selectionName=selection.getActiveObject();
            String ogNodeName = SelectionHelper.getOGNodeName(selectionName);
            String sgNodeName = SelectionHelper.getSGNodeName(selectionName);
            

            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            Node ogNode = OperatorGraphHelper.insert(selectionName,"MakeEdge","input","output");
            ogNode.getProperty("name").setValue(sgNodeName);
            
            if(selection.getSelection().size() != 2) {
                state.logger().warning(L10N.translate("exactly two vertices need to be selected"));
                return;
            }

            // only handle the first two vertices in the selection - ignore any rest if present
            int index=0;
            ArrayList<Object> selectedObjects=selection.getSelectionObjects();
            for(int i=0;i<selectedObjects.size();i++) {
                // TODO: do sanity check on selection (should be one object only for now)
                ogNode.getProperty("vertices").setArrayValue(index,((eu.moonlight3d.ml3d.modelling.mesh.Mesh.Vertex)(selectedObjects.get(i))).vertexID);
                index=index+1;
                if(index==2) {
                    break;
                }
            }
			ogManager.update();
	        selection.updateBase(ogNodeName,ogNode.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
