/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 3, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;
/**
 * \package eu.moonlight3d.ml3d.modelling.mesh
 *
 * This plugin contains the Mesh scene graph object and all tools for creating and editing meshes.
 * 
 * This package is a plugin depending on 
 * eu.moonlight3d.ml3d.modelling.map, eu.moonlight3d.ml3d.plugins.ui.toolchest, eu.moonlight3d.ml3d.plugins.ui.view, eu.moonlight3d.ml3d.ui.plugins.ui.widgets
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.modelling.basic.SceneStatisticsDialog;
import eu.moonlight3d.ml3d.modelling.basic.DeleteSelectionTool;
import eu.moonlight3d.ml3d.modelling.basic.TransformMode;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.ModeManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;

/**
 * Plugin class for the mesh modelling plugin
 *
 * @author gregor
 */
public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	/**
	 * Reference to instance of MeshConverter class for later unregistering
	 */
	private MeshConverter meshConverter;
	/**
	 * Reference to create box tool for later unregistering
	 */
	private CreateBoxTool createBoxTool;
	/**
	 * Reference to create cone tool for later unregistering
	 */
	private CreateConeTool createConeTool;
	/**
	 * Reference to create cylinder tool for later unregistering
	 */
	private CreateCylinderTool createCylinderTool;
	/**
	 * Reference to create pipe tool for later unregistering
	 */
	private CreatePipeTool createPipeTool;
	/**
	 * Reference to create plane tool for later unregistering
	 */
	private CreatePlaneTool createPlaneTool;
	/**
	 * Reference to create sphere tool for later unregistering
	 */
	private CreateSphereTool createSphereTool;
	/**
	 * Reference to add vertex tool for later unregistering
	 */
	private AddVertexTool addVertexTool;
	/**
	 * Reference to select vertex mode for later unregistering
	 */
	private SelectVertexMode selectVertexMode;
	/**
	 * Reference to select vertex tool for later unregistering
	 */
	private SelectVertexTool selectVertexTool;
	/**
	 * Reference to make edge tool for later unregistering
	 */
	private MakeEdgeTool makeEdgeTool;
	/**
	 * Reference to select edges mode for later unregistering
	 */
	private SelectEdgesMode selectEdgesMode;
	/**
	 * Reference to select edges tool for later unregistering
	 */
	private SelectEdgesTool selectEdgesTool;
	/**
	 * Reference to divide edges tool for later unregistering
	 */
	private DivideEdgesTool divideEdgesTool;
	/**
	 * Reference to make face tool for later unregistering
	 */
	private MakeFaceTool makeFaceTool;
	/**
	 * Reference to select faces mode for later unregistering
	 */
	private SelectFacesMode selectFacesMode;
	/**
	 * Reference to select faces tool for later unregistering
	 */
	private SelectFacesTool selectFacesTool;
	/**
	 * Reference to split face tool for later unregistering
	 */
	private SplitFaceTool splitFaceTool;
	/**
	 * Reference to flip face tool for later unregistering
	 */
	private FlipFaceTool flipFaceTool;
	/**
	 * Reference to extrude mode for later unregistering
	 */
	private ExtrudeMode extrudeMode;
	/**
	 * Reference to extrude tool for later unregistering
	 */
	private ExtrudeTool extrudeTool;
	/**
	 * Reference to subdivide tool for later unregistering
	 */
	private SubdivideTool subdivideTool;
	/**
	 * Reference to triangulate tool for later unregistering
	 */
	private TriangulateTool triangulateTool;
	/**
	 * Reference to merge meshes tool for later unregistering
	 */
	private MergeMeshesTool mergeMeshesTool;
	/**
	 * Reference to set mesh normals mode tool for later unregistering
	 */
	private SetMeshNormalsModeTool setMeshNormalsModeTool;
	/**
	 * Reference to grow selection tool for later unregistering
	 */
	private GrowSelection growSelectionTool;
	
	/**
	 * Reference to delete edges handler for later unregistering
	 */
	private DeleteEdgesHandler deleteEdgesHandler;
	/**
	 * Reference to delete faces handler for later unregistering
	 */
	private DeleteFacesHandler deleteFacesHandler;
	/**
	 * Reference to delete vertices handler for later unregistering
	 */
	private DeleteVerticesHandler deleteVerticesHandler;
	/**
	 * {@link MeshElementCounter} instance.
	 */
	private MeshElementCounter meshElementCounter;
	/**
	 * {@link InstancedMeshElementCounter} instance.
	 */
	private InstancedMeshElementCounter instancedMeshElementCounter;
	
	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.mesh.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin"); // depend on 3d view, Tool, HandleWidget and Gizmo
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("Mesh",Mesh.class));

		eu.moonlight3d.ml3d.backend.selection.Manager.registerSelectionType(new MeshVertexType());
		eu.moonlight3d.ml3d.backend.selection.Manager.registerSelectionType(new MeshEdgeType());
		eu.moonlight3d.ml3d.backend.selection.Manager.registerSelectionType(new MeshFaceType());

        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("AddVertex",AddVertex.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateBox",CreateBox.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateCone",CreateCone.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateCylinder",CreateCylinder.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreatePipe",CreatePipe.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreatePlane",CreatePlane.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateSphere",CreateSphere.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DeleteEdges",DeleteEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DeleteFaces",DeleteFaces.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DeleteVertices",DeleteVertices.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DivideEdges",DivideEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ExtrudeEdges",ExtrudeEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ExtrudeFaces",ExtrudeFaces.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ExtrudeVertices",ExtrudeVertices.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MakeEdge",MakeEdge.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MakeFace",MakeFace.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MoveVertices",MoveVertices.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("RotateVertices",RotateVertices.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ScaleVertices",ScaleVertices.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MoveEdges",MoveEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("RotateEdges",RotateEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ScaleEdges",ScaleEdges.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MoveFaces",MoveFaces.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("RotateFaces",RotateFaces.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ScaleFaces",ScaleFaces.class));        
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("SplitFace",SplitFace.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("FlipFace",FlipFace.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("SubdivideMesh",SubdivideMesh.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("Triangulate",Triangulate.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MergeMeshes",MergeMeshes.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("SetMeshNormalsMode",SetMeshNormalsMode.class));

        if(!state.getBatchMode()) {
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        meshConverter=new MeshConverter();
	        converter.registerSGNodeConverter(meshConverter);
			
			UIManager manager=state.getUIManager();
		
			manager.registerAction(new ToolAction("CreatePlaneAction",L10N.translate("Create Plane"),"icons/createPolygon.png","Mesh","Meshes","CreatePlaneTool"));
			manager.registerAction(new ToolAction("CreateBoxAction",L10N.translate("Create Box"),"icons/createPolyBox.png","Mesh","Meshes","CreateBoxTool"));
			manager.registerAction(new ToolAction("CreateConeAction",L10N.translate("Create Cone"),"icons/createPolyCone.png","Mesh","Meshes","CreateConeTool"));
			manager.registerAction(new ToolAction("CreateCylinderAction",L10N.translate("Create Cylinder"),"icons/createPolyCylinder.png","Mesh","Meshes","CreateCylinderTool"));
			manager.registerAction(new ToolAction("CreateSphereAction",L10N.translate("Create Sphere"),"icons/createPolySphere.png","Mesh","Meshes","CreateSphereTool"));
			manager.registerAction(new ToolAction("CreatePipeAction",L10N.translate("Create Pipe"),"icons/createPolyPipe.png","Mesh","Meshes","CreatePipeTool"));
			manager.registerAction(new ToolAction("AddVertexAction",L10N.translate("Add Vertex"),null,"Mesh","Meshes","AddVertexTool"));
			manager.registerAction(new ToolAction("SelectVertexAction",L10N.translate("Select Vertex"),null,"Mesh","Meshes","SelectVertexTool","s"));
			manager.registerAction(new ToolAction("MakeEdgeAction",L10N.translate("Make Edge"),null,"Mesh","Meshes","MakeEdgeTool"));
			manager.registerAction(new ToolAction("SelectEdgeAction",L10N.translate("Select Edge"),null,"Mesh","Meshes","SelectEdgesTool","d"));
			manager.registerAction(new ToolAction("DivideEdgesAction",L10N.translate("Divide Edges"),null,"Mesh","Meshes","DivideEdgesTool"));
			manager.registerAction(new ToolAction("MakeFaceAction",L10N.translate("Make Face"),null,"Mesh","Meshes","MakeFaceTool"));
			manager.registerAction(new ToolAction("SelectFaceAction",L10N.translate("Select Face"),null,"Mesh","Meshes","SelectFacesTool","f"));
			manager.registerAction(new ToolAction("SplitFaceAction",L10N.translate("Split Face"),null,"Mesh","Meshes","SplitFaceTool"));
			manager.registerAction(new ToolAction("FlipFaceAction",L10N.translate("Flip Face"),null,"Mesh","Meshes","FlipFaceTool"));
			manager.registerAction(new ToolAction("ExtrudeAction",L10N.translate("Extrude Selection"),null,"Mesh","Meshes","ExtrudeTool","g"));
			manager.registerAction(new ToolAction("SubdivideAction",L10N.translate("Subdivide"),null,"Mesh","Meshes","SubdivideTool"));
			manager.registerAction(new ToolAction("TriangulateAction",L10N.translate("Triangulate"),null,"Mesh","Meshes","TriangulateTool"));
			manager.registerAction(new ToolAction("MergeMeshesAction",L10N.translate("Merge Meshes"),null,"Mesh","Meshes","MergeMeshesTool"));
			manager.registerAction(new ToolAction("SetMeshNormalsModeAction",L10N.translate("Set Normals mode"),null,"Mesh","Meshes","SetMeshNormalsModeTool"));
	
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"D_Mesh","menubar");
			MenuNode meshMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Mesh"),"");
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Plane"),"CreatePlaneAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Box"),"CreateBoxAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Cone"),"CreateConeAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Cylinder"),"CreateCylinderAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Sphere"),"CreateSphereAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Pipe"),"CreatePipeAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Add Vertex"),"AddVertexAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Select Vertex"),"SelectVertexAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Make Edge"),"MakeEdgeAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Select Edge"),"SelectEdgeAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Divide Edge"),"DivideEdgesAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Make Face"),"MakeFaceAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Select Face"),"SelectFaceAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Split Face"),"SplitFaceAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Flip Face"),"FlipFaceAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Extrude Selection"),"ExtrudeAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Subdivide"),"SubdivideAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Triangulate"),"TriangulateAction"));
			meshMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			meshMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Set Normals Mode"),"SetMeshNormalsModeAction"));
			contribution.add(meshMenu);
			state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);
	
			selectVertexMode=new SelectVertexMode();
			ModeManager.getInstance().addMode(selectVertexMode);
			selectEdgesMode=new SelectEdgesMode();
			ModeManager.getInstance().addMode(selectEdgesMode);
			selectFacesMode=new SelectFacesMode();
			ModeManager.getInstance().addMode(selectFacesMode);
			extrudeMode=new ExtrudeMode();
			ModeManager.getInstance().addMode(extrudeMode);
			
			createBoxTool=new CreateBoxTool();
			ToolChestView.addTool(createBoxTool);
			createConeTool=new CreateConeTool();
			ToolChestView.addTool(createConeTool);
			createCylinderTool=new CreateCylinderTool();
			ToolChestView.addTool(createCylinderTool);
			createPipeTool=new CreatePipeTool();
			ToolChestView.addTool(createPipeTool);
			createPlaneTool=new CreatePlaneTool();
			ToolChestView.addTool(createPlaneTool);
			createSphereTool=new CreateSphereTool();
			ToolChestView.addTool(createSphereTool);
			addVertexTool=new AddVertexTool();
			ToolChestView.addTool(addVertexTool);
			selectVertexTool=new SelectVertexTool(selectVertexMode);
			ToolChestView.addTool(selectVertexTool);
			makeEdgeTool=new MakeEdgeTool();
			ToolChestView.addTool(makeEdgeTool);
			selectEdgesTool=new SelectEdgesTool(selectEdgesMode);
			ToolChestView.addTool(selectEdgesTool);
			divideEdgesTool=new DivideEdgesTool();
			ToolChestView.addTool(divideEdgesTool);
			makeFaceTool=new MakeFaceTool();
			ToolChestView.addTool(makeFaceTool);
			selectFacesTool=new SelectFacesTool(selectFacesMode);
			ToolChestView.addTool(selectFacesTool);
			splitFaceTool=new SplitFaceTool();
			ToolChestView.addTool(splitFaceTool);
			flipFaceTool=new FlipFaceTool();
			ToolChestView.addTool(flipFaceTool);
			extrudeTool=new ExtrudeTool(extrudeMode);
			ToolChestView.addTool(extrudeTool);
			subdivideTool=new SubdivideTool();
			ToolChestView.addTool(subdivideTool);
			triangulateTool=new TriangulateTool();
			ToolChestView.addTool(triangulateTool);
			mergeMeshesTool=new MergeMeshesTool();
			ToolChestView.addTool(mergeMeshesTool);
			setMeshNormalsModeTool=new SetMeshNormalsModeTool();
			ToolChestView.addTool(setMeshNormalsModeTool);
			growSelectionTool = new GrowSelection();
            ToolChestView.addTool(growSelectionTool);
			
			TransformMode.registerGizmoListener("MeshVertex", new TransformVerticesGizmoListener());
			TransformMode.registerGizmoListener("MeshEdge", new TransformEdgesGizmoListener());
			TransformMode.registerGizmoListener("MeshFace", new TransformFacesGizmoListener());
			
			deleteEdgesHandler=new DeleteEdgesHandler();
			DeleteSelectionTool.registerHandler(deleteEdgesHandler);
			deleteFacesHandler=new DeleteFacesHandler();
			DeleteSelectionTool.registerHandler(deleteFacesHandler);
			deleteVerticesHandler=new DeleteVerticesHandler();
			DeleteSelectionTool.registerHandler(deleteVerticesHandler);
			
			meshElementCounter=new MeshElementCounter();
			SceneStatisticsDialog.addElementCounter(meshElementCounter);
			instancedMeshElementCounter=new InstancedMeshElementCounter();
			SceneStatisticsDialog.addElementCounter(instancedMeshElementCounter);
        }
	}

	public void unload() throws Exception {
		State state=State.getInstance();

		eu.moonlight3d.ml3d.backend.selection.Manager.unregisterSelectionType("MeshVertex");
		eu.moonlight3d.ml3d.backend.selection.Manager.unregisterSelectionType("MeshEdge");
		eu.moonlight3d.ml3d.backend.selection.Manager.unregisterSelectionType("MeshFace");

		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("Mesh");

        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("AddVertex");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateBox");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateCone");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateCylinder");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreatePipe");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreatePlane");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateSphere");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DeleteEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DeleteFaces");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DeleteVertices");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DivideEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ExtrudeEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ExtrudeFaces");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ExtrudeVertices");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MakeEdge");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MakeFace");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MoveVertices");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("RotateVertices");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ScaleVertices");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MoveEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("RotateEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ScaleEdges");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MoveFaces");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("RotateFaces");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ScaleFaces");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("SplitFace");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("FlipFace");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("SubdivideMesh");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("Triangulate");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MergeMeshes");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("SetMeshNormalsMode");

        if(!state.getBatchMode()) {
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        converter.unregisterSGNodeConverter(meshConverter);
			
			UIManager manager=state.getUIManager();
			
			manager.unregisterAction("CreatePlaneAction");
			manager.unregisterAction("CreateBoxAction");
			manager.unregisterAction("CreateConeAction");
			manager.unregisterAction("CreateCylinderAction");
			manager.unregisterAction("CreateSphereAction");
			manager.unregisterAction("CreatePipeAction");
			manager.unregisterAction("AddVertexAction");
			manager.unregisterAction("SelectVertexAction");
			manager.unregisterAction("MakeEdgeAction");
			manager.unregisterAction("SelectEdgeAction");
			manager.unregisterAction("DivideEdgesAction");
			manager.unregisterAction("MakeFaceAction");
			manager.unregisterAction("SelectFaceAction");
			manager.unregisterAction("SplitFaceAction");
			manager.unregisterAction("FlipFaceAction");
			manager.unregisterAction("ExtrudeAction");
			manager.unregisterAction("SubdivideAction");
			manager.unregisterAction("TriangulateAction");
			manager.unregisterAction("MergeMeshesAction");
			manager.unregisterAction("SetMeshNormalsModeAction");
	
			ToolChestView.removeTool(createBoxTool);
			ToolChestView.removeTool(createConeTool);
			ToolChestView.removeTool(createCylinderTool);
			ToolChestView.removeTool(createPipeTool);
			ToolChestView.removeTool(createPlaneTool);
			ToolChestView.removeTool(createSphereTool);
			ToolChestView.removeTool(addVertexTool);
			ToolChestView.removeTool(selectVertexTool);
			ToolChestView.removeTool(makeEdgeTool);
			ToolChestView.removeTool(selectEdgesTool);
			ToolChestView.removeTool(divideEdgesTool);
			ToolChestView.removeTool(makeFaceTool);
			ToolChestView.removeTool(selectFacesTool);
			ToolChestView.removeTool(splitFaceTool);
			ToolChestView.removeTool(flipFaceTool);
			ToolChestView.removeTool(extrudeTool);
			ToolChestView.removeTool(subdivideTool);
			ToolChestView.removeTool(triangulateTool);
			ToolChestView.removeTool(mergeMeshesTool);
			ToolChestView.removeTool(setMeshNormalsModeTool);
			ToolChestView.removeTool(growSelectionTool);
	
			ModeManager.getInstance().removeMode(selectEdgesMode);
			ModeManager.getInstance().removeMode(selectFacesMode);
			ModeManager.getInstance().removeMode(extrudeMode);
	
			TransformMode.unregisterGizmoListener("MeshVertex");
			TransformMode.unregisterGizmoListener("MeshEdge");
			TransformMode.unregisterGizmoListener("MeshFace");
			
			DeleteSelectionTool.unregisterHandler(deleteEdgesHandler);
			DeleteSelectionTool.unregisterHandler(deleteFacesHandler);
			DeleteSelectionTool.unregisterHandler(deleteVerticesHandler);
			
			SceneStatisticsDialog.removeElementCounter(meshElementCounter);
			SceneStatisticsDialog.removeElementCounter(instancedMeshElementCounter);
        }
	}
}
