/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jul 21, 2008
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.math.Matrix3;
import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

public class Revolve extends Node {
	private Property angle;
	private Property axis;
	private Property closed;
	private Property divisions;
	private Property name;
	private Property pivot;
	
	public Revolve() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		angle=new Property(this);
		angle.setName("angle");
		angle.setType(Value.Type.Float);
		angle.setValue(360);
		axis=new Property(this);
		axis.setName("axis");
		axis.setType(Value.Type.Vector3D);
		axis.setValue(new Vector3D(0,0,1));
		closed=new Property(this);
		closed.setName("closed");
		closed.setType(Value.Type.Boolean);
		closed.setValue(true);
		divisions=new Property(this);
		divisions.setName("divisions");
		divisions.setType(Value.Type.Integer);
		divisions.setValue(32);
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		name.setValue("None");
		pivot=new Property(this);
		pivot.setName("pivot");
		pivot.setType(Value.Type.Vector3D);
		pivot.setValue(new Vector3D(0,0,0));
	}
	
	private void revolveVertices(Mesh mesh) {
		Quaternion rotation=new Quaternion();
		
		int divisions=this.divisions.getIntegerValue();
		
		if(closed.getBooleanValue()==true) {
			divisions--;
		}
		
		for(int i=0;i<mesh.getNumVertices();i++) {
			Vector3D vertexPosition=mesh.getVertexPosition(i);
			int lastVertex=i;
			for(int j=0;j<divisions;j++) {
				Vector3D newPosition=vertexPosition.sub(pivot.getVectorValue());
		        Matrix3 result;
		        rotation.fromAxisAngle(axis.getVectorValue(),(j+1)*angle.getFloatValue()/divisions);
		        result=rotation.toRotationMatrix3();
		        Matrix3.multiply(result, newPosition);
				Vector3D.add(newPosition, pivot.getVectorValue());
				
				int newVertex=mesh.addVertex(newPosition);
				
				mesh.addEdge(lastVertex, newVertex);
			}
			
			if(closed.getBooleanValue()==true) {
				mesh.addEdge(i, lastVertex);
			}
		}
	}
	
	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();
		// TODO Auto-generated method stub
		
		Mesh mesh=(Mesh) graph.getNodeByName(name.getStringValue());
		
		revolveVertices(mesh);

		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}
}
