/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 23, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * An operator graph node that scales edges of a mesh
 * by a certain amount.
 *
 * @author gregor
 */
public class ScaleEdges extends Node {
    private Property name;
    private Property edges;
    private Property pivot;
    private Property scale;

    public ScaleEdges()
    {
        super();

        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);

        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Nothing");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        edges=new Property(this);
        edges.setName("edges");
        edges.setType(Property.Type.IntegerArray);
        try {
            edges.setMaxSize(0);
        } catch (Exception e1) {
            // This exception shouldn't occur at all in this limited case
            e1.printStackTrace();
        }

        pivot=new Property(this);
        pivot.setName("pivot");
        pivot.setType(Property.Type.Vector3D);
        try {
            pivot.setValue(new Vector3D());
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
        scale=new Property(this);
        scale.setName("scale");
        scale.setType(Property.Type.Vector3D);
        try {
            scale.setValue(new Vector3D());
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;
        Vector3D pivot=new Vector3D();
        Vector3D scale=new Vector3D();
        
        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        
        if(!(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh)) {
            throw new Exception(L10N.translate("ScaleEdges can only operate on mesh nodes"));
        }
        
        eu.moonlight3d.ml3d.modelling.mesh.Mesh mesh=(eu.moonlight3d.ml3d.modelling.mesh.Mesh)node;
        
        ArrayList<Integer> vertices=new ArrayList<Integer>();
        for(int i=0;i<edges.getSize();i++) {
        	for(int j=0;j<2;j++) {
        		int vertexID=mesh.getEdge(edges.getArrayIntegerValue(i)).vertexID[j];
        		if(!vertices.contains(vertexID)) {
        			vertices.add(vertexID);
        		}
        	}
        }
        
        pivot=this.pivot.getVectorValue();
        scale=this.scale.getVectorValue();
        
        for(int i=0;i<vertices.size();i++) {
        	int vertexID=vertices.get(i);
            Vector3D position=mesh.getVertexPosition(vertexID);
            
            position.X1=(position.X1-pivot.X1)*scale.X1+pivot.X1;
            position.X2=(position.X2-pivot.X2)*scale.X2+pivot.X2;
            position.X3=(position.X3-pivot.X3)*scale.X3+pivot.X3;
            
            mesh.setVertexPosition(vertexID, position);
        }
        
        getSlot("output").setGraph(graph);
    }

	@Override
	public void removeDependencies() {
	}

}
