/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jul 19, 2008
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

/**
 * Operator graph node that sets the transformation mode of
 * mesh normals.
 *
 * @author gregor
 */
public class SetMeshNormalsMode extends Node {
	/**
	 * IDs of the edges for which to set the normals mode
	 */
	private Property edges;
	/**
	 * IDs of the faces for which to set the normals mode
	 */
	private Property faces;
	/**
	 * The mode to set
	 */
	private Property mode;
	/**
	 * The name of the mesh to operate on
	 */
	private Property name;
	/**
	 * IDs of the vertices for which to set the normals mode
	 */
	private Property vertices;

	/**
	 * Default constructor
	 */
	public SetMeshNormalsMode() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		edges=new Property(this);
		edges.setName("edges");
		edges.setType(Value.Type.IntegerArray);
		edges.setMaxSize(0);
		faces=new Property(this);
		faces.setName("faces");
		faces.setType(Value.Type.IntegerArray);
		faces.setMaxSize(0);
		mode=new Property(this);
		mode.setName("mode");
		mode.setType(Value.Type.Integer);
		mode.setValue(0);
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		name.setValue("None");
		vertices=new Property(this);
		vertices.setName("vertices");
		vertices.setType(Value.Type.IntegerArray);
		vertices.setMaxSize(0);
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();

		Mesh mesh=(Mesh) graph.getNodeByName(name.getStringValue());
		
		for(int i=0;i<faces.getSize();i++) {
			int faceID=faces.getArrayIntegerValue(i);
			int[] edges=mesh.getFaceEdges(faceID);
			for(int j=0;j<edges.length;j++) {
				mesh.setFaceVertexNormalMode(faceID, j, (byte) mode.getIntegerValue());
			}
		}
		
		for(int i=0;i<edges.getSize();i++) {
			int edgeID=edges.getArrayIntegerValue(i);
			int edgeVertices[]=mesh.getEdgeVertices(edgeID);
			int faces[]=mesh.getEdgeFaces(edgeID);
			for(int j=0;j<faces.length;j++) {
				int faceID=faces[j];
				ArrayList<Integer> vertexIDs=mesh.getVertexIDs(mesh.getFace(faceID));
				for(int k=0;k<vertexIDs.size();k++) {
					int vertexID=vertexIDs.get(k);
					if(vertexID==edgeVertices[0] || vertexID==edgeVertices[1]) {
						mesh.setFaceVertexNormalMode(faceID, k, (byte) mode.getIntegerValue());
					}
				}
			}
		}
		
		for(int i=0;i<vertices.getSize();i++) {
			int vertexID=vertices.getArrayIntegerValue(i);
			int[] vertexEdges=mesh.getVertexEdges(vertexID);
			for(int j=0;j<vertexEdges.length;j++) {
				int edgeID=vertexEdges[j];
				int[] edgeFaces=mesh.getEdgeFaces(edgeID);
				for(int k=0;k<edgeFaces.length;k++) {
					int faceID=edgeFaces[k];
					ArrayList<Integer> vertexIDs=mesh.getVertexIDs(mesh.getFace(faceID));
					for(int l=0;l<vertexIDs.size();l++) {
						if(vertexIDs.get(l)==vertexID) {
							mesh.setFaceVertexNormalMode(faceID, l, (byte) mode.getIntegerValue());
						}
					}
				}
			}
		}
		
		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}

}
