/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Edge;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Face;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh.Vertex;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A tool that sets the transformation modes of mesh normals
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class SetMeshNormalsModeTool extends Tool {
	/**
	 * The drop down list to select the mode to apply.
	 */
	private QComboBox mode;
	
	/**
	 * Default constructor.
	 */
	public SetMeshNormalsModeTool() {
		super();
		setName("SetMeshNormalsModeTool");
		setDisplayName(L10N.translate("Set Normals Mode"));
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshEdge") && 
               selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshFace") &&
               selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshVertex")) {
                state.logger().warning(L10N.translate("current selection must be a mesh edges, faces or vertices"));
                return;            	
            }
            
            String selectionString = selection.getActiveObject();
            String ogNodeName = SelectionHelper.getOGNodeName(selectionString);
            String slotName = SelectionHelper.getOGSlotName(selectionString);
            String sgNodeName = SelectionHelper.getSGNodeName(selectionString);
            
            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            Node ogNode = ogManager.getNodeByName(ogNodeName);
            Slot slot = ogNode.getSlot(slotName);
            Node setMeshNormalsMode = ogManager.createNodeByName("SetMeshNormalsMode");
            setMeshNormalsMode.getProperty("name").setValue(sgNodeName);
            
            if(selection.getType()==eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshEdge")) {
            	int index=0;
            	for(Object object : selection.getSelectionObjects()) {
            		Mesh.Edge edge=(Edge) object;
            		setMeshNormalsMode.getProperty("edges").setArrayValue(index, edge.edgeID);
            		index++;
            	}
            } else if(selection.getType()==eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshFace")) {
            	int index=0;
            	for(Object object : selection.getSelectionObjects()) {
            		Mesh.Face face=(Face) object;
            		setMeshNormalsMode.getProperty("faces").setArrayValue(index, face.faceID);
            		index++;
            	}
            } else if(selection.getType()==eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("MeshVertex")) {
            	int index=0;
            	for(Object object : selection.getSelectionObjects()) {
            		Mesh.Vertex vertex=(Vertex) object;
            		setMeshNormalsMode.getProperty("vertices").setArrayValue(index, vertex.vertexID);
            		index++;
            	}
            }

            if(mode.currentIndex()==0) {
            	setMeshNormalsMode.getProperty("mode").setValue(Mesh.NORMAL_MODE_LOCKED);
            } else if(mode.currentIndex()==1) {
            	setMeshNormalsMode.getProperty("mode").setValue(Mesh.NORMAL_MODE_SMOOTH);
            } else if(mode.currentIndex()==2) {
            	setMeshNormalsMode.getProperty("mode").setValue(Mesh.NORMAL_MODE_HARD);
            }
            
            // set links
            ogManager.insertNodeAfter(slot,setMeshNormalsMode.getSlot("input"),setMeshNormalsMode.getSlot("output"));
			ogManager.update();
	        selection.updateBase(ogNode.getName(),setMeshNormalsMode.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget editor=new QWidget(parent);
		
		QLabel modeLabel=new QLabel(editor);
		modeLabel.setText("mode:");
		mode=new QComboBox(editor);
		mode.addItem(L10N.translate("Locked"));
		mode.addItem(L10N.translate("Smooth"));
		mode.addItem(L10N.translate("Hard"));
		mode.setEditable(false);
		
		QVBoxLayout layout=new QVBoxLayout(editor);
		layout.addWidget(modeLabel);
		layout.addWidget(mode);
		
		editor.setLayout(layout);
		
		return editor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
