/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Created on Jul 23, 2006
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * An operator graph node that splits edges into several segments.
 *
 * @author gregor
 */
public class SplitFace extends Node {
	private Property meshName;
	private Property vertices;
	private Property face;

	public SplitFace() {
		super();
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);

		meshName=new Property(this);
		meshName.setName("meshName");
		meshName.setType(Value.Type.String);
		try {
			meshName.setValue("none");
		} catch (Exception e) {
			// cannot happen  here
			e.printStackTrace();
		}
		vertices=new Property(this);
		vertices.setName("vertices");
		vertices.setType(Value.Type.IntegerArray);
		try {
			vertices.setMaxSize(2);
		} catch (Exception e) {
			// cannot happen  here
			e.printStackTrace();
		}
		face=new Property(this);
		face.setName("face");
		face.setType(Value.Type.Integer);
		try {
			face.setValue(1);
		} catch (Exception e) {
			// cannot happen  here
			e.printStackTrace();
		}
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();
		Mesh mesh=(Mesh) graph.getNodeByName(meshName.getStringValue());

		if(vertices.getSize()!=2)  {
			throw new IllegalStateException(L10N.translate("SplitFace takes only exactly two vertices"));
		}

		int newEdge;

		newEdge = mesh.findOrCreateEdge(vertices.getArrayIntegerValue(0), vertices.getArrayIntegerValue(1));

		int[] faceEdges=mesh.getFace(face.getIntegerValue()).edgeIDs;

		// build new face loops
		ArrayList<Integer> newFace1=new ArrayList<Integer>();
		ArrayList<Integer> newFace2=new ArrayList<Integer>();

		/*
		 * the plan:
		 * start with first vertex of newEdge and walk around the old face
		 * until you hit the second vertex of newEdge and build a new face
		 * from that.
		 */

		// find the two edges in the face edge loop that lie before the newly connected edge
		int index=0;
		int lastEdges[]=new int[2];
		int i=0;
		while(i<faceEdges.length) {
			int vertexIDs[]=mesh.getEdgeVertices(faceEdges[i]);
			int nextVertexIDs[]=mesh.getEdgeVertices(faceEdges[(i+1)%faceEdges.length]);

			// important; also check if the following edge in the loop also has the same vertex in common with the new edge
			if(((vertexIDs[0]==vertices.getArrayIntegerValue(0) || vertexIDs[1]==vertices.getArrayIntegerValue(0)) &&
			    (nextVertexIDs[0]==vertices.getArrayIntegerValue(0) || nextVertexIDs[1]==vertices.getArrayIntegerValue(0))) ||
			   ((vertexIDs[0]==vertices.getArrayIntegerValue(1) || vertexIDs[1]==vertices.getArrayIntegerValue(1)) &&
				(nextVertexIDs[0]==vertices.getArrayIntegerValue(1) || nextVertexIDs[1]==vertices.getArrayIntegerValue(1)))) {
				lastEdges[index]=i;
				index++;
				i++; // skip the next edge
			}
			i++;
		}

		// the edges for the first new face are between lastEdges[0] and lastEdges[1]
		i=(lastEdges[0]+1)%faceEdges.length;
		while(i<=lastEdges[1]) {
			newFace1.add(faceEdges[i]);
			i++;
		}
		newFace1.add(newEdge);
		mesh.addFace(newFace1);

		// the edges for the second new face are between lastEdges[1] and lastEdges[0]
		i=(lastEdges[1]+1)%faceEdges.length;
		while(i>lastEdges[1] || i<=lastEdges[0]) {
			newFace2.add(faceEdges[i]);
			i=(i+1)%faceEdges.length;
		}
		newFace2.add(newEdge);
		mesh.addFace(newFace2);

		mesh.removeFace(face.getIntegerValue());

		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}

}
