/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on May 7, 2007
 */

package eu.moonlight3d.ml3d.modelling.mesh;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.modelling.basic.TransformGizmoListener;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.Gizmo;

class TransformEdgesGizmoListener extends TransformGizmoListener {
	private eu.moonlight3d.ml3d.backend.threedim.og.Node currentOperation;
	private boolean didChange=false;
	private Manager ogManager;
	
	private Vector3D oldDisplacement;
	private Quaternion oldRotation;
	private Vector3D oldScale;
	
	@Override
	public boolean canTransformCurrentSelection() {
		// this fortunately always possible
		return true;
	}
	
	@Override
	public Vector3D getInitialPosition() throws Exception {
		State state=eu.moonlight3d.framework.State.getInstance();
		Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
		Mesh.Edge edge=(Mesh.Edge)selection.getObjectFromSelectionString(selection.getActiveObject());
        String selectionName=selection.getActiveObject();
        String ownerOGNodeName=SelectionHelper.getOGNodeName(selectionName);
        String ownerOGSlotName=SelectionHelper.getOGSlotName(selectionName);
        String ownerSGNodePath=SelectionHelper.getSGPath(selectionName);
        String ownerSGNodeName=SelectionHelper.getSGNodeName(selectionName);
        
        eu.moonlight3d.ml3d.backend.threedim.og.Node ownerOGNode=((Document) state.getDocumentManager().getMainDocument()).getOGManager().getNodeByName(ownerOGNodeName);
        eu.moonlight3d.ml3d.backend.threedim.og.Slot ownerOGSlot=ownerOGNode.getSlot(ownerOGSlotName);
        Mesh mesh=(Mesh) ownerOGSlot.getGraph().getNodeByName(ownerSGNodeName);
        
        Vector3D position=mesh.getVertexPosition(edge.vertexID[0]).add(mesh.getVertexPosition(edge.vertexID[1])).multiply(0.5);

//        position=mesh.transformPointLocalToWorld(ownerSGNodePath, position);
        
        return position;
	}
	
	private void createNewOperation(String name) throws Exception {
		State state=State.getInstance();
        ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
        currentOperation=ogManager.createNodeByName(name);
        
        // get name of currently active object, og node and slot
        Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
        String selectionName=selection.getActiveObject();
        String[] selectionParts=selectionName.split("/");
        String ownerOGNodeName=selectionParts[1];
        String ownerOGSlotName=selectionParts[2];
        String ownerSGNodeName=selectionParts[selectionParts.length-1].split(":")[0]; // remove appendix from SG node name
        eu.moonlight3d.framework.State.getInstance().logger().info("SG node name: " + ownerSGNodeName);

        // connect MoveObject OG node to parent OG node of currently active SG node
        eu.moonlight3d.ml3d.backend.threedim.og.Node ownerOGNode=ogManager.getNodeByName(ownerOGNodeName);
        eu.moonlight3d.ml3d.backend.threedim.og.Slot ownerOGSlot=ownerOGNode.getSlot(ownerOGSlotName);
        ogManager.insertNodeAfter(ownerOGSlot,currentOperation.getSlot("input"),currentOperation.getSlot("output"));
        
        // pass name of SG node to OG node
        currentOperation.getProperty("name").setValue(ownerSGNodeName);
        // build list of vertices
        int index=0;
        for(Object i : selection.getSelectionObjects()) {
            // TODO: do sanity check on selection (should be one object only for now)
            currentOperation.getProperty("edges").setArrayValue(index,((Mesh.Edge)i).edgeID);
            index=index+1;
        }
        eu.moonlight3d.framework.State.getInstance().logger().info(currentOperation.getProperty("name").getStringValue());
        
        // update selection to point to output of new node
        ogManager.update();
        selection.updateBase("OperatorGraph/" + ownerOGNodeName,"OperatorGraph/" + currentOperation.getName());

        didChange=false;			
	}
	
	public void begin(Gizmo.DragMode mode) {
		if(mode==Gizmo.DragMode.TranslateX || mode==Gizmo.DragMode.TranslateY || mode==Gizmo.DragMode.TranslateZ ||
		   mode==Gizmo.DragMode.TranslateXY || mode==Gizmo.DragMode.TranslateXZ || mode==Gizmo.DragMode.TranslateYZ ||
		   mode==Gizmo.DragMode.TranslateXYZ) {
			if(currentOperation instanceof MoveEdges) {
				try {
					oldDisplacement=currentOperation.getProperty("displacement").getVectorValue();
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("MoveEdges");
					oldDisplacement = new Vector3D();
					oldDisplacement.clear();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		} else if(mode==Gizmo.DragMode.RotateX || mode==Gizmo.DragMode.RotateY || mode==Gizmo.DragMode.RotateZ ||
				  mode==Gizmo.DragMode.RotateXY || mode==Gizmo.DragMode.RotateXZ || mode==Gizmo.DragMode.RotateYZ) {
			if(currentOperation instanceof RotateEdges) {
				try {
					Vector3D oldAxis=currentOperation.getProperty("axis").getVectorValue();
					double oldAngle=currentOperation.getProperty("angle").getFloatValue();
					oldRotation=new Quaternion();
					oldRotation.fromAxisAngle(oldAxis, oldAngle);
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("RotateEdges");
					oldRotation=new Quaternion();
					oldRotation.clear();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		} else if(mode==Gizmo.DragMode.ScaleX || mode==Gizmo.DragMode.ScaleY || mode==Gizmo.DragMode.ScaleZ ||
				  mode==Gizmo.DragMode.ScaleXY || mode==Gizmo.DragMode.ScaleXZ || mode==Gizmo.DragMode.ScaleYZ ||
				  mode==Gizmo.DragMode.ScaleXYZ) {
			if(currentOperation instanceof ScaleEdges) {
				try {
					oldScale=currentOperation.getProperty("scale").getVectorValue();
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("ScaleEdges");
					oldScale = new Vector3D();
					oldScale.X1 = 1.0;
					oldScale.X2 = 1.0;
					oldScale.X3 = 1.0;
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		}
	}
	
	public void translate(Vector3D displacment) {
		try {
			// update displacement
			currentOperation.getProperty("displacement").setValue(oldDisplacement.add(displacment));
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
	}

	public void rotate(Vector3D center, Quaternion rotation) {
		try {
			// update axis and angle
			currentOperation.getProperty("pivot").setValue(center);
	        currentOperation.getProperty("axis").setValue(oldRotation.multiply(rotation).getRotationAxis());
	        currentOperation.getProperty("angle").setValue(oldRotation.multiply(rotation).getRotationAngle());
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
		
	}

	public void scale(Vector3D center, Vector3D amount) {
		try {
			// update displacement
			Vector3D scaleAmount=new Vector3D();
			scaleAmount.X1=oldScale.X1*amount.X1;
			scaleAmount.X2=oldScale.X2*amount.X2;
			scaleAmount.X3=oldScale.X3*amount.X3;
			currentOperation.getProperty("pivot").setValue(center);
			currentOperation.getProperty("scale").setValue(scaleAmount);
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
	}

	public void apply() {
		if(!didChange && currentOperation!=null) {
            try {
            	// remove OG node because it is a NOP and restore any OG links

            	String previousOGNodeName=currentOperation.getSlot("input").getCurrentLink().getInput().getNode().getName();
                Selection selection = ((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
                selection.updateBase("OperatorGraph/" + currentOperation.getName(), "OperatorGraph/" + previousOGNodeName);
            	
            	if(currentOperation.getSlot("output").getCurrentLink()!=null) {
            		eu.moonlight3d.ml3d.backend.threedim.og.Slot previous, next;
            		previous=currentOperation.getSlot("input").getCurrentLink().getInput();
            		next=currentOperation.getSlot("output").getCurrentLink().getOutput();
            		ogManager.removeLink(currentOperation.getSlot("output").getCurrentLink());
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            		ogManager.addLink(previous, next);
            	} else {
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            	}
				ogManager.removeNode(currentOperation);
				currentOperation=null;
			} catch (Exception e) {
				// this should not happen
				e.printStackTrace();
			}
		}
	}

	public void end() {
		apply();
		currentOperation=null;
	}
}