/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.mesh;


import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.OperatorGraphHelper;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A tool that adds a single vertex to a selected mesh.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class TriangulateTool extends Tool {
	/**
	 * Default constructor
	 */
	public TriangulateTool() {
		super();
		setName("TriangulateTool");
		setDisplayName(L10N.translate("Triangulate"));
		setCategory("Edit/Mesh");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning(L10N.translate("current selection must be a mesh object"));
                return;
            }
            if(selection.getActiveObject()==null) {
            	State.getInstance().logger().info(L10N.translate("no active object to work on"));
            	return;
            }
            if(!(selection.getObjectFromSelectionString(selection.getActiveObject()) instanceof Mesh)) {
                state.logger().warning(L10N.translate("current selection must be a mesh object"));
                return;            	
            }
            
            String selectionName = selection.getActiveObject();
            String ogNodeName = SelectionHelper.getOGNodeName(selectionName);
            String sgNodeName = SelectionHelper.getSGNodeName(selectionName);
            
            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            Node triangulate = OperatorGraphHelper.insert(selectionName, "Triangulate", "input", "output");
            triangulate.getProperty("name").setValue(sgNodeName);

            ogManager.update();
	        selection.updateBase(ogNodeName,triangulate.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
