/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.objects;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.selection.SelectionType;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.modelling.basic.DeleteSelectionHandler;

public class DeleteObjectsHandler implements DeleteSelectionHandler {
	/**
	 * Default constructor
	 */
	public DeleteObjectsHandler() {
	}

	public boolean canHandleSelectionType(SelectionType type) throws Exception {
        if(type==eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
            return true;
        }
		return false;
	}

	public void run() {
        try {
            State state=State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning(L10N.translate("current selection must be a scene graph node"));
                return;
            }
            
            if(selection.getActiveObject()==null) {
            	state.logger().warning(L10N.translate("no object selected"));
            	return;
            }
            
            String ogNodeName=SelectionHelper.getOGNodeName(selection.getActiveObject());
            String slotName=SelectionHelper.getOGSlotName(selection.getActiveObject());

            Manager ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            Node ogNode=ogManager.getNodeByName(ogNodeName);
            Slot slot=ogNode.getSlot(slotName);
            ogNode=ogManager.createNodeByName("DeleteObjects");
            
            ArrayList<Object> objects = selection.getSelectionObjects();
            for(int i=0;i<objects.size();i++) {
                ogNode.getProperty("name").setArrayValue(i,((eu.moonlight3d.ml3d.backend.threedim.sg.Node) objects.get(i)).getName());
            }
            // set links
            ogManager.insertNodeAfter(slot,ogNode.getSlot("input"),ogNode.getSlot("output"));
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}
}
