/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.modelling.objects;

import java.util.ArrayList;
import java.util.List;

import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.modelling.basic.OGOutputMerger;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

public class GroupObjectsTool extends Tool {
	/**
	 * Default constructor
	 */
	public GroupObjectsTool() {
		setName("GroupObjectsTool");
		setDisplayName("Group Objects");
		setCategory("Edit/Object");
	}

	@Override
	public void activated() {
        try {
            State state=State.getInstance();
            
            // don't do anything if selected object is not a SG node
            Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning(L10N.translate("current selection must be a scene graph node"));
                return;
            }
            
            String[] path=selection.getActiveObject().split("/");
            String ogNodeName=path[1];
            String slotName=path[2];
            
            OGOutputMerger ogOutputMerger=new OGOutputMerger();

            List<String> selectedObjects=selection.getSelection();
            
            List<String> ogNodes=ogOutputMerger.getOGNodeNames(selection);
            Manager ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
            if(ogNodes.size()>1) {
            	// insert required merge nodes
            	ogNodeName=ogOutputMerger.mergeOGOutputSlots(ogNodes);            	
            	slotName="output";
            	
            	// update selection list with renamed objects from merge node output
            	selectedObjects=ogOutputMerger.getMergedSGNodeNames(selectedObjects);

            	// run OG update to get the name mappings
            	ogManager.update();
            	
            }

            Node ogNode=ogManager.getNodeByName(ogNodeName);
            Slot slot=ogNode.getSlot(slotName);
            ogNode=ogManager.createNodeByName("GroupObjects");
            
            int i=0;
            for(String name : selectedObjects) {
            	String nodePath=SelectionHelper.getSGPath(name);
                ogNode.getProperty("objects").setArrayValue(i,nodePath);
                i++;
            }
            
            // set links
            ogManager.insertNodeAfter(slot,ogNode.getSlot("input"),ogNode.getSlot("output"));
                
            // clear selection to prevent rogue selection items
            selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode"));
            
			ogManager.update();
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		return false;
	}

}
