/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 3, 2005
 */
package eu.moonlight3d.ml3d.modelling.objects;
/**
 * \package eu.moonlight3d.ml3d.modelling.objects
 *
 * This plugin contains tools to manipulate 3D objects of all types
 * 
 * This package is a plugin depending on eu.moonlight3d.ml3d.modelling.basic, 
 * eu.moonlight3d.ml3d.modelling.basicselectiontypes, eu.moonlight3d.ml3d.ui.plugins.view
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.modelling.basic.DeleteSelectionTool;
import eu.moonlight3d.ml3d.modelling.basic.TransformMode;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.ModeManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private DuplicateObjectsTool duplicateObjectsTool;
	private GroupObjectsTool groupObjectsTool;
	private SelectObjectMode selectObjectMode;
	private SelectObjectTool selectObjectTool;
	private UngroupObjectsTool ungroupObjectsTool;
	
	private DeleteObjectsHandler deleteObjectsHandler;

	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.objects.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.modelling.basic.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.modelling.basicselectiontypes.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
        
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DeleteObjects",DeleteObjects.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("DuplicateObjects",DuplicateObjects.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("GroupObjects",GroupObjects.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("MoveObject",MoveObject.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("RotateObject",RotateObject.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("ScaleObject",ScaleObject.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("UngroupObjects",UngroupObjects.class));

        if(!state.getBatchMode()) {
	        UIManager manager=State.getInstance().getUIManager();
			
			manager.registerAction(new ToolAction("SelectObjectAction",L10N.translate("Select Object"),null,"Object","All Objects","SelectObjectTool","a"));
			manager.registerAction(new ToolAction("DuplicateObjectsAction",L10N.translate("Duplicate Objects"),null,"Object","All Objects","DuplicateObjectsTool"));
			manager.registerAction(new ToolAction("GroupObjectsAction",L10N.translate("Group Objects"),null,"Object","All Objects","GroupObjectsTool"));
			manager.registerAction(new ToolAction("UngroupObjectsAction",L10N.translate("Ungroup Objects"),null,"Object","All Objects","UngroupObjectsTool"));
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"C_Objects","menubar");
			MenuNode objectsMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Objects"),"");
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Select Object"),"SelectObjectAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Duplicate Objects"),"DuplicateObjectsAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Group Objects"),"GroupObjectsAction"));
			objectsMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Ungroup Objects"),"UngroupObjectsAction"));
			contribution.add(objectsMenu);
			state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);
			
			selectObjectMode=new SelectObjectMode();
			ModeManager.getInstance().addMode(selectObjectMode);
			selectObjectTool=new SelectObjectTool(selectObjectMode);
			ToolChestView.addTool(selectObjectTool);
			duplicateObjectsTool=new DuplicateObjectsTool();
			ToolChestView.addTool(duplicateObjectsTool);
			groupObjectsTool=new GroupObjectsTool();
			ToolChestView.addTool(groupObjectsTool);
			ungroupObjectsTool=new UngroupObjectsTool();
			ToolChestView.addTool(ungroupObjectsTool);
			
			TransformMode.registerGizmoListener("SGNode",new TransformObjectGizmoListener());
			
			deleteObjectsHandler=new DeleteObjectsHandler();
			DeleteSelectionTool.registerHandler(deleteObjectsHandler);
        }
	}

	public void unload() throws Exception {
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DeleteObjects");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("DuplicateObjects");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("GroupObjects");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("MoveObject");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("RotateObject");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("ScaleObject");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("UngroupObjects");

        State state=State.getInstance();
        if(!state.getBatchMode()) {
	        UIManager manager=State.getInstance().getUIManager();
	
			manager.unregisterAction("SelectObjectAction");
			manager.unregisterAction("DeleteObjectsAction");
			manager.unregisterAction("DuplicateObjectsAction");
			manager.unregisterAction("GroupObjectsAction");
			manager.unregisterAction("UngroupObjectsAction");
			manager.unregisterAction("TransformObjectAction");
			
			ToolChestView.removeTool(selectObjectTool);
			ToolChestView.removeTool(duplicateObjectsTool);
			ToolChestView.removeTool(groupObjectsTool);
			ToolChestView.removeTool(ungroupObjectsTool);
			
			ModeManager.getInstance().removeMode(selectObjectMode);
	
			TransformMode.unregisterGizmoListener("SGNode");

			DeleteSelectionTool.unregisterHandler(deleteObjectsHandler);
        }
	}
}
