/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 21, 2007
 */
package eu.moonlight3d.ml3d.modelling.objects;

import java.util.Arrays;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.math.AxisAlignedBoundingBox;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.selection.Manager;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.DrainSlotIterator;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.Node;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;
import eu.moonlight3d.ml3d.backend.threedim.sg.TransformableNode;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TreeIterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TypeFilter;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.Mode;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewEventHandler;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.Handle;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.HandleContainer;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.HandleWidget;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.WidgetViewEventHandler;

public class SelectObjectMode extends Mode {
	public SelectObjectMode() {
		super("SelectObjectMode");
	}
	
	private class Populator extends eu.moonlight3d.ml3d.ui.mainwindow.widgets.HandleContainerPopulator {

		public Populator(HandleContainer container) {
			super(container);
		}

		@Override
		public void repopulate(HandleContainer handleContainer) {
			handleContainer.clearAllHandles();
			
			// generate handles
			DrainSlotIterator slotIterator=new DrainSlotIterator(((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager());
			while(slotIterator.hasNext()) {
				Slot slot = slotIterator.next();
				String ogNodeName = "OperatorGraph/" + slot.getNode().getName();
				String slotName = ogNodeName + "/" + slot.getName();
				SceneGraph sg;
				try {
					sg = slot.getGraph();
					TreeIterator iterator = new TreeIterator(sg);
					iterator.addFilter(new TypeFilter("eu.moonlight3d.ml3d.backend.threedim.sg.TransformableNode", true));
					while (iterator.hasNext()) {
						Node[] path = iterator.next();
						TransformableNode node = (TransformableNode) path[path.length-1];
						AxisAlignedBoundingBox boundingBox=node.getBoundingBox();
						
						if(boundingBox==null) {
							continue;
						}
						
						Vector3D midpoint=boundingBox.min.add(boundingBox.max).multiply(0.5);
						if(path.length>1) {
							Node[] parentPath=Arrays.copyOf(path, path.length-1);
							midpoint=node.transformPointLocalToWorld(parentPath, midpoint);
						}

						Handle handle=new Handle();
							
						String sgNodeName = slotName + "/" + path[0].getName();
						for(int i=1;i<path.length;i++) {
							sgNodeName+="/" + path[i].getName();
						}

						handle.setElementName(sgNodeName);
						handle.setPosition(midpoint);
						handleContainer.addHandle(handle);
					}
				} catch (Exception e) {
					// this shouldn't happen here
					e.printStackTrace();
				}

			}
		}
		
	}

	@Override
	public void activated() {
		try {
			WidgetViewEventHandler widgetEventHandler=new WidgetViewEventHandler();
			HandleWidget handleWidget=new HandleWidget(widgetEventHandler,"SGNode");
			handleWidget.setPopulator(new Populator(handleWidget));
			
			Manager selectionManager=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager();
			selectionManager.getSelection("viewSelection").clearSelection(Manager.getSelectionType("SGNode"));
			ViewEventHandler.setEventHandler(widgetEventHandler);
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}

	}

	@Override
	public void deactivated() {
		ViewEventHandler.setEventHandler(ViewEventHandler.getDefaultEventHandler());
	}

}
