/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on May 7, 2007
 */

package eu.moonlight3d.ml3d.modelling.objects;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.math.AxisAlignedBoundingBox;
import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.OperatorGraphHelper;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.sg.TransformableNode;
import eu.moonlight3d.ml3d.modelling.basic.TransformGizmoListener;
import eu.moonlight3d.ml3d.ui.mainwindow.widgets.Gizmo;

class TransformObjectGizmoListener extends TransformGizmoListener {
	private eu.moonlight3d.ml3d.backend.threedim.og.Node currentOperation;
	private boolean didChange=false;
	private Manager ogManager;
	
	private Vector3D oldDisplacement;
	private Quaternion oldRotation;
	private Vector3D oldScale;
	
	@Override
	public boolean canTransformCurrentSelection() {
		try {
			Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			for(Object selected : selection.getSelectionObjects()) {
				if(!(selected instanceof TransformableNode)) {
					return false;
				}
			}
			
			return true;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}
	
	@Override
	public Vector3D getInitialPosition() throws Exception {
		Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
		AxisAlignedBoundingBox box=((eu.moonlight3d.ml3d.backend.threedim.sg.TransformableNode)selection.getObjectFromSelectionString(selection.getActiveObject())).getBoundingBox();
		
        Vector3D position=(box.min.add(box.max.sub(box.min).multiply(0.5)));
        
		return position;
	}
	
	private void createNewOperation(String name) throws Exception {
        ogManager=((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager();
        
        // get name of currently active object, og node and slot
        Selection selection = ((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
        String selectionName=selection.getActiveObject();
        String ownerOGNodeName=SelectionHelper.getOGNodeName(selectionName);
        String ownerSGNodeName=SelectionHelper.getSGNodeName(selectionName);
        eu.moonlight3d.framework.State.getInstance().logger().info("SG node name: " + ownerSGNodeName);
        
        currentOperation=OperatorGraphHelper.insert(selectionName,name,"input","output");

        // pass name of SG node to MoveObject OG node
        currentOperation.getProperty("name").setValue(ownerSGNodeName);
        eu.moonlight3d.framework.State.getInstance().logger().info(currentOperation.getProperty("name").getStringValue());
        
        // update selection to point to output of new node
        ogManager.update();
        selection.updateBase("OperatorGraph/" + ownerOGNodeName,"OperatorGraph/" + currentOperation.getName());
        
        didChange=false;
	}
	
	public void begin(Gizmo.DragMode mode) {
		if(mode==Gizmo.DragMode.TranslateX || mode==Gizmo.DragMode.TranslateY || mode==Gizmo.DragMode.TranslateZ ||
		   mode==Gizmo.DragMode.TranslateXY || mode==Gizmo.DragMode.TranslateXZ || mode==Gizmo.DragMode.TranslateYZ ||
		   mode==Gizmo.DragMode.TranslateXYZ) {
			if(currentOperation instanceof MoveObject) {
				try {
					oldDisplacement=currentOperation.getProperty("displacement").getVectorValue();
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("MoveObject");
					oldDisplacement = new Vector3D();
					oldDisplacement.clear();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		} else if(mode==Gizmo.DragMode.RotateX || mode==Gizmo.DragMode.RotateY || mode==Gizmo.DragMode.RotateZ ||
				  mode==Gizmo.DragMode.RotateXY || mode==Gizmo.DragMode.RotateXZ || mode==Gizmo.DragMode.RotateYZ) {
			if(currentOperation instanceof RotateObject) {
				try {
					Vector3D oldAxis=currentOperation.getProperty("axis").getVectorValue();
					double oldAngle=currentOperation.getProperty("angle").getFloatValue();
					oldRotation=new Quaternion();
					oldRotation.fromAxisAngle(oldAxis, oldAngle);
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("RotateObject");
					oldRotation=new Quaternion();
					oldRotation.clear();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		} else if(mode==Gizmo.DragMode.ScaleX || mode==Gizmo.DragMode.ScaleY || mode==Gizmo.DragMode.ScaleZ ||
				  mode==Gizmo.DragMode.ScaleXY || mode==Gizmo.DragMode.ScaleXZ || mode==Gizmo.DragMode.ScaleYZ ||
				  mode==Gizmo.DragMode.ScaleXYZ) {
			if(currentOperation instanceof ScaleObject) {
				try {
					oldScale=currentOperation.getProperty("scale").getVectorValue();
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			} else {
				// add a new operation to OG
				try {
					createNewOperation("ScaleObject");
					oldScale = new Vector3D();
					oldScale.X1 = 1.0;
					oldScale.X2 = 1.0;
					oldScale.X3 = 1.0;
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		}
	}
	
	public void translate(Vector3D displacment) {
		try {
			// update displacement
			currentOperation.getProperty("displacement").setValue(oldDisplacement.add(displacment));
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
	}

	public void rotate(Vector3D center, Quaternion rotation) {
		try {
			// update axis and angle
			currentOperation.getProperty("pivot").setValue(center);
	        currentOperation.getProperty("axis").setValue(oldRotation.multiply(rotation).getRotationAxis());
	        currentOperation.getProperty("angle").setValue(oldRotation.multiply(rotation).getRotationAngle());
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
		
	}

	public void scale(Vector3D center, Vector3D amount) {
		try {
			// update displacement
			Vector3D scaleAmount=new Vector3D();
			scaleAmount.X1=oldScale.X1*amount.X1;
			scaleAmount.X2=oldScale.X2*amount.X2;
			scaleAmount.X3=oldScale.X3*amount.X3;
			currentOperation.getProperty("pivot").setValue(center);
			currentOperation.getProperty("scale").setValue(scaleAmount);
			didChange=true;
			ogManager.update();
		} catch(Exception e) {
			// this cannot happen here
			e.printStackTrace();
		}
	}

	public void apply() {
		if(!didChange) {
            try {
            	// remove OG node because it is a NOP and restore any OG links
            	if(currentOperation==null) {
            		return;
            	}
            	
            	String previousOGNodeName=currentOperation.getSlot("input").getCurrentLink().getInput().getNode().getName();
                Selection selection = ((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
                selection.updateBase("OperatorGraph/" + currentOperation.getName(), "OperatorGraph/" + previousOGNodeName);
            	
            	if(currentOperation.getSlot("output").getCurrentLink()!=null) {
            		eu.moonlight3d.ml3d.backend.threedim.og.Slot previous, next;
            		previous=currentOperation.getSlot("input").getCurrentLink().getInput();
            		next=currentOperation.getSlot("output").getCurrentLink().getOutput();
            		ogManager.removeLink(currentOperation.getSlot("output").getCurrentLink());
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            		ogManager.addLink(previous, next);
            	} else {
            		ogManager.removeLink(currentOperation.getSlot("input").getCurrentLink());
            	}
				ogManager.removeNode(currentOperation);
				currentOperation=null;
			} catch (Exception e) {
				// this should not happen
				ExceptionDialog.run(e);
			}
		}
	}
	
	public void end() {
		apply();
		currentOperation=null;
	}
}
