/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 21, 2008
 */

package eu.moonlight3d.ml3d.modelling.objects;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.Group;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;
import eu.moonlight3d.ml3d.backend.threedim.sg.TransformableNode;

/**
 * An operator graph node that ungroups a group of objects
 */
public class UngroupObjects extends Node {
	/**
	 * Whether the group's transformation should be kept for the
	 * elements contained in it 
	 */
	private Property keepTransformation;
	/**
	 * The name of the group object to remove
	 */
	private Property name;

	/**
	 * Default constructor
	 */
	public UngroupObjects() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		keepTransformation=new Property(this);
		keepTransformation.setName("keepTransformation");
		keepTransformation.setType(Value.Type.Boolean);
		keepTransformation.setValue(false);
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		name.setValue("None");
	}
	
	@Override
	public void update() throws Exception {
		SceneGraph graph=getSlot("input").getGraph();
		
		Group group=(Group) graph.getNodeByName(name.getStringValue());
		
		for(eu.moonlight3d.ml3d.backend.threedim.sg.Node child : group.getChildren()) {
			graph.removeChild(group, child);
			
			if(keepTransformation.getBooleanValue()) {
				if(child instanceof TransformableNode) {
					TransformableNode transformable=(TransformableNode) child;
					
					transformable.applyMatrix(group.getTransformationMatrix());
				}
			}
			
			for(eu.moonlight3d.ml3d.backend.threedim.sg.Node parent : group.getParents()) {
				graph.addChild(parent, child);
			}
		}
		
		graph.removeNode(group);
		
		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}
}
