/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 27, 2007
 */
package eu.moonlight3d.ml3d.modelling.textures;


import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

public class LoadTextureTool extends Tool {
	public LoadTextureTool() {
		super();
		setName("LoadTextureTool");
		setDisplayName("Load Texture");
		setCategory("Create/Material");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            Document document=(Document) state.getDocumentManager().getMainDocument();
/*            Manager ogManager = document.getOGManager();

            // don't do anything if selected object is not a SG node
            Selection selection=document.getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning("current selection must be a scene graph object");
                return;
            } */
            
            // build open file dialog
            String filterName="";
            boolean usableFormatsPresent=false;
            for(ImageFormat format : FormatManager.getInstance().getFormats()) {
            	if (format.isLoadable()) {
            		usableFormatsPresent=true;
					FileType fileType = format.getFileType();
					if (!filterName.equals("")) {
						filterName += ";;";
					}
					filterName = filterName + fileType.getName() + " (";
					for (String pattern : fileType.getPatterns()) {
						filterName += pattern + " ";
					}
					filterName += ")";
				}
            }
            
            if(!usableFormatsPresent) {
    			QMessageBox.warning(QApplication.activeWindow(),"No image format handlers present","There are no suitable image format handlers present. No images can be loaded.");
    			return;        	
            }
            
            QFileDialog.Filter filter=new QFileDialog.Filter(filterName);
            
            String filename=QFileDialog.getOpenFileName(state.getMainWindow(),"Load texture","",filter);

            // abort if user pressed cancel:
            if(filename==null) {
            	return;
            }
            if(filename.equals("")) {
            	return;
            }
            
            document.getResourceManager().addResource("Texture", "Texture", filename);

/*            // create operator graph node:
            String[] path=selection.getActiveObject().split("/");
            String ogNodeName=path[1];
            String slotName=path[2];
            
            Node ogNode=ogManager.getNodeByName(ogNodeName);
            Slot slot=ogNode.getSlot(slotName);
            Node loadTexture=ogManager.createNodeByName("LoadTexture");
            
			loadTexture.getProperty("fileName").setValue(filename);
	        
	        // set links
	        ogManager.insertNodeAfter(slot,loadTexture.getSlot("input"),loadTexture.getSlot("output"));
	        ogManager.update();
	        selection.updateBase(ogNodeName,loadTexture.getName());
*/
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
