/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 16, 2006
 */
package eu.moonlight3d.ml3d.modelling.textures;
/**
 * \package eu.moonlight3d.ml3d.modelling.textures
 *
 * This plugin contains the texture material and related tools.
 * 
 * This package is a plugin depending on eu.moonlight3d.ml3d.plugins.modelliig.map,
 * eu.moonlight3d.ml3d.plugins.ui.view, eu.moonlight3d.ml3d.rendering.simple
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private TextureFactory textureFactory;
	private GenerateUVMapTool generateUVMapTool;
	private LoadTextureTool loadTextureTool;

	public String getName() {
		return "eu.moonlight3d.ml3d.modelling.textures.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.modelling.mesh.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.rendering.simple.Plugin"); // because of materialMenuExtensionPoint
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("Texture",Texture.class));

		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("GenerateUVMap",GenerateUVMap.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("TweakUVMap",TweakUVMap.class));
        
        textureFactory=new TextureFactory();
        ResourceManager.registerResourceFactory(textureFactory);

        if(!state.getBatchMode()) {			
			UIManager manager=state.getUIManager();
			manager.registerAction(new ToolAction("GenerateUVMapAction",L10N.translate("Generate UV Map"),null,"Material","Materials","GenerateUVMapTool"));
			manager.registerAction(new ToolAction("LoadTextureAction",L10N.translate("Load Texture"),null,"Material","Materials","LoadTextureTool"));
			manager.registerAction(new OpenUVEditorAction(null));
			manager.registerAction(new SaveUVMapImageAction(null));
			
			generateUVMapTool=new GenerateUVMapTool();
			ToolChestView.addTool(generateUVMapTool);
			loadTextureTool=new LoadTextureTool();
			ToolChestView.addTool(loadTextureTool);
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,L10N.translate("Texture"),"materialMenuExtensionPoint");
			contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
			contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Load Texture"),"LoadTextureAction"));
			contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Generate UV Map"),"GenerateUVMapAction"));
			contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Open UV Editor"),"OpenUVEditorAction"));
			manager.getLayout("mainwindow").addMenuContribution(contribution);
	
			manager.registerMainWidgetFactory(new UVEditorFactory());
			manager.addOrReplaceLayout(new FrameLayout("uveditor","UV Editor"));
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"FileMenu","menubar");
			MenuNode fileMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("File"),"");
			fileMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Save UV map as image..."),"SaveUVMapImageAction"));
			fileMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			fileMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Close"),"CloseCurrentFrameAction"));
			contribution.add(fileMenu);
			manager.getLayout("uveditor").addMenuContribution(contribution);
        }
	}

	public void unload() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("Texture");

		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("GenerateUVMap");

		State state=State.getInstance();
		if(!state.getBatchMode()) {
			ToolChestView.removeTool(generateUVMapTool);
			ToolChestView.removeTool(loadTextureTool);
			
			state.getUIManager().unregisterAction("GenerateUVMapAction");
			state.getUIManager().unregisterAction("LoadTextureAction");
			state.getUIManager().unregisterAction("OpenUVEditorAction");
			state.getUIManager().unregisterAction("SaveUVMapImageAction");
		}
	}

}
