/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jul 15, 2008
 */

package eu.moonlight3d.ml3d.modelling.textures;

import java.util.HashMap;
import java.util.List;

import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;

public class SaveUVMapImageAction extends Action {
	public SaveUVMapImageAction(QWidget parent) {
		super("SaveUVMapImageAction",null,L10N.translate("Save UV map as an image"),parent);
        triggered.connect(this,"run()");
	}
	
	@Override
	public void run() {
		UVEditor uvEditor=UVEditor.getInstance();
		
		if(uvEditor==null) {
			return;
		}
		if(uvEditor.getCurrentMesh()==null || uvEditor.getUVMapName()==null) {
			QMessageBox.critical(QApplication.activeWindow(), L10N.translate("No UV map"), L10N.translate("There is no UV map active in the editor."));
			return;
		}
		
		FormatManager formatManager=FormatManager.getInstance();
		List<ImageFormat> imageFormats=formatManager.getFormats();
		
		if(imageFormats.size()==0) {
			QMessageBox.warning(QApplication.activeWindow(),L10N.translate("No image format handlers present"),L10N.translate("There are no image format handlers loaded. The image cannot be saved."));
			return;
		}
		
		HashMap<String, ImageFormat> imageFormatMap=new HashMap<String, ImageFormat>();
        String filterNames="";
        boolean usableFormatsPresent=false;
        for(ImageFormat imageFormat : imageFormats) {
        	if (imageFormat.isSaveable()) {
        		usableFormatsPresent=true;
				FileType fileType = imageFormat.getFileType();
				String filterName = fileType.getName() + " (";
				for (String pattern : fileType.getPatterns()) {
					filterName += pattern + " ";
				}
				filterName += ")";
				imageFormatMap.put(filterName, imageFormat);
				filterNames = filterNames + filterName + ";;";
			}
        }
        filterNames=filterNames+L10N.translate("By filename");
        
        if(!usableFormatsPresent) {
			QMessageBox.warning(QApplication.activeWindow(),L10N.translate("No image format handlers present"),L10N.translate("There are no suitable image format handlers loaded. The Image cannot be saved."));
			return;        	
        }

        QFileDialog.Filter fileDialogFilter=new QFileDialog.Filter(filterNames);

        State state=State.getInstance();
		SaveUVMapImageDialog saveUVMapImageDialog=new SaveUVMapImageDialog(state.getUIManager().getActiveApplicationWindow(),fileDialogFilter, uvEditor.getCurrentMesh(), uvEditor.getUVMapName());
		
		saveUVMapImageDialog.exec();
	}
}
