/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jul 20, 2008
 */

package eu.moonlight3d.ml3d.modelling.textures;

import java.util.List;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QCursor;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIntValidator;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.common.image.core.FramebufferPainter;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh;

public class SaveUVMapImageDialog extends QDialog {
	private QLineEdit filename;
	private QPushButton selectButton;
	private QLineEdit imageWidth;
	private QLineEdit imageHeight;
	private QPushButton okButton;
	private QPushButton cancelButton;
    private QFileDialog.Filter fileDialogFilter;
    private Mesh mesh;
    private String uvMapName;
    
	public SaveUVMapImageDialog(QWidget parent, QFileDialog.Filter fileDialogFilter, Mesh mesh, String uvMapName) {
		super(parent);
		
		this.fileDialogFilter=fileDialogFilter;
		this.mesh=mesh;
		this.uvMapName=uvMapName;
		
		setWindowTitle(L10N.translate("Save UV map as image"));
		
		QLabel filenameLabel=new QLabel(this);
		filenameLabel.setText(L10N.translate("File name:"));
		filename=new QLineEdit(this);
		selectButton=new QPushButton(this);
		selectButton.setText(L10N.translate("Select..."));
		selectButton.clicked.connect(this,"select()");
		QLabel imageWidthLabel=new QLabel(this);
		imageWidthLabel.setText(L10N.translate("Image width:"));
		imageWidth=new QLineEdit(this);
		imageWidth.setText("512");
		imageWidth.setValidator(new QIntValidator(imageWidth));
		QLabel imageHeightLabel=new QLabel(this);
		imageHeightLabel.setText(L10N.translate("Image height:"));
		imageHeight=new QLineEdit(this);
		imageHeight.setText("512");
		imageHeight.setValidator(new QIntValidator(imageHeight));
		okButton=new QPushButton(this);
		okButton.setText(L10N.translate("OK"));
		okButton.clicked.connect(this,"ok()");
		cancelButton=new QPushButton(this);
		cancelButton.setText(L10N.translate("Cancel"));
		cancelButton.clicked.connect(this,"cancel()");
		
		QGridLayout layout=new QGridLayout();
		QHBoxLayout buttonLayout=new QHBoxLayout();
		buttonLayout.addStretch();
		buttonLayout.addWidget(cancelButton);
		buttonLayout.addWidget(okButton);
		
		layout.addWidget(filenameLabel,0,0);
		layout.addWidget(filename,0,1);
		layout.addWidget(selectButton,0,2);
		layout.addWidget(imageWidthLabel,1,0);
		layout.addWidget(imageWidth,1,1);
		layout.addWidget(imageHeightLabel,2,0);
		layout.addWidget(imageHeight,2,1);
		
		layout.addLayout(buttonLayout, 3, 0, 1, 3);
		
		setLayout(layout);
	}
	
	protected void select() {
        String filename=QFileDialog.getSaveFileName(this,"Generated image file name","",fileDialogFilter);
		
		if(filename!=null && !filename.equals("")) {
			this.filename.setText(filename);
		}
	}
	
	private String imageFormatFromFilename() {
		String filename=this.filename.text();
		FormatManager formatManager=FormatManager.getInstance();
		List<ImageFormat> imageFormats=formatManager.getFormats();

		String formatName=null;
        if(fileDialogFilter.selectedFilter.equals(L10N.translate("By filename"))) {
        	for(ImageFormat format : imageFormats) {
        		if(format.getFileType().matches(filename)) {
        			formatName=format.getName();
        		}
        	}
        } else {
        	for(ImageFormat imageFormat : imageFormats) {
				FileType fileType = imageFormat.getFileType();
				String filterName = fileType.getName() + " (";
				for (String pattern : fileType.getPatterns()) {
					filterName += pattern + " ";
				}
				filterName += ")";
        		if(filterName.equals(fileDialogFilter.selectedFilter)) {
        			formatName=imageFormat.getName();
        			break;
        		}
        	}
        }
        if(formatName==null) {
        	QMessageBox.warning(QApplication.activeWindow(), L10N.translate("File format not recognised"), L10N.translate("The file extension you entered does not correspond to any known image format. The image cannot be saved."));
        	return "";
        }

        return formatName;
	}
	
	private void paintUVs(Framebuffer buffer, Mesh mesh, String mapName, Colour background, Colour foreground) {
		FramebufferPainter.fillRect(buffer, 0, 0, buffer.getWidth()-1, buffer.getHeight()-1, background);
		
		for(int i=0;i<mesh.getNumFaces();i++) {
			for(int j=0;j<mesh.getFaceEdges(i).length-1;j++) {
				double uv1[];
				double uv2[];
				
				uv1=mesh.getUVCoordinates(mapName, i, j);
				uv2=mesh.getUVCoordinates(mapName, i, j+1);
				
				uv1[0]=uv1[0]*buffer.getWidth();
				uv1[1]=uv1[1]*buffer.getHeight();
				uv2[0]=uv2[0]*buffer.getWidth();
				uv2[1]=uv2[1]*buffer.getHeight();
				
				FramebufferPainter.drawLine(buffer, (int) uv1[0], (int) uv1[1], (int) uv2[0], (int) uv2[1], foreground);
			}
		}
	}

	/**
	 * Slot for the OK button that tries to save the requested image file
	 * and closes the dialog if successful. Otherwise, it shows an appropriate
	 * error message and leaves the dialog open. 
	 */
	protected void ok() {
		String filename=this.filename.text();
		
		if(filename.equals("")) {
			QMessageBox.critical(this, L10N.translate("No filename"), L10N.translate("You did not provide a file name for the output file."));
			return;
		}
		
		String formatName=imageFormatFromFilename();
		
		if(formatName.equals("")) {
			return;
		}

		QApplication.setOverrideCursor(new QCursor(Qt.CursorShape.WaitCursor));
		
		int imageWidth=Integer.parseInt(this.imageWidth.text());
		int imageHeight=Integer.parseInt(this.imageHeight.text());
		
		Framebuffer framebuffer=new Framebuffer(imageWidth,imageHeight);
		framebuffer.addChannel("red");
		framebuffer.addChannel("green");
		framebuffer.addChannel("blue");
		framebuffer.addChannel("alpha");
		
		Colour background=new Colour(1,1,1,1);
		Colour foreground=new Colour(0,0,0,1);
		
		paintUVs(framebuffer, mesh, uvMapName, background, foreground);

		try {
			FormatManager.getInstance().saveImage(framebuffer, filename, formatName);
		} catch (Exception e) {
			QApplication.restoreOverrideCursor();
			ExceptionDialog.run(e);
			return;
		}
		
		QApplication.restoreOverrideCursor();

		close();
	}
	
	/**
	 * Slot for the cancel button that closes the dialog without further action.
	 */
	protected void cancel() {
		close();
	}
}
