/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Dec 9, 2005
 */
package eu.moonlight3d.ml3d.modelling.textures;


import java.io.File;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.framework.helper.RelativePath;
import eu.moonlight3d.ml3d.backend.threedim.resource.Resource;

public class Texture extends Resource {
	protected String filename;
	protected Framebuffer image;

	public Texture() {
		super();
	}
	
	public void load(String filename) throws Exception {
		image=FormatManager.getInstance().loadImage(filename);
		this.filename=filename;
	}
	
	public String getFilename() {
		return filename;
	}
	
	public Framebuffer getImage() {
		return image;
	}

	@Override
	public String getType() {
		return "Texture";
	}

	protected void read(Element dataElement) throws Exception {
		Element textureElement=(Element)dataElement.getElementsByTagName("texture").item(0);
		String relativeFilename=textureElement.getAttribute("filename");
		String absoluteFilename=new File(relativeFilename).getAbsolutePath();
		load(absoluteFilename);
	}
	
	@Override
	public void write(Document document, Element resourceElement) {
		Element textureElement=document.createElement("texture");
		
		// save relative filename to directory containing the file
		File documentDirectory=new File(getDocument().getFilename()).getParentFile();
		String relativeFilename=RelativePath.getRelativePath(documentDirectory, new File(filename));
		textureElement.setAttribute("filename",relativeFilename);
		resourceElement.appendChild(textureElement);
	}
}
