/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 27, 2007
 */
package eu.moonlight3d.ml3d.modelling.textures;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;
import eu.moonlight3d.ml3d.modelling.mesh.Mesh;

public class TweakUVMap extends Node {
	private Property meshName;
	private Property mapName;
	private Property faceIDs;
	private Property vertexIDs;
	private Property coordinates;
	
	public TweakUVMap() {
		super();
		
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		meshName=new Property(this);
		meshName.setName("meshName");
		meshName.setType(Value.Type.String);
		try {
			meshName.setValue("None");
		} catch (Exception e) {
			// this cannot happen here, type safety is guaranteed
			e.printStackTrace();
		}
		mapName=new Property(this);
		mapName.setName("mapName");
		mapName.setType(Value.Type.String);
		try {
			mapName.setValue("None");
		} catch (Exception e) {
			// this cannot happen here, type safety is guaranteed
			e.printStackTrace();
		}
		faceIDs=new Property(this);
		faceIDs.setName("faceIDs");
		faceIDs.setType(Value.Type.IntegerArray);
		try {
			faceIDs.setMaxSize(0);
		} catch (Exception e) {
			// this cannot happen here, type safety is guaranteed
			e.printStackTrace();
		}
		vertexIDs=new Property(this);
		vertexIDs.setName("vertexIDs");
		vertexIDs.setType(Value.Type.IntegerArray);
		try {
			vertexIDs.setMaxSize(0);
		} catch (Exception e) {
			// this cannot happen here, type safety is guaranteed
			e.printStackTrace();
		}
		coordinates=new Property(this);
		coordinates.setName("coordinates");
		coordinates.setType(Value.Type.FloatArray);
		try {
			coordinates.setMaxSize(0);
		} catch (Exception e) {
			// this cannot happen here, type safety is guaranteed
			e.printStackTrace();
		}
	}
	
	@Override
	public void update() throws Exception {
		SceneGraph graph= getSlot("input").getGraph();
		
		Mesh mesh=(Mesh)graph.getNodeByName(meshName.getStringValue());
		String mapName=this.mapName.getStringValue();
		
		for(int i=0;i<faceIDs.getSize();i++) {
			int faceID=faceIDs.getArrayIntegerValue(i);
			int vertexIndex=vertexIDs.getArrayIntegerValue(i);
			double u=coordinates.getArrayFloatValue(2*i);
			double v=coordinates.getArrayFloatValue(2*i+1);
			mesh.setUVCoordinates(mapName, faceID, vertexIndex, u, v);
		}

		getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}

}
