/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 6, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;


import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.Property;

public class AddShaderParameterDialog extends QDialog {
	private QLineEdit parameterName;
	private QComboBox parameterType;
	private CreateGLSLShader createGLSLShaderNode;
	
	protected AddShaderParameterDialog(QWidget parent, CreateGLSLShader createGLSLShaderNode) {
		super(parent);
		this.createGLSLShaderNode=createGLSLShaderNode;

		setWindowTitle("Add shader parameter");

		QLabel parameterNameLabel=new QLabel(this);
		parameterNameLabel.setText("Parameter name:");
		
		parameterName=new QLineEdit(this);
		
		QLabel parameterTypeLabel=new QLabel(this);
		parameterTypeLabel.setText("Parameter type:");

		parameterType=new QComboBox(this);
		parameterType.setEditable(false);
		parameterType.addItem("bool");
		parameterType.addItem("int");
		parameterType.addItem("float");
		parameterType.addItem("vec2");
		parameterType.addItem("vec3");
		parameterType.addItem("vec4");		
		parameterType.addItem("bvec2");
		parameterType.addItem("bvec3");
		parameterType.addItem("bvec4");		
		parameterType.addItem("ivec2");
		parameterType.addItem("ivec3");
		parameterType.addItem("ivec4");		
		parameterType.addItem("mat2");
		parameterType.addItem("mat3");
		parameterType.addItem("mat4");
//		parameterType.addItem("sampler1D");
//		parameterType.addItem("sampler2D");
//		parameterType.addItem("sampler3D");
//		parameterType.addItem("samplerCube");
//		parameterType.addItem("sampler1DShadow");
//		parameterType.addItem("sampler2DShadow");
		
		QPushButton cancel=new QPushButton(this);
		cancel.setText("Cancel");
		cancel.clicked.connect(this,"close()");

		QPushButton ok=new QPushButton(this);
		ok.setText("OK");
		ok.clicked.connect(this,"handleOK()");
	}
	
	private void processInput() {
		String parameterType=this.parameterType.itemText(this.parameterType.currentIndex());
		if(parameterType.equals("bool")) {
			Property nameProperty=createGLSLShaderNode.getProperty("boolUniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("boolUniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+1);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				valueProperty.setArrayValue(nameProperty.getSize(),false);
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("int")) {
			Property nameProperty=createGLSLShaderNode.getProperty("intUniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("intUniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+1);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				valueProperty.setArrayValue(nameProperty.getSize(),0);
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("float")) {
			Property nameProperty=createGLSLShaderNode.getProperty("floatUniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("floatUniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+1);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				valueProperty.setArrayValue(nameProperty.getSize(),0.0);
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("vec2")) {
			Property nameProperty=createGLSLShaderNode.getProperty("vec2UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("vec2UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+2);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<2;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("vec3")) {
			Property nameProperty=createGLSLShaderNode.getProperty("vec3UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("vec3UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+3);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<3;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("vec4")) {
			Property nameProperty=createGLSLShaderNode.getProperty("vec4UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("vec4UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+4);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<4;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("bvec2")) {
			Property nameProperty=createGLSLShaderNode.getProperty("bvec2UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("bvec2UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+2);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<2;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),false);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("bvec3")) {
			Property nameProperty=createGLSLShaderNode.getProperty("bvec3UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("bvec3UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+3);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<3;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),false);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("bvec4")) {
			Property nameProperty=createGLSLShaderNode.getProperty("bvec4UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("bvec4UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+4);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<4;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),false);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("ivec2")) {
			Property nameProperty=createGLSLShaderNode.getProperty("ivec2UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("ivec2UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+2);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<2;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("ivec3")) {
			Property nameProperty=createGLSLShaderNode.getProperty("ivec3UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("ivec3UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+3);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<3;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("ivec4")) {
			Property nameProperty=createGLSLShaderNode.getProperty("ivec4UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("ivec4UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+4);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<4;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("mat2")) {
			Property nameProperty=createGLSLShaderNode.getProperty("mat2UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("mat2UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+4);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<4;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("mat3")) {
			Property nameProperty=createGLSLShaderNode.getProperty("mat3UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("mat3UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+9);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<9;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("mat4")) {
			Property nameProperty=createGLSLShaderNode.getProperty("mat4UniformNames");
			Property valueProperty=createGLSLShaderNode.getProperty("mat4UniformValues");
			try {
				nameProperty.setMaxSize(nameProperty.getSize()+1);
				valueProperty.setMaxSize(valueProperty.getSize()+16);
				nameProperty.setArrayValue(nameProperty.getSize(),parameterName.text());
				for(int i=0;i<16;i++) {
					valueProperty.setArrayValue(nameProperty.getSize(),0.0);
				}
			} catch (Exception e) {
				// should not happen here
				e.printStackTrace();
			}
		} else if(parameterType.equals("sampler1D")) {
			// TODO implement this type of shader parameter
		} else if(parameterType.equals("sampler2D")) {
			// TODO implement this type of shader parameter
		} else if(parameterType.equals("sampler3D")) {
			// TODO implement this type of shader parameter
		} else if(parameterType.equals("samplerCube")) {
			// TODO implement this type of shader parameter
		} else if(parameterType.equals("sampler1DShadow")) {
			// TODO implement this type of shader parameter
		} else if(parameterType.equals("sampler2DShadow")) {
			// TODO implement this type of shader parameter
		}
	}
	
	public void handleOK() {
		processInput();
		close();
	}
}
