/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 25, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

public class CreateGLSLShader extends Node {
	private Property ownerName;
	private Property name;
	private Property vertexShader;
	private Property fragmentShader;
	
	private Property boolUniformNames;
	private Property boolUniformValues;
	private Property intUniformNames;
	private Property intUniformValues;
	private Property floatUniformNames;
	private Property floatUniformValues;
	private Property vec2UniformNames;
	private Property vec2UniformValues;
	private Property vec3UniformNames;
	private Property vec3UniformValues;
	private Property vec4UniformNames;
	private Property vec4UniformValues;
	private Property bvec2UniformNames;
	private Property bvec2UniformValues;
	private Property bvec3UniformNames;
	private Property bvec3UniformValues;
	private Property bvec4UniformNames;
	private Property bvec4UniformValues;
	private Property ivec2UniformNames;
	private Property ivec2UniformValues;
	private Property ivec3UniformNames;
	private Property ivec3UniformValues;
	private Property ivec4UniformNames;
	private Property ivec4UniformValues;
	private Property mat2UniformNames;
	private Property mat2UniformValues;
	private Property mat3UniformNames;
	private Property mat3UniformValues;
	private Property mat4UniformNames;
	private Property mat4UniformValues;

	public CreateGLSLShader() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		ownerName=new Property(this);
		ownerName.setName("ownerName");
		ownerName.setType(Value.Type.String);
		try {
			ownerName.setValue("none");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		try {
			name.setValue("GLSLShader");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		vertexShader=new Property(this);
		vertexShader.setName("vertexShader");
		vertexShader.setType(Value.Type.String);
		try {
			vertexShader.setValue("void main()\n{\n  gl_Position = ftransform();\n}\n");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		fragmentShader=new Property(this);
		fragmentShader.setName("fragmentShader");
		fragmentShader.setType(Value.Type.String);
		try {
			fragmentShader.setValue("void main()\n{\n  gl_FragColor = vec4(1.0,0.0,1.0,1.0);\n}\n");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		boolUniformNames=new Property(this);
		boolUniformNames.setName("boolUniformNames");
		boolUniformNames.setType(Value.Type.StringArray);
		try {
			boolUniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		boolUniformValues=new Property(this);
		boolUniformValues.setName("boolUniformValues");
		boolUniformValues.setType(Value.Type.BooleanArray);
		try {
			boolUniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		intUniformNames=new Property(this);
		intUniformNames.setName("intUniformNames");
		intUniformNames.setType(Value.Type.StringArray);
		try {
			intUniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		intUniformValues=new Property(this);
		intUniformValues.setName("intUniformValues");
		intUniformValues.setType(Value.Type.IntegerArray);
		try {
			intUniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		floatUniformNames=new Property(this);
		floatUniformNames.setName("floatUniformNames");
		floatUniformNames.setType(Value.Type.StringArray);
		try {
			floatUniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		floatUniformValues=new Property(this);
		floatUniformValues.setName("floatUniformValues");
		floatUniformValues.setType(Value.Type.FloatArray);
		try {
			floatUniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec2UniformNames=new Property(this);
		vec2UniformNames.setName("vec2UniformNames");
		vec2UniformNames.setType(Value.Type.StringArray);
		try {
			vec2UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec2UniformValues=new Property(this);
		vec2UniformValues.setName("vec2UniformValues");
		vec2UniformValues.setType(Value.Type.FloatArray);
		try {
			vec2UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec3UniformNames=new Property(this);
		vec3UniformNames.setName("vec3UniformNames");
		vec3UniformNames.setType(Value.Type.StringArray);
		try {
			vec3UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec3UniformValues=new Property(this);
		vec3UniformValues.setName("vec3UniformValues");
		vec3UniformValues.setType(Value.Type.FloatArray);
		try {
			vec3UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec4UniformNames=new Property(this);
		vec4UniformNames.setName("vec4UniformNames");
		vec4UniformNames.setType(Value.Type.StringArray);
		try {
			vec4UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		vec4UniformValues=new Property(this);
		vec4UniformValues.setName("vec4UniformValues");
		vec4UniformValues.setType(Value.Type.FloatArray);
		try {
			vec4UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec2UniformNames=new Property(this);
		bvec2UniformNames.setName("bvec2UniformNames");
		bvec2UniformNames.setType(Value.Type.StringArray);
		try {
			bvec2UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec2UniformValues=new Property(this);
		bvec2UniformValues.setName("bvec2UniformValues");
		bvec2UniformValues.setType(Value.Type.BooleanArray);
		try {
			bvec2UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec3UniformNames=new Property(this);
		bvec3UniformNames.setName("bvec3UniformNames");
		bvec3UniformNames.setType(Value.Type.StringArray);
		try {
			bvec3UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec3UniformValues=new Property(this);
		bvec3UniformValues.setName("bvec3UniformValues");
		bvec3UniformValues.setType(Value.Type.BooleanArray);
		try {
			bvec3UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec4UniformNames=new Property(this);
		bvec4UniformNames.setName("bvec4UniformNames");
		bvec4UniformNames.setType(Value.Type.StringArray);
		try {
			bvec4UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		bvec4UniformValues=new Property(this);
		bvec4UniformValues.setName("bvec4UniformValues");
		bvec4UniformValues.setType(Value.Type.BooleanArray);
		try {
			bvec4UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec2UniformNames=new Property(this);
		ivec2UniformNames.setName("vec2UniformNames");
		ivec2UniformNames.setType(Value.Type.StringArray);
		try {
			ivec2UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec2UniformValues=new Property(this);
		ivec2UniformValues.setName("ivec2UniformValues");
		ivec2UniformValues.setType(Value.Type.IntegerArray);
		try {
			ivec2UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec3UniformNames=new Property(this);
		ivec3UniformNames.setName("ivec3UniformNames");
		ivec3UniformNames.setType(Value.Type.StringArray);
		try {
			ivec3UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec3UniformValues=new Property(this);
		ivec3UniformValues.setName("ivec3UniformValues");
		ivec3UniformValues.setType(Value.Type.IntegerArray);
		try {
			ivec3UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec4UniformNames=new Property(this);
		ivec4UniformNames.setName("ivec4UniformNames");
		ivec4UniformNames.setType(Value.Type.StringArray);
		try {
			ivec4UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		ivec4UniformValues=new Property(this);
		ivec4UniformValues.setName("ivec4UniformValues");
		ivec4UniformValues.setType(Value.Type.IntegerArray);
		try {
			ivec4UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat2UniformNames=new Property(this);
		mat2UniformNames.setName("mat2UniformNames");
		mat2UniformNames.setType(Value.Type.StringArray);
		try {
			mat2UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat2UniformValues=new Property(this);
		mat2UniformValues.setName("mat2UniformValues");
		mat2UniformValues.setType(Value.Type.FloatArray);
		try {
			mat2UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat3UniformNames=new Property(this);
		mat3UniformNames.setName("mat3UniformNames");
		mat3UniformNames.setType(Value.Type.StringArray);
		try {
			mat3UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat3UniformValues=new Property(this);
		mat3UniformValues.setName("mat3UniformValues");
		mat3UniformValues.setType(Value.Type.FloatArray);
		try {
			mat3UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat4UniformNames=new Property(this);
		mat4UniformNames.setName("mat4UniformNames");
		mat4UniformNames.setType(Value.Type.StringArray);
		try {
			mat4UniformNames.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
		mat4UniformValues=new Property(this);
		mat4UniformValues.setName("mat4UniformValues");
		mat4UniformValues.setType(Value.Type.FloatArray);
		try {
			mat4UniformValues.setMaxSize(0);
		} catch (Exception e) {
			// cannot happen here
			e.printStackTrace();
		}
	}

	private void setBooleanTypeUniforms(GLSLShader shader, Property uniformNames, Property uniformValues, Value.Type valueType, Uniform.GLSLType glslType, int arraySize) {
		int index=0;
		for(int i=0;i<uniformNames.getSize();i++) {
			Uniform uniform=new Uniform();
			try {
				uniform.setName(uniformNames.getArrayStringValue(i));
				uniform.setType(valueType);
				uniform.setGLSLType(glslType);
				if(arraySize==1) {
					uniform.setValue(uniformValues.getArrayBooleanValue(index));
					index++;
				} else {
					for(int j=0;j<arraySize;j++) {
						uniform.setArrayValue(j,uniformValues.getArrayBooleanValue(index));
						index++;
					}
				}
				shader.setUniform(uniformNames.getArrayStringValue(i),uniform);
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
		}
	}
	
	private void setIntegerTypeUniforms(GLSLShader shader, Property uniformNames, Property uniformValues, Value.Type valueType, Uniform.GLSLType glslType, int arraySize) {
		int index=0;
		for(int i=0;i<uniformNames.getSize();i++) {
			Uniform uniform=new Uniform();
			try {
				uniform.setName(uniformNames.getArrayStringValue(i));
				uniform.setType(valueType);
				uniform.setGLSLType(glslType);
				if(arraySize==1) {
					uniform.setValue(uniformValues.getArrayIntegerValue(index));
					index++;
				} else {
					for(int j=0;j<arraySize;j++) {
						uniform.setArrayValue(j,uniformValues.getArrayIntegerValue(index));
						index++;
					}
				}
				shader.setUniform(uniformNames.getArrayStringValue(i),uniform);
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
		}
	}

	private void setFloatTypeUniforms(GLSLShader shader, Property uniformNames, Property uniformValues, Value.Type valueType, Uniform.GLSLType glslType, int arraySize) {
		int index=0;
		for(int i=0;i<uniformNames.getSize();i++) {
			Uniform uniform=new Uniform();
			try {
				uniform.setName(uniformNames.getArrayStringValue(i));
				uniform.setType(valueType);
				uniform.setGLSLType(glslType);
				if(arraySize==1) {
					uniform.setValue(uniformValues.getArrayFloatValue(index));
					index++;
				} else {
					for(int j=0;j<arraySize;j++) {
						uniform.setArrayValue(j,uniformValues.getArrayFloatValue(index));
						index++;
					}
				}
				shader.setUniform(uniformNames.getArrayStringValue(i),uniform);
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
		}
	}

	@Override
	public void update() throws Exception {
		SceneGraph graph; 
        eu.moonlight3d.ml3d.backend.threedim.sg.Node owner;
        GLSLShader glslShader;
        GLSLProgram vertexProgram, fragmentProgram; 
        
        graph=getSlot("input").getGraph();
        owner=graph.getNodeByName(ownerName.getStringValue());

        glslShader=(GLSLShader) graph.createNode("GLSLShader",name.getStringValue());
        graph.addNode(glslShader);
        graph.addChild(owner, glslShader);
        
        setBooleanTypeUniforms(glslShader,boolUniformNames,boolUniformValues,Value.Type.Boolean,Uniform.GLSLType.Bool,1);
        setBooleanTypeUniforms(glslShader,bvec2UniformNames,bvec2UniformValues,Value.Type.BooleanArray,Uniform.GLSLType.BVec2,2);
        setBooleanTypeUniforms(glslShader,bvec3UniformNames,bvec3UniformValues,Value.Type.BooleanArray,Uniform.GLSLType.BVec2,3);
        setBooleanTypeUniforms(glslShader,bvec4UniformNames,bvec4UniformValues,Value.Type.BooleanArray,Uniform.GLSLType.BVec2,4);
        setIntegerTypeUniforms(glslShader,intUniformNames,intUniformValues,Value.Type.Integer,Uniform.GLSLType.Int,1);
        setIntegerTypeUniforms(glslShader,ivec2UniformNames,ivec2UniformValues,Value.Type.IntegerArray,Uniform.GLSLType.IVec2,2);
        setIntegerTypeUniforms(glslShader,ivec3UniformNames,ivec3UniformValues,Value.Type.IntegerArray,Uniform.GLSLType.IVec2,3);
        setIntegerTypeUniforms(glslShader,ivec4UniformNames,ivec4UniformValues,Value.Type.IntegerArray,Uniform.GLSLType.IVec2,4);
        setFloatTypeUniforms(glslShader,floatUniformNames,floatUniformValues,Value.Type.Float,Uniform.GLSLType.Float,1);
        setFloatTypeUniforms(glslShader,vec2UniformNames,vec2UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Vec2,2);
        setFloatTypeUniforms(glslShader,vec3UniformNames,vec3UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Vec2,3);
        setFloatTypeUniforms(glslShader,vec4UniformNames,vec4UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Vec2,4);
        setFloatTypeUniforms(glslShader,mat2UniformNames,mat2UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Mat2,4);
        setFloatTypeUniforms(glslShader,mat3UniformNames,mat3UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Mat2,9);
        setFloatTypeUniforms(glslShader,mat4UniformNames,mat4UniformValues,Value.Type.FloatArray,Uniform.GLSLType.Vec2,16);
        
        vertexProgram=(GLSLProgram) graph.createNode("GLSLProgram",name.getStringValue() + "VertexProgram");
        vertexProgram.setProgramType(GLSLProgram.Type.Vertex);
        vertexProgram.setProgram(vertexShader.getStringValue());
        graph.addNode(glslShader);
        graph.addChild(glslShader,vertexProgram);
        
        fragmentProgram=(GLSLProgram) graph.createNode("GLSLProgram",name.getStringValue() + "FragmentProgram");
        fragmentProgram.setProgramType(GLSLProgram.Type.Fragment);
        fragmentProgram.setProgram(fragmentShader.getStringValue());
        graph.addChild(glslShader,fragmentProgram);
      
        getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}
}
