/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 21, 2007
 */
package eu.moonlight3d.ml3d.rendering.glsl;


import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

public class CreateGLSLShaderTool extends Tool {
	public CreateGLSLShaderTool() {
		setName("CreateGLSLShaderTool");
		setDisplayName("Create GLSL Shader");
		setCategory("Create/Material");
	}

	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();

            // don't do anything if selected object is not a SG node
            Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning("current selection must be a scene graph object");
                return;
            }
            
            String[] path=selection.getActiveObject().split("/");
            String ogNodeName=path[1];
            String slotName=path[2];
            String sgNodeName=path[3];
            
            Node ogNode=ogManager.getNodeByName(ogNodeName);
            Slot slot=ogNode.getSlot(slotName);
            Node createGLSLShader=ogManager.createNodeByName("CreateGLSLShader");
            createGLSLShader.getProperty("ownerName").setValue(sgNodeName);
            // set links
            ogManager.insertNodeAfter(slot,createGLSLShader.getSlot("input"),createGLSLShader.getSlot("output"));
			ogManager.update();
	        selection.updateBase(ogNodeName,createGLSLShader.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
