/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 3, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;

import eu.moonlight3d.ml3d.backend.threedim.sg.Node;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TreeIterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TypeFilter;
import eu.moonlight3d.ml3d.ui.mainwindow.view.LightSource;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SGNodeConverter;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SceneNode;

/**
 * A scene graph node converter that convertes GLSL shader nodes
 * to a preview.
 *
 * @author gregor
 */
public class GLSLConverter implements SGNodeConverter {

	public boolean canConvertNode(Node node) {
		if(node instanceof GLSLShader) {
			return true;
		}
		return false;
	}

	public SceneNode[] convert(Node node) {
		GLSLShader glslShaderNode=(GLSLShader) node;
		
		GLSLMaterial glslMaterial=new GLSLMaterial();
		
		// extract programs from GLSLProgram/GLSLShader nodes
		TreeIterator iterator=new TreeIterator(glslShaderNode.getSceneGraph(), glslShaderNode);
		iterator.addFilter(new TypeFilter("eu.moonlight3d.ml3d.rendering.glsl.GLSLProgram",true));
		
		while(iterator.hasNext()) {
			Node[] nodes=iterator.next();
			GLSLProgram glslProgram=(GLSLProgram) nodes[nodes.length-1];
			
		    if(glslProgram.getProgramType()==GLSLProgram.Type.Vertex) {
		    	glslMaterial.setVertexProgram(glslProgram.getProgram());
		    } else {
		    	glslMaterial.setFragmentProgram(glslProgram.getProgram());		    	
		    }
		}
	    glslMaterial.setUniformParameters(glslShaderNode.getUniforms());

		SceneNode[] results=new SceneNode[1];
		results[0]=glslMaterial;
		return results;
	}

	public LightSource[] convertLights(Node node) {
		return null;
	}

}
