/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 31, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;


import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QSyntaxHighlighter;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Link;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;

public class GLSLEditor extends QWidget {
	private static GLSLEditor instance = null;
	
	public static GLSLEditor getInstance() {
		return instance;
	}
	
	private QTextEdit editor;
	private PropertySheet parameterSheet;
	private QComboBox programList;
	private QComboBox shader;
	
	private CreateGLSLShader createGLSLShader;
	
	private boolean internalEditorUpdate=false;
	
	private class OperatorGraphListener implements eu.moonlight3d.ml3d.backend.threedim.og.OperatorGraphListener {

		private void rebuildProgramList() {
			String currentProgram=programList.itemText(programList.currentIndex());
			boolean maintainedSelection=false;
			
			programList.clear();
			for(eu.moonlight3d.ml3d.backend.threedim.og.Node node : ((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().getNodeList()) {
				if(node instanceof CreateGLSLShader) {
					String programName="OperatorGraph/" + node.getName();
					programList.addItem(programName);
					if(programName.equals(currentProgram)) {
						programList.setCurrentIndex(programList.count()-1);
						maintainedSelection=true;
					}
				}
			}
			if(!maintainedSelection && programList.count()>=1) {
				programList.setCurrentIndex(0);
			}
		}
		
		public void nodeInserted(Node node) {
			rebuildProgramList();
		}

		public void nodeRemoved(Node node) {
			rebuildProgramList();
		}

		public void nodeUpdated(Node node) {
//			rebuildProgramList();
		}

		public void nodeUpdateFailed(Node node, Exception exception) {
		}
		
		public void graphUpdated() {
			rebuildProgramList();
		}

		public void graphCleared() {
			rebuildProgramList();
		}

		public void linkInserted(Link link) {
		}

		public void linkRemoved(Link link) {
		}

	}
	
	private class LifecycleListener implements eu.moonlight3d.framework.document.LifecycleListener {

		public void afterNew() {
			operatorGraphListener=new OperatorGraphListener();
			((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().registerOperatorGraphListener(operatorGraphListener);
		}

		public void afterOpen(String filename) {
			operatorGraphListener=new OperatorGraphListener();
			((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().registerOperatorGraphListener(operatorGraphListener);
		}

		public void afterSave(String filename) {
			// TODO Auto-generated method stub
			
		}

		public void beforeNew() {
			// TODO Auto-generated method stub
			
		}

		public void beforeOpen(String filename) {
			// TODO Auto-generated method stub
			
		}

		public void beforeSave(String filename) {
			// TODO Auto-generated method stub
			
		}

		public void onClose() {
			((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().unregisterOperatorGraphListener(operatorGraphListener);
			operatorGraphListener=null;
		}
		
	}
	
	private OperatorGraphListener operatorGraphListener;
	private LifecycleListener lifecycleListener;

	public GLSLEditor(QWidget parent) {
		super(parent);
		
		instance=this;
		
		programList=new QComboBox(this);
		programList.setEditable(false);
		programList.currentIndexChanged.connect(this,"programChanged()");

		shader=new QComboBox(this);
		shader.addItem("Vertex program");
		shader.addItem("Fragment program");
		shader.setEditable(false);
		shader.currentIndexChanged.connect(this,"shaderChanged()");
		
        QFont font=new QFont();
        font.setFamily("Courier");
        font.setFixedPitch(true);
        font.setPointSize(10);

        editor=new QTextEdit(this);
		editor.setLineWrapMode(QTextEdit.LineWrapMode.NoWrap);
		editor.setAcceptRichText(false);
		editor.textChanged.connect(this,"textEdited()");
		editor.setFont(font);

		updateEditor();
		
		parameterSheet=new PropertySheet(this);
		
		this.createGLSLShader=null;
		
		State state=State.getInstance();
		lifecycleListener=new LifecycleListener();
		state.getDocumentManager().addLifecycleListener(lifecycleListener);
		operatorGraphListener=new OperatorGraphListener();
		((Document) state.getDocumentManager().getMainDocument()).getOGManager().registerOperatorGraphListener(operatorGraphListener);
		operatorGraphListener.rebuildProgramList();
		
		QVBoxLayout layout=new QVBoxLayout();
		QHBoxLayout bottomLayout=new QHBoxLayout();
		bottomLayout.addWidget(editor,3);
		bottomLayout.addWidget(parameterSheet,1);
		
		layout.addWidget(programList);
		layout.addWidget(shader);
		layout.addLayout(bottomLayout);
		
		setLayout(layout);
	}

	private void updateEditor() {
		internalEditorUpdate=true;
		if(createGLSLShader!=null) {
//			editor.setEnabled(true);
			if(shader.itemText(shader.currentIndex()).equals("Vertex program")) {
				try {
					editor.clear();
					editor.textCursor().insertText(createGLSLShader.getProperty("vertexShader").getStringValue());
				} catch (Exception e) {
					// this should not happen
					e.printStackTrace();
				}
			} else {
				try {
					editor.clear();
					editor.textCursor().insertText(createGLSLShader.getProperty("fragmentShader").getStringValue());
				} catch (Exception e) {
					// this should not happen
					e.printStackTrace();
				}						
			}
			QSyntaxHighlighter highlighter=State.getInstance().getUIManager().getSyntaxHighlighter("GLSL");
			if(highlighter!=null) {
				highlighter.setDocument(editor.document());
			}	
		} else {
//			editor.setEnabled(false);
		}
		internalEditorUpdate=false;
	}
	
	public void shaderChanged() {
		updateEditor();		
	}
	
	public void programChanged() {
		updateCreateGLSLShaderNode();		
	}
	
	public void textEdited() {
		if(internalEditorUpdate) {
			return;
		}
		
		if(shader.itemText(shader.currentIndex()).equals("Vertex program")) {
			try {
				createGLSLShader.getProperty("vertexShader").setValue(editor.toPlainText());
				((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().update();
			} catch (Exception e1) {
				// this cannot happen here
				e1.printStackTrace();
			}
		} else {
			try {
				createGLSLShader.getProperty("fragmentShader").setValue(editor.toPlainText());
				((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().update();
			} catch (Exception e1) {
				// this cannot happen here
				e1.printStackTrace();
			}					
		}		
	}

	private void updateCreateGLSLShaderNode() {
		if(programList.currentIndex()==-1) {
			// this happens; just haven't figure out yet, when
			return;
		}
		String programNodeName=programList.itemText(programList.currentIndex());
		String nodeName=programNodeName.split("/")[1];
		try {
			this.createGLSLShader=(CreateGLSLShader) ((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager().getNodeByName(nodeName);
		} catch (Exception e1) {
			// TODO handle exception properly
			e1.printStackTrace();
			createGLSLShader=null;
		}				
	}
	
	public CreateGLSLShader getCreateGLSLShaderNode() {
		return createGLSLShader;
	}
	
	public String getShaderText() {
		return editor.toPlainText();
	}
	
	public void setShaderText(String text) {
		editor.clear();
		editor.textCursor().insertText(text);
	}

	@Override
	public void disposed() {
		State state=State.getInstance();
		state.getDocumentManager().removeLifecycleListener(lifecycleListener);
		((Document) state.getDocumentManager().getMainDocument()).getOGManager().unregisterOperatorGraphListener(operatorGraphListener);
	}
}
