/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 26, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;
/**
 * \package eu.moonlight3d.ml3d.rendering.glsl
 *
 * This plugin contains the support for GLSL shaders including the GLSL shader editor
 * 
 * This package is a plugin that depends on eu.moonlight3d.ml3d.rendering.simple, eu.moonlight3d.ml3d.plugins.ui.view 
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private GLSLConverter glslConverter;
	private CreateGLSLShaderTool createGLSLShaderTool;

	public String getName() {
		return "eu.moonlight3d.ml3d.rendering.glsl.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.rendering.simple.Plugin"); // because of materialMenuExtensionPoint
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.propertysheet.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("GLSLProgram",GLSLProgram.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("GLSLShader",GLSLShader.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateGLSLShader",CreateGLSLShader.class));
		
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			manager.registerMainWidgetFactory(new GLSLEditorFactory());
			manager.addOrReplaceLayout(new FrameLayout("glsleditor","GLSL Editor"));
			manager.registerAction(new ToolAction("CreateGLSLShaderAction","Create GLSL Shader",null,"Material","Materials","CreateGLSLShaderTool"));
			manager.registerAction(new OpenGLSLEditorAction(null));
			manager.registerAction(new LoadShaderAction(null));
			manager.registerAction(new SaveShaderAction(null));
	//		manager.registerTool(new AddShaderParameterTool());
	//		manager.registerTool(new RemoveShaderParameterTool());
			
			createGLSLShaderTool=new CreateGLSLShaderTool();
			ToolChestView.addTool(createGLSLShaderTool);
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"GLSL","materialMenuExtensionPoint");
			contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
			contribution.add(new MenuNode(MenuNode.Type.Entry,"Create GLSL Shader","CreateGLSLShaderAction"));
			contribution.add(new MenuNode(MenuNode.Type.Entry,"Open GLSL Editor","OpenGLSLEditorAction"));
			manager.getLayout("mainwindow").addMenuContribution(contribution);
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"Editor","menubar");
			MenuNode programMenu=new MenuNode(MenuNode.Type.Menu,"Program","");
			programMenu.add(new MenuNode(MenuNode.Type.Entry,"Load Shader...","LoadShaderAction"));
			programMenu.add(new MenuNode(MenuNode.Type.Entry,"Save Shader...","SaveShaderAction"));
			programMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			programMenu.add(new MenuNode(MenuNode.Type.Entry,"Close","CloseCurrentFrameAction"));
			MenuNode parametersMenu=new MenuNode(MenuNode.Type.Menu,"Parameters","");
			parametersMenu.add(new MenuNode(MenuNode.Type.Entry,"Add Shader Parameter...","AddShaderParameterAction"));
			parametersMenu.add(new MenuNode(MenuNode.Type.Entry,"Remove Shader Parameter...","RemoveShaderParameterAction"));
			contribution.add(programMenu);
			contribution.add(parametersMenu);
			manager.getLayout("glsleditor").addMenuContribution(contribution);
			
			manager.registerSyntaxHighlighterFactory(new DefaultFactory("GLSL",SyntaxHighlighter.class));
	
			glslConverter=new GLSLConverter();
			OpenGLConverter.getInstance().registerSGNodeConverter(glslConverter);
		}
	}

	public void unload() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("GLSLProgram");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("GLSLShader");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateGLSLShader");
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager uiManager=State.getInstance().getUIManager();
			uiManager.unregisterMainWidgetFactory("glsleditorview");
			uiManager.unregisterAction("CreateGLSLShaderAction");
			uiManager.unregisterAction("OpenGLSLEditorAction");
			uiManager.unregisterAction("LoadShaderAction");
			uiManager.unregisterAction("SaveShaderAction");
	//		State.getInstance().getToolsManager().unregisterTool("AddShaderParameterTool");
	//		State.getInstance().getToolsManager().unregisterTool("RemoveShaderParameterTool");
			
			ToolChestView.removeTool(createGLSLShaderTool);
			
			State.getInstance().getUIManager().unregisterSyntaxHighlighterFactory("GLSL");
	
			OpenGLConverter.getInstance().unregisterSGNodeConverter(glslConverter);
		}
	}

}
