/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 1, 2006
 */
package eu.moonlight3d.ml3d.rendering.glsl;

import com.trolltech.qt.core.QRegExp;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QSyntaxHighlighter;
import com.trolltech.qt.gui.QTextDocument;

public class SyntaxHighlighter extends QSyntaxHighlighter {
	private String[] keywords = { "attribute", "const", "uniform", "varying", 
			                      "break", "continue", "do", "for", "while",
			                      "if", "else",
			                      "in", "out", "inout",
			                      "float", "int", "void", "bool", "true", "false",
			                      "discard", "return",
			                      "mat2", "mat3", "mat4",
			                      "vec2", "vec3", "vec4", "ivec2", "ivec3", "ivec4", "bvec2", "bvec3", "bvec4",
			                      "sampler1D", "sampler2D", "sampler3D", "samplerCube", "sampler1DShadow", "sampler2DShadow",
			                      "struct",
			                      // reserved for future use (as of GLSL Specification 1.10.59)
			                      "asm",
			                      "class", "union", "enum", "typedef", "template", "this", "packed",
			                      "goto", "switch", "default",
			                      "inline", "noinline", "volatile", "public", "static", "extern", "external", "interface:",
			                      "long", "short", "double", "half", "fixed", "unsigned",
			                      "input", "output",
			                      "hvec2", "hvec3", "hvec4", "dvec2", "dvec3", "dvec4", "fvec2", "fvec3", "fvec4",
			                      "sampler2DRect", "sampler3DRect", "sampler2DRectShadow",
			                      "sizeof", "cast",
			                      "namespace", "using"
			                      };
	
	private String[] preprocessor = { "#", "#define", "#undef",
			                          "#if", "#ifdef", "#ifndef", "#else", "#elif", "#endif",
			                          "#error", "#pragma",
			                          "#extension", "#version", 
			                          "#line" };
	
	private String[] builtinVariables = { "gl_Position", "gl_PointSize", "glClipVertex",
			                              "gl_FragCoord", "gl_FrontFacing", "gl_FragColor", "gl_FragData", "glFragDepth"};
	
	private String[] builtinAttributes = { "gl_Color", "gl_SecondaryColor", "gl_Normal", "gl_Vertex", 
			                               "gl_MulitTexCoord0", "gl_MulitTexCoord1", "gl_MulitTexCoord2", "gl_MulitTexCoord3", 
			                               "gl_MulitTexCoord4", "gl_MulitTexCoord5", "gl_MulitTexCoord6", "gl_MulitTexCoord7", 
			                               "gl_FogCoord" };
	
	private String[] builtinConstants = { "gl_MaxLights", "gl_MaxClipPlanes", "gl_MaxTextureUnits", "gl_MaxTextureCoords", 
			                              "gl_MaxVertexAttribs", "gl_MaxVertexUniformComponents", "gl_MaxVaryingFloats",
			                              "gl_MaxVertexTextureImageUnits", "gl_MaxCombinedTextureImageUnits", 
			                              "gl_MaxFragmentUniformComponents", "gl_MaxDrawBufers" };
	
	private String[] builtinUniforms = { "gl_ModelViewMatrix", "gl_ProjectionMatrix", "gl_ModelviewProjectionMatrix", 
			                             "gl_TextureMatrix", "gl_NormalMatrix", "gl_ModelViewMatrixInverse",
			                             "gl_ProjectionMatrixInverse", "gl_ModelViewProjectionMatrixInverse",
			                             "gl_TextureMatrixInverse", "gl_ModelViewMatrixTranspose", 
			                             "gl_ProjectionMatrixTranspose", "gl_ModelViewProjectionMatrixTranspose", 
			                             "gl_TextureMatrixTranspose", "gl_ModelViewMatrixInverseTranspose",
			                             "gl_ProjectionMatrixInverseTranspose", "gl_ModelViewProjectionMatrixInverseTranspose",
			                             "gl_TextureMatrixInverseTranspose", "gl_NormalScale", "gl_DepthRange",
			                             "gl_ClipPlane", "gl_Point", "gl_FrontMaterial", "gl_BackMaterial", 
			                             "gl_LightSource", "gl_LightModel", "gl_FrontLightModelProduct",
			                             "gl_BackLightModelProduct", "gl_FrontLightProduct", "gl_BackLightProduct",
			                             "gl_TextureEnvColor", "gl_EyePlaneS", "gl_EyePlaneT", "gl_EyePlaneR",
			                             "gl_EyePlaneQ", "gl_ObjectPlaneS", "glObjectPlaneT", "glObjectPlaneR",
			                             "gl_ObjectPlaneQ", "gl_Fog" };
	
	private String[] builtinVarying = { "gl_FrontColor", "gl_BackColor", "glFrontSecondaryColor", "gl_BackSecondaryColor",
			                            "gl_TexCoord", "gl_FogFragCoord", "gl_Color", "gl_SeconaryColor" };
	
	public SyntaxHighlighter() {
		super((QTextDocument)null);
	}
	
	private void highlightPatternList(String text, String[] patternList, QColor color) {
		for(String pattern : patternList) {
			QRegExp expression=new QRegExp("\\b" + pattern + "\\b");
            int index = expression.indexIn(text, 0);
            while (index >= 0) {
                int length = expression.matchedLength();
                setFormat(index, length, color);
                index = expression.indexIn(text, index+length);
            }			
		}		
	}

	@Override
	protected void highlightBlock(String text) {
		highlightPatternList(text, keywords, QColor.blue);
		highlightPatternList(text, preprocessor, QColor.green);
		highlightPatternList(text, builtinVariables, QColor.darkRed);
		highlightPatternList(text, builtinAttributes, QColor.red);
		highlightPatternList(text, builtinConstants, QColor.darkCyan);
		highlightPatternList(text, builtinUniforms, QColor.darkRed);
		highlightPatternList(text, builtinVarying, QColor.red);
	}

}
