/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 27, 2006
 */
package eu.moonlight3d.ml3d.rendering.materialgraph;

import java.util.ArrayList;
import java.util.List;

import eu.moonlight3d.framework.helper.NamedObject;
import eu.moonlight3d.framework.helper.PropertyContainer;


public abstract class MaterialGraphNode extends PropertyContainer implements NamedObject {
	private String name;
	private ArrayList<MaterialGraphSlot> slots;

	public abstract void evaluate(RendererInterface renderer);
	
	public boolean isPureSource() {
		for(MaterialGraphSlot slot : slots) {
			if(slot.getType()==MaterialGraphSlot.Type.Input) {
				return false;
			}
		}
		return true;
	}
	
	public boolean isSource() {
		for(MaterialGraphSlot slot : slots) {
			if(slot.getType()==MaterialGraphSlot.Type.Input && slot.getConnectedSlot()!=null) {
				return false;
			}
		}
		return true;
	}
	
	public void setName(String name) {
		this.name=name;
		slots=new ArrayList<MaterialGraphSlot>();
	}
	
	public String getName() {
		return name;
	}

	protected void createSlot(String name, MaterialGraphSlot.Type type) {
		slots.add(new MaterialGraphSlot(name,type));
	}
	
	protected MaterialGraphSlot getSlot(String name) {
		for(MaterialGraphSlot slot : slots) {
			if(slot.getName().equals(name)) {
				return slot;
			}
		}
		return null;
	}
	
	public List<MaterialGraphSlot> getSlots() {
		return (List<MaterialGraphSlot>) slots.clone();
	}
}
