/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 29, 2007
 */
package eu.moonlight3d.ml3d.rendering.materialgraph;

import java.util.ArrayList;
import java.util.HashMap;

import eu.moonlight3d.graph.GraphModel;
import eu.moonlight3d.graph.Node;
import eu.moonlight3d.graph.Slot;
import eu.moonlight3d.graph.layout.SimpleLayouter;

public class ModelGenerator {
	private GraphModel graphModel;
	private MaterialGraph materialGraph;
	private HashMap<MaterialGraphSlot, Slot> slotMap;
	private ArrayList<MaterialGraphSlot> outputSlotList;
	
	public ModelGenerator() {
		graphModel=new GraphModel();
		graphModel.setLayouter(new SimpleLayouter());
		slotMap=new HashMap<MaterialGraphSlot, Slot>();
		outputSlotList=new ArrayList<MaterialGraphSlot>();
	}
	
	private void addNodes() {
		for(MaterialGraphNode node : materialGraph.getNodes()) {
			Node graphNode=new Node(node.getName());
			
			// add slots, build list of links
			for(MaterialGraphSlot slot : node.getSlots()) {
				Slot.Type slotType;
				if(slot.getType()==MaterialGraphSlot.Type.Input) {
					slotType=Slot.Type.Input;
				} else {
					slotType=Slot.Type.Output;
					if(slot.getConnectedSlot()!=null) {
						outputSlotList.add(slot);
					}
				}
				Slot graphSlot=new Slot(graphNode,slot.getName(),slotType);
				graphNode.addSlot(graphSlot);
				slotMap.put(slot, graphSlot);
			}
			graphModel.addNode(graphNode);
		}
	}
	
	private void addLinks() {
		for(MaterialGraphSlot slot : outputSlotList) {
			graphModel.addLink(slotMap.get(slot), slotMap.get(slot.getConnectedSlot()));
		}
	}
	
	public void updateGraph() {
		graphModel.clearGraph();
		slotMap.clear();
		outputSlotList.clear();
		addNodes();
		addLinks();
		slotMap.clear();
		outputSlotList.clear();
		graphModel.recalculateLayout();
	}
	
	public void clearGraph() {
		graphModel.clearGraph();
		slotMap.clear();
		outputSlotList.clear();
		graphModel.recalculateLayout();
	}
	
	public void setMaterialGraph(MaterialGraph materialGraph) {
		this.materialGraph=materialGraph;
		updateGraph();
	}

	public GraphModel getGraphModel() {
		return graphModel;
	}
}
