/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 27, 2006
 */
package eu.moonlight3d.ml3d.rendering.materialgraph;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private CreateMaterialGraphTool createMaterialGraphTool;

	public String getName() {
		return "eu.moonlight3d.ml3d.rendering.materialgraph.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.rendering.simple.Plugin"); // because of materialMenuExtensionPoint
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.propertysheet.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("MaterialGraph",MaterialGraph.class));
        
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateMaterialGraph",CreateMaterialGraph.class));

        State state=State.getInstance();
        if(!state.getBatchMode()) {
			UIManager uiManager=state.getUIManager();
			
			uiManager.registerAction(new OpenMaterialGraphEditorAction(null));
			
			uiManager.registerMainWidgetFactory(new MaterialGraphEditorFactory());
			uiManager.addOrReplaceLayout(new FrameLayout("materialgrapheditor","Material Graph Editor"));
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"MaterialGraph","materialMenuExtensionPoint");
			contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
			contribution.add(new MenuNode(MenuNode.Type.Entry,"Open Material Graph Editor","OpenMaterialGraphEditorAction"));
			uiManager.getLayout("mainwindow").addMenuContribution(contribution);
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"A_Material","menubar");
			MenuNode materialMenu=new MenuNode(MenuNode.Type.Menu,"Material", "Material");
			materialMenu.add(new MenuNode(MenuNode.Type.Entry,"Close","CloseCurrentFrameAction"));
			contribution.add(materialMenu);
			uiManager.getLayout("materialgrapheditor").addMenuContribution(contribution);
			
			createMaterialGraphTool=new CreateMaterialGraphTool();
			ToolChestView.addTool(createMaterialGraphTool);
        }
	}

	public void unload() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("MaterialGraph");

		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager uiManager=state.getUIManager();
			
			uiManager.unregisterMainWidgetFactory("materialgrapheditor");
			uiManager.unregisterAction("OpenMaterialGraphEditorAction");
			
			ToolChestView.removeTool(createMaterialGraphTool);
		}
	}

}
