/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 28, 2006
 */
package eu.moonlight3d.ml3d.rendering.materialgraph;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.math.Vector3D;

public class SubtractNode extends MaterialGraphNode {

	public SubtractNode() {
		createSlot("input1",MaterialGraphSlot.Type.Input);
		createSlot("input2",MaterialGraphSlot.Type.Input);
		createSlot("output",MaterialGraphSlot.Type.Output);
	}
	
	@Override
	public void evaluate(RendererInterface renderer) {
		if(getSlot("input1").getValue().getType()==Value.Type.Integer && getSlot("input2").getValue().getType()==Value.Type.Integer) {
			Value value=new Value();
			value.setType(Value.Type.Integer);
			try {
				value.setValue(getSlot("input1").getValue().getIntegerValue()-getSlot("input2").getValue().getIntegerValue());
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else if(getSlot("input1").getValue().getType()==Value.Type.Float && getSlot("input2").getValue().getType()==Value.Type.Float) {
			Value value=new Value();
			value.setType(Value.Type.Float);
			try {
				value.setValue(getSlot("input1").getValue().getFloatValue()-getSlot("input2").getValue().getFloatValue());
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else if(getSlot("input1").getValue().getType()==Value.Type.Integer && getSlot("input2").getValue().getType()==Value.Type.Float) {
			Value value=new Value();
			value.setType(Value.Type.Float);
			try {
				value.setValue(getSlot("input1").getValue().getIntegerValue()-getSlot("input2").getValue().getFloatValue());
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else if(getSlot("input1").getValue().getType()==Value.Type.Float && getSlot("input2").getValue().getType()==Value.Type.Integer) {
			Value value=new Value();
			value.setType(Value.Type.Float);
			try {
				value.setValue(getSlot("input1").getValue().getFloatValue()-getSlot("input2").getValue().getIntegerValue());
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else if(getSlot("input1").getValue().getType()==Value.Type.Colour && getSlot("input2").getValue().getType()==Value.Type.Colour) {
			Value value=new Value();
			value.setType(Value.Type.Colour);
			try {
				Colour input1, input2;
				Colour colour=new Colour();
				input1=getSlot("input1").getValue().getColourValue();
				input2=getSlot("input2").getValue().getColourValue();
				colour.red=input1.red-input2.red;
				colour.green=input1.green-input2.green;
				colour.blue=input1.blue-input2.blue;
				colour.alpha=input1.alpha-input2.alpha;
				value.setValue(colour);
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else if(getSlot("input1").getValue().getType()==Value.Type.Vector3D && getSlot("input2").getValue().getType()==Value.Type.Vector3D) {
			Value value=new Value();
			value.setType(Value.Type.Vector3D);
			try {
				Vector3D input1, input2;
				input1=getSlot("input1").getValue().getVectorValue();
				input2=getSlot("input2").getValue().getVectorValue();
				Vector3D output=input1.sub(input2);
				value.setValue(output);
				getSlot("output").setValue(value);
			} catch (Exception e) {
				// this cannot happen - type safety is guaranteed
				e.printStackTrace();
			}
		} else {
			// TODO handle this error
		}
	}

}
