/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 20, 2007
 */
package eu.moonlight3d.ml3d.rendering.simple;


import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.common.ui.colourdialog.ColourDialog;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.GeometryNode;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

public class CreateSimpleMaterialTool extends Tool {
    public class ColourEditor extends QWidget {
    	class Canvas extends QWidget {
    		public Canvas(QWidget parent) {
    			super(parent);
    			setMinimumWidth(40);
    		}
    		
    		private QColor getQColor() {
    			Colour colour=getColour();
				Colour.RGB rgb=colour.getsRGB8();
    			return QColor.fromRgb(rgb.red, rgb.green, rgb.blue);    			
    		}
    		
    		@Override 
    		public void paintEvent(QPaintEvent event) {
    			QPainter painter=new QPainter();
    			painter.begin(this);
    			painter.setPen(QColor.black);
    			painter.setBrush(getQColor());
    			painter.drawRect(rect());
    			painter.end();
    		}
    		
    		@Override 
    		public void mouseDoubleClickEvent(QMouseEvent event) {
    			ColourDialog dialog=new ColourDialog(State.getInstance().getUIManager().getActiveApplicationWindow(),getColour(),false);
    			dialog.exec();
    			if(dialog.getOKPressed()) {
    				setColour(dialog.getColour());
        			update();
    			}
    		}
    	}
    	
        Canvas canvas;
        Colour colour;

        ColourEditor(QWidget parent, String labelText, Colour colour)
        {
        	super(parent);
            
        	this.colour=colour;
        	
            QLabel label=new QLabel(this);
            label.setText(labelText);
            canvas=new Canvas(this);
            
            QHBoxLayout layout=new QHBoxLayout();
            layout.addWidget(label);
            layout.addStretch(1);
            layout.addWidget(canvas);
            setLayout(layout);
        }
        
        public Colour getColour()
        {
        	return colour;            
        }
        
        public void setColour(Colour value)
        {
        	colour=value.clone();
        	update();
        }
    }
    
    private ColourEditor colourEditor;
    private ColourEditor specularEditor;
    private ColourEditor ambientEditor;
	
	public CreateSimpleMaterialTool() {
		setName("CreateSimpleMaterialTool");
		setDisplayName("Create Simple Material");
		setCategory("Create/Material");
	}
	
	@Override
	public void activated() {
        try {
            State state=eu.moonlight3d.framework.State.getInstance();
            Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();

            // don't do anything if selected object is not a SG node
            Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
            if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
                state.logger().warning(L10N.translate("current selection must be a scene graph object"));
                return;
            }
            
            if(!(selection.getObjectFromSelectionString(selection.getActiveObject()) instanceof GeometryNode)) {
            	state.logger().warning(L10N.translate("materials can only be assigned to scene graph nodes which contain visible geometry (e.g. Meshes)"));
            	return;
            }
            
            String ogNodeName=SelectionHelper.getOGNodeName(selection.getActiveObject());
            String slotName=SelectionHelper.getOGSlotName(selection.getActiveObject());
            String sgNodeName=SelectionHelper.getSGNodeName(selection.getActiveObject());
            
            Node ogNode=ogManager.getNodeByName(ogNodeName);
            Slot slot=ogNode.getSlot(slotName);
            Node createSimpleColourMaterial=ogManager.createNodeByName("CreateSimpleMaterial");
            createSimpleColourMaterial.getProperty("ownerName").setValue(sgNodeName);
            createSimpleColourMaterial.getProperty("colour").setValue(colourEditor.getColour());
            createSimpleColourMaterial.getProperty("specular").setValue(specularEditor.getColour());
            createSimpleColourMaterial.getProperty("ambient").setValue(ambientEditor.getColour());
            // set links
            ogManager.insertNodeAfter(slot,createSimpleColourMaterial.getSlot("input"),createSimpleColourMaterial.getSlot("output"));
			ogManager.update();
	        selection.updateBase(ogNodeName,createSimpleColourMaterial.getName());
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);
		colourEditor=new ColourEditor(optionEditor,L10N.translate("Colour:"),new Colour(0.5,0.5,0.5,1.0));
		specularEditor=new ColourEditor(optionEditor,L10N.translate("Specular:"),new Colour(1.0,1.0,1.0,1.0));
		ambientEditor=new ColourEditor(optionEditor,L10N.translate("Ambient:"),new Colour(0.2,0.2,0.2,1.0));
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(colourEditor);
		layout.addWidget(specularEditor);
		layout.addWidget(ambientEditor);
		optionEditor.setLayout(layout);
		
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
