/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 16, 2006
 */
package eu.moonlight3d.ml3d.rendering.simple;

import java.nio.ByteBuffer;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.common.image.core.Channel;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.ml3d.ui.mainwindow.view.GLHelper;
import eu.moonlight3d.ml3d.ui.mainwindow.view.MaterialNode;


public class GLTextureMaterial extends MaterialNode {
	private eu.moonlight3d.common.image.core.Framebuffer textureImage;
	private boolean textureInitialised;
	private int textureID;
	private long lastUsedMillis;
	private GL gl;
	private String uvMapName;
	
	public GLTextureMaterial(Framebuffer textureImage, String uvMapName) {
		this.textureImage=textureImage;
		textureInitialised=false;
		lastUsedMillis=System.currentTimeMillis();
		this.uvMapName=uvMapName;
	}

	@Override
	public int getNumPasses(GL gl) {
		lastUsedMillis=System.currentTimeMillis();
		this.gl=gl;

		if(!textureInitialised) {
			GLHelper.checkAndLogGLError(gl,"[TextureMaterial] before glEnable");
			gl.glEnable(GL.GL_TEXTURE_2D);
			GLHelper.checkAndLogGLError(gl,"[TextureMaterial] glEnable");
			int textureIDs[]=new int[1];
			gl.glGenTextures(1,textureIDs,0);
			textureID=textureIDs[0];
			gl.glBindTexture(GL.GL_TEXTURE_2D,textureID);
			GLHelper.checkAndLogGLError(gl,"[TextureMaterial] glBindTexture");
			
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_REPEAT);
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_REPEAT);
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);
            gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);

            Channel redChannel=textureImage.getChannel("red");
			Channel greenChannel=textureImage.getChannel("green");
			Channel blueChannel=textureImage.getChannel("blue");
			if(textureImage.getChannel("alpha")!=null) {
				Channel alphaChannel=textureImage.getChannel("alpha");
				byte[] openGLTexture=new byte[4*textureImage.getWidth()*textureImage.getHeight()];
				int index=0;
				for(int y=0;y<textureImage.getHeight();y++) {
					for(int x=0;x<textureImage.getWidth();x++) {
						Colour.RGB colour=new Colour(redChannel.getPixel(x,y),greenChannel.getPixel(x,y),blueChannel.getPixel(x,y),alphaChannel.getPixel(x,y)).getsRGB8();
						openGLTexture[index]=(byte)colour.red;
						index++;
						openGLTexture[index]=(byte)colour.green;
						index++;
						openGLTexture[index]=(byte)colour.blue;
						index++;
						openGLTexture[index]=(byte)colour.alpha;
						index++;
					}
				}

				// TODO: don't copy array to bytebyffer
				ByteBuffer buffer=ByteBuffer.allocateDirect(openGLTexture.length);
				buffer.put(openGLTexture);
				buffer.rewind();
				gl.glTexImage2D(GL.GL_TEXTURE_2D,0,GL.GL_RGBA,textureImage.getWidth(),textureImage.getHeight(),0,GL.GL_RGBA,GL.GL_UNSIGNED_BYTE,buffer);
//				gl.glTexImage2D(GL.GL_TEXTURE_2D,0,GL.GL_RGB,textureImage.getWidth(),textureImage.getHeight(),0,GL.GL_RGB,GL.GL_UNSIGNED_BYTE,buffer);
				GLHelper.checkAndLogGLError(gl,"[TextureMaterial] glTexImage2D");
			} else {
				byte[] openGLTexture=new byte[3*textureImage.getWidth()*textureImage.getHeight()];
				int index=0;
				for(int y=0;y<textureImage.getHeight();y++) {
					for(int x=0;x<textureImage.getWidth();x++) {
						Colour.RGB colour=new Colour(redChannel.getPixel(x,y),greenChannel.getPixel(x,y),blueChannel.getPixel(x,y),1.0).getsRGB8();
						openGLTexture[index]=(byte)colour.red;
						index++;
						openGLTexture[index]=(byte)colour.green;
						index++;
						openGLTexture[index]=(byte)colour.blue;
						index++;
					}
				}				
				// TODO: don't copy array to bytebyffer
				ByteBuffer buffer=ByteBuffer.allocateDirect(openGLTexture.length);
				buffer.put(openGLTexture);
				gl.glTexImage2D(GL.GL_TEXTURE_2D,0,GL.GL_RGBA,textureImage.getWidth(),textureImage.getHeight(),0,GL.GL_RGB,GL.GL_UNSIGNED_BYTE,buffer);
				GLHelper.checkAndLogGLError(gl,"[TextureMaterial] glTexImage2D");
			}
			
			gl.glDisable(GL.GL_TEXTURE_2D);
			GLHelper.checkAndLogGLError(gl,"[TextureMaterial] glDisable");

			textureInitialised=true;
		}
		
		return 1; // this material needs only one pass
	}

	@Override
	public String getUVMapForPass(GL gl, int numPass) {
		return uvMapName;
	}

	@Override
	public void doPass(GL gl, int numPass) {
		lastUsedMillis=System.currentTimeMillis();

		gl.glEnable(GL.GL_TEXTURE_2D);
		gl.glBindTexture(GL.GL_TEXTURE_2D,textureID);
		gl.glColor3f(1.0f, 1.0f, 1.0f);
	}
	
	@Override
	public void reset(GL gl) {
		gl.glDisable(GL.GL_TEXTURE_2D);
	}

	public long getLastUsedMillis() {
		return lastUsedMillis;
	}
	
	public void disposeGLTexture() {
		if(!textureInitialised) {
			return;
		}
		
		int textureIDs[]=new int[1];
		textureIDs[0]=textureID;
		
		gl.glDeleteTextures(1,textureIDs,0);
	}
}
