/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 29, 2005
 */
package eu.moonlight3d.ml3d.rendering.simple;
/**
 * \package eu.moonlight3d.ml3d.plugins.rendering.simplematerial
 * 
 * This is the plugin for the simple material.
 * 
 * This package is a seperate plugin.
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.preferences.PreferencesGroup;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;


/**
 * This is the plugin class for the create box plugin
 *
 * @author gregor
 */
public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	private SimpleMaterialConverter simpleMaterialConverter;
	private TextureMaterialConverter textureMaterialConverter;
	private CreateSimpleMaterialTool createSimpleMaterialTool;
	private CreateTextureMaterialTool createTextureMaterialTool;
	
	private LifecycleListener lifecycleListener;

    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.core.plugins.Plugin#getName()
     */
    public String getName()
    {
        return "eu.moonlight3d.ml3d.rendering.simple.Plugin";
    }

    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.core.plugins.Plugin#getDependencies()
     */
    public ArrayList<String> getDependencies()
    {
        ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
        dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
        return dependencyList;
    }

    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.core.plugins.Plugin#load()
     */
    public void load() throws Exception
    {
        eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("SimpleMaterial",SimpleMaterial.class));
        eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("TextureMaterial",TextureMaterial.class));
        
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateSimpleMaterial",CreateSimpleMaterial.class));
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateTextureMaterial",CreateTextureMaterial.class));
        
        State state=State.getInstance();
        
        PreferencesGroup preferences=new PreferencesGroup();
        preferences.setName("Texture Cache");
        Property textureGraceTime=new Property(preferences);
        textureGraceTime.setName("textureGraceTime");
        textureGraceTime.setType(Value.Type.Integer);
        textureGraceTime.setValue(600);
        Property cleanupInterval=new Property(preferences);
        cleanupInterval.setName("cleanupInterval");
        cleanupInterval.setType(Value.Type.Integer);
        cleanupInterval.setValue(60);
        state.getPreferencesManager().getCurrentPreferences().addPreferencesGroup(preferences);
        
        lifecycleListener=new LifecycleListener();
        state.getDocumentManager().addLifecycleListener(lifecycleListener);
        
        if(!state.getBatchMode()) {
			TextureCache.getInstance();

			UIManager uiManager=state.getUIManager();
			uiManager.registerAction(new ToolAction("CreateSimpleMaterialAction","Create Simple Material",null,"Material","Materials","CreateSimpleMaterialTool"));
			uiManager.registerAction(new ToolAction("CreateTextureMaterialAction","Create Texture Material",null,"Material","Materials","CreateTextureMaterialTool"));
	        
	        MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"Material","menubar");
	        MenuNode materialMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Material"),"");
	        materialMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Simple Material"),"CreateSimpleMaterialAction"));
	        materialMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Create Texture Material"),"CreateTextureMaterialAction"));
	        materialMenu.add(new MenuNode(MenuNode.Type.ExtensionPoint,"materialMenuExtensionPoint",""));
	        contribution.add(materialMenu);
	        uiManager.getLayout("mainwindow").addMenuContribution(contribution);
	
	        createSimpleMaterialTool=new CreateSimpleMaterialTool();
	        ToolChestView.addTool(createSimpleMaterialTool);
	        createTextureMaterialTool=new CreateTextureMaterialTool();
	        ToolChestView.addTool(createTextureMaterialTool);
	        
			simpleMaterialConverter=new SimpleMaterialConverter();
			OpenGLConverter.getInstance().registerSGNodeConverter(simpleMaterialConverter);
			textureMaterialConverter=new TextureMaterialConverter();
			OpenGLConverter.getInstance().registerSGNodeConverter(textureMaterialConverter);
        }
    }

    /* (non-Javadoc)
     * @see eu.moonlight3d.ml3d.core.plugins.Plugin#unload()
     */
    public void unload() throws Exception
    {
        eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("SimpleMaterial");

        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateSimpleMaterial");
        eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateTextureMaterial");

        State state=State.getInstance();
        state.getDocumentManager().removeLifecycleListener(lifecycleListener);

        if(!state.getBatchMode()) {        
	        state.getUIManager().unregisterAction("CreateSimpleMaterialAction");
	        
	        ToolChestView.removeTool(createSimpleMaterialTool);
	
			OpenGLConverter.getInstance().unregisterSGNodeConverter(simpleMaterialConverter);
			OpenGLConverter.getInstance().unregisterSGNodeConverter(textureMaterialConverter);
        }
    }

}
