/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Aug 21, 2005
 */
package eu.moonlight3d.ml3d.rendering.simple;


import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.ml3d.backend.threedim.sg.MaterialNode;

public class SimpleMaterial extends MaterialNode {
    private Colour colour;
    private Colour specular;
    private Colour ambient;
    
    public SimpleMaterial() 
    {
        colour=new Colour();
        specular=new Colour();
        ambient=new Colour();
    }
    
    @Override
	public eu.moonlight3d.ml3d.backend.threedim.sg.Node clone() {
    	SimpleMaterial m=(SimpleMaterial)super.clone();
    	m.colour=colour.clone();
    	return m;
    }
    
    public Colour getColour()
    {
        return colour;
    }
    
    public void setColour(Colour colour)
    {
        this.colour=colour;
    }
    
    public Colour getSpecular() {
    	return specular;
    }
    
    public void setSpecular(Colour specular) {
    	this.specular=specular;
    }
    
    public Colour getAmbient() {
    	return ambient;
    }
    
    public void setAmbient(Colour ambient) {
    	this.ambient=ambient;
    }

    public boolean isValidAppendixName(String appendix)
    {
        if(appendix.equals("colour") || appendix.equals("specular") || appendix.equals("ambient")) {
            return true;
        } else {
            return false;
        }
    }

    public Object getFromAppendix(String appendix)
    {
        if(appendix.equals("colour")) {
            return getColour();
        } else if(appendix.equals("specular")) {
        	return getSpecular();
        } else if(appendix.equals("ambient"));
        return null;
    }

	@Override
	public String getType() {
		return "SimpleMaterial";
	}

	@Override
	protected void loadXMLData(Element dataElement) {
		Element colourElement=(Element) dataElement.getElementsByTagName("colour").item(0);
		colour.red=Double.parseDouble(colourElement.getAttribute("r"));
		colour.green=Double.parseDouble(colourElement.getAttribute("g"));
		colour.blue=Double.parseDouble(colourElement.getAttribute("b"));
		colour.alpha=Double.parseDouble(colourElement.getAttribute("a"));
		
		Element specularElement=(Element) dataElement.getElementsByTagName("specular").item(0);
		specular.red=Double.parseDouble(specularElement.getAttribute("r"));
		specular.green=Double.parseDouble(specularElement.getAttribute("g"));
		specular.blue=Double.parseDouble(specularElement.getAttribute("b"));
		specular.alpha=Double.parseDouble(specularElement.getAttribute("a"));

		Element ambientElement=(Element) dataElement.getElementsByTagName("ambient").item(0);
		ambient.red=Double.parseDouble(ambientElement.getAttribute("r"));
		ambient.green=Double.parseDouble(ambientElement.getAttribute("g"));
		ambient.blue=Double.parseDouble(ambientElement.getAttribute("b"));
		ambient.alpha=Double.parseDouble(ambientElement.getAttribute("a"));
	}
	
	@Override
	protected void saveXMLData(Document document, Element dataElement) {
		Element colourElement=document.createElement("colour");
		colourElement.setAttribute("r",Double.toString(colour.red));
		colourElement.setAttribute("g",Double.toString(colour.green));
		colourElement.setAttribute("b",Double.toString(colour.blue));
		colourElement.setAttribute("a",Double.toString(colour.alpha));
		dataElement.appendChild(colourElement);
		Element specularElement=document.createElement("specular");
		specularElement.setAttribute("r",Double.toString(specular.red));
		specularElement.setAttribute("g",Double.toString(specular.green));
		specularElement.setAttribute("b",Double.toString(specular.blue));
		specularElement.setAttribute("a",Double.toString(specular.alpha));
		dataElement.appendChild(specularElement);
		Element ambientElement=document.createElement("ambient");
		ambientElement.setAttribute("r", Double.toString(ambient.red));
		ambientElement.setAttribute("g", Double.toString(ambient.green));
		ambientElement.setAttribute("b", Double.toString(ambient.blue));
		ambientElement.setAttribute("a", Double.toString(ambient.alpha));
		dataElement.appendChild(ambientElement);
	}
}
