/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Dec 24, 2008
 */

package eu.moonlight3d.ml3d.rendering.simple;

import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.resource.Resource;
import eu.moonlight3d.ml3d.backend.threedim.resource.ResourceRemoveCheck;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.Iterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TypeFilter;
import eu.moonlight3d.ml3d.modelling.textures.Texture;

public class TextureRemoveCheck implements ResourceRemoveCheck {
	private Document document;
	
	TextureRemoveCheck(Document document) {
		this.document=document;
	}

	public boolean canRemove(Resource resource) {
		if(!(resource instanceof Texture)) {
			return true;
		}

		// scan scene graphs for TextureMaterial nodes that reference this texture resource
		for(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode : document.getOGManager().getNodeList()) {
			for(Slot slot : ogNode.getSlots()) {
				if(slot.getType()==Slot.Type.Output) {
					try {
						Iterator iterator=new Iterator(slot.getGraph());
						
						iterator.addFilter(new TypeFilter("eu.moonlight3d.ml3d.rendering.simple.TextureMaterial",true));
						
						while(iterator.hasNext()) {
							eu.moonlight3d.ml3d.backend.threedim.sg.Node sgNode=iterator.next();
							
							if(sgNode instanceof TextureMaterial) {
								TextureMaterial textureMaterial=(TextureMaterial) sgNode;
								if(textureMaterial.getTextureName().equals(resource.getName())) {
									return false;
								}
							}
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}
		
		return true;
	}

}
