/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 16, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;


import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.ml3d.backend.threedim.sg.MaterialNode;

public class AmbientOcclusionMaterial extends MaterialNode {
	private Colour darkColour;
	private Colour lightColour;
	private int numSamples;
	private double maxDistance;

	public AmbientOcclusionMaterial() {
		this.darkColour=new Colour();
		darkColour.red=0.0;
		darkColour.green=0.0;
		darkColour.blue=0.0;
		darkColour.alpha=0.0;
		this.lightColour=new Colour();
		lightColour.red=1.0;
		lightColour.green=1.0;
		lightColour.blue=1.0;
		lightColour.alpha=0.0;
		numSamples=16;
		maxDistance=20;
	}
	
	public void setDarkColour(Colour colour) {
		darkColour=colour.clone();
	}
	
	public Colour getDarkColour() {
		return darkColour.clone();
	}
	
	public void setLightColour(Colour colour) {
		lightColour=colour.clone();
	}
	
	public Colour getLightColour() {
		return lightColour.clone();
	}
	
	public void setNumSamples(int numSamples) {
		this.numSamples=numSamples;
	}
	
	public int getNumSamples() {
		return numSamples;
	}
	
	public void setMaxDistance(double maxDistance) {
		this.maxDistance=maxDistance;
	}
	
	public double getMaxDistance() {
		return maxDistance;
	}
	
	@Override
	protected void saveXMLData(Document document, Element dataElement) {
		Element darkColourElement=document.createElement("darkcolour");
		darkColourElement.setAttribute("r",Double.toString(darkColour.red));
		darkColourElement.setAttribute("g",Double.toString(darkColour.green));
		darkColourElement.setAttribute("b",Double.toString(darkColour.blue));
		darkColourElement.setAttribute("a",Double.toString(darkColour.alpha));
		dataElement.appendChild(darkColourElement);
		Element lightColourElement=document.createElement("lightcolour");
		lightColourElement.setAttribute("r",Double.toString(lightColour.red));
		lightColourElement.setAttribute("g",Double.toString(lightColour.green));
		lightColourElement.setAttribute("b",Double.toString(lightColour.blue));
		lightColourElement.setAttribute("a",Double.toString(lightColour.alpha));
		dataElement.appendChild(lightColourElement);
		Element numSamplesElement=document.createElement("numsamples");
		numSamplesElement.setAttribute("value",Integer.toString(numSamples));
		dataElement.appendChild(numSamplesElement);
		Element maxDistanceElement=document.createElement("maxdistance");
		maxDistanceElement.setAttribute("value",Double.toString(maxDistance));
		dataElement.appendChild(maxDistanceElement);
	}
	
	@Override
	protected void loadXMLData(Element dataElement) {
		Element darkColourElement=(Element) dataElement.getElementsByTagName("darkcolour").item(0);
		darkColour.red=Double.parseDouble(darkColourElement.getAttribute("r"));
		darkColour.green=Double.parseDouble(darkColourElement.getAttribute("g"));
		darkColour.blue=Double.parseDouble(darkColourElement.getAttribute("b"));
		darkColour.alpha=Double.parseDouble(darkColourElement.getAttribute("a"));
		Element lightColourElement=(Element) dataElement.getElementsByTagName("lightcolour").item(0);
		lightColour.red=Double.parseDouble(lightColourElement.getAttribute("r"));
		lightColour.green=Double.parseDouble(lightColourElement.getAttribute("g"));
		lightColour.blue=Double.parseDouble(lightColourElement.getAttribute("b"));
		lightColour.alpha=Double.parseDouble(lightColourElement.getAttribute("a"));
		Element numSamplesElement=(Element) dataElement.getElementsByTagName("numsamples").item(0);
		numSamples=Integer.parseInt(numSamplesElement.getAttribute("value"));
		Element maxDistanceElement=(Element) dataElement.getElementsByTagName("maxdistance").item(0);
		maxDistance=Double.parseDouble(maxDistanceElement.getAttribute("value"));
	}
	
	@Override
	public String getType() {
		return "AmbientOcclusionMaterial";
	}

	public boolean isValidAppendixName(String appendix) {
		return false;
	}

	public Object getFromAppendix(String appendix) {
		return null;
	}

}
