/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 26, 2007
 */

package eu.moonlight3d.ml3d.rendering.sunflow;


import org.sunflow.image.Color;

import eu.moonlight3d.common.image.core.Channel;
import eu.moonlight3d.common.image.core.Framebuffer;

/**
 * Wrapper Bitmap implementation for Moonlights {@link Framebuffer} images.
 *
 * @author gregor
 */
public class Bitmap extends org.sunflow.image.Bitmap {
	private Framebuffer framebuffer;
	private Channel red;
	private Channel green;
	private Channel blue;
	private Channel luminance;
	private Channel alpha;
	
	/**
	 * Constructor taking the framebuffer to wrap.
	 * 
	 * @param framebuffer the framebuffer to wrap
	 */
	public Bitmap(Framebuffer framebuffer) {
		this.framebuffer=framebuffer;
		red=framebuffer.getChannel("red");		
		green=framebuffer.getChannel("green");		
		blue=framebuffer.getChannel("blue");		
		luminance=framebuffer.getChannel("luminance");		
		alpha=framebuffer.getChannel("alpha");		
	}

	@Override
	public int getHeight() {
		return framebuffer.getHeight();
	}

	@Override
	public int getWidth() {
		return framebuffer.getWidth();
	}

	@Override
	public float readAlpha(int x, int y) {
		if(alpha!=null) {
			return alpha.getPixel(x, y);
		} else {
			return 1;
		}
	}

	@Override
	public Color readColor(int x, int y) {
		float red, green, blue;
		if(luminance!=null) {
			red=green=blue=luminance.getPixel(x, y);
		} else {
			red=this.red.getPixel(x, y);
			green=this.green.getPixel(x, y);
			blue=this.blue.getPixel(x, y);
		}
		
		Color color=new Color(red, green, blue);
		return color;
	}
}
