/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 17, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

public class CreateMirrorMaterial extends Node {
	private Property name;
	private Property ownerName;
	private Property reflectedColour;
	
	public CreateMirrorMaterial() {
		createSlot("input",Slot.Type.Input);
		createSlot("output",Slot.Type.Output);
		
		name=new Property(this);
		name.setName("name");
		name.setType(Value.Type.String);
		try {
			name.setValue("MirrorMaterial");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		ownerName=new Property(this);
		ownerName.setName("ownerName");
		ownerName.setType(Value.Type.String);
		try {
			ownerName.setValue("None");
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
		reflectedColour=new Property(this);
		reflectedColour.setName("reflectedColour");
		reflectedColour.setType(Value.Type.Colour);
		Colour colourValue=new Colour();
		colourValue.red=0.0;
		colourValue.green=0.0;
		colourValue.blue=0.0;
		colourValue.alpha=1.0;
		try {
			reflectedColour.setValue(colourValue);
		} catch (Exception e) {
			// this cannot happen
			e.printStackTrace();
		}
	}
	
	@Override
	public void update() throws Exception {
        SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node owner;
        MirrorMaterial mirrorMaterial;
        
        graph=getSlot("input").getGraph();
        owner=graph.getNodeByName(ownerName.getStringValue());

        mirrorMaterial=(MirrorMaterial) graph.createNode("MirrorMaterial",name.getStringValue());
        
        mirrorMaterial.setReflectedColour(reflectedColour.getColourValue());
        
        graph.addNode(mirrorMaterial);
        graph.addChild(owner, mirrorMaterial);
      
        getSlot("output").setGraph(graph);
	}

	@Override
	public void removeDependencies() {
	}

}
