/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 19, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;


import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.helper.SelectionHelper;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.GeometryNode;

public class CreateSunflowMaterialAction extends Action {
	String graphNodeName;
	
	public CreateSunflowMaterialAction(QWidget parent, String actionName, String statusBarText, String graphNodeName) {
		super(actionName,statusBarText,parent);
		this.graphNodeName=graphNodeName;
        triggered.connect(this,"run()");
	}

	@Override
	public void run() throws Exception {
        State state=State.getInstance();
        Manager ogManager = ((Document) state.getDocumentManager().getMainDocument()).getOGManager();

        // don't do anything if selected object is not a SG node
        Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
        if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode")) {
            state.logger().warning(L10N.translate("current selection must be a scene graph object"));
            return;
        }
        
        if(!(selection.getObjectFromSelectionString(selection.getActiveObject()) instanceof GeometryNode)) {
        	state.logger().warning(L10N.translate("materials can only be assigned to scene graph nodes which contain visible geometry (e.g. Meshes)"));
        	return;
        }
        
        String ogNodeName=SelectionHelper.getOGNodeName(selection.getActiveObject());
        String slotName=SelectionHelper.getOGSlotName(selection.getActiveObject());
        String sgNodeName=SelectionHelper.getSGNodeName(selection.getActiveObject());
        
        Node ogNode=ogManager.getNodeByName(ogNodeName);
        Slot slot=ogNode.getSlot(slotName);
        Node createSimpleColorMaterial=ogManager.createNodeByName(graphNodeName);
        createSimpleColorMaterial.getProperty("ownerName").setValue(sgNodeName);
        // set links
        ogManager.insertNodeAfter(slot,createSimpleColorMaterial.getSlot("input"),createSimpleColorMaterial.getSlot("output"));
        ogManager.update();
        selection.updateBase("OperatorGraph/"+ogNodeName,"OperatorGraph/"+createSimpleColorMaterial.getName());
	}
}
