/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 16, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;


import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.ml3d.backend.threedim.sg.MaterialNode;

public class PhongMaterial extends MaterialNode {
	private Colour diffuseColour;
	private Colour specularColour;
	private int numRays;
	private double power;

	public PhongMaterial() {
		this.diffuseColour=new Colour();
		diffuseColour.red=0.0;
		diffuseColour.green=0.0;
		diffuseColour.blue=0.0;
		diffuseColour.alpha=0.0;
		this.specularColour=new Colour();
		specularColour.red=1.0;
		specularColour.green=1.0;
		specularColour.blue=1.0;
		specularColour.alpha=0.0;
		numRays=4;
		power=2;
	}
	
	public void setDiffuseColour(Colour colour) {
		diffuseColour=colour.clone();
	}
	
	public Colour getDiffuseColour() {
		return diffuseColour.clone();
	}
	
	public void setSpecularColour(Colour colour) {
		specularColour=colour.clone();
	}
	
	public Colour getSpecularColour() {
		return specularColour.clone();
	}
	
	public void setNumRays(int numRays) {
		this.numRays=numRays;
	}
	
	public int getNumRays() {
		return numRays;
	}
	
	public void setPower(double power) {
		this.power=power;
	}
	
	public double getPower() {
		return power;
	}
	
	@Override
	protected void saveXMLData(Document document, Element dataElement) {
		Element diffuseColourElement=document.createElement("diffusecolour");
		diffuseColourElement.setAttribute("r",Double.toString(diffuseColour.red));
		diffuseColourElement.setAttribute("g",Double.toString(diffuseColour.green));
		diffuseColourElement.setAttribute("b",Double.toString(diffuseColour.blue));
		diffuseColourElement.setAttribute("a",Double.toString(diffuseColour.alpha));
		dataElement.appendChild(diffuseColourElement);
		Element specularColourElement=document.createElement("specularcolour");
		specularColourElement.setAttribute("r",Double.toString(specularColour.red));
		specularColourElement.setAttribute("g",Double.toString(specularColour.green));
		specularColourElement.setAttribute("b",Double.toString(specularColour.blue));
		specularColourElement.setAttribute("a",Double.toString(specularColour.alpha));
		dataElement.appendChild(specularColourElement);
		Element numRaysElement=document.createElement("numrays");
		numRaysElement.setAttribute("value",Integer.toString(numRays));
		dataElement.appendChild(numRaysElement);
		Element powerElement=document.createElement("power");
		powerElement.setAttribute("value",Double.toString(power));
		dataElement.appendChild(powerElement);
	}
	
	@Override
	protected void loadXMLData(Element dataElement) {
		Element diffuseColourElement=(Element) dataElement.getElementsByTagName("diffusecolour").item(0);
		diffuseColour.red=Double.parseDouble(diffuseColourElement.getAttribute("r"));
		diffuseColour.green=Double.parseDouble(diffuseColourElement.getAttribute("g"));
		diffuseColour.blue=Double.parseDouble(diffuseColourElement.getAttribute("b"));
		diffuseColour.alpha=Double.parseDouble(diffuseColourElement.getAttribute("a"));
		Element specularColourElement=(Element) dataElement.getElementsByTagName("specularcolour").item(0);
		specularColour.red=Double.parseDouble(specularColourElement.getAttribute("r"));
		specularColour.green=Double.parseDouble(specularColourElement.getAttribute("g"));
		specularColour.blue=Double.parseDouble(specularColourElement.getAttribute("b"));
		specularColour.alpha=Double.parseDouble(specularColourElement.getAttribute("a"));
		Element numRaysElement=(Element) dataElement.getElementsByTagName("numrays").item(0);
		numRays=Integer.parseInt(numRaysElement.getAttribute("value"));
		Element powerElement=(Element) dataElement.getElementsByTagName("power").item(0);
		power=Double.parseDouble(powerElement.getAttribute("value"));
	}

	@Override
	public String getType() {
		return "PhongMaterial";
	}

	public boolean isValidAppendixName(String appendix) {
		// TODO Auto-generated method stub
		return false;
	}

	public Object getFromAppendix(String appendix) {
		// TODO Auto-generated method stub
		return null;
	}

}
