/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 22, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;
/**
 * \package eu.moonlight3d.ml3d.rendering.sunflow
 * 
 * This is the plugin for the Sunflow render engine.
 * 
 * It depends on the sunflow render engine to be available
 * in the classpath, otherwise loading will fail.
 */

import java.util.ArrayList;

import org.sunflow.PluginRegistry;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.DefaultFactory;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	AboutSunflowPage aboutSunflowPage;

	public String getName() {
		return "eu.moonlight3d.ml3d.rendering.sunflow.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.rendering.simple.Plugin"); // because of materialMenuExtensionPoint
		return dependencyList;
	}

	public void load() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("AmbientOcclusionMaterial",AmbientOcclusionMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("ConstantMaterial",ConstantMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("DiffuseMaterial",DiffuseMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("GlassMaterial",GlassMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("MirrorMaterial",MirrorMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("PhongMaterial",PhongMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.registerNodeFactory(new DefaultFactory("ShinyDiffuseMaterial",ShinyDiffuseMaterial.class));

		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateAmbientOcclusionMaterial",CreateAmbientOcclusionMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateConstantMaterial",CreateConstantMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateDiffuseMaterial",CreateDiffuseMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateGlassMaterial",CreateGlassMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateMirrorMaterial",CreateMirrorMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreatePhongMaterial",CreatePhongMaterial.class));
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.registerNodeFactory(new DefaultFactory("CreateShinyDiffuseMaterial",CreateShinyDiffuseMaterial.class));

		eu.moonlight3d.ml3d.backend.threedim.renderer.Manager.addRenderEngine(new SunflowRenderEngine());

		// register "moonlight" format handler in Sunflow for handling in-memory textures (as proposed by Chris):
        PluginRegistry.bitmapReaderPlugins.registerPlugin("moonlight", BitmapReader.class);
		
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			
//			manager.registerAction(new CreateAmbientOcclusionMaterialAction(null));
//			manager.registerAction(new CreateConstantMaterialAction(null));
//			manager.registerAction(new CreateDiffuseMaterialAction(null));
//			manager.registerAction(new CreateGlassMaterialAction(null));
//			manager.registerAction(new CreateMirrorMaterialAction(null));
//			manager.registerAction(new CreatePhongMaterialAction(null));
//			manager.registerAction(new CreateShinyDiffuseMaterialAction(null));
//			
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateAmbientOcclusionMaterialAction","Create ambient occlusion material","CreateAmbientOcclusionMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateConstantMaterialAction","Create constant material","CreateConstantMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateDiffuseMaterialAction","Create diffuse material","CreateDiffuseMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateGlassMaterialAction","Create glass material","CreateGlassMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateMirrorMaterialAction","Create mirror material","CreateMirrorMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreatePhongMaterialAction","Create phong material","CreatePhongMaterial"));
			manager.registerAction(new CreateSunflowMaterialAction(null,"CreateShinyDiffuseMaterialAction","Create shiny diffuse material","CreateShinyDiffuseMaterial"));
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"Sunflow","materialMenuExtensionPoint");
			MenuNode materialsMenu=new MenuNode(MenuNode.Type.Menu,"Create Sunflow Material","");
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Ambient Occlusion Material","CreateAmbientOcclusionMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Constant Material","CreateConstantMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Diffuse Material","CreateDiffuseMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Glass Material","CreateGlassMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Mirror Material","CreateMirrorMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Phong Material","CreatePhongMaterialAction"));
			materialsMenu.add(new MenuNode(MenuNode.Type.Entry,"Shiny Diffuse Material","CreateShinyDiffuseMaterialAction"));
			contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
			contribution.add(materialsMenu);
			manager.getLayout("mainwindow").addMenuContribution(contribution);
			
			aboutSunflowPage=new AboutSunflowPage();
			eu.moonlight3d.framework.ui.AboutDialog.registerAboutDialogPageFactory(aboutSunflowPage);
		}
		
		/*
		 * Add an own plugin to Sunflow's bitmap reader plugin list to get the possibility
		 * to feed it in-memory textures through the backdoor that this own plugin creates. 
		 */
//    	PluginRegistry.bitmapReaderPlugins.registerPlugin("MLFramebuffer", BitmapReader.class);
	}

	public void unload() throws Exception {
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("AmbientOcclusionMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("ConstantMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("DiffuseMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("GlassMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("MirrorMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("PhongMaterial");
		eu.moonlight3d.ml3d.backend.threedim.sg.Manager.unregisterNodeFactory("ShinyDiffuseMaterial");

		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateAmbientOcclusionMaterial");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateConstantMaterial");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateGlassMaterial");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateMirrorMaterial");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreatePhongMaterial");
		eu.moonlight3d.ml3d.backend.threedim.og.Manager.unregisterNodeFactory("CreateShinyDiffuseMaterial");

		eu.moonlight3d.ml3d.backend.threedim.renderer.Manager.removeRenderEngine("Sunflow");

		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			
			manager.unregisterAction("CreateAmbientOcclusionMaterialAction");
			manager.unregisterAction("CreateConstantMaterialAction");
			manager.unregisterAction("CreateDiffuseMaterialAction");
			manager.unregisterAction("CreateGlassMaterialAction");
			manager.unregisterAction("CreateMirrorMaterialAction");
			manager.unregisterAction("CreatePhongMaterialAction");
			manager.unregisterAction("CreateShinyDiffuseMaterialAction");
			eu.moonlight3d.framework.ui.AboutDialog.unregisterAboutDialogPageFactory(aboutSunflowPage);
		}
	}

}
