/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 18, 2007
 */
package eu.moonlight3d.ml3d.rendering.sunflow;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.math.Vector3D;

public class SunflowProperties extends PropertyContainer {
    private Property resX;
    private Property resY;
    private Property threads;
    private Property cameraName;
    private Property antiAliasMin;
    private Property antiAliasMax;
    private Property superSample;
    private Property filter;
    private Property diffuseTraceDepth;
    private Property reflectionTraceDepth;
    private Property refractionTraceDepth;
    private Property causticsEnable;
    private Property causticsEmit;
    private Property causticsGather;
    private Property causticsRadius;
    private Property causticsFilter;
    private Property giMethod;
    private Property fakeUp;
    private Property fakeSky;
    private Property fakeGround;
    private Property irrCacheSamples;
    private Property irrCacheTolerance;
    private Property irrCacheMinSpacing;
    private Property irrCacheMaxSpacing;
    private Property irrCacheGMap;
    private Property irrCacheEmit;
    private Property irrCacheGather;
    private Property irrCacheRadius;
    private Property pathSamples;
    private Property ambOccBright;
    private Property ambOccDark;
    private Property ambOccSamples;
    private Property ambOccMaxDistance;
    private Property instantSamples;
    private Property instantSets;
    private Property instantBiasBound;
    private Property instantBiasRays;
    
	public SunflowProperties() {
        resX=new Property(this);
        resX.setName("resX");
        resX.setType(Property.Type.Integer);
        try {
            resX.setValue(640);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        resY=new Property(this);
        resY.setName("resY");
        resY.setType(Property.Type.Integer);
        try {
            resY.setValue(480);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        threads=new Property(this);
        threads.setName("threads");
        threads.setType(Property.Type.Integer);
        try {
            threads.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        cameraName=new Property(this);
        cameraName.setName("cameraName");
        cameraName.setType(Property.Type.String);
        try {
            cameraName.setValue("none");
        } catch(Exception e) {
            // this can never happen
            e.printStackTrace();            
        }
        antiAliasMin=new Property(this);
        antiAliasMin.setName("antiAliasMin");
        antiAliasMin.setType(Property.Type.Integer);
        try {
            antiAliasMin.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        antiAliasMax=new Property(this);
        antiAliasMax.setName("antiAliasMax");
        antiAliasMax.setType(Property.Type.Integer);
        try {
            antiAliasMax.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        superSample=new Property(this);
        superSample.setName("superSample");
        superSample.setType(Property.Type.Integer);
        try {
            superSample.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        filter=new Property(this);
        filter.setName("filter");
        filter.setType(Property.Type.Integer);
        try {
            filter.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        diffuseTraceDepth=new Property(this);
        diffuseTraceDepth.setName("diffuseTraceDepth");
        diffuseTraceDepth.setType(Property.Type.Integer);
        try {
            diffuseTraceDepth.setValue(9);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        reflectionTraceDepth=new Property(this);
        reflectionTraceDepth.setName("reflectionTraceDepth");
        reflectionTraceDepth.setType(Property.Type.Integer);
        try {
            reflectionTraceDepth.setValue(9);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        refractionTraceDepth=new Property(this);
        refractionTraceDepth.setName("refractionTraceDepth");
        refractionTraceDepth.setType(Property.Type.Integer);
        try {
            refractionTraceDepth.setValue(9);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        causticsEnable=new Property(this);
        causticsEnable.setName("causticsEnable");
        causticsEnable.setType(Property.Type.Boolean);
        try {
            causticsEnable.setValue(false);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        causticsEmit=new Property(this);
        causticsEmit.setName("causticsEmit");
        causticsEmit.setType(Property.Type.Integer);
        try {
            causticsEmit.setValue(10000);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        causticsGather=new Property(this);
        causticsGather.setName("causticsGather");
        causticsGather.setType(Property.Type.Integer);
        try {
            causticsGather.setValue(50);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        causticsRadius=new Property(this);
        causticsRadius.setName("causticsRadius");
        causticsRadius.setType(Property.Type.Float);
        try {
            causticsRadius.setValue(0.5);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        causticsFilter=new Property(this);
        causticsFilter.setName("causticsFilter");
        causticsFilter.setType(Property.Type.Float);
        try {
            causticsFilter.setValue(1.1);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        giMethod=new Property(this);
        giMethod.setName("giMethod");
        giMethod.setType(Property.Type.Integer);
        try {
            giMethod.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        fakeUp=new Property(this);
        fakeUp.setName("fakeUp");
        fakeUp.setType(Property.Type.Vector3D);
        try {
        	Vector3D up=new Vector3D();
        	up.clear();
        	up.X3=1;
			fakeUp.setValue(up);
		} catch (Exception e) {
			// this can never happen
			e.printStackTrace();
		}
		fakeSky=new Property(this);
		fakeSky.setName("fakeSky");
		fakeSky.setType(Property.Type.Colour);
		try {
			Colour colour=new Colour(0.3,0.3,1.0,1.0);
			fakeSky.setValue(colour);
		} catch (Exception e) {
            // this can never happen
			e.printStackTrace();
		}
		fakeGround=new Property(this);
		fakeGround.setName("fakeGround");
		fakeGround.setType(Property.Type.Colour);
		try {
			Colour colour=new Colour(0.2,0.2,0.2,1.0);
			fakeGround.setValue(colour);
		} catch (Exception e) {
            // this can never happen
			e.printStackTrace();
		}
        irrCacheSamples=new Property(this);
        irrCacheSamples.setName("irrCacheSamples");
        irrCacheSamples.setType(Property.Type.Integer);
        try {
            irrCacheSamples.setValue(256);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheTolerance=new Property(this);
        irrCacheTolerance.setName("irrCacheTolerance");
        irrCacheTolerance.setType(Property.Type.Float);
        try {
            irrCacheTolerance.setValue(0.05);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheMinSpacing=new Property(this);
        irrCacheMinSpacing.setName("irrCacheMinSpacing");
        irrCacheMinSpacing.setType(Property.Type.Float);
        try {
            irrCacheMinSpacing.setValue(0.05);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheMaxSpacing=new Property(this);
        irrCacheMaxSpacing.setName("irrCacheMaxSpacing");
        irrCacheMaxSpacing.setType(Property.Type.Float);
        try {
            irrCacheMaxSpacing.setValue(5.0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheGMap=new Property(this);
        irrCacheGMap.setName("irrCacheGMap");
        irrCacheGMap.setType(Property.Type.Integer);
        try {
            irrCacheGMap.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheEmit=new Property(this);
        irrCacheEmit.setName("irrCacheEmit");
        irrCacheEmit.setType(Property.Type.Integer);
        try {
            irrCacheEmit.setValue(256);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheGather=new Property(this);
        irrCacheGather.setName("irrCacheGather");
        irrCacheGather.setType(Property.Type.Integer);
        try {
            irrCacheGather.setValue(256);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        irrCacheRadius=new Property(this);
        irrCacheRadius.setName("irrCacheRadius");
        irrCacheRadius.setType(Property.Type.Float);
        try {
            irrCacheRadius.setValue(0.5);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        pathSamples=new Property(this);
        pathSamples.setName("pathSamples");
        pathSamples.setType(Property.Type.Integer);
        try {
            pathSamples.setValue(16);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
		ambOccBright=new Property(this);
		ambOccBright.setName("ambOccBright");
		ambOccBright.setType(Property.Type.Colour);
		try {
			Colour colour=new Colour(1.0,1.0,1.0,1.0); // TODO check default value
			ambOccBright.setValue(colour);
		} catch (Exception e) {
            // this can never happen
			e.printStackTrace();
		}
		ambOccDark=new Property(this);
		ambOccDark.setName("ambOccDark");
		ambOccDark.setType(Property.Type.Colour);
		try {
			Colour colour=new Colour(0.3,0.3,0.3,1.0); // TODO check default value
			ambOccDark.setValue(colour);
		} catch (Exception e) {
            // this can never happen
			e.printStackTrace();
		}
        ambOccSamples=new Property(this);
        ambOccSamples.setName("ambOccSamples");
        ambOccSamples.setType(Property.Type.Integer);
        try {
            ambOccSamples.setValue(16);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        ambOccMaxDistance=new Property(this);
        ambOccMaxDistance.setName("ambOccMaxDistance");
        ambOccMaxDistance.setType(Property.Type.Float);
        try {
            ambOccMaxDistance.setValue(20d);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        instantSamples=new Property(this);
        instantSamples.setName("instantSamples");
        instantSamples.setType(Property.Type.Integer);
        try {
            instantSamples.setValue(64);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        instantSets=new Property(this);
        instantSets.setName("instantSets");
        instantSets.setType(Property.Type.Integer);
        try {
            instantSets.setValue(1);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        instantBiasBound=new Property(this);
        instantBiasBound.setName("instantBiasBound");
        instantBiasBound.setType(Property.Type.Float);
        try {
            instantBiasBound.setValue(0.00003);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
        instantBiasRays=new Property(this);
        instantBiasRays.setName("instantBiasRays");
        instantBiasRays.setType(Property.Type.Integer);
        try {
            instantBiasRays.setValue(0);
        } catch (Exception e) {
            // this can never happen
            e.printStackTrace();
        }
	}
	
	public int getResX() {
		try {
			return resX.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getResY() {
		try {
			return resY.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getThreads() {
		try {
			return threads.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public String getCameraName() {
		try {
			return cameraName.getStringValue();
		} catch (Exception e) {
			return "Camera";
		}
	}
	
	public int getAntiAliasMin() {
		try {
			return antiAliasMin.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getAntiAliasMax() {
		try {
			return antiAliasMax.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getSuperSample() {
		try {
			return superSample.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getFilter() {
		try {
			return filter.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getDiffuseTraceDepth() {
		try {
			return diffuseTraceDepth.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getReflectionTraceDepth() {
		try {
			return reflectionTraceDepth.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public int getRefractionTraceDepth() {
		try {
			return refractionTraceDepth.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public boolean getCausticsEnable() {
		try {
			return causticsEnable.getBooleanValue();
		} catch(Exception e) {
			return false;
		}
	}
	
	public int getCausticsEmit() {
		try {
			return causticsEmit.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getCausticsGather() {
		try {
			return causticsGather.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getCausticsRadius() {
		try {
			return causticsRadius.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getCausticsFilter() {
		try {
			return causticsFilter.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getGIMethod() {
		try {
			return giMethod.getIntegerValue();
		} catch (Exception e) {
			return 0;
		}
	}
	
	public Vector3D getFakeUp() {
		try {
			return fakeUp.getVectorValue();
		} catch(Exception e) {
			return null;
		}
	}
	
	public Colour getFakeSky() {
		try {
			return fakeSky.getColourValue();
		} catch(Exception e) {
			return null;
		}		
	}

	public Colour getFakeGround() {
		try {
			return fakeGround.getColourValue();
		} catch(Exception e) {
			return null;
		}
	}
	
	public int getIrrCacheSamples() {
		try {
			return irrCacheSamples.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getIrrCacheTolerance() {
		try {
			return irrCacheTolerance.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getIrrCacheMinSpacing() {
		try {
			return irrCacheMinSpacing.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getIrrCacheMaxSpacing() {
		try {
			return irrCacheMaxSpacing.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getIrrCacheGMap() {
		try {
			return irrCacheSamples.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getIrrCacheEmit() {
		try {
			return irrCacheEmit.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getIrrCacheGather() {
		try {
			return irrCacheGather.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getIrrCacheRadius() {
		try {
			return irrCacheRadius.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getPathSamples() {
		try {
			return pathSamples.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public Colour getAmbOccBright() {
		try {
			return ambOccBright.getColourValue();
		} catch(Exception e) {
			return null;
		}		
	}
	
	public Colour getAmbOccDark() {
		try {
			return ambOccDark.getColourValue();
		} catch(Exception e) {
			return null;
		}		
	}

	public int getAmbOccSamples() {
		try {
			return ambOccSamples.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public double getAmbOccMaxDistance() {
		try {
			return ambOccMaxDistance.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}
	
	public int getInstantSamples() {
		try {
			return instantSamples.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}

	public int getInstantSets() {
		try {
			return instantSets.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}

	public double getInstantBiasBound() {
		try {
			return instantBiasBound.getFloatValue();
		} catch(Exception e) {
			return 0;
		}
	}

	public int getInstantBiasRays() {
		try {
			return instantBiasRays.getIntegerValue();
		} catch(Exception e) {
			return 0;
		}
	}
}
