/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Mar 22, 2006
 */
package eu.moonlight3d.ml3d.rendering.sunflow;


import org.sunflow.SunflowAPI;
import org.sunflow.image.Color;
import org.sunflow.system.UI;

import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngine;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngineMonitor;

/**
 * RenderEngine implementation for the Sunflow renderer.
 *
 * @author gregor
 */
public class SunflowRenderEngine implements RenderEngine {
	
    /**
     * Reimplementation of Sunflow's display class that writes the resulting
     * image data into a eu.moonlight3d.ml3d.image.Framebuffer.
     *
     * @author gregor
     */
    private class Display implements org.sunflow.core.Display {
    	/**
    	 * Reference to the target framebuffer for the rendering.
    	 */
    	private Framebuffer framebuffer;
    	
    	/**
    	 * Reference to render engine monitor to report to
    	 */
    	private RenderEngineMonitor monitor;
    	
    	/**
    	 * Number of pixels rendered so far by Sunflow
    	 */
    	private int renderedPixels;
    	/**
    	 * Total number of pixels in the image
    	 */
    	private int totalPixels;
    	
    	/**
    	 * Constructor that takes the target framebuffer into
    	 * which the rendering result should be written.
    	 * 
    	 * @param framebuffer the target framebuffer
    	 * @param monitor the render engine monitor to report progress to
    	 */
    	public Display(Framebuffer framebuffer, RenderEngineMonitor monitor) {
    		this.framebuffer=framebuffer;
    		this.monitor=monitor;
    		
    		renderedPixels=0;
    		totalPixels=framebuffer.getWidth()*framebuffer.getHeight();
    	}

		public void imageBegin(int x, int y, int bucketsize) {
			// nothing to do here
		}

		public void imagePrepare(int x, int y, int w, int h, int id) {
			// nothing to do here
		}

		public void imageUpdate(int x, int y, int w, int h, Color[] data, float[] alpha) {
			for(int j=0;j<h;j++) {
				for(int i=0;i<w;i++) {
					framebuffer.getChannel("red").putPixel(x+i,y+j,data[j*w+i].getRGB()[0]);
					framebuffer.getChannel("green").putPixel(x+i,y+j,data[j*w+i].getRGB()[1]);
					framebuffer.getChannel("blue").putPixel(x+i,y+j,data[j*w+i].getRGB()[2]);
					framebuffer.getChannel("alpha").putPixel(x+i,y+j,alpha[j*w+i]);
				}
			}
			
			if(monitor!=null) {
				renderedPixels+=w*h;
				monitor.progress(renderedPixels/(float)totalPixels);
				if(monitor.wantCancelation()) {
					UI.taskCancel();
				}
			}
		}

		public void imageFill(int x, int y, int w, int h, Color color, float alpha) {
			float[] rgb=color.getRGB();
			for(int j=0;j<h;j++) {
				for(int i=0;i<w;i++) {
					framebuffer.getChannel("red").putPixel(x+i,y+j,rgb[0]);
					framebuffer.getChannel("green").putPixel(x+i,y+j,rgb[1]);
					framebuffer.getChannel("blue").putPixel(x+i,y+j,rgb[2]);
					framebuffer.getChannel("alpha").putPixel(x+i,y+j,alpha);
				}
			}
		}

		public void imageEnd() {
			// nothing to do here
		}
    	
    }

    /**
     * Default constructor.
     */
    public SunflowRenderEngine() {
    }
    
	public String getName() {
		return "Sunflow";
	}

	public PropertyContainer getProperties() {
		return new SunflowProperties();
	}

	public void initialise() {
		// nothing to do here
	}

	public Framebuffer renderCurrentOG(PropertyContainer rendererProperties, RenderEngineMonitor monitor) {
		SunflowProperties properties=(SunflowProperties) rendererProperties;
        Framebuffer framebuffer;
        framebuffer=new Framebuffer(properties.getResX(),properties.getResY());
        
        framebuffer.addChannel("red");
        framebuffer.addChannel("green");
        framebuffer.addChannel("blue");
        framebuffer.addChannel("alpha");
        framebuffer.addChannel("depth");
        
        SunflowAPI sunflowAPI=new SunflowAPI();
        
        SunflowSceneConverter converter=new SunflowSceneConverter(sunflowAPI);
        converter.convertCurrentOG(rendererProperties);
        
        if(monitor!=null) {
        	monitor.renderingStarted(framebuffer);
        }
        
        Display display=new Display(framebuffer,monitor);  
        sunflowAPI.render(SunflowAPI.DEFAULT_OPTIONS,display);
        sunflowAPI=null;
        
        if(monitor!=null) {
        	monitor.renderingEnded();
        }

        return framebuffer;
	}

}
