/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 20, 2008
 */

package eu.moonlight3d.ml3d.rendering.ui;

import java.text.MessageFormat;

import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QProgressBar;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.localization.L10N;

public class AnimationProgressDialog extends QDialog {
	private QLabel frameProgressLabel;
	private QProgressBar frameProgress;
	private QLabel animationProgressLabel;
	private QProgressBar animationProgress;
	private QPushButton cancelButton;
	
	private String currentFilename;
	private int totalNumFrames;
	private RenderAnimationThread thread;
	
	public AnimationProgressDialog(QWidget parent, int totalNumFrames) {
		super(parent);
		
		this.totalNumFrames=totalNumFrames;
		this.thread=thread;
		
		setWindowTitle(L10N.translate("Animation Progress"));
		setModal(true);
		
		frameProgressLabel=new QLabel(this);
		frameProgressLabel.setText(L10N.translate("Initialising..."));
		frameProgress=new QProgressBar(this);
		frameProgress.setRange(0, 100);
		
		animationProgressLabel=new QLabel(this);
		animationProgressLabel.setText(L10N.translate("Initialising..."));
		animationProgress=new QProgressBar(this);
		animationProgress.setRange(0, totalNumFrames);
		
		cancelButton=new QPushButton(this);
		cancelButton.setText(L10N.translate("Cancel"));
		cancelButton.clicked.connect(this,"cancel()");
		
		QVBoxLayout layout=new QVBoxLayout();
		
		layout.addWidget(frameProgressLabel);
		layout.addWidget(frameProgress);
		layout.addWidget(animationProgressLabel);
		layout.addWidget(animationProgress);
		layout.addWidget(cancelButton);
		
		setLayout(layout);
	}
	
	void setRenderAnimationThread(RenderAnimationThread thread) {
		this.thread=thread;
	}
	
	protected void cancel() {
		cancelButton.setEnabled(false);
		cancelButton.clicked.connect(null);
		thread.cancelRendering();
	}
	
	void updateFrameProgress(double completed) {
		frameProgressLabel.setText(MessageFormat.format(L10N.translate("Rendering frame {0}..."),currentFilename));
		frameProgress.setValue((int) (completed*100));
	}
	
	void updateAnimationProgress(String filename, int numFrames) {
		currentFilename=filename;
		animationProgressLabel.setText(MessageFormat.format(L10N.translate("Frame {0} of {1}..."), numFrames, totalNumFrames));
		animationProgress.setValue(numFrames);
	}
	
	void setFinished() {
		cancelButton.setText(L10N.translate("Close"));
		cancelButton.setEnabled(true);
		cancelButton.clicked.connect(this,"close()");
	}
}
