/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2005
 */
package eu.moonlight3d.ml3d.rendering.ui;
/**
 * \package eu.moonlight3d.ml3d.rendering.ui
 *
 * This plugin contains the render settings dialog and other interface elements
 * for interacting with renders
 * 
 * This package is a plugin that depends on eu.moonlight3d.ml3d.plugins.ui.toolchest
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;


/**
 * This is the plugin class for the UI test plugin
 * 
 * @author gregor
 */
public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	RenderTool renderTool;

	/* (non-Javadoc)
	 * @see eu.moonlight3d.ml3d.core.plugins.Plugin#getName()
	 */
	public String getName() {
		return "eu.moonlight3d.ml3d.rendering.ui.Plugin";
	}

	/* (non-Javadoc)
	 * @see eu.moonlight3d.ml3d.core.plugins.Plugin#getDependencies()
	 */
	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");		
		dependencyList.add("eu.moonlight3d.common.ui.propertysheet.Plugin");		
		return dependencyList;
	}

	/* (non-Javadoc)
	 * @see eu.moonlight3d.ml3d.core.plugins.Plugin#load()
	 */
	public void load() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			manager.registerAction(new RenderSettingsAction(null));
			manager.registerAction(new RenderAnimationAction(null));
			manager.registerActionGenerator(new RenderActionGenerator());
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"Render","menubar");
			MenuNode renderMenu=new MenuNode(MenuNode.Type.Menu,"Render","");
			MenuNode renderUsingMenu=new MenuNode(MenuNode.Type.Menu,"Render using","");
			renderUsingMenu.add(new MenuNode(MenuNode.Type.Generator,"RenderActionGenerator",""));
			renderMenu.add(new MenuNode(MenuNode.Type.Entry,"Render Settings","RenderSettingsAction"));
			renderMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			renderMenu.add(renderUsingMenu);
			renderMenu.add(new MenuNode(MenuNode.Type.Entry,"Render Animation...","RenderAnimationAction"));
			renderMenu.add(new MenuNode(MenuNode.Type.ExtensionPoint,"renderMenuExtensionPoint",""));
			contribution.add(renderMenu);
			manager.getLayout("mainwindow").addMenuContribution(contribution);
			
			renderTool=new RenderTool();
			eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.addTool(renderTool);
			
			eu.moonlight3d.common.ui.propertysheet.PropertySheet.addPropertyEditorFactory(new SelectCameraFactory());
		}
	}

	/* (non-Javadoc)
	 * @see eu.moonlight3d.ml3d.core.plugins.Plugin#unload()
	 */
	public void unload() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			state.getUIManager().unregisterAction("RenderSettingsAction");
			state.getUIManager().unregisterAction("RenderAnimationAction");
			eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView.removeTool(renderTool);
		}
	}

}
