/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 18, 2006
 */
package eu.moonlight3d.ml3d.rendering.ui;


import java.util.List;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngine;

/**
 * Render a range of frames in an animation as a batch job.
 *
 * @author gregor
 */
public class RenderAnimationAction extends Action {
	/**
	 * Default constructor
	 * 
	 * @param parent the action's parent widget
	 */
	public RenderAnimationAction(QWidget parent) {
		super("RenderAnimationAction","Render Animation",parent);
        triggered.connect(this,"run()");		
	}

	@Override
	public void run() {
		State state=State.getInstance();
		Document document=(Document) state.getDocumentManager().getMainDocument();
		
		if(document.getAnimationManager().getAnimations().size()==0) {
			state.logger().warning(L10N.translate("There are no animations that can be rendered in this scene. Create one first."));
		}

		QWidget activeWindow=state.getUIManager().getActiveApplicationWindow();
		RenderAnimationDialog dialog=new RenderAnimationDialog(activeWindow);
		
		if(dialog.exec()==0) {
			return;
		}
		
		// start rendering
		
		String animationName=dialog.getAnimationName();
		Animation animation=document.getAnimationManager().getAnimationByName(animationName);
		String renderEngineName=dialog.getRenderEngine();
		RenderEngine renderEngine=eu.moonlight3d.ml3d.backend.threedim.renderer.Manager.getRenderEngine(renderEngineName);
		String imageFormatName=dialog.getImageFileFormat();
		List<ImageFormat> formats=eu.moonlight3d.common.image.core.FormatManager.getInstance().getFormats();
		ImageFormat imageFormat=null;				
		for(ImageFormat format : formats) {
			if(format.getName().equals(imageFormatName)) {
				imageFormat=format;				
			}
		}
		if(imageFormat==null) {
			state.logger().severe(L10N.translate("internal error: unable to retrieve selected output image file format"));
			return;
		}
		
		AnimationProgressDialog progressDialog=new AnimationProgressDialog(activeWindow,dialog.getEndFrame()-dialog.getStartFrame()+1);
		RenderAnimationThread renderThread=new RenderAnimationThread(animation, dialog.getStartFrame(), dialog.getEndFrame(), renderEngine, dialog.getOutputDirectory(), dialog.getBaseName(), imageFormat, progressDialog);
		progressDialog.setRenderAnimationThread(renderThread);
		progressDialog.show();
		
		renderThread.start();
	}
}
