/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 18, 2008
 */

package eu.moonlight3d.ml3d.rendering.ui;

import java.util.List;

import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;

public class RenderAnimationDialog extends QDialog {
	/**
	 * The dialog's OK button
	 */
	QPushButton okButton;
	/**
	 * The dialog's cancel button
	 */
	QPushButton cancelButton;
	
	/**
	 * A list of animations to select the right one from
	 */
	QComboBox animationList;
	
	/**
	 * Text entry for the number of the start frame
	 */
	QLineEdit rangeStart;
	/**
	 * Text entry for the number of the end frame
	 */
	QLineEdit rangeEnd;
	
	/**
	 * A list of available render engines to choose from
	 */
	QComboBox renderEngineList;

	/**
	 * The dropdown list for selecting the output file format
	 */
	QComboBox imageFileFormat;
	/**
	 * The output directory for the rendered image files
	 */
	QLineEdit directory;
	/**
	 * Button to open a directory selector for the output directory.
	 */
	QPushButton selectDirectoryButton;
	/**
	 * The base name for the output files 
	 */
	QLineEdit baseName;
	
	/**
	 * Default constructor 
	 * @param parent parent window for the dialog
	 */
	public RenderAnimationDialog(QWidget parent) {
		super(parent);
		
		State state=State.getInstance();
		Document document=(Document) state.getDocumentManager().getMainDocument();
		
		setWindowTitle(L10N.translate("Render Animation"));
		
		
		QLabel animationListLabel=new QLabel(this);
		animationListLabel.setText(L10N.translate("Animation:"));
		animationList=new QComboBox(this);
		animationList.setEditable(false);
		for(Animation animation : document.getAnimationManager().getAnimations()) {
			animationList.addItem(animation.getName());
		}
		
		QLabel rangeStartLabel=new QLabel(this);
		rangeStartLabel.setText(L10N.translate("Range start (frame):"));
		rangeStart=new QLineEdit(this);
		QLabel rangeEndLabel=new QLabel(this);
		rangeEndLabel.setText(L10N.translate("Range end (frame:"));
		rangeEnd=new QLineEdit(this);
		
		QLabel renderEngineListLabel=new QLabel(this);
		renderEngineListLabel.setText(L10N.translate("Render engine:"));
		renderEngineList=new QComboBox(this);
		renderEngineList.setEditable(false);
		List<String> engines=eu.moonlight3d.ml3d.backend.threedim.renderer.Manager.getRenderEngines();
		for(String engine : engines) {
			renderEngineList.addItem(engine);
		}
		
		QLabel imageFileFormatLabel=new QLabel(this);
		imageFileFormatLabel.setText(L10N.translate("Ouput image file format:"));
		imageFileFormat=new QComboBox(this);
		imageFileFormat.setEditable(false);

		List<ImageFormat> imageFormats=FormatManager.getInstance().getFormats();
		for(ImageFormat imageFormat : imageFormats) {
			if(imageFormat.isSaveable()) {
				imageFileFormat.addItem(imageFormat.getFileType().getName());
			}
		}
		
		QLabel directoryLabel=new QLabel(this);
		directoryLabel.setText(L10N.translate("Output directory:"));
		directory=new QLineEdit(this);
		directory.setReadOnly(true);
		// TODO populate that with a sane default
		selectDirectoryButton=new QPushButton(this);
		selectDirectoryButton.setText(L10N.translate("Select"));
		selectDirectoryButton.clicked.connect(this,"selectDirectory()");
		
		QLabel baseNameLabel=new QLabel(this);
		baseNameLabel.setText(L10N.translate("Base filename:"));
		baseName=new QLineEdit(this);
		
		// TODO populate widgets with sane default values

		okButton=new QPushButton(this);
		okButton.setText(L10N.translate("OK"));
		okButton.clicked.connect(this,"ok()");
		cancelButton=new QPushButton(this);
		cancelButton.setText(L10N.translate("Cancel"));
		cancelButton.clicked.connect(this,"cancel()");
		
		QVBoxLayout layout=new QVBoxLayout();
		
		QGridLayout topLayout=new QGridLayout();
		topLayout.addWidget(animationListLabel,0,0);
		topLayout.addWidget(animationList,0,1,1,2);
		topLayout.addWidget(rangeStartLabel,1,0);
		topLayout.addWidget(rangeStart,1,1,1,2);
		topLayout.addWidget(rangeEndLabel,2,0);
		topLayout.addWidget(rangeEnd,2,1,1,2);
		topLayout.addWidget(renderEngineListLabel,3,0);
		topLayout.addWidget(renderEngineList,3,1,1,2);
		topLayout.addWidget(directoryLabel,4,0);
		topLayout.addWidget(directory,4,1);
		topLayout.addWidget(selectDirectoryButton,4,2);
		topLayout.addWidget(baseNameLabel,5,0);
		topLayout.addWidget(baseName,5,1,1,2);
		topLayout.addWidget(imageFileFormatLabel,6,0);
		topLayout.addWidget(imageFileFormat,6,1,1,2);
		
		QHBoxLayout buttonsLayout=new QHBoxLayout();
		buttonsLayout.addStretch();
		buttonsLayout.addWidget(cancelButton);
		buttonsLayout.addWidget(okButton);
		
		layout.addLayout(topLayout);
		layout.addLayout(buttonsLayout);
		
		setLayout(layout);
	}
	
	protected void selectDirectory() {
		String directory=QFileDialog.getExistingDirectory();
		if(directory==null || directory.equals("")) {
			return;
		}
		this.directory.setText(directory);
	}
	
	protected void cancel() {
		done(0);
	}
	
	protected void ok() {
		if(rangeStart.text().equals("")) {
			QMessageBox.critical(this,L10N.translate("Invalid Range"),L10N.translate("You must enter a start frame number."));
		}
		if(rangeEnd.text().equals("")) {
			QMessageBox.critical(this,L10N.translate("Invalid Range"),L10N.translate("You must enter a end frame number."));
		}
		
		int startFrame=Integer.parseInt(rangeStart.text());
		int endFrame=Integer.parseInt(rangeEnd.text());
				
		State state=State.getInstance();
		Document document=(Document) state.getDocumentManager().getMainDocument();
		String animationName=getAnimationName();
		Animation animation=document.getAnimationManager().getAnimationByName(animationName);
		double duration=animation.getEndTime()-animation.getStartTime();			
		int numAnimationFrames=(int)(duration*animation.getFramesPerSecond());
		
		if(startFrame<1 || startFrame>=endFrame) {
			QMessageBox.critical(this, L10N.translate("Invalid Range"), L10N.translate("The number of the start frame must be >=1 and lower or equal to the end frame number."));
			return;
		}
		
		if(endFrame>=numAnimationFrames+1) {
			QMessageBox.critical(this, L10N.translate("Invalid Range"), L10N.translate("The number of the end frame is higher than the number of frames in the selected animation."));
			return;
		}
		
		if(directory.text().equals("")) {
			QMessageBox.critical(this, L10N.translate("Invalid directory"), L10N.translate("You must select an output directory for the created image files."));
		}

		done(1);
	}
	
	String getAnimationName() {
		return animationList.itemText(animationList.currentIndex());
	}
	
	int getStartFrame() {
		return Integer.parseInt(rangeStart.text())-1;
	}
	
	int getEndFrame() {
		return Integer.parseInt(rangeEnd.text())-1;
	}
	
	String getRenderEngine() {
		return renderEngineList.itemText(renderEngineList.currentIndex());
	}
	
	String getOutputDirectory() {
		return directory.text();
	}
	
	String getBaseName() {
		return baseName.text();
	}
	
	String getImageFileFormat() {
		return imageFileFormat.itemText(imageFileFormat.currentIndex());
	}
}
