/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 20, 2008
 */

package eu.moonlight3d.ml3d.rendering.ui;

import java.io.File;
import java.text.DecimalFormat;

import com.trolltech.qt.gui.QApplication;

import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngine;

public class RenderAnimationThread extends Thread {
	private Animation animation;
	private int startFrame;
	private int endFrame;
	private RenderEngine renderEngine;
	private String outputDirectory;
	private String baseName;
	private ImageFormat imageFormat;
	
	private boolean wantCancelation;
	private AnimationProgressDialog dialog;
	
	private class RenderEngineMonitor implements eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngineMonitor {

		public void progress(final double fraction) {
			QApplication.invokeLater(new Runnable() {

				public void run() {
					dialog.updateFrameProgress(fraction);
				}
				
			});
		}

		public void renderingEnded() {
			// TODO Auto-generated method stub
			
		}

		public void renderingStarted(Framebuffer temporaryImage) {
			// TODO Auto-generated method stub
			
		}

		public boolean wantCancelation() {
			// TODO Auto-generated method stub
			return wantCancelation;
		}
		
	}
	
	private class SetTimeRunnable implements Runnable {
		private Animation animation;
		private double time;
		private Object notifyTarget;
		private boolean wasNotified;
		
		public SetTimeRunnable(Animation animation, double time, Object notifyTarget) {
			// TODO Auto-generated constructor stub
			this.animation=animation;
			this.time=time;
			this.notifyTarget=notifyTarget;
			wasNotified=false;
		}

		public void run() {
			try {
				animation.setToTime(time);
				Document document=(Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument();
				document.getOGManager().update();
			} catch (Exception e) {
				e.printStackTrace();
			}
			synchronized(notifyTarget) {
				notifyTarget.notify();
				wasNotified=true;
			}
		}
		
		public boolean wasNotified() {
			return wasNotified;
		}
	}
	
	RenderAnimationThread(Animation animation, int startFrame, int endFrame, RenderEngine renderEngine, String outputDirectory, String baseName, ImageFormat imageFormat, AnimationProgressDialog dialog) {
		this.animation=animation;
		this.startFrame=startFrame;
		this.endFrame=endFrame;
		this.renderEngine=renderEngine;
		this.outputDirectory=outputDirectory;
		this.baseName=baseName;
		this.imageFormat=imageFormat;
		this.dialog=dialog;
		
		wantCancelation=false;
	}
	
	private void signalFinished() {
		QApplication.invokeLater(new Runnable() {
			public void run() {
				dialog.setFinished();
			}	
		});		
	}
	
	@Override
	public void run() {
		try {
			final DecimalFormat decimalFormat=new DecimalFormat();

			double duration=animation.getEndTime()-animation.getStartTime();			
			int numAnimationFrames=(int)(duration*animation.getFramesPerSecond());
			
			int numDigits=(int) Math.ceil(Math.log10(numAnimationFrames));
			String formatString="";
			for(int i=0;i<numDigits;i++) {
				formatString+="0";
			}
			decimalFormat.applyPattern(formatString);
			Document document=(Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument();
			String engineName=renderEngine.getName();
			
			double startTime=animation.getStartTime()+startFrame/animation.getFramesPerSecond();
			
			for(int i=startFrame;i<=endFrame;i++) {
				final double time=startTime+i/animation.getFramesPerSecond();

				SetTimeRunnable setTimeOutRunnable=new SetTimeRunnable(animation,time,this);
				QApplication.invokeLater(setTimeOutRunnable);
				synchronized(this) {
					while(!setTimeOutRunnable.wasNotified()) {
						wait(200);
					}
				}
				PropertyContainer rendererProperties=document.getRendererManager().getProperties(engineName);
				
				final int frameNumber=i;
				QApplication.invokeLater(new Runnable() {
					public void run() {
						dialog.updateAnimationProgress(baseName + decimalFormat.format(frameNumber+1), frameNumber);
					}
				});

				RenderEngineMonitor monitor=new RenderEngineMonitor();
				Framebuffer currentFrame=renderEngine.renderCurrentOG(rendererProperties,monitor);
				
				if(wantCancelation) {
					signalFinished();
					return;
				}
				
				String imageFilename=outputDirectory;
				if(!outputDirectory.endsWith(File.separator)) {
					imageFilename+=File.separator;
				}
				imageFilename+=baseName + decimalFormat.format(i+1); // TODO: file extension
				imageFormat.saveFile(imageFilename, currentFrame);
			}
			signalFinished();
		} catch(Exception e) {
			// TODO proper error handling here, including UI updates
			signalFinished();
			e.printStackTrace();
		}
	}
	
	void cancelRendering() {
		wantCancelation=false;
	}
}
