/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2005
 */
package eu.moonlight3d.ml3d.rendering.ui;


import java.util.List;


import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.renderer.Manager;

/**
 * This is the renderer settings dialog. It displays a property sheet
 * for the currently selected rendering engine that lets the user
 * edit settings specific to this rendering engine.
 * 
 * @author gregor
 */
public class RenderSettingsDialog extends QDialog {
	/**
	 * Label for render engine combo box
	 */
    QLabel renderEngineLabel;
    /**
     * Render engine selection combo box
     */
    QComboBox renderEngine;
    /**
     * Property sheet used to show render engine properties
     */
    PropertySheet propertySheet;
    
    /**
     * Default constructor
     * @param parent parent widget
     */
	public RenderSettingsDialog(QWidget parent) {
		super(parent);
		setWindowTitle(L10N.translate("Render settings"));
		setMinimumSize(300,400);

        renderEngineLabel=new QLabel(this);
        renderEngineLabel.setText(L10N.translate("Render engine:"));
        renderEngine=new QComboBox(this);
        List<String> renderEngines=Manager.getRenderEngines();
        for(int i=0;i<renderEngines.size();i++) {
        		renderEngine.addItem(renderEngines.get(i));
        }
        renderEngine.setEditable(false);
        renderEngine.currentIndexChanged.connect(this,"renderEngineChanged()");
        
        propertySheet=new PropertySheet(this);
        State state=State.getInstance();
        Document document=((Document) state.getDocumentManager().getMainDocument());
        propertySheet.setPropertyContainer(document.getRendererManager().getProperties(renderEngines.get(0)));
        
		QPushButton close=new QPushButton(this);
		close.setText(L10N.translate("Close"));
		close.clicked.connect(this,"close()");
		
		QVBoxLayout layout=new QVBoxLayout();
		QHBoxLayout topLayout=new QHBoxLayout();
		
		topLayout.addWidget(renderEngineLabel);
		topLayout.addWidget(renderEngine);
		
		layout.addLayout(topLayout);
		layout.addWidget(propertySheet,1);
		layout.addWidget(close,0,Qt.AlignmentFlag.AlignRight);
		
		setLayout(layout);
	}
	
	/**
	 * Handle selection of a different rendering engine and display the corresponding
	 * properties.
	 */
	public void renderEngineChanged() {
		String itemText=renderEngine.itemText(renderEngine.currentIndex());
        State state=State.getInstance();
        Document document=((Document) state.getDocumentManager().getMainDocument());
		propertySheet.setPropertyContainer(document.getRendererManager().getProperties(itemText));
	}
}
