/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 6, 2005
 */
package eu.moonlight3d.ml3d.rendering.ui;

import java.text.MessageFormat;
import java.util.List;

import com.trolltech.qt.core.QCoreApplication;

import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.tasks.Task;
import eu.moonlight3d.framework.tasks.TaskManager;
import eu.moonlight3d.framework.ui.ApplicationWindow;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.renderer.Manager;
import eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngine;
import eu.moonlight3d.ml3d.images.viewer.ImageView;

public class RenderTask extends Task {
	private final String renderEngine;
	
	private class RenderEngineMonitor implements eu.moonlight3d.ml3d.backend.threedim.renderer.RenderEngineMonitor {

		public void progress(double fraction) {
			setActionDescription(MessageFormat.format(L10N.translate("rendering... ({0}% finished)"),(int)(fraction*100)));
		}

		public void renderingEnded() {
			// TODO Auto-generated method stub
			
		}

		public void renderingStarted(Framebuffer temporaryImage) {
			// TODO Auto-generated method stub
			
		}

		public boolean wantCancelation() {
			return shouldTerminate();
		}
		
	}
	
	public RenderTask(String renderEngine, TaskManager manager) {
		super(manager);
		this.renderEngine=renderEngine;
		setName(renderEngine);
		setActionDescription(L10N.translate("starting..."));
	}

	@Override
	public void run() {
		setActionDescription("rendering...");
		
        RenderEngine engine = Manager.getRenderEngine(renderEngine);
        State state=State.getInstance();
        Document document=((Document) state.getDocumentManager().getMainDocument());
        RenderEngineMonitor monitor=new RenderEngineMonitor();
        final Framebuffer framebuffer = engine.renderCurrentOG(document.getRendererManager().getProperties(engine.getName()),monitor);
        
        setActionDescription(L10N.translate("finishing..."));

        QCoreApplication.invokeLater(new Runnable() {
        	public void run() {
        		try {
        	        List<ImageView> imageViews = eu.moonlight3d.ml3d.images.viewer.ImageView.getImageViews();
        	        if(imageViews.size()==0) {
        	            UIManager frameManager = eu.moonlight3d.framework.State.getInstance().getUIManager();
        	            ApplicationWindow imageViewer = new ApplicationWindow(frameManager.getLayout("imageviewer"));
        	            imageViewer.show();
        	            imageViews=eu.moonlight3d.ml3d.images.viewer.ImageView.getImageViews();
        	        }
        	        imageViews.get(0).setImage(framebuffer);
        		} catch(Exception e) {
        			ExceptionDialog.run(e);
        		}        		
        	}
        });
	}

}
