/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 15, 2007
 */
package eu.moonlight3d.ml3d.rendering.ui;

import java.util.List;

import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.tasks.TaskManager;
import eu.moonlight3d.ml3d.backend.threedim.renderer.Manager;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

/**
 * A modeless tool to render an image.
 * 
 * @see eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool
 *
 * @author gregor
 */
public class RenderTool extends Tool {
	QComboBox renderEngine;
	
	public RenderTool() {
		super();
		setName("RenderTool");
		setDisplayName("Render");
		setCategory("Render");		
	}
	
	@Override
	public void activated() {
		String renderEngineName=renderEngine.itemText(renderEngine.currentIndex());
		TaskManager manager = State.getInstance().getTaskManager();
		manager.addTask(new RenderTask(renderEngineName,manager));
	}

	@Override
	public void deactivated() {
		// non-modal tool, no deactivation
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		QWidget optionEditor=new QWidget(parent);

        QLabel renderEngineLabel=new QLabel(optionEditor);
        renderEngineLabel.setText("Render engine:");
        renderEngine=new QComboBox(optionEditor);
        List<String> renderEngines=Manager.getRenderEngines();
        for(int i=0;i<renderEngines.size();i++) {
        		renderEngine.addItem(renderEngines.get(i));
        }
        renderEngine.setEditable(false);
		
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(renderEngineLabel);
		layout.addWidget(renderEngine);
		optionEditor.setLayout(layout);
		return optionEditor;
	}

	@Override
	public boolean hasOptionEditor() {
		return true;
	}

}
