/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 8, 2006
 */
package eu.moonlight3d.ml3d.rendering.ui;

import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertyEditorFactory;
import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.common.ui.propertysheet.ValueEditor;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.DrainSlotIterator;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.Iterator;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TypeFilter;

public class SelectCameraFactory implements PropertyEditorFactory {
	private class Parameters {
		public String labelText;
	}
	
	public class SelectCameraEditor extends ValueEditor {
		PropertySheet propertySheet;
	    Property property;
	    QComboBox entry;
	    Parameters parameters;
	
	    SelectCameraEditor(PropertySheet sheet, QWidget parent, final Property property, Parameters parameters)
	    {
	    	super(parent);
	    	propertySheet=sheet;
	        this.property=property;
	        this.parameters=parameters;
	        
	        QLabel label;
	        
	        label=new QLabel(this);
	        if(parameters.labelText!=null) {
	        	label.setText(parameters.labelText);
	        } else {
	        	label.setText(property.getName());
	        }

	        entry=new QComboBox(this);
			// generate handles
			DrainSlotIterator slotIterator = new DrainSlotIterator(((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager());
			while(slotIterator.hasNext()) {
				Slot slot = slotIterator.next();
				SceneGraph sg;
				try {
					sg = slot.getGraph();

					Iterator iterator = new Iterator(sg);
					iterator.addFilter(new TypeFilter("eu.moonlight3d.ml3d.modelling.camera.Camera", true));
					while (iterator.hasNext()) {
						entry.addItem(iterator.next().getName());
					}
				} catch (Exception e) {
					// this should not happen
					e.printStackTrace();
				}
			}
	        updateFromPropertyValue();
	        entry.setEditable(false);
	        entry.currentIndexChanged.connect(this,"currentIndexChanged()");
	        entry.setCurrentIndex(0);
	        try {
		    	String item=null;
				try {
					item = property.getStringValue();
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}

				boolean propertyValueExists=false;
		    	for(int i=0;i<entry.count();i++) {
		    		if(entry.itemText(i).equals(item)) {
		    			entry.setCurrentIndex(i);
		    			propertyValueExists=true;
		    			break;
		    		}
		    	}
				if(entry.count()>0 && !propertyValueExists) {
					property.setValue(entry.itemText(entry.currentIndex()));
				}
			} catch (Exception e) {
				// this exception should not happen here
				e.printStackTrace();
			}
	        
	        QHBoxLayout layout=new QHBoxLayout();
	        layout.addWidget(label);
	        layout.addStretch(1);
	        layout.addWidget(entry);
	        layout.setMargin(0);
	        setLayout(layout);
	    }
	    
	    public void currentIndexChanged() {
        	setPropertyValue(entry.itemText(entry.currentIndex()));
            propertySheet.notifyPropertyUpdate(property);	    	
	    }
	    
	    @Override
		public void updateFromPropertyValue() {
	    	String item=null;
			try {
				item = property.getStringValue();
			} catch (Exception e) {
				ExceptionDialog.run(e);
			}
	    	
	    	for(int i=0;i<entry.count();i++) {
	    		if(entry.itemText(i).equals(item)) {
	    			entry.setCurrentIndex(i);
	    			break;
	    		}
	    	}
	    }
	    
	    public String getPropertyValue() {
	    	try {
				return property.getStringValue();
			} catch (Exception e) {
				// this cannot happen
				e.printStackTrace();
				return "";
			}
	    }
	    
	    public void setPropertyValue(String value) {
	    	try {
				property.setValue(value);
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
	    }
	}
	
	public String getName() {
		return "SelectCamera";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		Parameters parameters=new Parameters();
		
		if(parameterRoot==null) {
			return parameters;
		}

		NodeList labelList=parameterRoot.getElementsByTagName("label");
		if(labelList.getLength()>1) {
			throw new Exception(L10N.translate("only one label can be defined for a camera selection property editor"));
		} else if(labelList.getLength()==1) {
			Element labelElement=(Element) labelList.item(0);
			parameters.labelText=labelElement.getAttribute("text");
		}
		
		return parameters;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container,
			ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		return new SelectCameraEditor(sheet,parent,property,(Parameters) additionalParams);
	}

}
