/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 1, 2006
 */
package eu.moonlight3d.ml3d.scripting.python;

import java.util.Vector;

import com.trolltech.qt.core.QRegExp;
import com.trolltech.qt.core.Qt.BrushStyle;
import com.trolltech.qt.gui.QBrush;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QSyntaxHighlighter;
import com.trolltech.qt.gui.QTextCharFormat;
import com.trolltech.qt.gui.QTextDocument;

public class SyntaxHighlighter extends QSyntaxHighlighter {
	
	QRegExp commentStartExpression;
    QRegExp commentEndExpression;
    
    QTextCharFormat keywordFormat = new QTextCharFormat();
    QTextCharFormat classFormat = new QTextCharFormat();
    QTextCharFormat commentFormat = new QTextCharFormat();
    QTextCharFormat quotationFormat = new QTextCharFormat();
    QTextCharFormat functionFormat = new QTextCharFormat();
    
    public class HighlightingRule{
    	public QRegExp pattern;
    	public QTextCharFormat format;
    	
    	public HighlightingRule(QRegExp pattern, QTextCharFormat format){
    		this.pattern = pattern;
    		this.format = format;
    	}
    }
    
    Vector<HighlightingRule> highlightingRules = new Vector<HighlightingRule>();
	public SyntaxHighlighter() {
		super((QTextDocument)null);
		
		HighlightingRule rule;
		QBrush brush;
		QRegExp pattern;
		
		brush = new QBrush(QColor.blue,BrushStyle.SolidPattern);
		keywordFormat.setForeground(brush);
		keywordFormat.setFontWeight(QFont.Weight.Bold.value());
		
		String[] keywords = { "and", "as", "assert", "break", "class", "continue", "def", 
                "del", "elif", "else", "except", "exec", "finally", "for", 
                "from", "global", "if", "import", "in", "is", "lambda", 
                "not", "or", "pass", "print", "raise", "return", "try",
                "while", "with", "yield" };
		
		for (String keyword : keywords) {
			pattern = new QRegExp("\\b" + keyword +"\\b");
			rule = new HighlightingRule(pattern, keywordFormat);
			highlightingRules.add(rule);
		}
		
		brush = new QBrush(QColor.darkMagenta);
		pattern = new QRegExp("\\bQ[A-Za-z]+\\b");
		classFormat.setForeground(brush);
		classFormat.setFontWeight(QFont.Weight.Bold.value());
		rule =  new HighlightingRule(pattern, classFormat);
		highlightingRules.add(rule);
		brush = new QBrush(QColor.gray, BrushStyle.SolidPattern);
		pattern = new QRegExp("//[^\n]*");
		commentFormat.setForeground(brush);
		rule=new HighlightingRule(pattern, commentFormat);
		highlightingRules.add(rule);
		brush = new QBrush(QColor.blue, BrushStyle.SolidPattern);
		pattern = new QRegExp("\".*\"");
		quotationFormat.setForeground(brush);
		rule = new HighlightingRule(pattern, quotationFormat);
		highlightingRules.add(rule);
		brush = new QBrush(QColor.gray, BrushStyle.SolidPattern);
		pattern = new QRegExp("#[^\n]*");
		commentFormat.setForeground(brush);
		rule = new HighlightingRule(pattern, commentFormat);
		highlightingRules.add(rule);
		commentStartExpression = new QRegExp("/\\*");
		commentEndExpression = new QRegExp("\\*/");
	}
	
	@Override
	protected void highlightBlock(String text) {

	    for (HighlightingRule rule : highlightingRules){
	    	QRegExp expression = rule.pattern;
	    	int index = expression.indexIn(text);
	    	while (index >=0) {
	    		int length = expression.matchedLength();
	    		setFormat(index, length, rule.format);
	    		index = expression.indexIn(text, index + length);
	    	}
	    }
	    
	    setCurrentBlockState(0);
	    
	    int startIndex = 0;
	    if (previousBlockState() != 1)
	    	startIndex= commentStartExpression.indexIn(text);
	    while (startIndex >= 0) {
	    	int endIndex = commentEndExpression.indexIn(text, startIndex);
	    	int commentLength;
	    	if (endIndex == -1) {
	    		setCurrentBlockState(1);
	    		commentLength = text.length() - startIndex;
	    	} else {
	    		commentLength = endIndex - startIndex + commentEndExpression.matchedLength();
	    	}
	    	setFormat(startIndex, commentLength, commentFormat);
	    	startIndex = commentStartExpression.indexIn(text, startIndex + commentLength);
	    }
	}

}
