/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 21, 2005
 */
package eu.moonlight3d.ml3d.scripting.scripteditor;
/**
 * \package eu.moonlight3d.ml3d.plugins.scripting. scripteditor
 *
 * This plugins contains the script editor view
 * 
 * This package is a standalone plugin.
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;


public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {

    public String getName()
    {
        return "eu.moonlight3d.ml3d.scripting.scripteditor.Plugin";
    }

    public ArrayList<String> getDependencies()
    {
        ArrayList<String> dependencyList=new ArrayList<String>();
        dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
        return dependencyList;
    }

    public void load() throws Exception
    {
    	State state=State.getInstance();
    	if(!state.getBatchMode()) {
	        UIManager uiManager=state.getUIManager();
	        
	        uiManager.registerMainWidgetFactory(new ScriptEditorFactory());
	        uiManager.addOrReplaceLayout(new FrameLayout("scripteditor","Script Editor"));
	
	        uiManager.registerAction(new OpenScriptEditorAction(null));
	        uiManager.registerAction(new OpenScriptAction(null));
	        uiManager.registerAction(new SaveScriptAction(null));
	        
	        // Scripts menu in mainwindow:
	        MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"Scripts","menubar");
	        MenuNode scriptsMenu=new MenuNode(MenuNode.Type.Menu,"Scripts","Scripts");
	        contribution.add(scriptsMenu);
			scriptsMenu.add(new MenuNode(MenuNode.Type.Entry,"Open Script Editor","OpenScriptEditorAction"));
			uiManager.getLayout("mainwindow").addMenuContribution(contribution);
			
			// Script menu in script editor:
			contribution=new MenuNode(MenuNode.Type.Contribution,"Script","menubar");
			MenuNode scriptMenu=new MenuNode(MenuNode.Type.Menu,"Script","Script");
			contribution.add(scriptMenu);
			scriptMenu.add(new MenuNode(MenuNode.Type.Entry,"Open Script...","OpenScriptAction"));
			scriptMenu.add(new MenuNode(MenuNode.Type.Entry,"Save Script...","SaveScriptAction"));
			scriptMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			scriptMenu.add(new MenuNode(MenuNode.Type.Entry,"Close Window","CloseCurrentFrameAction"));
			uiManager.getLayout("scripteditor").addMenuContribution(contribution);
    	}
    }

    public void unload() throws Exception
    {
    	State state=State.getInstance();
    	if(!state.getBatchMode()) {
	        UIManager uiManager=state.getUIManager();
	        
	        uiManager.unregisterMainWidgetFactory("scripteditor");
	        
	        uiManager.unregisterAction("OpenScriptEditorAction");
	        uiManager.unregisterAction("OpenScriptAction");
	        uiManager.unregisterAction("SaveScriptAction");
    	}
    }

}
