/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 21, 2005
 */
package eu.moonlight3d.ml3d.scripting.scripteditor;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;


import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSyntaxHighlighter;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.scripts.Interpreter;
import eu.moonlight3d.framework.ui.ExceptionDialog;

public class ScriptEditor extends QWidget {
    private QComboBox language;
    private QPushButton loadButton;
    private QPushButton saveButton;
    private QPushButton runButton;
    private QTextEdit textViewer;

    public ScriptEditor(QWidget parent)
    {
        super(parent);
        
        language=new QComboBox(this);
        List<String> languages=State.getInstance().getScriptManager().getLanguages();
        for(int i=0;i<languages.size();i++) {
            language.addItem(languages.get(i));
        }
        if(languages.size()==0) {
       		language.setEnabled(false);
        }
        language.setEditable(false);
        language.currentIndexChanged.connect(this,"languageChanged()");
        
        loadButton=new QPushButton(this);
        loadButton.setText("load");
        loadButton.clicked.connect(this,"loadScript()");
        
        saveButton=new QPushButton(this);
        saveButton.setText("save");
        saveButton.clicked.connect(this,"saveScript()");
        
        runButton=new QPushButton(this);
        runButton.setText("run");
        runButton.clicked.connect(this,"runScript()");

        if(languages.size()==0) {
        	runButton.setEnabled(false);
        }

        QFont font=new QFont();
        font.setFamily("Courier");
        font.setFixedPitch(true);
        font.setPointSize(10);

        textViewer=new QTextEdit(this);
        textViewer.setAcceptRichText(false);
        textViewer.setLineWrapMode(QTextEdit.LineWrapMode.NoWrap);
        textViewer.setFont(font);
        
        QVBoxLayout layout=new QVBoxLayout();
        QHBoxLayout topLayout=new QHBoxLayout();
        
        topLayout.addWidget(language);
        topLayout.addWidget(loadButton);
        topLayout.addWidget(saveButton);
        topLayout.addWidget(runButton);
        
        layout.addLayout(topLayout);
        layout.addWidget(textViewer,1);
        
        setLayout(layout);
        
        setSyntaxHighlighter();
    }
    
    private void setSyntaxHighlighter() {
    	String languageName=language.itemText(language.currentIndex());
    	QSyntaxHighlighter highlighter=eu.moonlight3d.framework.State.getInstance().getUIManager().getSyntaxHighlighter(languageName);
    	if(highlighter==null) {
    		return;
    	}
    	highlighter.setDocument(textViewer.document());
    }
    
    public void languageChanged() {
    	setSyntaxHighlighter();
    }

	public void loadScript() {
		// TODO build a list of known file extensions for script files as a filter
		String filename=QFileDialog.getOpenFileName(window(),"Open Script");
		
		if(filename.equals("")) {
			return;
		}
		
		try {
			BufferedReader reader=new BufferedReader(new FileReader(filename));
			
			String line, text;
			
			text="";
			while((line=reader.readLine())!=null) {
				text=text+line+"\n";
			}
			reader.close();
			
			textViewer.clear();
			textViewer.textCursor().insertText(text);
			setSyntaxHighlighter();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			QMessageBox.critical(this,"Load script failed","The script file could not be opened. Reason: " + e.getMessage());
		} catch (IOException e) {
			e.printStackTrace();
			QMessageBox.critical(this,"Load script failed","The script file could not be opened. Reason: " + e.getMessage());
		}
	}

	public void saveScript() {
		String filename=QFileDialog.getSaveFileName(window(),"Save Script");
		
		if(filename.equals("")) {
			return;
		}
		
		try {
			FileWriter writer=new FileWriter(filename);
			
			writer.write(textViewer.toPlainText(),0,textViewer.toPlainText().length());
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
			QMessageBox.critical(this,"Save script failed","The script could not be saved. Reason: " + e.getMessage());
		}
	}
	
	public void runScript() {
        try {
            String languageName=language.itemText(language.currentIndex());
            Interpreter interpreter=eu.moonlight3d.framework.State.getInstance().getScriptManager().createInterpreter(languageName);
            interpreter.runText(textViewer.toPlainText());
        } catch(Exception e) {
            ExceptionDialog.run(e);
        }		
	}
}
