/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 25, 2007
 */
package eu.moonlight3d.ml3d.ui.mainwindow;

import com.trolltech.qt.gui.QTabWidget;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.ui.mainwindow.ogeditor.OGEditorView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewWindowSpace;

/**
 * The main window's main widget. It contains the the tab widget
 * with the 3d views and the operator graph editor
 *
 * @author gregor
 */
public class MainWindow extends QWidget {
	/**
	 * The operator graph editor widget
	 */
	private OGEditorView ogEditor;
	/**
	 * The 3D view area widget
	 */
	private ViewWindowSpace view;
	
	/**
	 * Default constructor
	 * 
	 * @param parent the parent widget
	 */
	MainWindow(QWidget parent) {
		super(parent);
		
		QTabWidget upperTab=new QTabWidget(this);
		view=new ViewWindowSpace(upperTab);
		upperTab.addTab(view, L10N.translate("3D View"));
		ogEditor=new OGEditorView(upperTab);
		upperTab.addTab(ogEditor, L10N.translate("Operator Graph"));
		
		QVBoxLayout layout=new QVBoxLayout(this);
		layout.addWidget(upperTab);
		layout.setMargin(0);
		setLayout(layout);
	}

	/**
	 * Get the operator graph editor widget
	 * 
	 * @return the operator graph editor widget
	 */
	public OGEditorView getOgEditor() {
		return ogEditor;
	}

	/**
	 * Get the view area widget
	 * 
	 * @return the view area widget
	 */
	public ViewWindowSpace getView() {
		return view;
	}
}
