/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 25, 2007
 */
package eu.moonlight3d.ml3d.ui.mainwindow;

import java.util.ArrayList;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.Value;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.preferences.PreferencesGroup;
import eu.moonlight3d.framework.ui.DockWidgetFactory;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.threedim.document.DocumentFactory;
import eu.moonlight3d.ml3d.ui.mainwindow.logger.LoggerView;
import eu.moonlight3d.ml3d.ui.mainwindow.ogeditor.ArrangeGraphAction;
import eu.moonlight3d.ml3d.ui.mainwindow.ogeditor.DeleteOGNodeTool;
import eu.moonlight3d.ml3d.ui.mainwindow.ogeditor.DocumentContributionFactory;
import eu.moonlight3d.ml3d.ui.mainwindow.ogeditor.TruncateHistoryTool;
import eu.moonlight3d.ml3d.ui.mainwindow.outline.OutlineView;
import eu.moonlight3d.ml3d.ui.mainwindow.propertyeditor.PropertyEditorView;
import eu.moonlight3d.ml3d.ui.mainwindow.tasklist.TaskListView;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolAction;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.ToolChestView;
import eu.moonlight3d.ml3d.ui.mainwindow.view.InstanceConverter;
import eu.moonlight3d.ml3d.ui.mainwindow.view.OpenGLConverter;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ResetViewportAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SetFourPaneViewAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.SetOnePaneViewAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.StandardViewEventHandler;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewEventHandler;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewProjectionTypeAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewPreviewModeAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.ViewStructureModeAction;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Viewport;

/**
 * Main window plugin
 *
 * @author gregor
 */
public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	/**
	 * Reference to the Delete Operator Graph Node Tool
	 */
	private DeleteOGNodeTool deleteOGNodeTool;
	/**
	 * Reference to the Truncate History Tool
	 */
	private TruncateHistoryTool truncateHistoryTool;
	/**
	 * Reference to the plugin's lifecycle listener
	 */
	private LifecycleListener lifecycleListener;
	/**
	 * The OG editor document contribution factory
	 */
	private DocumentContributionFactory ogContributionFactory;

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.common.ui.colourdialog.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.coreactions.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.fileactions.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.mainwindow.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.propertysheet.Plugin");
		dependencyList.add("eu.moonlight3d.common.ui.viewactions.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.modelling.basicselectiontypes.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.animation.basicselectiontypes.Plugin");
		return dependencyList;
	}

	public String getName() {
		return "eu.moonlight3d.ml3d.ui.mainwindow.Plugin";
	}

	public void load() throws Exception {
		State state=State.getInstance();

        // register preferences
        PreferencesGroup preferences=new PreferencesGroup();
        preferences.setName("3D View");
        Property selectedColour=new Property(preferences);
        selectedColour.setName("selectedColour");
        selectedColour.setType(Value.Type.Colour);
        selectedColour.setValue(new Colour(1.0,0.0,0.0,1.0));
        Property activeColour=new Property(preferences);
        activeColour.setName("activeColour");
        activeColour.setType(Value.Type.Colour);
        activeColour.setValue(new Colour(1.0,0.5,0.5,1.0));
        Property gridColour=new Property(preferences);
        gridColour.setName("gridColour");
        gridColour.setType(Value.Type.Colour);
        gridColour.setValue(new Colour(0.0,0.0,0.0,1.0));
        Property drawGrid=new Property(preferences);
        drawGrid.setName("drawGrid");
        drawGrid.setType(Value.Type.Boolean);
        drawGrid.setValue(true);
        Property gridSize=new Property(preferences);
        gridSize.setName("gridSize");
        gridSize.setType(Value.Type.Float);
        gridSize.setValue(1.0);
        Property gridLines=new Property(preferences);
        gridLines.setName("gridLines");
        gridLines.setType(Value.Type.Integer);
        gridLines.setValue(100);
        Property invertXRotation=new Property(preferences);
        invertXRotation.setName("invertXRotation");
        invertXRotation.setType(Value.Type.Boolean);
        invertXRotation.setValue(false);
        Property invertYRotation=new Property(preferences);
        invertYRotation.setName("invertYRotation");
        invertYRotation.setType(Value.Type.Boolean);
        invertYRotation.setValue(false);
        Property cameraPOIDistance=new Property(preferences);
        cameraPOIDistance.setName("cameraPOIDistance");
        cameraPOIDistance.setType(Value.Type.Float);
        cameraPOIDistance.setValue(1.0);
        Property standardLightPosition=new Property(preferences);
        standardLightPosition.setName("standardLightPosition");
        standardLightPosition.setType(Value.Type.Vector3D);
        standardLightPosition.setValue(new Vector3D(-3.0,3.0,0));
        Property standardLightColour=new Property(preferences);
        standardLightColour.setName("standardLightColour");
        standardLightColour.setType(Value.Type.Colour);
        standardLightColour.setValue(new Colour(0.28,0.28,0.28,1.0));
        Property enableMultisampling=new Property(preferences);
        enableMultisampling.setName("enableMultisampling");
        enableMultisampling.setType(Value.Type.Boolean);
        enableMultisampling.setValue(true);
        
        state.getPreferencesManager().getCurrentPreferences().addPreferencesGroup(preferences);

        if(!state.getBatchMode()) {
        	// main widget factory:
        	UIManager uiManager=state.getUIManager();
        	uiManager.registerMainWidgetFactory(new MainWindowFactory());
        	FrameLayout mainWindowLayout=uiManager.getLayout("mainwindow");

        	// operator graph actions:
        	uiManager.registerAction(new ToolAction("DeleteOGNodeAction",L10N.translate("Delete OG Node"),null,"Operator Graph","Operator Graph","DeleteOGNodeTool"));
        	uiManager.registerAction(new ToolAction("TruncateHistoryAction",L10N.translate("Truncate History"),null,"Operator Graph","Operator Graph","TruncateHistoryTool"));
        	uiManager.registerAction(new ArrangeGraphAction(null));
        	// 3D view actions:
        	uiManager.registerAction(new ViewProjectionTypeAction(Viewport.ProjectionType.Perspective,"ViewProjectionTypePerspectiveAction","Perspective",null));
        	uiManager.registerAction(new ViewProjectionTypeAction(Viewport.ProjectionType.Orthographic,"ViewProjectionTypeOrthographicAction","Orthographic",null));
        	uiManager.registerAction(new ViewPreviewModeAction(Viewport.PreviewMode.None,"ViewSetPreviewModeNoneAction","None",null));
	        uiManager.registerAction(new ViewPreviewModeAction(Viewport.PreviewMode.Wireframe,"ViewSetPreviewModeWireframeAction","Wireframe",null));
	        uiManager.registerAction(new ViewPreviewModeAction(Viewport.PreviewMode.Shaded,"ViewSetPreviewModeShadedAction","Shaded",null));
	        uiManager.registerAction(new ViewPreviewModeAction(Viewport.PreviewMode.Lit,"ViewSetPreviewModeLitAction","Lit",null));
	        uiManager.registerAction(new ViewStructureModeAction(Viewport.StructureMode.Full,"ViewSetStructureModeFullAction","Full",null));
	        uiManager.registerAction(new ViewStructureModeAction(Viewport.StructureMode.Occluded,"ViewSetStructureModeOccludedAction","Occluded",null));
	        uiManager.registerAction(new ViewStructureModeAction(Viewport.StructureMode.None,"ViewSetStructureModeNoneAction","None",null));
	        uiManager.registerAction(new ResetViewportAction(null,"ResetViewportPerspectiveAction","Reset to perspective","1",Viewport.StandardPerspective.Perspective));
	        uiManager.registerAction(new ResetViewportAction(null,"ResetViewportRightAction","Reset to right","2",Viewport.StandardPerspective.Right));
	        uiManager.registerAction(new ResetViewportAction(null,"ResetViewportFrontAction","Reset to front","3",Viewport.StandardPerspective.Front));
	        uiManager.registerAction(new ResetViewportAction(null,"ResetViewportTopAction","Reset to top","4",Viewport.StandardPerspective.Top));
	        uiManager.registerAction(new SetOnePaneViewAction(null));
	        uiManager.registerAction(new SetFourPaneViewAction(null));
	
			// add menu contributions for operator graph editor
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"OGActions","menubar");
			MenuNode ogMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Operator Graph"),"");
			ogMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Delete Graph Nodes"),"DeleteOGNodeAction"));
			ogMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Truncate History"),"TruncateHistoryAction"));
			ogMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			ogMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Arrange Graph"),"ArrangeGraphAction"));
			contribution.add(ogMenu);
			mainWindowLayout.addMenuContribution(contribution);
	
			// /add menu contributions for 3D view 
	        contribution=new MenuNode(MenuNode.Type.Contribution,"A_ViewActions","viewMenuExtensionPoint");
	        MenuNode previewModeMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Preview Mode"),"");
	        previewModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("None"),"ViewSetPreviewModeNoneAction"));
	        previewModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Wireframe"),"ViewSetPreviewModeWireframeAction"));
	        previewModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Shaded"),"ViewSetPreviewModeShadedAction"));
	        previewModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Lit"),"ViewSetPreviewModeLitAction"));
	        contribution.add(previewModeMenu);
	        
	        MenuNode structureModeMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Structure Mode"),"");
	        structureModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Full"),"ViewSetStructureModeFullAction"));
	        structureModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Occluded"),"ViewSetStructureModeOccludedAction"));
	        structureModeMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("None"),"ViewSetStructureModeNoneAction"));
	        contribution.add(structureModeMenu);

	        contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
	        contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("One Pane View Layout"),"SetOnePaneViewAction"));
	        contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Four Pane View Layout"),"SetFourPaneViewAction"));
	        contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));
	        contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Perspective projection"),"ViewProjectionTypePerspectiveAction"));
	        contribution.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Orthographic projection"),"ViewProjectionTypeOrthographicAction"));        
	        contribution.add(new MenuNode(MenuNode.Type.Separator,"",""));

	        MenuNode resetViewportMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Reset Viewport"),"");
	        resetViewportMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Perspective"),"ResetViewportPerspectiveAction"));
	        resetViewportMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Right"),"ResetViewportRightAction"));
	        resetViewportMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Front"),"ResetViewportFrontAction"));
	        resetViewportMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Top"),"ResetViewportTopAction"));
	        contribution.add(resetViewportMenu);
	        
	        contribution.add(new MenuNode(MenuNode.Type.ExtensionPoint,"viewMenuExtensionPoint",""));
	        mainWindowLayout.addMenuContribution(contribution);
	        
	        // register tools:
	        deleteOGNodeTool=new DeleteOGNodeTool();
	        ToolChestView.addTool(deleteOGNodeTool);
	        truncateHistoryTool=new TruncateHistoryTool();
	        ToolChestView.addTool(truncateHistoryTool);
	        
	        // register dock widget factories:
	        uiManager.registerDockWidgetFactory(new DockWidgetFactory("LoggerView",L10N.translate("Log"),LoggerView.class));
	        uiManager.registerDockWidgetFactory(new DockWidgetFactory("TaskListView",L10N.translate("Background tasks"),TaskListView.class));
	        uiManager.registerDockWidgetFactory(new DockWidgetFactory("PropertyEditorView",L10N.translate("Property editor"),PropertyEditorView.class));
	        uiManager.registerDockWidgetFactory(new DockWidgetFactory("OutlineView",L10N.translate("Outline"),OutlineView.class));
	        uiManager.registerDockWidgetFactory(new DockWidgetFactory("ToolChestView",L10N.translate("Toolchest"),ToolChestView.class));
	        
	        mainWindowLayout.addDockWidget("LoggerView");
	        mainWindowLayout.addDockWidget("TaskListView");
	        mainWindowLayout.addDockWidget("PropertyEditorView");
	        mainWindowLayout.addDockWidget("OutlineView");
	        mainWindowLayout.addDockWidget("ToolChestView");
	        
	        // other initialisation:
	        
	        // force generation of OGObserver singleton for 3D view:
	        OpenGLConverter converter=OpenGLConverter.getInstance();
	        converter.registerSGNodeConverter(new InstanceConverter());
	        // set standard view event handler for 3D view:
	        ViewEventHandler.setDefaultEventHandler(new StandardViewEventHandler());
	        ViewEventHandler.setEventHandler(eu.moonlight3d.ml3d.ui.mainwindow.view.ViewEventHandler.getDefaultEventHandler());
        
        	lifecycleListener=new LifecycleListener();
        	state.getDocumentManager().addLifecycleListener(lifecycleListener);
        	
        	ogContributionFactory = new DocumentContributionFactory();
        	DocumentFactory factory=(DocumentFactory) state.getDocumentManager().getDocumentFactory("MoonlightScene");
        	factory.registerDocumentContributionFactory(ogContributionFactory);
        }
	}

	public void unload() throws Exception {
		State state=State.getInstance();

        if(!state.getBatchMode()) {
        	lifecycleListener.unregisterAll();
        	state.getDocumentManager().removeLifecycleListener(lifecycleListener);

        	DocumentFactory factory=(DocumentFactory) state.getDocumentManager().getDocumentFactory("MoonlightScene");
        	factory.unregisterDocumentContributionFactory(ogContributionFactory);
        	
        	UIManager uiManager=state.getUIManager();

        	// unregister views:
        	uiManager.unregisterMainWidgetFactory("mainwindow");

        	// unregister operator graph actions:
        	uiManager.unregisterAction("DeleteOGNodeAction");
        	uiManager.unregisterAction("TruncateHistoryAction");
        	uiManager.unregisterAction("ArrangeGraphAction");

        	// unregister 3D view actions
        	uiManager.unregisterAction("ViewSetRenderModeNoneAction");
        	uiManager.unregisterAction("ViewSetRenderModeWireframeAction");
        	uiManager.unregisterAction("ViewSetRenderModeShadedAction");
        	uiManager.unregisterAction("ViewSetRenderModeLitAction");
        	uiManager.unregisterAction("ResetViewportAction");
        	uiManager.unregisterAction("SetOnePaneViewAction");
        	uiManager.unregisterAction("SetFourPaneViewAction");

        	// unregister tools:
        	ToolChestView.removeTool(deleteOGNodeTool);
        	ToolChestView.removeTool(truncateHistoryTool);
        }
	}

}
