/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 24, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.logger;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSpinBox;
import com.trolltech.qt.gui.QStandardItemModel;
import com.trolltech.qt.gui.QTableView;
import com.trolltech.qt.gui.QVBoxLayout;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.DockWidget;

/**
 * The LoggerView provides a view for looking at the log messages generated by
 * the application. The log messages are put into a table.
 * 
 * @author riishigh
 */
public class LoggerView extends DockWidget {

	private LoggerViewHandler mHandler;

	private int mMaxMessages = 100;

	private QStandardItemModel mLogMessageTable;

	private QTableView mLogTableViewer;

	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");

	static final String[] columnNames = { "Type", "Message", "Time", "Class" };

	Level[] levels = { Level.ALL, Level.FINE, Level.FINER, Level.FINEST, Level.CONFIG, Level.INFO, Level.WARNING, Level.SEVERE, Level.OFF };

	private QComboBox mLevelCombo;

	private QSpinBox mHistorySpinner;

	/**
	 * Accessor for the level set in the State. This should be the same as the
	 * level set in the CCombo
	 * 
	 * @return
	 */
	public Level logLevel() {
		if (mLevelCombo != null && 
				!mLevelCombo.itemText(mLevelCombo.currentIndex()).equals(logger().getLevel().toString())) {
			// Whoops the ui isn't reflecting the state let's change that.
			updateToCurrentLogLevel();
		}
		return logger().getLevel();
	}

	public void updateToCurrentLogLevel() {

		if (mLevelCombo != null) {
			// Figure out the selected level
			int j = 0;
			int selectIndex = 0;
			for (Level l : levels) {
				if (logger().getLevel().equals(l)) {
					selectIndex = j;
				}
				++j;
			}
			mLevelCombo.setCurrentIndex(selectIndex);
		}
	}

	public int historySize() {
		return mMaxMessages;
	}

	/**
	 * This method allows the user to set the log level using this method. The
	 * method is supposed to be exposed in a MBean
	 * 
	 * @param aLevel
	 */
	public void logLevel(Level aLevel) {
		logger().setLevel(aLevel);
		updateToCurrentLogLevel();
	}

	public LoggerView() {
		super("Log");
		
		setDefaultArea(Qt.DockWidgetArea.BottomDockWidgetArea);
		
		mHandler = new LoggerViewHandler();
		logger().addHandler(mHandler);

		// for( Handler h : State.getInstance().logger().getHandlers() ) {
		// State.getInstance().logger().info(h.getClass().getName());
		// }
		// Group toolsGroup = new Group(this, SWT.BORDER);

		QLabel levelLabel = new QLabel(this);
		levelLabel.setText("Level");

		mLevelCombo = new QComboBox(this);
		mLevelCombo.setEditable(false);
		if (logger().getLevel() == null) {
			logger().setLevel(Level.INFO);
		}
		// Figure out the selected level essentially the same code as in
		// updateToCurrentLogLevel above.
		int j = 0;
		int selectIndex = 0;
		for (Level l : levels) {
			mLevelCombo.addItem(l.getName());
			if (logger().getLevel().equals(l)) {
				selectIndex = j;
			}
			++j;
		}
		mLevelCombo.setCurrentIndex(selectIndex);
		mLevelCombo.currentIndexChanged.connect(this,"levelComboChanged()");

		QLabel historyLabel = new QLabel(this);
		historyLabel.setText("history");

		mHistorySpinner = new QSpinBox(this);
		mHistorySpinner.setEnabled(true);
		mHistorySpinner.setMaximum(Integer.MAX_VALUE);
		mHistorySpinner.setValue(mMaxMessages);
		mHistorySpinner.valueChanged.connect(this,"historySpinnerChanged()");

		QPushButton clearButton = new QPushButton(this);
		clearButton.setText("clear");
		clearButton.clicked.connect(this,"clear()");

		mLogTableViewer = new QTableView(this);
		mLogMessageTable=new QStandardItemModel(0,4);
		initialiseModel();
		mLogTableViewer.setModel(mLogMessageTable);

		QVBoxLayout layout=new QVBoxLayout();
		QHBoxLayout topLayout=new QHBoxLayout();
		
		topLayout.addWidget(levelLabel);
		topLayout.addWidget(mLevelCombo);
		topLayout.addWidget(historyLabel);
		topLayout.addWidget(mHistorySpinner);
		topLayout.addWidget(clearButton);
		topLayout.addStretch(1);
		
		layout.addLayout(topLayout);
		layout.addWidget(mLogTableViewer,1);
		layout.setMargin(0);
		
		setLayout(layout);
		
		logger().info("Logging messages will appear here");
	}

	private void initialiseModel() {
		mLogMessageTable.clear();
		mLogMessageTable.insertColumns(0, 4);
		mLogMessageTable.setHeaderData(0, Qt.Orientation.Horizontal, "Level",Qt.ItemDataRole.DisplayRole);
		mLogMessageTable.setHeaderData(1, Qt.Orientation.Horizontal, "Message",Qt.ItemDataRole.DisplayRole);
		mLogMessageTable.setHeaderData(2, Qt.Orientation.Horizontal, "Date",Qt.ItemDataRole.DisplayRole);
		mLogMessageTable.setHeaderData(3, Qt.Orientation.Horizontal, "Source",Qt.ItemDataRole.DisplayRole);
		mLogMessageTable.layoutChanged.emit();				
	}
	
	private Logger logger() {
		return State.getInstance().logger();
	}
	
	public void levelComboChanged() {
		Level newLevel = Level.parse(mLevelCombo.itemText(mLevelCombo.currentIndex()));
		logger().config("Log level set to : [" + newLevel + "]");
		logger().setLevel(newLevel);		
	}
	
	public void historySpinnerChanged() {
		mMaxMessages = mHistorySpinner.value();
	}
	
	public void clear() {
		initialiseModel();
		mLogMessageTable.layoutChanged.emit();		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Widget#dispose()
	 */
	@Override
	public void disposed() {
		logger().removeHandler(mHandler);
	}

	public class LoggerViewHandler extends Handler {

		
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.util.logging.Handler#publish(java.util.logging.LogRecord)
		 */
		@Override
		public void publish(final LogRecord aRecord) {
			QApplication.invokeLater(new Runnable() {

				public void run() {
					mLogMessageTable.insertRow(mLogMessageTable.rowCount());
					int rowIndex=mLogMessageTable.rowCount()-1;
					String data;
					data = aRecord.getLevel().toString();
					mLogMessageTable.setData(mLogMessageTable.index(rowIndex,0),data,Qt.ItemDataRole.DisplayRole);
					data = aRecord.getMessage();
					mLogMessageTable.setData(mLogMessageTable.index(rowIndex,1),data,Qt.ItemDataRole.DisplayRole);
					data = sdf.format(new Date(aRecord.getMillis()));
					mLogMessageTable.setData(mLogMessageTable.index(rowIndex,2),data,Qt.ItemDataRole.DisplayRole);
					data = aRecord.getSourceClassName() + "." + aRecord.getSourceMethodName();
					mLogMessageTable.setData(mLogMessageTable.index(rowIndex,3),data,Qt.ItemDataRole.DisplayRole);
					
					while(mLogMessageTable.rowCount() > mMaxMessages) {
						mLogMessageTable.removeRow(0);
					}
//					mLogMessageTable.layoutChanged.emit();
					mLogTableViewer.resizeColumnsToContents();
					mLogTableViewer.scrollTo(mLogMessageTable.index(rowIndex, 0));
					
				}
				
			});
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.util.logging.Handler#flush()
		 */
		@Override
		public void flush() {
			// flush shouln't do anything here.
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.util.logging.Handler#close()
		 */
		@Override
		public void close() throws SecurityException {
			mLogMessageTable.clear();
		}
	}

}
