/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 3, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.modes;

import java.util.ArrayList;

public class ModeManager {
	private static ModeManager instance;
	
	public static ModeManager getInstance() {
		if(instance==null) {
			instance=new ModeManager();
		}
		return instance;
	}
	
	private ArrayList<Mode> modes;
	private Mode currentMode;
	private Mode defaultMode;
	private ArrayList<ModeSwitchListener> modeListeners;
	
	private ModeManager() {
		modes=new ArrayList<Mode>();
		modeListeners=new ArrayList<ModeSwitchListener>();
	}
	
	public void addMode(Mode mode) {
		modes.add(mode);
	}
	
	public void removeMode(Mode mode) {
		modes.remove(mode);
	}
	
	public void addModeListener(ModeSwitchListener listener) {
		modeListeners.add(listener);
	}
	
	public void removeModeListener(ModeSwitchListener listener) {
		modeListeners.remove(listener);
	}
	
	/** 
	 * Return the currently active mode or null if there is none.
	 */ 
	public Mode getCurrentMode() {
		return currentMode;
	}
	
	public Mode getModeByName(String modeName) {
		for(Mode mode : modes) {
			if(mode.getName()==null) {
				continue;
			}
			if(mode.getName().equals(modeName)) {
				return mode;
			}
		}
		return null;
	}

	public Mode getModeByOGNodeName(String modeName) {
		for(Mode mode : modes) {
			if(mode.getOGNodeName()==null) {
				continue;
			}
			if(mode.getOGNodeName().equals(modeName)) {
				return mode;
			}
		}
		return null;
	}
	
	/**
	 * Switch the user interface to a different mode.
	 * 
	 * @param mode the newe mode or null for default mode
	 */
	public void setMode(Mode mode) {
		if(currentMode==mode) {
			return;
		}
		Mode oldMode=currentMode;
		if(currentMode!=null) {
			currentMode.deactivate();
		}
		if(mode==null) {
			mode=defaultMode;
		}
		currentMode=mode;
		mode.activate();
		for(ModeSwitchListener listener : modeListeners) {
			listener.modeSwitched(oldMode,currentMode);
		}
	}
	
	public void setMode(String modeName) {
		setMode(getModeByName(modeName));
	}

	public void setModeByOGNodeName(String modeName) {
		setMode(getModeByOGNodeName(modeName));
	}
	
	/**
	 * Set the default mode. This mode will be activated when
	 * no other mode is present.
	 * 
	 * @param mode new default mode
	 */
	public void setDefaultMode(Mode mode) {
		if(currentMode==null || currentMode==defaultMode) {
			setMode(mode);
		}
		defaultMode=mode;
	}
	
	/**
	 * Return the current default mode.
	 * 
	 * @return default mode
	 */
	public Mode getDefaultMode() {
		return defaultMode;
	}
}
