/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 30, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.modes;

import java.util.List;

import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;


public class SelectionListener implements eu.moonlight3d.ml3d.backend.selection.SelectionListener {
	private Selection selection;
	private String currentModeNode; // OG node the current mode is running on
	
	public SelectionListener(Selection selection) {
		this.selection=selection;
	}
	
	public void elementAdded(String objectName) {
		if(selection.getType().getName().equals("OGNode")) {
			// find corresponding mode and activate
			try {
				Node ogNode=(Node) selection.getObjectFromSelectionString(objectName);
				ModeManager manager=ModeManager.getInstance();
				manager.setModeByOGNodeName(ogNode.getType());
				currentModeNode=objectName;
			} catch (Exception e) {
				ExceptionDialog.run(e);
			}
		}
	}

	public void elementsAdded(List<String> objectNames) {
		if(selection.getType().getName().equals("OGNode")) {
			// find corresponding mode and activate
			try {
				Node ogNode=(Node) selection.getObjectFromSelectionString(selection.getActiveObject());
				ModeManager manager=ModeManager.getInstance();
				manager.setModeByOGNodeName(ogNode.getType());
				currentModeNode=selection.getActiveObject();
			} catch (Exception e) {
				ExceptionDialog.run(e);
			}
		}
	}

	public void elementRemoved(String objectName) {
		if(selection.getType().getName().equals("OGNode")) {
			// check if current mode is associated and deactivate
			if(currentModeNode.equals(objectName)) {
				ModeManager manager=ModeManager.getInstance();
				manager.setMode(manager.getDefaultMode());
			}
		}
	}

	public void elementUpdated(String oldName, String newName) {
		if(selection.getType().getName().equals("OGNode")) {
			// check if current mode is associated and replace
			if(currentModeNode.equals(oldName)) {
				try {
					Node ogNode=(Node) selection.getObjectFromSelectionString(newName);
					ModeManager manager=ModeManager.getInstance();
					manager.setModeByOGNodeName(ogNode.getType());
					currentModeNode=newName;				
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		}
	}

	public void selectionCleared() {
		if(selection.getType()==null) {
			return;
		}
		if(selection.getType().getName().equals("OGNode")) {
			// fall back to default mode (is this the right thing to do?)
			ModeManager manager=ModeManager.getInstance();
			manager.setMode(manager.getDefaultMode());
		}
	}

}
