/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 20, 2007
 */
package eu.moonlight3d.ml3d.ui.mainwindow.ogeditor;


import java.text.MessageFormat;
import java.util.ArrayList;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Manager;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.ui.mainwindow.toolchest.Tool;

public class DeleteOGNodeTool extends Tool {
	public DeleteOGNodeTool() {
		setName("DeleteOGNodeTool");
		setDisplayName("Delete OG Nodes");
		setCategory("Edit/Operator Graph");
	}
	
	private void deleteDrainNode(Manager ogManager, Node node) throws Exception {
		State state=State.getInstance();
		
		for(Slot slot : node.getSlots()) {
			if(slot.getType()==Slot.Type.Input) {
				if(slot.getCurrentLink()!=null) {
					slot.getCurrentLink().getInput().setCurrentLink(null);
					((Document) state.getDocumentManager().getMainDocument()).getOGManager().removeLink(slot.getCurrentLink());
					slot.setCurrentLink(null);
				}
			}
		}
		ogManager.removeNode(node);
	}
	
	private boolean deleteNonDrainNode(Manager ogManager, Node node) throws Exception {
		State state=State.getInstance();

		int numInputSlots=0;
		int numOutputSlots=0;
		Slot inputSlot=null;
		Slot outputSlot=null;
		
		for(Slot slot : node.getSlots()) {
			if(slot.getType()==Slot.Type.Input) {
				if(slot.getCurrentLink()!=null) {
					numInputSlots++;
					inputSlot=slot;
				} else {
					return false;
				}
			} else if(slot.getType()==Slot.Type.Output) {
				if(slot.getCurrentLink()!=null) {
					numOutputSlots++;
					outputSlot=slot;
				} else {
					return false;
				}
			}
		}
		
		if(numInputSlots!=1 || numOutputSlots!=1) {
			return false;
		}
		
		Slot otherOutput=inputSlot.getCurrentLink().getInput();
		Slot otherInput=outputSlot.getCurrentLink().getOutput();
		
		ogManager.removeLink(inputSlot.getCurrentLink());
		ogManager.removeLink(outputSlot.getCurrentLink());
		ogManager.addLink(otherOutput, otherInput);
		ogManager.removeNode(node);
		outputSlot.getNode().setHasUpdatedProperties();
		
		return true;
	}
	
	@Override
	public void activated() {
		try {
			State state=State.getInstance();
			Manager ogManager=((Document) state.getDocumentManager().getMainDocument()).getOGManager();
			Selection selection=((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			if(selection.getType()!=eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("OGNode")) {
				state.logger().warning(L10N.translate("Delete operator graph node tool only works on operator graphs"));
				return;
			}
			
			if(selection.getActiveObject()==null) {
				state.logger().warning(L10N.translate("no active operator graph node"));
				return;
			}
						
			ArrayList<String> selectedObjects=selection.getSelection();
			if(selectedObjects.size()==0) {
				state.logger().warning(L10N.translate("no operator graph nodes selected"));
				return;				
			}
			
			boolean deletedNodes=false;
			do {
				deletedNodes=false;
				ArrayList<String> deletedObjects=new ArrayList<String>();
				for(String selectedObject : selectedObjects) {
					Node node=(Node) selection.getObjectFromSelectionString(selectedObject);
			
					selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("OGNode"));

					if(!node.isDrain()) {
						if(deleteNonDrainNode(ogManager, node)) {
							deletedNodes=true;
							deletedObjects.add(selectedObject);
						}
					} else {
						deleteDrainNode(ogManager, node);
						deletedNodes=true;
						deletedObjects.add(selectedObject);
					}
				}
				for(String deletedObject : deletedObjects) {
					selectedObjects.remove(deletedObject);
				}
			} while(selectedObjects.size()!=0 && deletedNodes);
			
			if(selectedObjects.size()!=0) {
				String translation=L10N.translate("could not delete {0} selected nodes, possibly because they have more than one input or output slot and are connected");
				state.logger().warning(MessageFormat.format(translation,selectedObjects.size()));
			}

			ogManager.update();
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}

	@Override
	public void deactivated() {
	}

	@Override
	public QWidget getOptionEditor(QWidget parent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasOptionEditor() {
		// TODO Auto-generated method stub
		return false;
	}

}
