/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 16, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.ogeditor;

import java.util.ArrayList;
import java.util.List;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QTreeWidgetItem;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.exceptions.ExceptionHelper;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.graph.GraphModel;
import eu.moonlight3d.graph.GraphModelListener;
import eu.moonlight3d.graph.Link;
import eu.moonlight3d.graph.Node;
import eu.moonlight3d.graph.layout.SimpleLayouter;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;
import eu.moonlight3d.ml3d.backend.threedim.sg.iterator.TreeIterator;

public class ModelGenerator implements GraphModelListener {
	private static ModelGenerator instance=null;

	private GraphModel graphModel;

	public static ModelGenerator getInstance() {
		if(instance==null) {
			instance=new ModelGenerator();
		}
		return instance;
	}

	public ModelGenerator() {
		graphModel=new GraphModel();
		graphModel.registerListener(this);
		graphModel.setLayouter(new SimpleLayouter());
	}
	
	public GraphModel getGraphModel() {
		return graphModel;
	}
	
	public void updateSelection() {
		try {
			Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			if(selection.getType()==null) {
				return;
			}
			if(selection.getType().getName()!="OGNode") {
				return;
			}
			
			ArrayList<String> nodeNames=new ArrayList<String>();
			for(String selectionName : selection.getSelection()) {
				nodeNames.add(selectionName.split("/")[1]);
			}
			
			for(eu.moonlight3d.graph.Node node : graphModel.getNodes()) {
				if(nodeNames.contains(node.getName())) {
					node.setActive(true);
				} else {
					node.setActive(false);
				}
			}
		} catch (Exception e) {
			// TODO handle exception more appropriately
			e.printStackTrace();
		}
	}
	
	void addNode(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode) {
		Node node=new Node(ogNode.getName());
		for(eu.moonlight3d.ml3d.backend.threedim.og.Slot ogSlot : ogNode.getSlots()) {
			eu.moonlight3d.graph.Slot.Type type;
			if(ogSlot.getType()==eu.moonlight3d.ml3d.backend.threedim.og.Slot.Type.Input) {
				type=eu.moonlight3d.graph.Slot.Type.Input;
			} else {
				type=eu.moonlight3d.graph.Slot.Type.Output;					
			}
			eu.moonlight3d.graph.Slot slot=new eu.moonlight3d.graph.Slot(node,ogSlot.getName(),type);
			node.addSlot(slot);
		}
		graphModel.addNode(node);		
	}
	
	private void addNodes(Document document) {
		List<eu.moonlight3d.ml3d.backend.threedim.og.Node> ogNodes;
		
		ogNodes=document.getOGManager().getNodeList();
		for(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode : ogNodes) {
			addNode(ogNode);
		}
	}
	
	void addLink(eu.moonlight3d.ml3d.backend.threedim.og.Link ogLink) {
		eu.moonlight3d.ml3d.backend.threedim.og.Slot inputSlot, outputSlot;
		eu.moonlight3d.graph.Node startNode, endNode;
		eu.moonlight3d.graph.Slot startSlot, endSlot;
		String nodeName;
		
		inputSlot=ogLink.getInput();
		outputSlot=ogLink.getOutput();
		nodeName=inputSlot.getNode().getName();
		startNode=graphModel.getNode(nodeName);
		nodeName=outputSlot.getNode().getName();
		endNode=graphModel.getNode(nodeName);
		
		startSlot=startNode.getSlot(inputSlot.getName());
		endSlot=endNode.getSlot(outputSlot.getName());
		
		graphModel.addLink(startSlot,endSlot);		
	}
	
	private void addLinks(Document document) {
		ArrayList<eu.moonlight3d.ml3d.backend.threedim.og.Link> ogLinks;
		
		ogLinks=document.getOGManager().getLinkList();
		for(eu.moonlight3d.ml3d.backend.threedim.og.Link ogLink : ogLinks) {
			addLink(ogLink);
		}
	}
	
	private Node findGraphNode(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode) {
		for(Node node : graphModel.getNodes()) {
			if(node.getName().equals(ogNode.getName())) {
				return node;
			}
		}
		return null;
	}
	
	void removeLink(eu.moonlight3d.ml3d.backend.threedim.og.Link ogLink) {
		eu.moonlight3d.ml3d.backend.threedim.og.Node startOGNode;
		
		startOGNode=ogLink.getInput().getNode();
		
		Node startNode=findGraphNode(startOGNode);
		
		Link link=startNode.getSlot(ogLink.getInput().getName()).getLink();
		graphModel.removeLink(link);
	}
	
	void removeNode(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode) {
		Node node=findGraphNode(ogNode);
		if(node!=null) {
			graphModel.removeNode(node);
		}
	}
	
	public void updateGraph(Document document) {
		graphModel.clearGraph();
		addNodes(document);
		addLinks(document);
		graphModel.recalculateLayout();
	}
	
	public void updateGraph() {
		Document document=((Document) State.getInstance().getDocumentManager().getMainDocument());
		
		graphModel.clearGraph();
		addNodes(document);
		addLinks(document);
		graphModel.recalculateLayout();
	}
	
	public void clearGraph() {
		graphModel.clearGraph();
		graphModel.recalculateLayout();
	}

	public void handleSelection(GraphModel model, Node node, boolean selected) {
		try {
			Selection selection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
//			selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("OGNode"));
			if(selected) {
				if(selection.getType()== null || !selection.getType().getName().equals("OGNode")) {
					selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("OGNode"));					
				}
				selection.add("OperatorGraph/" + node.getName());
			} else {
				if(!selection.getType().getName().equals("OGNode")) {
					return;
				}
				selection.remove("OperatorGraph/" + node.getName());				
			}
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
		
	}

	private String populateTree(int level, eu.moonlight3d.ml3d.backend.threedim.sg.Node node) {
		String result="";
		
		for(int i=0;i<level-1;i++) {
			result+="&nbsp;&nbsp;&nbsp;&nbsp;";
		}
		if(level>=1) {
			result+="&nbsp;+-&nbsp;";
		}
		if(node.getName()!=null) {
			result+=node.getName();
		} else {
			// unnamed objects aren't supposed to happen; warn here
			State.getInstance().logger().warning("operator graph: discovered unnamed node");
			result+="(unnamed)";
		}
		result+="<br>";
		
		for(eu.moonlight3d.ml3d.backend.threedim.sg.Node child : node.getChildren()) {
			result+=populateTree(level+1,child);
		}
		
		return result;
	}

	public void markUpdateOK(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode) {
		Node node=graphModel.getNode(ogNode.getName());
		
		node.setNormalColor(null);
		node.setSelectedColor(null);
		
		String toolTip="<b>" + L10N.translate("Node output:") + "</b><br><br>";
		for(eu.moonlight3d.ml3d.backend.threedim.og.Slot slot : ogNode.getSlots()) {
			if(slot.getType()==Slot.Type.Output) {
				toolTip+="<u>" + L10N.translate("Slot") + " <b><i>" + slot.getName() + "</i></b></u>:<br>";
				try {
					SceneGraph sceneGraph=slot.getGraph();
					for(eu.moonlight3d.ml3d.backend.threedim.sg.Node rootNode : sceneGraph.getRootNodes()) {
						toolTip+=populateTree(0,rootNode);
					}
				} catch (Exception e) {
					ExceptionDialog.run(e);
				}
			}
		}
		
		node.setToolTip(toolTip);
	}

	public void markUpdateFailure(eu.moonlight3d.ml3d.backend.threedim.og.Node ogNode, Exception exception) {
		Node node=graphModel.getNode(ogNode.getName());
		
		node.setNormalColor(QColor.red);
		node.setSelectedColor(QColor.darkMagenta);
		// generate tooltip
		if(exception!=null) {
			String toolTip;
			toolTip=L10N.translate("<b>Node update failed.</b>");
			toolTip+="<br>";
			toolTip+=L10N.translate("An exception occurred during node update:<br>");
			toolTip+="<p style='white-space:pre'>";
			toolTip+=ExceptionHelper.getReasons(exception);
			toolTip+="</p>";
			node.setToolTip(toolTip);
		} else {
			node.setToolTip(L10N.translate("<b>Node update failed.</b><br>Check input nodes for failures."));
		}
	}
}
