/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 20, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.outline;

import java.util.ArrayList;
import java.util.List;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAbstractItemView;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QTreeWidget;
import com.trolltech.qt.gui.QTreeWidgetItem;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.DockWidget;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.og.DrainSlotIterator;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.Node;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;

public class OutlineView extends DockWidget {
	private QTreeWidget outlineTree;
	private boolean internalSelectionChange;
	
	private static OutlineView instance=null;
	
	public OutlineView() {
		super("Outline");
		
		setDefaultArea(Qt.DockWidgetArea.RightDockWidgetArea);
		
		instance=this;
		
		outlineTree=new QTreeWidget(this);
		outlineTree.setColumnCount(1);
		outlineTree.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection);
		outlineTree.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows);
		List<String> headerLabels=new ArrayList<String>();
		headerLabels.add("Objects");
		outlineTree.setHeaderLabels(headerLabels);

		QHBoxLayout fillLayout=new QHBoxLayout(this);
		fillLayout.setMargin(1);
		fillLayout.setSpacing(1);
		fillLayout.addWidget(outlineTree,1);
		setLayout(fillLayout);
	
		outlineTree.itemSelectionChanged.connect(this,"updateSelection()");
		internalSelectionChange=false;
	}
	
	@SuppressWarnings("unused") // this is a slot
	private void updateSelection() {
		if(internalSelectionChange) {
			return;
		}
		
		List<QTreeWidgetItem> selection=outlineTree.selectedItems();
		Selection viewSelection;
		
		try {
			viewSelection=((Document) State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			viewSelection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType("SGNode"));

			for(int i=0;i<selection.size();i++) {
				viewSelection.add((String)selection.get(i).data(0,Qt.ItemDataRole.UserRole));
			}
		} catch (Exception e1) {
			// this should not happen here - but cancel operation just in case
			e1.printStackTrace();
			return;
		}
	}

	private QTreeWidgetItem findItem(String itemName) {
		ArrayList<QTreeWidgetItem> items=new ArrayList<QTreeWidgetItem>();
		String[] pathElements=itemName.split(":")[0].split("/");
		if(pathElements.length<4) {
			return null;
		}
		String currentPath=pathElements[0] + "/" + pathElements[1] + "/" + pathElements[2] + "/" + pathElements[3];

		int level=4;
		for(int i=0;i<outlineTree.topLevelItemCount();i++) {
			items.add(outlineTree.topLevelItem(i));
		}
		
		while(level<=pathElements.length) {
			for(int i=0;i<items.size();i++) {
				if(items.get(i).data(0,Qt.ItemDataRole.UserRole).equals(currentPath)) {
					if(level==pathElements.length) {
						return items.get(i);
					}

					QTreeWidgetItem item=items.get(i);
					items.clear();
					for(int j=0;j<item.childCount();j++) {
						items.add(item.child(j));
					}
					currentPath=currentPath + "/" + pathElements[level];
					level++;
					break;
				}
			}
			return null;
		}
		
		return null;
	}
	
	public void updateTreeSelection() {
		Selection viewSelection;
		ArrayList<QTreeWidgetItem> treeItems=new ArrayList<QTreeWidgetItem>();
		
		internalSelectionChange=true;
		outlineTree.clearSelection();
		try {
			viewSelection=((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
			
			for(String selectedName : viewSelection.getSelection()) {
				QTreeWidgetItem item=findItem(selectedName);
				if(item!=null) {
					if(!treeItems.contains(item)) {
						treeItems.add(item);
						item.setSelected(true);
//						outlineTree.setItemSelected(item, true);
					}
				}
			}
			internalSelectionChange=false;
		} catch (Exception e1) {
			// this should not happen here - but cancel operation just in case
			e1.printStackTrace();
			internalSelectionChange=false;
			return;
		}
	}

	private void populateTree(QTreeWidgetItem parent, String parentName, Node node) {
		QTreeWidgetItem item;
		
		if(parent==null) {
			item=new QTreeWidgetItem(outlineTree);
		} else {
			item=new QTreeWidgetItem(parent);			
		}
		
		if(node.getName()!=null) {
			item.setText(0,node.getName());
		} else {
			// unnamed objects aren't supposed to happen; warn here
			State.getInstance().logger().warning("outline: adding unnamed node");
			item.setText(0,"(unnamed)");
		}
		item.setData(0,Qt.ItemDataRole.UserRole,parentName + "/" + node.getName());
		
		for(Node child : node.getChildren()) {
			populateTree(item,parentName  + "/" + node.getName(),child);
		}
	}

	public void repopulate() {
		DrainSlotIterator drainSlotIterator=new DrainSlotIterator(((Document) State.getInstance().getDocumentManager().getMainDocument()).getOGManager());
		
		internalSelectionChange=true;
		outlineTree.clear();
		
		while(drainSlotIterator.hasNext()) {
			Slot slot=drainSlotIterator.next();
			
			try {
				if(slot.getGraph()==null) {
					continue;
				}
				
				SceneGraph sceneGraph=slot.getGraph();
				for(Node rootNode : sceneGraph.getRootNodes()) {
					String slotName="OperatorGraph/" + slot.getNode().getName() + "/" + slot.getName();
				
					if(rootNode!=null) {
						// ignore top-level root node here
//						for(Node child : rootNode.getChildren()) {
//							populateTree(null,slotName,child);
//						}
						populateTree(null,slotName,rootNode);
					}
				}
				
			} catch (Exception e) {
				// this should not happen
				e.printStackTrace();
			}
		}
		updateTreeSelection();
		internalSelectionChange=false;
	}
	
	public void clear() {
		internalSelectionChange=true;
		outlineTree.clear();
		internalSelectionChange=false;
	}
	
	public static OutlineView getInstance() {
		return instance;
	}
	
	@Override
	public void disposed() {
		instance=null;
	}
}
