/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Apr 5, 2008
 */

package eu.moonlight3d.ml3d.ui.mainwindow.propertyeditor;

import java.util.ArrayList;
import java.util.List;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.selection.SelectionListener;
import eu.moonlight3d.ml3d.backend.threedim.anim.Animation;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;

public class AnimationSelectionListener implements SelectionListener {
	/**
	 * The animation listener responsible for triggering keyframe button updates
	 */
	private AnimationListener animationListener;
	/**
	 * The animation that the animation listener is currently assigned to
	 */
	private Animation assignedAnimation;
	
	public AnimationSelectionListener() {
		animationListener=new AnimationListener();
	}
	
	private void unregisterAnimationListener() {
		if(assignedAnimation!=null) {
			assignedAnimation.unregisterAnimationListener(animationListener);
			assignedAnimation=null;
		}
	}
	
	private void reregisterAnimationListener() {
		unregisterAnimationListener();

		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();

		try {
			Selection animationSelection=document.getSelectionManager().getSelection("animationSelection");
			if(animationSelection.getActiveObject()!=null) {
				assignedAnimation=(Animation) animationSelection.getObjectFromSelectionString(animationSelection.getActiveObject());
			} else {
				assignedAnimation=null;
				return;
			}
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
		assignedAnimation.registerAnimationListener(animationListener);
	}
	
	private void updatePropertyEditors() {
		Animation newAnimation=null;
		
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		
		try {
			Selection animationSelection=document.getSelectionManager().getSelection("animationSelection");
			if(animationSelection.getActiveObject()!=null) {
				newAnimation=(Animation) animationSelection.getObjectFromSelectionString(animationSelection.getActiveObject());
			}
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}

		ArrayList<PropertyEditorView> propertyEditors=PropertyEditorView.getPropertyEditors();
        for(int i=0;i<propertyEditors.size();i++) {
            propertyEditors.get(i).handleAnimationChange(newAnimation);
        }		
	}
	
	public void elementAdded(String objectName) {
		// TODO remove animation listener on old element, add to new element
		updatePropertyEditors();
		reregisterAnimationListener();
	}

	public void elementRemoved(String objectName) {
		// TODO remove animation listener from old element
		updatePropertyEditors();
		reregisterAnimationListener();
	}

	public void elementUpdated(String oldName, String newName) {
		// TODO remove animation listener on old element, add to new element
		updatePropertyEditors();
		reregisterAnimationListener();
	}

	public void elementsAdded(List<String> objectNames) {
		// TODO remove animation listener on old element, add to laset new element
		updatePropertyEditors();
		reregisterAnimationListener();
	}

	public void selectionCleared() {
		// TODO remove animation listener on old element
		updatePropertyEditors();
		unregisterAnimationListener();
	}

}
