/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 24, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.propertyeditor;


import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.common.ui.propertysheet.ValueEditor;
import eu.moonlight3d.framework.helper.Property;

public abstract class KeyframeEditor extends ValueEditor {
	PropertySheet propertySheet;
    Property property;
    QLineEdit entry;
    KeyframeButton keyframeButton;

    KeyframeEditor(PropertySheet sheet, QWidget parent, final Property property, String labelText)
    {
    	super(parent);
    	propertySheet=sheet;
        this.property=property;
        
        QLabel label;
        label=new QLabel(this);
        if(labelText!=null) {
        	label.setText(labelText);
        } else {
        	label.setText(property.getName());
        }

        entry=new QLineEdit(this);
        entry.setText(getPropertyValue());
        entry.textEdited.connect(this,"textEdited(String)");
        entry.editingFinished.connect(this,"editingFinished()");
        
        keyframeButton=new KeyframeButton(this,sheet,property);        
        
        QHBoxLayout layout=new QHBoxLayout();
        layout.setMargin(0);
        layout.setWidgetSpacing(2);
        layout.addWidget(label);
        layout.addStretch(1);
        layout.addWidget(entry);
        layout.addWidget(keyframeButton);
        layout.setStretchFactor(keyframeButton, 0);
        setLayout(layout);
    }
    
    public void textEdited(String text) {    	
        setPropertyValue(entry.text());
    	propertySheet.notifyPropertyUpdate(property);
    }
    
    public void editingFinished() {
        setPropertyValue(entry.text());
    	propertySheet.notifyPropertyUpdate(property);    	
    }
    
    @Override
	public void updateFromPropertyValue() {
   		entry.setText(getPropertyValue());
    }
    
    public abstract String getPropertyValue();
    public abstract void setPropertyValue(String value);
}
