/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 24, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.tasklist;

import java.util.ArrayList;
import java.util.List;


import com.trolltech.qt.core.QCoreApplication;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QTreeWidget;
import com.trolltech.qt.gui.QTreeWidgetItem;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Observed;
import eu.moonlight3d.framework.helper.Observer;
import eu.moonlight3d.framework.tasks.Task;
import eu.moonlight3d.framework.tasks.TaskManager;
import eu.moonlight3d.framework.ui.DockWidget;

/**
 * The LoggerView provides a view for looking at the log messages generated by
 * the application. The log messages are put into a table.
 * 
 * @author riishigh
 */
public class TaskListView extends DockWidget implements Observer {

	private QTreeWidget taskTree;

	public TaskListView() {
		super("Background tasks");
		
		setDefaultArea(Qt.DockWidgetArea.BottomDockWidgetArea);

		State.getInstance().getTaskManager().registerObserver(this);

		taskTree = new QTreeWidget(this);
		taskTree.setColumnCount(2);
		List<String> columnNames = new ArrayList<String>();
		columnNames.add("Task");
		columnNames.add("Action");
		taskTree.setHeaderLabels(columnNames);

		QHBoxLayout layout = new QHBoxLayout(this);
		layout.addWidget(taskTree, 1);
		layout.setMargin(1);
		layout.setSpacing(1);
		setLayout(layout);
	}

	public void notify(final Observed observed) {
		QCoreApplication.invokeLater(new Runnable() {
			public void run() {
				taskTree.clear();
				List<Task> tasks = ((TaskManager) observed).getTasks();
				for (int i = 0; i < tasks.size(); i++) {
					QTreeWidgetItem taskItem = new QTreeWidgetItem(taskTree);
					taskItem.setText(0, tasks.get(i).getName());
					taskItem.setText(1, tasks.get(i).getActionDescription());
				}
			}
		});
	}

	@Override
	public void disposed() {
		State.getInstance().getTaskManager().unregisterObserver(this);
	}
}
