/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 27, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.toolchest;


import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.NamedObject;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.Mode;

public abstract class Tool implements NamedObject {
	private String name;
	private String displayName;
	private String category;
	private boolean active=false;
	private Mode mode;

	public abstract boolean hasOptionEditor();
	public abstract QWidget getOptionEditor(QWidget parent);
	public abstract void activated();
	public abstract void deactivated();
	
	public Tool() {
		mode=null;
	}
	
	public Tool(Mode mode) {
		this.mode=mode;
	}
	
	final public void activate() {
		if(active) {
			return;
		}
		if(isModal()) {
			active=true;
			ToolChestView.deactivateAllExcept(this);
			if(!mode.isActive()) {
				mode.activate();
			}
		}
		activated();
	}
	
	final public void deactivate() {
		if(!active) {
			return;
		}
		if(isModal()) {
			active=false;
			if(mode.isActive()) {
				mode.deactivate();
			}
		}
		deactivated();
	}
	
	final public void toggleActivation() {
		if(isModal()) {
			if(active) {
				deactivate();
			} else {
				activate();
			}
		} else {
			activate();
		}
	}
	
	final public boolean isActive() {
		return active;
	}
	
	final public boolean isModal() {
		if(mode==null) {
			return false;
		} else {
			return true;
		}
	}

	final public Mode getMode() {
		return mode;
	}
	
	public void setName(String name) {
		this.name=name;
	}
	
	public String getName() {
		return name;
	}
	
	public String getDisplayName() {
		return displayName;
	}
	
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}
	
	public String getCategory() {
		return category;
	}
	
	public void setCategory(String category) {
		this.category = category;
	}
}
