/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 27, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.toolchest;

import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QBrush;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QFontMetrics;
import com.trolltech.qt.gui.QLinearGradient;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QPolygon;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

class ToolButton extends QWidget {
	private Tool tool;
	private QWidget optionEditor;
	private boolean expanded;
	private ToolButtonHeader toolButtonHeader;
	private ToolCategory parentCategory;
	
	private class ToolButtonHeader extends QWidget {
		int textHeight;
		
		public ToolButtonHeader(QWidget parent) {
			super(parent);
			// determine minimal height:
			textHeight=new QFontMetrics(new QFont()).height();
			setFixedHeight(textHeight+4);
			setMinimumWidth(50);
		}
		
		@Override
		public void mouseReleaseEvent(QMouseEvent event) {
			if(tool.hasOptionEditor()) {
				// click on arrow toggles expansion
				if(event.x()<5+textHeight) {
					setExpanded(!expanded);
					return;
				}
			}
			// click on any other region activates tool
			tool.toggleActivation();
			repaint();
		}
		
		@Override
		public void paintEvent(QPaintEvent event) {
			super.paintEvent(event);
			
			QPainter painter=new QPainter();
			painter.begin(this);

			// draw background
			if(tool.isActive()) {
				// TODO draw activated background
				painter.setPen(Qt.PenStyle.NoPen);
				painter.setBrush(QApplication.palette().button());
				painter.drawRect(rect());
			} else if(optionEditor!=null && expanded) {
				QLinearGradient gradient=new QLinearGradient(0,0,0,height());
				gradient.setColorAt(0d, QApplication.palette().window().color());
				gradient.setColorAt(1d, new QColor(120,120,120));
//				gradient.setColorAt(1d, new QColor(210,210,210));
				painter.setBrush(new QBrush(gradient));
				painter.setPen(Qt.PenStyle.NoPen);
				painter.drawRect(rect());
			} else {
				// TODO draw normal background
			}
			
			// draw header name
			painter.setBrush(QApplication.palette().text());
			painter.setPen(QApplication.palette().text().color());
			painter.drawText(5+textHeight,textHeight,tool.getDisplayName());
			
			// draw marker triangle if appropriate
			if(tool.hasOptionEditor()) {
				QPolygon polygon=new QPolygon();
				painter.translate(3+textHeight/2,height()/2);
				polygon.append(new QPoint(0, -height()/2+4));
				polygon.append(new QPoint(0, height()/2-4));
				polygon.append(new QPoint(height()/2-4, 0));
				if(expanded) {
					painter.rotate(90);
				}
//				painter.setBrush(QColor.black);
//				painter.setPen(Qt.PenStyle.NoPen);
				painter.setBrush(QApplication.palette().window());
				painter.setPen(QApplication.palette().text().color());
				painter.drawPolygon(polygon);
				painter.end();
			}
		}
	}
	
	public ToolButton(QWidget parent, Tool tool, ToolCategory parentCategory) {
		super(parent);
		this.tool=tool;
		this.parentCategory=parentCategory;
		expanded=true;
		
		toolButtonHeader=new ToolButtonHeader(this);
		
		if(tool.hasOptionEditor()) {
			optionEditor=tool.getOptionEditor(this);
		}
		
		// set layout
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(toolButtonHeader);
		if(tool.hasOptionEditor()) {
			layout.addWidget(optionEditor);			
		}
		layout.setSpacing(0);
		layout.setMargin(0);
		setLayout(layout);

		if(tool.hasOptionEditor()) {
			setExpanded(false);
		}
	}
	
	public void setExpanded(boolean expanded) {
		if(!tool.hasOptionEditor()) {
			return;
		}
		
		this.expanded=expanded;
		optionEditor.setVisible(expanded);
		if(expanded) {
			optionEditor.setMinimumHeight(optionEditor.layout().minimumSize().height());				
		} else {
			optionEditor.setMinimumHeight(0);
		}
		setMinimumHeight(toolButtonHeader.minimumHeight()+optionEditor.minimumHeight()+2*layout().getContentsMargins().top);
		toolButtonHeader.repaint();
		if(parentCategory!=null) {
			parentCategory.resetSize();
		}
	}
	
	void updateHeader() {
		toolButtonHeader.update();
	}
	
	@Override
	public void paintEvent(QPaintEvent event) {
		super.paintEvent(event);
		if(optionEditor!=null && expanded) {
			QPainter painter=new QPainter();
			painter.begin(this);
			QBrush brush=new QBrush(new QColor(120,120,120));
//			QBrush brush=new QBrush(new QColor(210,210,210));
			painter.fillRect(0, toolButtonHeader.height(), width(), height(), brush);
			painter.end();
		}
	}
}
