/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 27, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.toolchest;

import java.util.ArrayList;

import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QFontMetrics;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QPolygon;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

class ToolCategory extends QWidget {
	private ArrayList<ToolCategory> childCategories;
	private ArrayList<ToolButton> buttons;
	private String name;
	private boolean expanded;
	private boolean headerVisible;
	private ToolCategory parentCategory;
	private ToolCategoryHeader toolCategoryHeader;
	private QWidget childFrame;
	private QGridLayout childFrameLayout;
	
	private class ToolCategoryHeader extends QWidget {
		public ToolCategoryHeader(QWidget parent) {
			super(parent);
			setMinimumWidth(150);
			setDefaultHeight();
		}

		@Override
		public void mouseReleaseEvent(QMouseEvent event) {
			setExpanded(!expanded);
		}

		@Override
		public void paintEvent(QPaintEvent event) {
			super.paintEvent(event);
			
			if(name==null) {
				return;
			}
			
			QPainter painter=new QPainter();
			painter.begin(this);

			// draw header name
			int textHeight=new QFontMetrics(new QFont()).height();
			painter.setPen(QApplication.palette().text().color());
			painter.drawText(5+textHeight,textHeight,name);
			
			// draw marker triangle
			QPolygon polygon=new QPolygon();
			painter.translate(3+textHeight/2,height()/2);
			polygon.append(new QPoint(0, -height()/2+4));
			polygon.append(new QPoint(0, height()/2-4));
			polygon.append(new QPoint(height()/2-4, 0));
			if(expanded) {
				painter.rotate(90);
			}
			painter.setBrush(QApplication.palette().text().color());
			painter.setPen(Qt.PenStyle.NoPen);
			painter.drawPolygon(polygon);
			painter.end();			
		}
		
		public void setDefaultHeight() {
			// determine minimal height:
			int textHeight=new QFontMetrics(new QFont()).height();
			setFixedHeight(textHeight+4);			
		}
	}
	
	public ToolCategory(QWidget parent, ToolCategory parentCategory) {
		super(parent);
		this.parentCategory=parentCategory;
		childCategories=new ArrayList<ToolCategory>();
		buttons=new ArrayList<ToolButton>();
		expanded=true;
		headerVisible=true;
		
		toolCategoryHeader=new ToolCategoryHeader(this);
		childFrame=new QWidget(this);

		// set layout
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(toolCategoryHeader);
		layout.addWidget(childFrame);
		layout.setSpacing(0);
//		layout.setMargin(0);
		layout.setContentsMargins(0,0,0,0);
		setLayout(layout);
		
		childFrameLayout=new QGridLayout();
		childFrameLayout.setSpacing(0);
//		childFrameLayout.setMargin(0);
		childFrameLayout.setContentsMargins(0,0,0,0);
		childFrameLayout.setColumnMinimumWidth(0, 5);
		childFrame.setLayout(childFrameLayout);
	}
	
	private void rebuildLayout() {
		for(ToolCategory category : childCategories) {
			childFrameLayout.removeWidget(category);
		}
		for(ToolButton button : buttons) {
			childFrameLayout.removeWidget(button);
		}
		int index=0;
		for(ToolCategory category : childCategories) {
			childFrameLayout.addWidget(category,index,1);
			index++;
		}
		for(ToolButton button : buttons) {
			childFrameLayout.addWidget(button,index,1);
			index++;
		}
		resetSize();
	}
	
	public void addToolCategory(ToolCategory category) {
		int i=0;
		while(i<childCategories.size()) {
			if(childCategories.get(i).getName().compareTo(category.getName())>0) {
				break;
			}
			i++;
		}
		childCategories.add(i,category);
		// add category to layout
		category.setParent(childFrame);
		rebuildLayout();
	}
	
	public ToolCategory getChildToolCategory(String name) {
		for(ToolCategory category : childCategories) {
			if(category.getName().equals(name)) {
				return category;
			}
		}
		return null;
	}
	
	public ArrayList<ToolCategory> getChildToolCategories() {
		return (ArrayList<ToolCategory>) childCategories.clone();
	}
	
	public void addToolButton(ToolButton button) {
		buttons.add(button);
		button.setParent(childFrame);
		rebuildLayout();
		setExpanded(expanded); // force size update if needed
	}
	
	public ArrayList<ToolButton> getChildToolButtons() {
		return (ArrayList<ToolButton>) buttons.clone();
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public void setExpanded(boolean expanded) {
		this.expanded=expanded;
		childFrame.setVisible(expanded);
		if(expanded) {
			childFrame.setMinimumHeight(childFrame.layout().minimumSize().height());
		} else {
			childFrame.setMinimumHeight(0);
		}
		setFixedHeight(toolCategoryHeader.minimumHeight()+childFrame.minimumHeight()+2*layout().getContentsMargins().top);
		if(parentCategory!=null) {
			parentCategory.resetSize();
		}
		toolCategoryHeader.repaint();
	}
	
	void setHeaderVisible(boolean visible)
	{
		// FIXME this is part of a hack to get an invisible root category without the need to change any present logic
		headerVisible=visible;
		if(headerVisible) {
			toolCategoryHeader.setDefaultHeight();
			childFrameLayout.setColumnMinimumWidth(0, 5);
		} else {
			toolCategoryHeader.setFixedHeight(0);
			childFrameLayout.setColumnMinimumWidth(0, 0);
		}
		// since the expansion control is hidden, category must be expanded or will be totally invisible
		setExpanded(true); 
	}
	
	public void resetSize() {
		childFrame.setMinimumSize(childFrameLayout.minimumSize());		
		childFrame.setMaximumHeight(childFrameLayout.minimumSize().height());
		setFixedHeight(toolCategoryHeader.minimumHeight()+childFrame.minimumHeight()+2*layout().getContentsMargins().top);
		if(parentCategory!=null) {
			parentCategory.resetSize();
		}
	}
	
	public void updateAll() {
		for(ToolCategory childCategory : childCategories) {
			childCategory.updateAll();
		}
		for(ToolButton button : buttons) {
			button.updateHeader();
			button.update();
		}
		update();
	}
}
