/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.toolchest;

import java.util.ArrayList;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QScrollArea;

import eu.moonlight3d.framework.ui.DockWidget;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.Mode;
import eu.moonlight3d.ml3d.ui.mainwindow.modes.ModeManager;

/**
 * This is the WindowSpace implementation of the UI toolchest plugin
 * 
 * @author riishigh
 */
public class ToolChestView extends DockWidget {
	ToolCategory rootCategory;
	QScrollArea scrollArea;
	
	private static ArrayList<Tool> tools;
	private static ArrayList<ToolChestView> views;
	
	public static class ModeSwitchListener implements eu.moonlight3d.ml3d.ui.mainwindow.modes.ModeSwitchListener {
		public void modeSwitched(Mode from, Mode to) {
			for(Tool tool : tools) {
				if(!tool.isModal()) {
					continue;
				}
				
				if(tool.getMode()==to) {
					tool.activate();
				} else {
					if(tool.isActive()) {
						tool.deactivate();
					}
				}
			}
			for(ToolChestView view : views) {
				view.updateAll();
			}
		}
	}
	
	private static ModeSwitchListener modeSwitchListener;

	static {
		tools=new ArrayList<Tool>();
		views=new ArrayList<ToolChestView>();
		modeSwitchListener=new ModeSwitchListener();
		ModeManager.getInstance().addModeListener(modeSwitchListener);
	}
	
	public static void addTool(Tool tool) {
		tools.add(tool);
		for(ToolChestView view : views) {
			view.rebuildLayout();
		}
	}
	
	public static void removeTool(Tool tool) {
		tools.remove(tool);
		for(ToolChestView view : views) {
			view.rebuildLayout();
		}
	}
	
	public static Tool getToolByName(String name) {
		for(Tool tool : tools) {
			if(tool.getName().equals(name)) {
				return tool;
			}
		}
		return null;
	}
	
	public static void activateTool(Tool tool) {
		tool.activate();
	}
	
	public static void activateTool(String toolName) {
		for(Tool tool : tools) {
			if(tool.getName().equals(toolName)) {
				activateTool(tool);
			}
		}
	}
	
	public static void deactivateTool(Tool tool) {
		tool.deactivate();
	}
	
	public static void deactivateTool(String toolName) {
		for(Tool tool : tools) {
			if(tool.getName().equals(toolName)) {
				deactivateTool(tool);
			}
		}
	}
	
	static void deactivateAllExcept(Tool currentTool)  {
		for(Tool tool : tools) {
			if(tool.isModal() && tool.isActive() && tool!=currentTool) {
				tool.deactivate();
			}
		}
		for(ToolChestView view : views) {
			view.updateAll();
		}
	}

	public ToolChestView() {
		super("Toolchest");
		
		setDefaultArea(Qt.DockWidgetArea.LeftDockWidgetArea);
		
		scrollArea=new QScrollArea(this);
		rootCategory=new ToolCategory(scrollArea,null);
		rootCategory.setName("Root");
		rootCategory.setHeaderVisible(false);
		scrollArea.setWidget(rootCategory);
		scrollArea.setWidgetResizable(true);
		
		QHBoxLayout layout=new QHBoxLayout();
		
		layout.addWidget(scrollArea,1);
		layout.setMargin(0);
		
		setLayout(layout);
		
		rebuildLayout();
		rootCategory.show();
		
		views.add(this);
	}

	private void addToolButton(Tool tool) {
		eu.moonlight3d.framework.State.getInstance().logger().info("addToolButton: adding tool " + tool.getName());
		// descend through tool category tree, inserting neccessary categories and finally the tool
		String[] name=tool.getCategory().split("/");
		ToolCategory category=rootCategory;
		for(int i=0;i<name.length;i++) {
			if(category.getChildToolCategory(name[i])==null) {
				// create new tool category
				eu.moonlight3d.framework.State.getInstance().logger().info("addToolButton: new tool category " + name[i]);
				ToolCategory newCategory=new ToolCategory(category,category);
				newCategory.setName(name[i]);
				category.addToolCategory(newCategory);
				category=newCategory;
			} else {
				category=category.getChildToolCategory(name[i]);
			}
		}
		ToolButton button=new ToolButton(category,tool,category);
		category.addToolButton(button);
	}
	
	private void rebuildLayout() {
		// TODO: clear layout
		for(Tool tool : tools) {
			addToolButton(tool);
		}
	}
	
	void updateAll() {
		rootCategory.updateAll();
	}
	
	@Override
	protected void disposed() {
		views.remove(this);
	}
}
