/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 31, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;

public class CoordinateGridRenderable extends RenderableNode {
	private Colour gridColour;
	private double gridSize;
	private int gridLines;
	private boolean drawGrid;
	private boolean displayListInitialised;
	private int linesDisplayList;
	private double displayListGridSize;
	private int displayListGridLines;
	
	public CoordinateGridRenderable() {
		displayListInitialised=false;
		try {
			drawGrid=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("drawGrid").getBooleanValue();
			gridColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridColour").getColourValue();
			gridSize=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridSize").getFloatValue();
			gridLines=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridLines").getIntegerValue();
		} catch (Exception e) {
			e.printStackTrace();
			drawGrid=true;
			gridColour=new Colour(0.0,0.0,0.0,1.0);
			gridSize=1;
			gridLines=100;
		}
	}
	
	private void renderDisplayLists(GL gl) {
		if(displayListInitialised && displayListGridSize==gridSize && displayListGridLines==gridLines) {
			return;
		} else if(!displayListInitialised) {
			linesDisplayList=gl.glGenLists(1);
			displayListInitialised=true;
		}
		
		displayListGridSize=gridSize;
		displayListGridLines=gridLines;

		gl.glNewList(linesDisplayList, GL.GL_COMPILE);
		
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glBegin(GL.GL_LINES);
        for(int i=-gridLines;i<=gridLines;i++) {
          gl.glVertex3d(i*gridSize,-gridLines*gridSize,0);
          gl.glVertex3d(i*gridSize,gridLines*gridSize,0);
        }
        for(int i=-gridLines;i<=gridLines;i++) {
          gl.glVertex3d(-gridLines*gridSize,i*gridSize,0);
          gl.glVertex3d(gridLines*gridSize,i*gridSize,0);
        }
        gl.glEnd();
        gl.glLineWidth(2.0f);
        gl.glBegin(GL.GL_LINES);
        gl.glVertex3d(0,-gridLines*gridSize,0);
        gl.glVertex3d(0,gridLines*gridSize,0);
        gl.glVertex3d(-gridLines*gridSize,0,0);
        gl.glVertex3d(gridLines*gridSize,0,0);
        gl.glEnd();
        gl.glLineWidth(1.0f);
        gl.glDisable(GL.GL_DEPTH_TEST);
		
        gl.glEndList();
	}
	
	@Override
	public void dispose(GL gl) {
		if(displayListInitialised) {
			gl.glDeleteLists(linesDisplayList, 1);
		}
	}
	
	@Override
	public void drawPreview(Viewport view, Scene scene) {
		// nothing to draw here
	}

	@Override
	public void drawVisibleStructure(Viewport view, Scene scene) {
		try {
			drawGrid=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("drawGrid").getBooleanValue();
			gridColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridColour").getColourValue();
			gridSize=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridSize").getFloatValue();
			gridLines=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("gridLines").getIntegerValue();
		} catch (Exception e) {
			drawGrid=true;
			gridColour=new Colour(0.0,0.0,0.0,1.0);
			gridSize=1;
			gridLines=100;
		}
		
		if(!drawGrid) {
			return;
		}
		
        GL gl=view.getGL();

        renderDisplayLists(gl);
		
		Colour.RGB sRGB=gridColour.getsRGB8();
		gl.glColor3d(sRGB.red/256.0,sRGB.green/256.0,sRGB.blue/256.0);
		gl.glCallList(linesDisplayList);
	}

	@Override
	public void drawOverlay(Viewport view, Scene scene) {
		// nothing to draw here
	}

	@Override
	public void drawOverlay2D(Viewport view, Scene scene) {
		// nothing to draw here
	}

}
