/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Created on Jul 14, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;

import com.trolltech.qt.core.QEvent;
import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCursor;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QResizeEvent;
import com.trolltech.qt.gui.QWidget;

public class FourSplit extends QWidget {
	private QWidget topLeft;
	private QWidget topRight;
	private QWidget bottomLeft;
	private QWidget bottomRight;

	private int splitterPosX;
	private int splitterPosY;
	private double splitterRatioX;
	private double splitterRatioY;

	private final int SPLITTERWIDTH=4;

	private boolean isDraggingHorizontal;
	private boolean isDraggingVertical;

	public FourSplit(QWidget parent, QWidget topLeft, QWidget topRight, QWidget bottomLeft, QWidget bottomRight) {
		super(parent);

		this.topLeft=topLeft;
		this.topRight=topRight;
		this.bottomLeft=bottomLeft;
		this.bottomRight=bottomRight;

		topLeft.setParent(this);
		topRight.setParent(this);
		bottomLeft.setParent(this);
		bottomRight.setParent(this);

		splitterRatioX=0.5;
		splitterRatioY=0.5;

		this.isDraggingHorizontal=false;
		this.isDraggingVertical=false;

		setMouseTracking(true);

		resetSizes();
	}

	private void resetSizes() {
		QSize size=size();

		splitterPosX=(int)(splitterRatioX*size.width());
		splitterPosY=(int)(splitterRatioY*size.height());

		// The Math.max()-Call prevents that negative sizes can be set
		topLeft.setFixedSize(Math.max(0, splitterPosX-SPLITTERWIDTH/2),
						     Math.max(0, splitterPosY-SPLITTERWIDTH/2));
		topRight.setGeometry(splitterPosX+SPLITTERWIDTH/2,
					         0,
					         size.width()-(splitterPosX+SPLITTERWIDTH/2),
					         Math.max(splitterPosY-SPLITTERWIDTH/2, 0));
		bottomLeft.setGeometry(0,
				               Math.max(0, splitterPosY+SPLITTERWIDTH/2),
				               splitterPosX-SPLITTERWIDTH/2,
				               size.height()-(splitterPosY+SPLITTERWIDTH/2));
		bottomRight.setGeometry(splitterPosX+SPLITTERWIDTH/2,
				                splitterPosY+SPLITTERWIDTH/2,
				                size.width()-(splitterPosX+SPLITTERWIDTH/2),
				                size.height()-(splitterPosY+SPLITTERWIDTH/2));
	}

	private void handleMouseDown(Qt.MouseButton button, int posX, int posY) {
		if(button!=Qt.MouseButton.LeftButton) {
			return;
		}
		if(posX>splitterPosX-SPLITTERWIDTH/2 && posX<splitterPosX+SPLITTERWIDTH/2) {
			isDraggingVertical=true;
		}
		if(posY>splitterPosY-SPLITTERWIDTH/2 && posY<splitterPosY+SPLITTERWIDTH/2) {
			isDraggingHorizontal=true;
		}

	}

	private void handleMouseUp(Qt.MouseButton button, int posX, int posY) {
		if(button!=Qt.MouseButton.LeftButton) {
			return;
		}
		if(isDraggingHorizontal || isDraggingVertical) {
			isDraggingHorizontal=false;
			isDraggingVertical=false;
		}
	}

	private void handleMouseMove(int posX, int posY) {
		if(!isDraggingHorizontal && !isDraggingVertical) {
			return;
		}

		QSize size=size();

		if(isDraggingHorizontal) {
			splitterRatioY=Math.min(Math.max(0, posY/(double)size.height()), 1);
		}
		if(isDraggingVertical) {
			splitterRatioX=Math.min(Math.max(0, posX/(double)size.width()), 1);
		}

		resetSizes();
	}

	@Override
	protected void mousePressEvent(QMouseEvent event) {
		handleMouseDown(event.button(),event.x(),event.y());
	}

	@Override
	protected void mouseMoveEvent(QMouseEvent event) {
		if(event.x()>=splitterRatioX*width()-SPLITTERWIDTH && event.x()<=splitterRatioX*width()+SPLITTERWIDTH) {
			if(event.y()>=splitterRatioY*height()-SPLITTERWIDTH && event.y()<=splitterRatioY*height()+SPLITTERWIDTH) {
				setCursor(new QCursor(Qt.CursorShape.SizeAllCursor));
			} else {
				setCursor(new QCursor(Qt.CursorShape.SplitHCursor));
			}
		} else if(event.y()>=splitterRatioY*height()-SPLITTERWIDTH && event.y()<=splitterRatioY*height()+SPLITTERWIDTH) {
			setCursor(new QCursor(Qt.CursorShape.SplitVCursor));
		} else {
			setCursor(new QCursor(Qt.CursorShape.ArrowCursor));
		}

		handleMouseMove(event.x(),event.y());
	}

	@Override
	protected void mouseReleaseEvent(QMouseEvent event) {
		handleMouseUp(event.button(),event.x(),event.y());
	}

	@Override
	protected void leaveEvent(QEvent event) {
		setCursor(new QCursor(Qt.CursorShape.ArrowCursor));
	}

	@Override
	protected void resizeEvent(QResizeEvent event) {
		resetSizes();
	}
}
