/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Oct 13, 2007
 */

package eu.moonlight3d.ml3d.ui.mainwindow.view;

import javax.media.opengl.GL;

/**
 * Base class for OpenGL material previews. These are used by the
 * 3D view to render objects with approximate representations of
 * their materials. 
 *
 * @author gregor
 */
public abstract class MaterialNode extends SceneNode {
	/**
	 * Determine and return the number of passes required for the
	 * Material to render itself properly. The number of passes
	 * may vary depending on the features that are available in
	 * the particular OpenGL implementation.
	 * 
	 * @param gl the OpenGL interface to work on
	 * @return the number of passes
	 */
	public abstract int getNumPasses(GL gl);
	
	/**
	 * Get the name of the UV map that should be used for rendering
	 * the given pass of the material. Different passes may have
	 * different maps. If no UV map should be used for that pass,
	 * null is returned.
	 * 
	 * @param gl the OpenGL interface to work on
	 * @param numPass the number of the pass
	 * @return the name of the UV map to use for that pass or null if
	 * no map should be used
	 */
	public abstract String getUVMapForPass(GL gl, int numPass);
	
	/**
	 * Setup the OpenGL state for a given pass. Material changes
	 * for passes have to be requested in numerical order, without
	 * skipping passes.
	 * 
	 * @param gl the OpenGL interface to work on
	 * @param numPass the number of the pass
	 */
	public abstract void doPass(GL gl, int numPass);
	
	/**
	 * Reset the parts of the OpenGL state that have
	 * been affected by the given material.
	 * 
	 * @param gl the OpenGL interface to work on
	 */
	public abstract void reset(GL gl);
}
