/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 31, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;

import eu.moonlight3d.ml3d.backend.threedim.sg.Node;

/**
 * Interface for converting scene graph elements into renderable
 * 3d view elements. Implementations of this interface are used
 * by the OpenGLConverter to convert the contents of the scene
 * graph to a representation that is renderable with OpenGL. 
 *
 * @author gregor
 */
public interface SGNodeConverter {
	/**
	 * Return true if this converter knows how to convert the
	 * given node.
	 * 
	 * @param node the node to check
	 * @return true if the converter applies to the given node, false otherwise
	 */
	public boolean canConvertNode(Node node);
	
	/**
	 * Create a set of light sources for the content of the given scene graph node,
	 * should it contain any.
	 * 
	 * @param node the scene graph node to process
	 * @return array of light sources or null if there are none
	 */
	public LightSource[] convertLights(Node node);
	
	/**
	/**
	 * Convert the contents of the given node to scene nodes for OpenGL rendering.
	 * 
	 * @param node the scene graph node to convert
	 * @return an array of scene nodes or null if there are no resulting nodes
	 */
	public SceneNode[] convert(Node node);
}
