/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 14, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;

import java.util.ArrayList;
import java.util.TreeMap;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.math.Matrix4;
import eu.moonlight3d.math.Vector3D;

/**
 * This class is the container for a scene which is renderable using OpenGL.
 * 
 * @author gregor
 */
public class Scene {
	public ArrayList<LightSource> lightSources;
	public ArrayList<SceneNode> sceneNodes;
	
	public Scene()
	{
		lightSources=new ArrayList<LightSource>();
		sceneNodes=new ArrayList<SceneNode>();
		
		sceneNodes.add(new CoordinateGridRenderable());
	}
	
	public void addLightSources(LightSource[] lightSources) {
		for(LightSource source : lightSources) {
			this.lightSources.add(source);
		}
	}
	
	public void removeLightSources(eu.moonlight3d.ml3d.backend.threedim.og.Node node) {
		ArrayList<LightSource> copiedLightSources=(ArrayList<LightSource>) lightSources.clone();
		for(LightSource lightSource : copiedLightSources) {
			if(node.getName().equals(lightSource.getOGNodeName())) {
				lightSources.remove(lightSource);
			}
		}
	}
	
	public void addSceneNodes(SceneNode[] sceneNodes) {
		for(SceneNode node : sceneNodes) {
			this.sceneNodes.add(node);
		}
	}
	
	public void removeSceneNode(SceneNode sceneNode) {
		sceneNodes.remove(sceneNode);
	}

	public void removeSceneNodes(eu.moonlight3d.ml3d.backend.threedim.og.Node node) {
		ArrayList<SceneNode> copiedSceneNodes=(ArrayList<SceneNode>) sceneNodes.clone();
		for(SceneNode sceneNode : copiedSceneNodes) {
			if(node.getName().equals(sceneNode.getOGNodeName())) {
				sceneNodes.remove(sceneNode);
			}
		}
	}
	
	void clearOGBasedNodes() {
		ViewWindowSpace.makeCommonContextCurrent();
		GL gl=ViewWindowSpace.getCommonContextGL();
		
		ArrayList<LightSource> copiedLightSources=(ArrayList<LightSource>) lightSources.clone();
		for(LightSource lightSource : copiedLightSources) {
			if(lightSource.getOGNodeName()!=null) {
				lightSources.remove(lightSource);
			}
		}		
		ArrayList<SceneNode> copiedSceneNodes=(ArrayList<SceneNode>) sceneNodes.clone();
		for(SceneNode sceneNode : copiedSceneNodes) {
			if(sceneNode.getOGNodeName()!=null) {
				sceneNode.dispose(gl);
				sceneNodes.remove(sceneNode);
			}
		}
	}
	
	private void setupCameraLocalLights(Viewport view) {
		if (view.getCurrentPreviewMode() == Viewport.PreviewMode.None ||
				view.getCurrentPreviewMode() == Viewport.PreviewMode.Wireframe ||
				view.getCurrentPreviewMode() == Viewport.PreviewMode.Lit) {
			return;
		}		
		GL gl=view.getGL();
		
        float[] ambient=new float[4];
        float[] diffuse=new float[4];
        float[] position=new float[4];
        
        gl.glEnable(GL.GL_LIGHTING);
        
        gl.glEnable(GL.GL_LIGHT0);
        Vector3D standardLightPosition;
        Colour standardLightColour;
        try {
			standardLightPosition=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("standardLightPosition").getVectorValue();
			standardLightColour=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("standardLightColour").getColourValue();
		} catch (Exception e) {
			standardLightPosition=new Vector3D(-3.0,3.0,0.0);
			standardLightColour=new Colour(0.28,0.28,0.28,1.0);
		}
        ambient[0]=(float)0.1;
        ambient[1]=(float)0.1;
        ambient[2]=(float)0.1;
        ambient[3]=(float)1.0;
		Colour.RGB sRGB=standardLightColour.getsRGB8();
        diffuse[0]=(float)(sRGB.red/256.0);
        diffuse[1]=(float)(sRGB.green/256.0);
        diffuse[2]=(float)(sRGB.blue/256.0);
        diffuse[3]=(float)(sRGB.alpha/256.0);
        position[0]=(float)standardLightPosition.X1;
        position[1]=(float)standardLightPosition.X2;
        position[2]=(float)standardLightPosition.X3;
        position[3]=(float)1.0;
        gl.glLightfv(GL.GL_LIGHT0,GL.GL_AMBIENT,ambient,0);
        gl.glLightfv(GL.GL_LIGHT0,GL.GL_DIFFUSE,diffuse,0);
        gl.glLightfv(GL.GL_LIGHT0,GL.GL_POSITION,position,0);
        
        gl.glEnable(GL.GL_COLOR_MATERIAL);
        gl.glColorMaterial(GL.GL_FRONT_AND_BACK,GL.GL_AMBIENT_AND_DIFFUSE);
        gl.glLightModeli(GL.GL_LIGHT_MODEL_TWO_SIDE,GL.GL_TRUE);
	}

	private void setupSceneLights(Viewport view) {
		if (view.getCurrentPreviewMode() == Viewport.PreviewMode.None ||
				view.getCurrentPreviewMode() == Viewport.PreviewMode.Wireframe ||
				view.getCurrentPreviewMode() == Viewport.PreviewMode.Shaded) {
			return;
		}		

		GL gl=view.getGL();
		
        gl.glEnable(GL.GL_LIGHTING);

        int maxLights[]=new int[1];
		gl.glGetIntegerv(GL.GL_MAX_LIGHTS,maxLights,0);
		

        Vector3D viewerPosition=new Vector3D();
		Matrix4 modelViewMatrix=view.getModelViewMatrix();
		viewerPosition.X1=modelViewMatrix.X[0][3];
		viewerPosition.X2=modelViewMatrix.X[1][3];
		viewerPosition.X3=modelViewMatrix.X[2][3];
		
		TreeMap<Double,LightSource> lights=new TreeMap<Double,LightSource>();
		
		for(LightSource light : lightSources) {
			double distance=light.getPosition().sub(viewerPosition).abs();
			lights.put(distance,light);
		}
		
		// now actually setup lights:
		int numLights=Math.min(maxLights[0],lights.size());

		int i=0;
		for(LightSource light : lights.values()) {
			light.setup(view,i);
			i++;
			if(i==numLights) {
				break;
			}
		}
	}
	
	private void disableLights(Viewport view) {
		GL gl=view.getGL();
		
        int maxLights[]=new int[1];
		gl.glGetIntegerv(GL.GL_MAX_LIGHTS,maxLights,0);
		
		for(int i=0;i<maxLights[0];i++) {
			gl.glDisable(GL.GL_LIGHT0+i);
		}
	}
	
	public void drawPreview(Viewport view) {
		for(SceneNode sceneNode : sceneNodes) {
			if(sceneNode.isToplevel() && sceneNode instanceof RenderableNode) {
				((RenderableNode) sceneNode).drawPreview(view, this);
			}
		}
	}
	
	public void drawOccludedStructure(Viewport view) {
		for(SceneNode sceneNode : sceneNodes) {
			if(sceneNode.isToplevel() && sceneNode instanceof RenderableNode) {
				((RenderableNode) sceneNode).drawOccludedStructure(view, this);
			}
		}
	}
	
	public void drawVisibleStructure(Viewport view) {
		for(SceneNode sceneNode : sceneNodes) {
			if(sceneNode.isToplevel() && sceneNode instanceof RenderableNode) {
				((RenderableNode) sceneNode).drawVisibleStructure(view, this);
			}
		}
	}
	
	public void drawOverlay(Viewport view) {
		for(SceneNode sceneNode : sceneNodes) {
			if(sceneNode.isToplevel() && sceneNode instanceof RenderableNode) {
				((RenderableNode) sceneNode).drawOverlay(view, this);
			}
		}
	}
	
	public void drawOverlay2D(Viewport view) {
		for(SceneNode sceneNode : sceneNodes) {
			if(sceneNode.isToplevel() && sceneNode instanceof RenderableNode) {
				((RenderableNode) sceneNode).drawOverlay2D(view, this);
			}
		}
	}
	
	public void render(Viewport view) {
		GL gl=view.getGL();
		
		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();
		
		setupCameraLocalLights(view);
		
		gl.glMultMatrixd(view.getModelViewMatrix().getOpenGLMatrix(), 0);
        
        setupSceneLights(view);
        
        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LESS);

        drawPreview(view);
        
        gl.glDisable(GL.GL_LIGHTING);
        
        disableLights(view);

        drawOccludedStructure(view);
        drawVisibleStructure(view);
        
        gl.glDisable(GL.GL_DEPTH_TEST);
        
        drawOverlay(view);
        
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glOrtho(0.0,1.0,0.0,1.0,-1.0,1.0);
        
        drawOverlay2D(view);
	}
}
