/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 4, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;

import com.trolltech.qt.core.Qt;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.math.Matrix4;
import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;

/**
 * TODO make modifier key for camera movement modifyable
 * 
 * @author gregor
 */
public class StandardViewEventHandler extends ViewEventHandler {
	protected boolean altPressed;

	protected boolean mouseButtonPressed[];

	private double initialScale;

	public StandardViewEventHandler() {
		altPressed = false;
		mouseButtonPressed = new boolean[3];
		for (int i = 0; i < 3; i++) {
			mouseButtonPressed[i] = false;
		}
	}

	@Override
	public void mouseMove(int oldX, int oldY, int newX, int newY) throws Exception {
		Viewport viewport;
		int deltaX, deltaY;
		
		viewport=getViewport();

		deltaX = newX - oldX;
		deltaY = newY - oldY;

		double scale=viewport.getModelViewMatrix().getTranslation().abs();
		
		if (altPressed) {
			if(getViewport().getProjectionType()==Viewport.ProjectionType.Perspective) {
				if (mouseButtonPressed[0] && mouseButtonPressed[1]) {
					// local Z rotation
					if(!getViewport().isCameraOrientationLocked()) {
						Matrix4 rotationMatrix = makeLocalZRotationMatrix(deltaX, deltaY);
						viewport.setModelViewMatrix(rotationMatrix.multiply(viewport.getModelViewMatrix()));
					}
				} else if (mouseButtonPressed[0]) {
					// local XY rotation
					if(!getViewport().isCameraOrientationLocked()) {
						Matrix4 rotationMatrix = makeLocalXYRotationMatrix(deltaX, deltaY,initialScale);
						viewport.setModelViewMatrix(rotationMatrix.multiply(viewport.getModelViewMatrix()));
					}
				} else if (mouseButtonPressed[2]) {
					// pan
					Matrix4 panningMatrix = makePanningMatrix(deltaX, deltaY,scale);
					viewport.setModelViewMatrix(panningMatrix.multiply(viewport.getModelViewMatrix()));
				} else if (mouseButtonPressed[1]) {
					// zoom
					Matrix4 zoomingMatrix = makeZoomingMatrix(deltaX, deltaY,scale);
					viewport.setModelViewMatrix(zoomingMatrix.multiply(viewport.getModelViewMatrix()));
					// NOTE adjust viewport scale here, too; else it becomes irritating on switching to orthographic projection (see orthograpic zoom below for further details)
					viewport.setOrthographicScale(viewport.getOrthographicScale()*(1.0+(-deltaY+deltaX)/100.0));
				}
			} else if(getViewport().getProjectionType()==Viewport.ProjectionType.Orthographic) {
				if (mouseButtonPressed[0] && mouseButtonPressed[1]) {
					// local Z rotation
					if(!getViewport().isCameraOrientationLocked()) {
						Matrix4 rotationMatrix = makeLocalZRotationMatrix(deltaX, deltaY);
						viewport.setModelViewMatrix(rotationMatrix.multiply(viewport.getModelViewMatrix()));
					}
				} else if (mouseButtonPressed[0]) {
					// local XY rotation
					if(!getViewport().isCameraOrientationLocked()) {
						Matrix4 rotationMatrix = makeLocalXYRotationMatrix(deltaX, deltaY,initialScale);
						viewport.setModelViewMatrix(rotationMatrix.multiply(viewport.getModelViewMatrix()));
					}
				} else if (mouseButtonPressed[2]) {
					// pan
					Matrix4 panningMatrix = makePanningMatrix(deltaX, deltaY,viewport.getOrthographicScale()/2);
					viewport.setModelViewMatrix(panningMatrix.multiply(viewport.getModelViewMatrix()));
				} else if (mouseButtonPressed[1]) {
					// zoom
					// NOTE zooming in orthographic mode works differently - the projection implements the scaling that is neccessary here 
					Matrix4 zoomingMatrix = makeZoomingMatrix(deltaX, deltaY,scale);
					viewport.setModelViewMatrix(zoomingMatrix.multiply(viewport.getModelViewMatrix()));
					viewport.setOrthographicScale(viewport.getOrthographicScale()*(1.0+(-deltaY+deltaX)/100.0));
				}
			}
		}
	}

	@Override
	public void mouseButtonDown(Qt.MouseButton button) throws Exception {
		if(button==Qt.MouseButton.LeftButton) {
			mouseButtonPressed[0]=true;
			initialScale=viewport.getModelViewMatrix().getTranslation().abs();
		} else if(button==Qt.MouseButton.RightButton) {
			mouseButtonPressed[1]=true;
		} else if(button==Qt.MouseButton.MidButton) {
			mouseButtonPressed[2]=true;
		}
	}

	@Override
	public void mouseButtonUp(Qt.MouseButton button) throws Exception {
		if(button==Qt.MouseButton.LeftButton) {
			mouseButtonPressed[0]=false;
		} else if(button==Qt.MouseButton.RightButton) {
			mouseButtonPressed[1]=false;
		} else if(button==Qt.MouseButton.MidButton) {
			mouseButtonPressed[2]=false;
		}
	}

	@Override
	public void mouseDoubleClick(Qt.MouseButton button) throws Exception {
	}
	
	@Override
	public void mouseWheelMove(double angle) {
		double scale=viewport.getModelViewMatrix().getTranslation().abs();
		// mouse wheel dolly
		Matrix4 zoomingMatrix = makeZoomingMatrix(angle,scale);
		// adjust both perspective and orthographic projection; else it becomes irritating on switching to orthographic projection (see orthograpic zoom below for further details)
		viewport.setModelViewMatrix(zoomingMatrix.multiply(viewport.getModelViewMatrix()));
		viewport.setOrthographicScale(viewport.getOrthographicScale()*(1.0-angle/150));
	}

	@Override
	public void keyPress(int key, Qt.KeyboardModifiers state) throws Exception {
		if (key == Qt.Key.Key_Alt.value()) {
			altPressed = true;
		}
	}

	@Override
	public void keyRelease(int key, Qt.KeyboardModifiers state) throws Exception {
		if (key == Qt.Key.Key_Alt.value()) {
			altPressed = false;
		}
	}

	@Override
	public void onRemove() {
		eu.moonlight3d.framework.State.getInstance().logger().warning("StandardViewEventHandler.onRemove() called - this should not happen!");
	}

	private Matrix4 makeLocalXYRotationMatrix(int deltaX, int deltaY, double scale) {
		double alpha, beta;
		Matrix4 rotationMatrix;
		Vector3D axisX = new Vector3D();
		Vector3D axisY = new Vector3D();
		Quaternion rotationX = new Quaternion();
		Quaternion rotationY = new Quaternion();
		Matrix4 translationMatrix=new Matrix4();

		double cameraPOIDistance;
		try {
			cameraPOIDistance = eu.moonlight3d.framework.State.getInstance().getPreferencesManager().getCurrentPreferences()
					.getGroupByName("3D View").getProperty("cameraPOIDistance").getFloatValue();
		} catch (Exception e) {
			e.printStackTrace();
			cameraPOIDistance=1.0;
		}
		boolean invertXRotation;
		try {
			invertXRotation=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("invertXRotation").getBooleanValue();
		} catch(Exception e) {
			e.printStackTrace();
			invertXRotation=false;
		}
		boolean invertYRotation;
		try {
			invertYRotation=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View").getProperty("invertYRotation").getBooleanValue();
		} catch(Exception e) {
			e.printStackTrace();
			invertYRotation=false;
		}
		
		cameraPOIDistance*=scale;
		
		translationMatrix.clear();
		for(int i=0;i<4;i++) {
			translationMatrix.X[i][i]=1;
		}
		translationMatrix.X[2][3]=cameraPOIDistance;

		axisX.X1 = 1;
		axisX.X2 = 0;
		axisX.X3 = 0;

		axisY.X1 = 0;
		axisY.X2 = 1;
		axisY.X3 = 0;

		alpha = - deltaY * Math.PI / 180.0;
		if(invertYRotation) {
			alpha=-alpha;
		}
		beta = - deltaX * Math.PI / 180.0;
		if(invertXRotation) {
			beta=-beta;
		}

		rotationX.fromAxisAngle(axisX, alpha);
		rotationY.fromAxisAngle(axisY, beta);

		rotationMatrix = rotationX.toRotationMatrix4().multiply(rotationY.toRotationMatrix4());
		
		rotationMatrix=rotationMatrix.multiply(translationMatrix);
		rotationMatrix=translationMatrix.invert().multiply(rotationMatrix);

		return rotationMatrix;
	}

	private Matrix4 makeLocalZRotationMatrix(int deltaX, int deltaY) {
		double alpha;
		Matrix4 rotationMatrix = new Matrix4();

		alpha = (deltaY-deltaX) * Math.PI / 180.0;

		rotationMatrix.clear();

		rotationMatrix.X[0][0] = Math.cos(alpha);
		rotationMatrix.X[0][1] = -Math.sin(alpha);
		rotationMatrix.X[1][0] = Math.sin(alpha);
		rotationMatrix.X[1][1] = Math.cos(alpha);

		rotationMatrix.X[2][2] = 1.0;
		rotationMatrix.X[3][3] = 1.0;

		return rotationMatrix;
	}

	private Matrix4 makePanningMatrix(int deltaX, int deltaY, double scale) {
		Matrix4 panningMatrix = new Matrix4();

		panningMatrix.clear();

		for (int i = 0; i < 4; i++) {
			panningMatrix.X[i][i] = 1.0;
		}

		panningMatrix.X[0][3] = scale * deltaX / 200.0;
		panningMatrix.X[1][3] = scale * -deltaY / 200.0;
		
		return panningMatrix;
	}

	private Matrix4 makeZoomingMatrix(int deltaX, int deltaY, double scale) {
		Matrix4 zoomingMatrix = new Matrix4();

		zoomingMatrix.clear();

		for (int i = 0; i < 4; i++) {
			zoomingMatrix.X[i][i] = 1.0;
		}

		zoomingMatrix.X[2][3] = scale * (-deltaY+deltaX) / 200.0;

		return zoomingMatrix;
	}

	private Matrix4 makeZoomingMatrix(double angle, double scale) {
		Matrix4 zoomingMatrix = new Matrix4();

		zoomingMatrix.clear();

		for (int i = 0; i < 4; i++) {
			zoomingMatrix.X[i][i] = 1.0;
		}

		zoomingMatrix.X[2][3] = scale * angle / 150.0;

		return zoomingMatrix;
	}
}
