/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.view;


import java.util.ArrayList;

import javax.media.opengl.GL;
import javax.media.opengl.GLContext;
import javax.media.opengl.GLDrawableFactory;

import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.opengl.QGLFormat;
import com.trolltech.qt.opengl.QGLWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.preferences.Preferences;
import eu.moonlight3d.framework.preferences.PreferencesGroup;

/**
 * This is the WindowSpace which renders the 3D views
 * 
 * @author gregor
 */
public class ViewWindowSpace extends QWidget {
	private QHBoxLayout layout;
	private ArrayList<Viewport> viewports;
	private QWidget topLevel;
	
	private static ArrayList<ViewWindowSpace> viewWindowSpaces;
	
	static {
		viewWindowSpaces=new ArrayList<ViewWindowSpace>();
	}
	
	private static void addViewWindowSpace(ViewWindowSpace view) {
		viewWindowSpaces.add(view);
	}
	
	private static void removeViewWindowSpace(ViewWindowSpace view) {
		viewWindowSpaces.remove(view);
	}
	
	static ArrayList<ViewWindowSpace> getViewWindowSpaces() {
		return (ArrayList<ViewWindowSpace>) viewWindowSpaces.clone();
	}
	
	private static QGLWidget commonGLWidget=null;
	private static GLContext commonGLContext=null;
	
	static QGLWidget getCommonGLWidget() {
		if(commonGLWidget==null) {
			if(State.getInstance().getBatchMode()==true) {
				throw new IllegalStateException(L10N.translate("access to common GL context widget not allowed in batch mode"));
			}
			
			QGLFormat format=new QGLFormat();
			commonGLWidget=new QGLWidget(format);
			commonGLWidget.setVisible(false);
		}
		
		return commonGLWidget;
	}
	
	static void makeCommonContextCurrent() {
		getCommonGLWidget().makeCurrent();		
	}
	
	static GL getCommonContextGL() {
        if(commonGLContext==null) {
        	commonGLContext=GLDrawableFactory.getFactory().createExternalGLContext();
        }
        commonGLContext.makeCurrent();
        GL gl=commonGLContext.getGL();
        return gl;
	}
	
    /**
	 * @param parent
	 * @param style
	 */
	public ViewWindowSpace(QWidget parent) {
		super(parent);
		
		addViewWindowSpace(this);
		
		viewports=new ArrayList<Viewport>();

		layout=new QHBoxLayout();
		layout.setMargin(1);
		layout.setSpacing(1);
		setLayout(layout);

//		setupOnePaneView();
		setupFourPaneView();
	}
	
	@Override
	public void disposed() {
		removeViewWindowSpace(this);
	}
	
	private Viewport createViewport(QWidget parent, QGLWidget shareWidget) {
		PreferencesGroup preferences=State.getInstance().getPreferencesManager().getCurrentPreferences().getGroupByName("3D View");
		boolean enableMultisampling=preferences.getProperty("enableMultisampling").getBooleanValue();
		
		QGLFormat format=new QGLFormat();
		format.setDepth(true);
		format.setDoubleBuffer(true);
		if(enableMultisampling) {
			format.setSampleBuffers(true);
			format.setSamples(4);
		}
		QGLFormat.setDefaultFormat(format);
		
		Viewport viewport;
       	if(shareWidget!=null) {
       		viewport=new Viewport(parent, format, shareWidget);        		
       	} else {
       		viewport=new Viewport(parent, format);
       	}
//   		viewport=new Viewport(parent, getContext());
		
		return viewport;
	}
	
	void setupFourPaneView() {
		viewports.clear();
		if(topLevel!=null) {
			layout.removeWidget(topLevel);
			topLevel=null;
		}
		Viewport topLeft=createViewport(this,getCommonGLWidget());
		Viewport topRight=createViewport(this,getCommonGLWidget());
		Viewport bottomLeft=createViewport(this,getCommonGLWidget());
		Viewport bottomRight=createViewport(this,getCommonGLWidget());
		topLeft.setStandardPerspective(Viewport.StandardPerspective.Perspective);
		topRight.setStandardPerspective(Viewport.StandardPerspective.Top);
		bottomLeft.setStandardPerspective(Viewport.StandardPerspective.Front);
		bottomRight.setStandardPerspective(Viewport.StandardPerspective.Right);
		FourSplit fourSplit=new FourSplit(this,topLeft,topRight,bottomLeft,bottomRight);
		layout.addWidget(fourSplit,1);
		setLayout(layout);
		topLevel=fourSplit;
		viewports.add(topLeft);
		viewports.add(topRight);
		viewports.add(bottomLeft);
		viewports.add(bottomRight);
		layout();
	}
	
	void setupOnePaneView() {
		viewports.clear();
		if(topLevel!=null) {
			layout.removeWidget(topLevel);
			topLevel=null;
		}
		Viewport viewport=createViewport(this,getCommonGLWidget());
		viewports.add(viewport);
		layout.addWidget(viewport,1);
		setLayout(layout);
		topLevel=viewport;
		layout();
	}

	public ArrayList<Viewport> getViewports() {
		return (ArrayList<Viewport>) viewports.clone();
	}
}
