/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 9, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.widgets;

import eu.moonlight3d.math.Quaternion;
import eu.moonlight3d.math.Vector3D;

/**
 * A listener for events emitted by the {@link Gizmo} 3D widget.
 * Gizmo listeners are typically used to apply the transformation
 * entered by the user to the target objects of the editing. They
 * therefore act as a glue between the generic gizmo widget and
 * the individual object types which can be edited.
 *
 * @author gregor
 */
public interface GizmoListener {
	/**
	 * Called when the {@link Gizmo} has started a dragging
	 * operation.
	 * 
	 * @param mode the mode of the dragging operation.
	 */
	public void begin(Gizmo.DragMode mode);
	/**
	 * Called whenever a new translation vector is calculated
	 * during a translation operation.
	 * 
	 * @param displacment the new translation vector
	 */
	public void translate(Vector3D displacment);
	/**
	 * Called whenever a new rotation quaternion is calculated
	 * for the rotation around the given center point during
	 * a rotation operation.
	 * 
	 * @param center center of the rotation
	 * @param rotation quaternion describing the rotation
	 */
	public void rotate(Vector3D center, Quaternion rotation);
	/**
	 * Called whenever a new set of scaling factors is calculated
	 * for the scaling around the given center point during a
	 * scaling operation.
	 * 
	 * @param center center of the scaling
	 * @param amount scaling factors along all axes
	 */
	public void scale(Vector3D center, Vector3D amount);
	/**
	 * Called whenever the transformation should be applied to the
	 * object for subsequent rendering in the 3D view.
	 */
	public void apply();
	/**
	 * Called whenever the dragging operation ends and the results
	 * can be applied permanently.
	 */
	public void end();
}
