/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Dec 28, 2005
 */
package eu.moonlight3d.ml3d.ui.mainwindow.widgets;

import javax.media.opengl.GL;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.framework.preferences.Preferences;
import eu.moonlight3d.math.Ray;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Scene;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Viewport;

public class Handle {
	protected Vector3D position;
	protected Colour colour;
	protected Colour selectedColour;
	protected Colour activeColour;
	protected float size;
	protected boolean active;
	protected boolean selected;
	protected String elementName;
	
	public Handle() {
		position=new Vector3D();
		position.clear();
		colour=new Colour();
		colour.red=1.0;
		colour.green=1.0;
		colour.blue=1.0;
		Preferences currentPreferences=eu.moonlight3d.framework.State.getInstance().getPreferencesManager().getCurrentPreferences();
		try {
			selectedColour=currentPreferences.getGroupByName("3D View").getProperty("selectedColour").getColourValue();
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			activeColour=currentPreferences.getGroupByName("3D View").getProperty("activeColour").getColourValue();
		} catch (Exception e) {
			e.printStackTrace();
		}
		size=(float) 3.0;
		active=false;
		selected=false;
	}
	
	public void setColour(Colour colour) {
		this.colour=colour.clone();
	}
	
	public Colour getColour() {
		return colour.clone();
	}
	
	public void setSelectedColour(Colour colour) {
		this.selectedColour=colour.clone();
	}
	
	public Colour getSelectedColour() {
		return selectedColour.clone();
	}
	
	public void setActiveColour(Colour colour) {
		this.selectedColour=colour.clone();
	}
	
	public Colour getActiveColour() {
		return selectedColour.clone();
	}
	
	public void setPosition(Vector3D position) {
		this.position=position.clone();
	}
	
	public Vector3D getPosition() {
		return position.clone();
	}
	
	public void setSize(float size) {
		this.size=size;
	}
	
	public float getSize() {
		return size;
	}
	
	public void setElementName(String name) {
		elementName=name;
	}
	
	public String getElementName() {
		return elementName;
	}
	
	public double getAngle(Ray ray) throws Exception {
		return position.sub(ray.start).angleTo(ray.direction);
	}

	public void draw(Viewport view, Scene scene) {
		GL gl = view.getGL();

		gl.glPointSize(size);
		gl.glBegin(GL.GL_POINTS);
		if (active) {
			Colour.RGB sRGB=activeColour.getsRGB8();
			gl.glColor3d(sRGB.red/256.0,sRGB.green/256.0,sRGB.blue/256.0);
//			gl.glColor3d(activeColour.red, activeColour.green, activeColour.blue);
		} else if (selected) {
			Colour.RGB sRGB=selectedColour.getsRGB8();
			gl.glColor3d(sRGB.red/256.0,sRGB.green/256.0,sRGB.blue/256.0);
//			gl.glColor3d(selectedColour.red, selectedColour.green, selectedColour.blue);
		} else {
			gl.glColor3d(colour.red,colour.green,colour.blue);
		}
		gl.glVertex3d(position.X1, position.X2, position.X3);
		gl.glEnd();
	}

    public void setSelected(boolean selected)
    {
      this.selected=selected;   
    }
   
    public boolean getSelected()
    {
        return selected;
    }
    
    public void setActive(boolean active)
    {
        this.active=active;
    }
    
    public boolean getActive()
    {
        return active;
    }
}
