/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 13, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.widgets;

import eu.moonlight3d.ml3d.backend.threedim.og.Link;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.OperatorGraphListener;

/**
 * This is an abstract base class for handle container populators, that is,
 * clases that fill the {@link HandleContainer} with the selection handles 
 * for their context.  
 *
 * @author gregor
 */
public abstract class HandleContainerPopulator implements OperatorGraphListener {
	/**
	 * The handle container to fill.
	 */
	HandleContainer container;
	
	/**
	 * Constructor.
	 * 
	 * @param container the handle container that this populator applies to
	 */
	public HandleContainerPopulator(HandleContainer container) {
		this.container=container;
	}
	
	/**
	 * Repopulate the handle container. Subclasses must implement this function.
	 * When called, this function must completely clear the handle container
	 * and add all its selection handles.
	 *  
	 * @param handleContainer the handle container to fill up
	 */
	public abstract void repopulate(HandleContainer handleContainer);
	
	public final void nodeInserted(Node node) {
//		repopulate(container);
	}

	public final void nodeRemoved(Node node) {
//		repopulate(container);
	}

	public final void nodeUpdated(Node node) {
//		repopulate(container);
	}
	
	public final void nodeUpdateFailed(Node node, Exception exception ) {
	}

	public final void graphUpdated() {
		repopulate(container);
	}
	
	public final void graphCleared() {
		repopulate(container);		
	}

	public final void linkInserted(Link link) {
	}

	public final void linkRemoved(Link link) {
	}
}
