/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 6, 2006
 */
package eu.moonlight3d.ml3d.ui.mainwindow.widgets;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.media.opengl.GL;

import com.trolltech.qt.core.Qt;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.math.Helper;
import eu.moonlight3d.math.Vector3D;
import eu.moonlight3d.ml3d.backend.selection.Selection;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Scene;
import eu.moonlight3d.ml3d.ui.mainwindow.view.Viewport;

public class HandleWidget extends Widget implements HandleContainer {
	private ArrayList<Handle> handles;
	private HashMap<String, Handle> handleMap;
	private HandleContainerPopulator populator;
	private boolean shiftPressed;
	private int currentX;
	private int currentY;
	private String selectionTypeName;
	private boolean dragging;
	private ArrayList<Vector3D> lassoPoints;
	private ArrayList<String> selectionList;
	
	private class SelectionListener implements eu.moonlight3d.ml3d.backend.selection.SelectionListener {

		public void selectionCleared() {
			updateSelectedHandles();
		}

		public void elementAdded(String objectName) {
			updateSelectedHandles();
		}

		public void elementsAdded(List<String> objectNames) {
			updateSelectedHandles();
		}
		
		public void elementRemoved(String objectName) {
			updateSelectedHandles();
		}

		public void elementUpdated(String oldName, String newName) {
			updateSelectedHandles();
		}

	}
	
	private SelectionListener selectionListener;
	
	public HandleWidget(WidgetViewEventHandler viewEventHandler, String selectionTypeName) {
		super(viewEventHandler);
		setPriority(Widget.HANDLEPRIORITY);
		handles=new ArrayList<Handle>();
		handleMap=new HashMap<String, Handle>();
		populator=null;
		shiftPressed=false;
		currentX=0;
		currentY=0;
		dragging=false;
		lassoPoints=new ArrayList<Vector3D>();
		selectionList=new ArrayList<String>();
		this.selectionTypeName=selectionTypeName;
		selectionListener=new SelectionListener();
		try {
			((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection").registerListener(selectionListener);
		} catch (Exception e) {
			// ignore that exception here - it simply means that registering the listener has failed, which is not a big deal
		}
	}
	
	public void setPopulator(HandleContainerPopulator populator) {
		this.populator=populator;
		((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getOGManager().registerOperatorGraphListener(populator);
		populator.repopulate(this);
	}
	
	public void addHandle(Handle handle) {
		handles.add(handle);
		handleMap.put(handle.getElementName(),handle);
	}
	
	public void removeHandle(Handle handle) {
		handles.remove(handle);
		handleMap.remove(handle.getElementName());
	}
	
	public void clearAllHandles() {
		handles.clear();
		handleMap.clear();
	}
	
	@Override
	public void onRemove() {
		if(populator!=null) {
			((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getOGManager().unregisterOperatorGraphListener(populator);
		}
		try {
			((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection").unregisterListener(selectionListener);
		} catch (Exception e) {
			// ignore that exception here - it simply means that unregistering the listener has failed, which is not a big deal
		}
	}
	
	/**
	 * Update displayed selection handle state when selection changed in the backend.
	 */
	private void updateSelectedHandles() {
		State state=State.getInstance();
		Selection selection;
		try {
			selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
		} catch (Exception e) {
			// just abort here - selection does not exist
			return;
		}
		for(Handle handle : handles) {
			handle.setSelected(false);
			handle.setActive(false);
		}
		for(String selectedObject : selection.getSelection()) {
			if(handleMap.get(selectedObject)!=null) {
				handleMap.get(selectedObject).setSelected(true);
			}
//			for(Handle handle : handles) {
//				if(handle.getElementName().equals(selectedObject)) {
//					handle.setSelected(true);
//				}
//			}
		}
		for(Handle handle : handles) {
			if(handle.getElementName().equals(selection.getActiveObject())) {
				handle.setActive(true);
			}
		}
	}
	
	private void toggleSelection(Handle hitHandle, boolean multipleSelect, boolean lasso) throws Exception {
		State state=State.getInstance();
		
		Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");

		String currentSelectionTypeName=selection.getType().getName();
		// no multiple selection or change of selection type -> clear selection
		if ((multipleSelect == false && lasso==false) || !currentSelectionTypeName.equals(selectionTypeName)) { 
			selection.clearSelection(eu.moonlight3d.ml3d.backend.selection.Manager.getSelectionType(selectionTypeName));
			for(Handle handle : handles) {
				handle.setSelected(false);
				handle.setActive(false);
			}
		} else {			
			// clear previously active handles here, but not selected:
			for(Handle handle : handles) {
				handle.setActive(false);
			}			
		}

		hitHandle.setSelected(true);
		hitHandle.setActive(true);
		
//		selection.add(hitHandle.getElementName());
		selectionList.add(hitHandle.getElementName());
	}
	
	void commitSelectionList() throws Exception
	{
		State state=State.getInstance();
		Selection selection = ((Document) state.getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");

		selection.add(selectionList);
		selectionList.clear();
	}
	
	/**
	 * Try to select handles based on the user input.
	 * 
	 * @param mouseX the x coordinate of the mouse pointer
	 * @param mouseY the y coordinate of the mouse pointer
	 * @param multipleSelect true if multiple selections are are wanted, false if existing selection should be cleared
	 * @throws Exception
	 */
	private void select(int mouseX, int mouseY, boolean multipleSelect) throws Exception {
		Handle hitHandle=null;
		boolean firstHit=true;
		double minDistance=0;

		for(Handle handle : handles) {
			Vector3D screenCoordinates=Helper.project(getViewEventHandler().getViewport().getModelViewMatrix(),getViewEventHandler().getViewport().getProjectionMatrix(),handle.position);
			screenCoordinates.X1=getViewEventHandler().getViewport().getWidth()*(screenCoordinates.X1+1)/2.0;
			screenCoordinates.X2=getViewEventHandler().getViewport().getHeight()*(-screenCoordinates.X2+1)/2.0;
			
			double distance=Math.sqrt((screenCoordinates.X1-mouseX)*(screenCoordinates.X1-mouseX)+(screenCoordinates.X2-mouseY)*(screenCoordinates.X2-mouseY));
			if(firstHit || distance < minDistance) {
				firstHit=false;
				minDistance=distance;
				hitHandle=handle;
			}
		}
		
		if(hitHandle==null) {
			return;
		}
		
		State.getInstance().logger().fine("selected element: " + hitHandle.getElementName());

		toggleSelection(hitHandle,multipleSelect,false);
		commitSelectionList();
	}

	private boolean checkInsideLasso(double x, double y) {
		boolean inside=false;
        // 4. determine whether point is inside the face
        for(int i=0;i<lassoPoints.size();i++) {
            if(lassoPoints.get(i).X2<y && lassoPoints.get((i+1)%lassoPoints.size()).X2<y) {
                continue; // edge is entirely below the test line
            }
            if(lassoPoints.get(i).X2>y && lassoPoints.get((i+1)%lassoPoints.size()).X2>y) {
                continue; // edge is entirely above the test line
            }
            // check, if intersection is on the left of the test line
            if(lassoPoints.get(i).X1+(y-lassoPoints.get(i).X2)/(lassoPoints.get((i+1)%lassoPoints.size()).X2-lassoPoints.get(i).X2)*(lassoPoints.get((i+1)%lassoPoints.size()).X1-lassoPoints.get(i).X1)<x) {
                inside=!inside;
            }
        }
		return inside;
	}
	
	private void lassoSelect(boolean multipleSelect) throws Exception {
		boolean firstHit=false;
		for(Handle handle : handles) {
			Vector3D screenCoordinates=Helper.project(getViewEventHandler().getViewport().getModelViewMatrix(),getViewEventHandler().getViewport().getProjectionMatrix(),handle.position);
			screenCoordinates.X1=(screenCoordinates.X1+1)/2.0;
			screenCoordinates.X2=(screenCoordinates.X2+1)/2.0;
			
			// check if inside lasso
			if(checkInsideLasso(screenCoordinates.X1,screenCoordinates.X2)) {
				toggleSelection(handle,multipleSelect,firstHit);
				firstHit=true;
			}
		}
		lassoPoints.clear();
		commitSelectionList();
	}
	
	@Override
	public boolean keyPress(int key, Qt.KeyboardModifiers state) {
		if(key==Qt.Key.Key_Shift.value()) {
			shiftPressed=true;
			return true;
		}
		return false;
	}
	
	@Override 
	public boolean keyRelease(int key, Qt.KeyboardModifiers state) {
		if(key==Qt.Key.Key_Shift.value()) {
			shiftPressed=false;
			return true;
		}
		return false;
	}
	
	@Override
	public boolean mouseButtonDown(Qt.MouseButton button) throws Exception {
		if(button==Qt.MouseButton.LeftButton) {
			dragging=true;
			return true;
		} else if(button==Qt.MouseButton.RightButton)  {
			// try to clear current selection
			try {
				Selection selection=((Document) eu.moonlight3d.framework.State.getInstance().getDocumentManager().getMainDocument()).getSelectionManager().getSelection("viewSelection");
				selection.clearSelection(selection.getType());
			} catch (Exception e) {
				// ignore that exception here - it's minor
			}
		}
		return false;
	}
	
	@Override
	public boolean mouseButtonUp(Qt.MouseButton button) throws Exception {
		if(button==Qt.MouseButton.LeftButton) {
			dragging=false;
			if(lassoPoints.size()>0) {
				lassoSelect(shiftPressed);
			} else {
				select(currentX,currentY,shiftPressed);				
			}
			return true;
		}
		return false;
	}
	
	@Override
    public boolean mouseMove(int oldX, int oldY, int newX, int newY) throws Exception {
		if(dragging==true) {
			Vector3D point=new Vector3D();
			point.clear();
			if(lassoPoints.size()==0) {
				point.X1=oldX/(double)getViewEventHandler().getViewport().getWidth();
				point.X2=1-oldY/(double)getViewEventHandler().getViewport().getHeight();
				lassoPoints.add(point.clone());
			}
			point.X1=newX/(double)getViewEventHandler().getViewport().getWidth();
			point.X2=1-newY/(double)getViewEventHandler().getViewport().getHeight();
			lassoPoints.add(point);
		}
		currentX=newX;
		currentY=newY;
        return dragging;
    }

	@Override
	public void drawVisibleStructure(Viewport view, Scene scene) {
		for(Handle handle : handles) {
			handle.draw(view,scene);
		}
	}
	
	@Override
	public void drawOverlay2D(Viewport view, Scene scene) {
		if(!view.hasKeyboardFocus()) {
			return;
		}
		GL gl=view.getGL();
		gl.glColor3d(0.0,0.0,0.0);
		gl.glBegin(GL.GL_LINE_STRIP);
		for(Vector3D point : lassoPoints) {
			gl.glVertex2d(point.X1,point.X2);
		}
		gl.glEnd();
	}
}
