/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Feb 16, 2006
 */
package eu.moonlight3d.ml3d.ui.resourcelist;
/**
 * \package eu.moonlight3d.ml3d.modelling.textures
 *
 * This plugin contains the texture material and related tools.
 * 
 * This package is a plugin depending on eu.moonlight3d.ml3d.plugins.modelliig.map,
 * eu.moonlight3d.ml3d.plugins.ui.view, eu.moonlight3d.ml3d.rendering.simple
 */

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {
	public String getName() {
		return "eu.moonlight3d.ml3d.ui.resourcelist.Plugin";
	}

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.ml3d.backend.selection.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.backend.threedim.Plugin");
		dependencyList.add("eu.moonlight3d.ml3d.ui.mainwindow.Plugin");
		return dependencyList;
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
        if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();

			manager.registerAction(new OpenResourceListAction(null));
			
			MenuNode contribution = new MenuNode(MenuNode.Type.Contribution, "Resource List", "fileMenuExtensionPoint");
			contribution.add(new MenuNode(MenuNode.Type.Entry, L10N.translate("Open Resource List..."), "OpenResourceListAction"));
			contribution.add(new MenuNode(MenuNode.Type.Separator, "", ""));
			state.getUIManager().getLayout("mainwindow").addMenuContribution(contribution);

			manager.registerMainWidgetFactory(new ResourceListFactory());
			manager.addOrReplaceLayout(new FrameLayout("resourcelist",L10N.translate("Resource List")));
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"ResourceListMenu","menubar");
			MenuNode fileMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Resource List"),"");
			fileMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Close"),"CloseCurrentFrameAction"));
			contribution.add(fileMenu);
			manager.getLayout("resourcelist").addMenuContribution(contribution);
        }
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			state.getUIManager().unregisterAction("OpenResourceListAction");
		}
	}

}
