/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Dec 6, 2008
 */

package eu.moonlight3d.ml3d.ui.resourcelist;

import java.util.ArrayList;
import java.util.List;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QListWidget;
import com.trolltech.qt.gui.QListWidgetItem;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;
import eu.moonlight3d.ml3d.backend.threedim.document.Document;
import eu.moonlight3d.ml3d.backend.threedim.resource.Resource;
import eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManager;

public class ResourceList extends QWidget {
	private class ResourceManagerListener implements eu.moonlight3d.ml3d.backend.threedim.resource.ResourceManagerListener {

		public void resourceAdded(Resource resource) {
			updateResourcesList();
		}

		public void resourceRemoved(Resource resource) {
			updateResourcesList();
		}
		
	}
	
	private ResourceManagerListener resourceManagerListener;
	
	private class LifecycleListener implements eu.moonlight3d.framework.document.LifecycleListener {
		
		public LifecycleListener() {
			registerResourceManagerListener();
		}
		
		private void registerResourceManagerListener() {
			Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
			document.getResourceManager().registerResourceManagerListener(resourceManagerListener);
		}
		
		private void unregisterResourceManagerListener() {
			Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
			document.getResourceManager().unregisterResourceManagerListener(resourceManagerListener);
		}

		public void afterNew() {
			registerResourceManagerListener();
		}

		public void afterOpen(String filename) {
			registerResourceManagerListener();
		}

		public void afterSave(String filename) {
		}

		public void beforeNew() {
		}

		public void beforeOpen(String filename) {
		}

		public void beforeSave(String filename) {
		}

		public void onClose() {
			unregisterResourceManagerListener();
		}
		
	}
	
	private LifecycleListener lifecycleListener;
	
	private QListWidget resourcesList;
	private QPushButton deleteResourceButton;
	
	public ResourceList(QWidget parent) {
		super(parent);
		
		resourcesList=new QListWidget(this);
		deleteResourceButton=new QPushButton(this);
		deleteResourceButton.setText(L10N.translate("Delete Resource"));
		deleteResourceButton.clicked.connect(this,"deleteResource()");
		
		QVBoxLayout layout=new QVBoxLayout(this);		
		layout.addWidget(resourcesList);
		layout.addWidget(deleteResourceButton);
		layout.setAlignment(deleteResourceButton, Qt.AlignmentFlag.AlignHCenter);
		setLayout(layout);
		
		resourceManagerListener=new ResourceManagerListener();
		lifecycleListener=new LifecycleListener();

		State.getInstance().getDocumentManager().addLifecycleListener(lifecycleListener);
		
		updateResourcesList();
	}
	
	private void updateResourcesList() {
		resourcesList.clear();
		
		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		
		List<Resource> resources=document.getResourceManager().getResources();
		for(Resource resource : resources) {
			resourcesList.addItem(resource.getName());
		}
	}
	
	public void deleteResource() {
		ArrayList<String> referencedResources=new ArrayList<String>();

		Document document=(Document) State.getInstance().getDocumentManager().getMainDocument();
		ResourceManager resourceManager=document.getResourceManager();
		
		for(QListWidgetItem item : resourcesList.selectedItems()) {
			String resourceName=item.text();
			
			Resource resource=resourceManager.getResource(resourceName);
			
			if(!resourceManager.canRemoveResource(resource)) {
				referencedResources.add(resourceName);
			} else {
				try {
					resourceManager.removeResource(resourceName);
				} catch (Exception e) {
					ExceptionDialog.run(e);
					return;
				}
			}
		}
		
		if(referencedResources.size()!=0) {
			QMessageBox messageBox=new QMessageBox(this);
			
			String message=L10N.translate("Some resources could not be deleted because they were still referenced:");
			
			messageBox.setWindowTitle(L10N.translate("Unable to delete resources"));
			messageBox.setText(message);

			message=L10N.translate("The following resources were not deleted:\n\n");
			
			for(String resourceName : referencedResources) {
				message=message+resourceName + "\n";
			}
			messageBox.setDetailedText(message);
			messageBox.setStandardButtons(QMessageBox.StandardButton.Ok);
				
			messageBox.show();
		}
	}
	
	@Override
	protected void disposed() {
		lifecycleListener.unregisterResourceManagerListener();
		State.getInstance().getDocumentManager().removeLifecycleListener(lifecycleListener);
	}
}
