/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Aug 12, 2007
 */

package eu.moonlight3d.graph.layout;

import eu.moonlight3d.graph.GraphModel;
import eu.moonlight3d.graph.Link;
import eu.moonlight3d.graph.Node;

/**
 * Basic interface for all graph layouting algorithms. Implementations
 * of this interface implement varying strategies.
 *
 * @author gregor
 */
public interface Layouter {
	/**
	 * Compute a new layout for the given graph.
	 * 
	 * @param model the model to layout
	 */
	public void layout(GraphModel model);
	
	/**
	 * Place a new node in the graph.
	 * 
	 * @param model the model to place the node in
	 * @param node the new node to place
	 */
	public void place(GraphModel model, Node node);
	
	/**
	 * Insert a new link in the graph, eventually moving
	 * nodes to avoid excessive overlapping.
	 *  
	 * @param model the model to insert the link in
	 * @param link the new link to add
	 */
	public void insertLink(GraphModel model, Link link);
}
