/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Aug 12, 2007
 */

package eu.moonlight3d.graph.test;

import java.util.ArrayList;

import eu.moonlight3d.graph.GraphModel;
import eu.moonlight3d.graph.Link;
import eu.moonlight3d.graph.Node;
import eu.moonlight3d.graph.Slot;

/**
 * Helper class for generating test graphs.
 *
 * @author gregor
 */
public class GraphModelGenerator {
	public static void generateLinearGraph(GraphModel model, int numSegments, int segmentLength) {
		model.clearGraph();
		
		ArrayList<Node> nodes=new ArrayList<Node>();
		for(int i=0;i<numSegments;i++) {
			for(int j=0;j<segmentLength;j++) {
				Node node=new Node("node-" + i + "-" + j);
				model.addNode(node);
				Slot input=new Slot(node,"input",Slot.Type.Input);
				node.addSlot(input);
				Slot output=new Slot(node,"output",Slot.Type.Output);
				node.addSlot(output);
				nodes.add(node);
			}
		}
		
		for(int i=0;i<numSegments;i++) {
			for(int j=i*segmentLength;j<(i+1)*segmentLength-1;j++) {
				Slot startSlot=nodes.get(j).getSlot("output");
				Slot endSlot=nodes.get(j+1).getSlot("input");
				model.addLink(startSlot, endSlot);
			}
		}
	}
	
	public static void generateBranchingGraph(GraphModel model, int numStars) {
		model.clearGraph();

		for (int i = 0; i < numStars; i++) {
			Slot input, output;

			Node topLeft = new Node("topLeft-" + i);
			input = new Slot(topLeft, "input", Slot.Type.Input);
			topLeft.addSlot(input);
			output = new Slot(topLeft, "output", Slot.Type.Output);
			topLeft.addSlot(output);
			
			Node topLeft2 = new Node("topLeft-2-" + i);
			input = new Slot(topLeft2, "input", Slot.Type.Input);
			topLeft2.addSlot(input);
			output = new Slot(topLeft2, "output", Slot.Type.Output);
			topLeft2.addSlot(output);

			Node bottomLeft = new Node("bottomLeft-" + i);
			input = new Slot(bottomLeft, "input", Slot.Type.Input);
			bottomLeft.addSlot(input);
			output = new Slot(bottomLeft, "output", Slot.Type.Output);
			bottomLeft.addSlot(output);
			Node topRight = new Node("topRight-" + i);
			input = new Slot(topRight, "input", Slot.Type.Input);
			topRight.addSlot(input);
			output = new Slot(topRight, "output", Slot.Type.Output);
			topRight.addSlot(output);
			Node bottomRight = new Node("bottomRight-" + i);
			input = new Slot(bottomRight, "input", Slot.Type.Input);
			bottomRight.addSlot(input);
			output = new Slot(bottomRight, "output", Slot.Type.Output);
			bottomRight.addSlot(output);
			Node center = new Node("center-" + i);
			input = new Slot(center, "input1", Slot.Type.Input);
			center.addSlot(input);
			input = new Slot(center, "input2", Slot.Type.Input);
			center.addSlot(input);
			output = new Slot(center, "output1", Slot.Type.Output);
			center.addSlot(output);
			output = new Slot(center, "output2", Slot.Type.Output);
			center.addSlot(output);

			model.addNode(topLeft);
			model.addNode(topLeft2);
			model.addNode(topRight);
			model.addNode(bottomLeft);
			model.addNode(bottomRight);
			model.addNode(center);

			model.addLink(topLeft.getSlot("output"), topLeft2.getSlot("input"));
			model.addLink(topLeft2.getSlot("output"), center.getSlot("input1"));
			model.addLink(bottomLeft.getSlot("output"), center.getSlot("input2"));
			model.addLink(center.getSlot("output1"), topRight.getSlot("input"));
			model.addLink(center.getSlot("output2"), bottomRight.getSlot("input"));
		}
	}
}
