/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * Created on Apr 16, 2005
 */
package eu.moonlight3d.ml3d.modelling.mesh;

import java.util.ArrayList;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.ml3d.backend.threedim.og.Node;
import eu.moonlight3d.ml3d.backend.threedim.og.Slot;
import eu.moonlight3d.ml3d.backend.threedim.sg.SceneGraph;


/**
 * The Bevel operator bevels edges of mesh objects
 *
 * @author gregor
 */
public class Triangulate extends Node {
	/**
	 * Name of the mesh to triangulate
	 */
    private Property name;

    /**
     * Default constructor
     */
    public Triangulate()
    {
    	super();
        createSlot("input",Slot.Type.Input);
        createSlot("output",Slot.Type.Output);
        name=new Property(this);
        name.setName("name");
        name.setType(Property.Type.String);
        try {
            name.setValue("Box");
        } catch (Exception e) {
            // This exception shouldn't occur at all in this limited case
            e.printStackTrace();
        }
    }

    @Override
    public void update() throws Exception
    {
    	SceneGraph graph;
        eu.moonlight3d.ml3d.backend.threedim.sg.Node node;

        graph=getSlot("input").getGraph();
        node=graph.getNodeByName(name.getStringValue());
        if(node instanceof eu.moonlight3d.ml3d.modelling.mesh.Mesh) {
        	Mesh mesh=(Mesh) node;
        	int numOldFaces=mesh.getNumFaces();
        	ArrayList<Integer> triangles;
        	ArrayList<ArrayList<Integer>> triangleEdges=new ArrayList<ArrayList<Integer>>();

        	for(int i=0;i<mesh.getNumFaces();i++) {
        		Mesh.Face face=mesh.getFace(i);
        		triangles=mesh.triangulated(face);
        		for(int j=0;j<triangles.size();j+=3) {
            		ArrayList<Integer> edges=new ArrayList<Integer>();
        			edges.add(mesh.findOrCreateEdge(triangles.get(j), triangles.get(j+1)));
        			edges.add(mesh.findOrCreateEdge(triangles.get(j+1), triangles.get(j+2)));
        			edges.add(mesh.findOrCreateEdge(triangles.get(j+2), triangles.get(j)));
            		triangleEdges.add(edges);
        		}
        	}

        	for(ArrayList<Integer> edges : triangleEdges) {
        		mesh.addFace(edges);
        	}

        	// exploit the fact that face IDs are continous
        	for(int faceID=numOldFaces-1;faceID>=0;faceID--) {
        		mesh.removeFace(faceID);
        	}
        } else {
            throw new Exception(L10N.translate("triangulation operator can only work on mesh objects"));
        }
        getSlot("output").setGraph(graph);
    }

    @Override
	public void removeDependencies() {
	}

}
